/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/
#ifndef _M3UAAPI_H_
#define _M3UAAPI_H_

/**************************************************************************
             SS7 MTP User Adaption Layer 3 Interface Specification

          File m3uaapi.h : Definitions & Function Prototypes
**************************************************************************/

#include "ss7type.h"

#ifdef UNIX
#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif
#endif

/* define the calling conventions for the API functions for various
 * operating systems - NOTE: API callers should never define the
 * preprocessor constant "_TX_M3UA_SOURCE_"; this is only for use
 * by the M3UA API source code itself.
 */

/* OS2: M3UA API is a DLL */
#ifdef __OS2__
#ifdef _TX_M3UA_SOURCE_
	/* M3UA API source definition */
#define     TXM3UAFUNC               extern _export _syscall
#else
	/* M3UA API caller's definition */
#define     TXM3UAFUNC               _syscall
#endif	/* _TX_M3UA_SOURCE_ */

/* WIN32: M3UA API is a DLL */
#elif  WIN32	                        /* WIN32 */
#ifdef _TX_M3UA_SOURCE_
	/* M3UA API source definition */
#define     TXM3UAFUNC               __declspec( dllexport ) WINAPI
#else
	/* M3UA API caller's definition */
#define     TXM3UAFUNC               WINAPI
#endif	/* _TX_M3UA_SOURCE_ */

/* all others: statically linked library */
#else
#define     TXM3UAFUNC
#endif

/* Miscellaneous values */
#define     MAXDATA         272     /* Max length of data to transmit */
#define     M3UA_PRESERVE   80

/* Values for Msg Indications */  /* JK TODO need to change these values */
#define M3UA_BIND_REQ       0x04
#define M3UA_BIND_CFM       0x05
#define M3UA_UBIND_REQ      0x08
#define M3UA_UBIND_CFM      0x09
#define M3UA_DATA_REQ       0x18
#define M3UA_FLOW_REQ       0x28
#define M3UA_XSTAT_REQ      0x38
#define M3UA_DATA_IND       0x1A
#define M3UA_XSTAT_IND      0x3A
#define M3UA_STAT_IND       0x7A
#define M3UA_STAT_REQ       0x80    /* Status Request */
#define M3UA_STAT_CFM       0x81    /* Status Confirm */

/* Values for the status field:                                     */
#define StatPaused              1    /* Paused                      */
#define StatResumed             2    /* Resumed                     */
#define StatCongested           3    /* Network Congestion          */
#define StatUsrUnavail          4    /* Remote User Unavailable     */
#define StatRestart             5    /* Restart Begins              */
#define StatRestartEnds         6    /* Restart Ends                */
#define StatCongestionEnds      7    /* Network Congestion Ends     */
#define StatPrimary             8    /* Now primary                 */
#define StatBackup              9    /* Now backup                  */
#define StatStandAlone         10    /* Now standalone              */
#define StatFlcStart           11    /* Flow control start          */
#define StatFlcEnd             12    /* Flow control end            */
#define StatAssociation        14    /* Association                 */

/* M3UA API error codes */
#define M3UA_SUCCESS            0    /* request successful */
#define M3UA_NO_MSG             1    /* no message waiting */
#define M3UA_NOT_BOUND          -1   /* M3UA primitive called before bind */
#define M3UA_OSERROR            -2   /* OS-specific error */
#define M3UA_INVBOARD           -3   /* Board number invalid */

/***** M3UA structure definitions *****/
typedef struct boardInfo
{
    TX_HANDLE   fd;
    Bool        boardState;
    U8          dprChan;
} BoardInfo;

typedef struct bind_req_s {
  U8      code;          /* bind request packet type 0x04           */
  U8      srvInfo;       /* value for the Service Information Octet */
  S16     sapId;         /* SAP identifier                          */
  S16     suId;          /* Service user identifier                 */
  U16     spare2;        /* Alignment                               */
} BIND_REQ;

typedef struct bind_cfm_s {
  U8      code;          /* bind request packet type 0x05           */
  U8      spare1;        /* Alignment                               */
  S16     suId;          /* SAP identifier                          */
  S16     spId;          /* Service user identifier                 */
  U16     spare2;        /* Alignment                               */
} BIND_CFM;

typedef struct data_req_s {
  U8      code;          /* data request packet type 0x18           */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  U32     opc;           /* the originating point code              */
  U32     dpc;           /* the destination point code              */
  U8      srvInfo;       /* value for the Service Information Octet */
  U8      lnkSel;        /* Signaling Link Selection field          */
  U8      priority;      /* Message priority                        */
  U8      spare2;        /* Alignment                               */
  U8      data[MAXDATA]; /* the data to be transmitted              */
} DATA_REQ;

typedef struct flow_req_s {
  U8      code;          /* flow control request packet type 0x28   */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  S16     action;        /* enable or disable flow control          */
  U16     spare2;        /* Alignment                               */
} FLOW_REQ;

typedef struct data_ind_s {
  U8      code;          /* data indication packet type 0x1A        */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  U32     opc;           /* the originating point code              */
  U32     dpc;           /* the destination point code              */
  U8      srvInfo;       /* the Service Information Octet           */
  U8      lnkSel;        /* the Signaling Link Selector field       */
  U16     spare2;        /* Alignment                               */
  U8      data[MAXDATA]; /* the data received                       */
} DATA_IND;

#define     DATA_IND_HDR_SIZE   12      /* Don't count spares because they're not packed.  Don't count code either. */

typedef struct stat_ind_s {
  U8      code;          /* status indication packet type 0x7A      */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  U32     pc;            /* the Point Code this status is about     */
  U32     opc;           /* the Originating Point Code this status is about     */
  S16     status;        /* status indicator value                  */
  U8      priority;      /* priority of the indication              */
  U8      spare2;        /* Alignment                               */
} STAT_IND;

typedef struct assoc_ind_s {
  U8      code;          /* status indication packet type 0x7A      */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  U32     associd;       /* Association id                          */
  U32     assoc_state;   /* M3UA_ASSOC_DOWN or M3UA_ASSOC_ACTIVE    */
  S16     status;        /* status type (StatAssociation)           */
  U8      spare2;        /* unused                                  */
  U8      spare3;        /* Alignment                               */
} ASSOC_IND;


/* status confirm */
typedef struct stat_cfm {
  U8      code;          /* status indication                       */
  U8      spare1;        /* Alignment                               */
  S16     suId;          /* SU id                                   */
  U32     dpc;           /* DPC for which status is returned        */
  U16     status;        /* Status of the dpc                       */
  U8      congLevel;     /* Congestion level                        */
  U8      spare2;        /* Spare for future use                    */
} STATUS_CFM;

#ifdef __cplusplus
extern "C" {
#endif

OS_HSYNC TXM3UAFUNC M3UASyncObj( U8 board, S16 *errval );

/* Establish a connection to the M3UA process */
short TXM3UAFUNC M3UABind(U8 board, U8 srvInfo, S16 sapId, U8 srcEnt,
                          U8 srcInst, S16 suId);

/* Break a connection to the M3UA process */
short TXM3UAFUNC M3UAUnbind(U8 board);

/* Send data to a remote node */
short TXM3UAFUNC M3UASendData( U8 board, S16 sapId, U32 opc, U32 dpc,
                               U8 lnkSel, U8 priority,
                               U8 *data, S16 length, U8 srvInfo );

/* Flow control the service access point */
short TXM3UAFUNC M3UAFlow( U8 board, S16 sapId, S16 action );
/* PC Status Request */
short TXM3UAFUNC M3UAPcStatus( U8 board, S16 sapId, U32 pc );

/* wait for a message to come in */
short TXM3UAFUNC M3UARetrieveMessage(U8 board, void *pMsgInd, U16 *Length);

#ifdef __cplusplus
}
#endif

#endif  /* _M3UAAPI_H_ */
