/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/*
Fordingbridge change history:

    22-Nov-12   MH      - CN028TXA - 'MTP3 + M3UA struct name clash'
                          Remove duplicate structures.
                          Include mtp3def.h.
*/

#ifndef _M3UADEF_H_
#define _M3UADEF_H_

/**************************************************************************
             SS7 MTP User adaption Layer 3 Interface Specification

          File m3uaapi.h : Definitions & Function Prototypes
**************************************************************************/

#include "nmstypes.h"
#include "ctadef.h"
#include "mtp3def.h"

#ifdef UNIX
#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif
#endif

/* define the calling conventions for the API functions for various
 * operating systems - NOTE: API callers should never define the
 * preprocessor constant "_TX_M3UA_SOURCE_"; this is only for use
 * by the M3UA API source code itself.
 */

/* OS2: M3UA API is a DLL */
#ifdef __OS2__
#ifdef _TX_M3UA_SOURCE_
	/* M3UA API source definition */
#define     TXM3UAFUNC               extern _export _syscall
#else
	/* M3UA API caller's definition */
#define     TXM3UAFUNC               _syscall
#endif	/* _TX_M3UA_SOURCE_ */

/* WIN32: M3UA API is a DLL */
#elif  WIN32	                        /* WIN32 */
#ifdef _TX_M3UA_SOURCE_
	/* M3UA API source definition */
#define     TXM3UAFUNC               __declspec( dllexport ) WINAPI
#else
	/* M3UA API caller's definition */
#define     TXM3UAFUNC               WINAPI
#endif	/* _TX_M3UA_SOURCE_ */

/* all others: statically linked library */
#else
#define     TXM3UAFUNC
#endif

/* Miscellaneous values */
#define     MAXDATA         272     /* Max length of data to transmit */
#define     M3UA_PRESERVE   80

/* Values for Msg Indications */
#define M3UA_BIND_REQ       0x04
#define M3UA_BIND_CFM       0x05
#define M3UA_UBIND_REQ      0x08
#define M3UA_UBIND_CFM      0x09
#define M3UA_DATA_REQ       0x18
#define M3UA_FLOW_REQ       0x28
#define M3UA_DATA_IND       0x1A
#define M3UA_STAT_IND       0x7A
#define M3UA_STAT_REQ       0x80    /* Status Request */
#define M3UA_STAT_CFM       0x81    /* Status Confirm */


/* Values for the status field:                                     */
#define StatPaused              1    /* Paused                      */
#define StatResumed             2    /* Resumed                     */
#define StatCongested           3    /* Network Congestion          */
#define StatUsrUnavail          4    /* Remote User Unavailable     */
#define StatRestart             5    /* Restart Begins              */
#define StatRestartEnds         6    /* Restart Ends                */
#define StatCongestionEnds      7    /* Network Congestion Ends     */
#define StatPrimary             8    /* Now primary                 */
#define StatBackup              9    /* Now backup                  */
#define StatStandAlone         10    /* Now standalone              */
#define StatFlcStart           11    /* Flow control start          */
#define StatFlcEnd             12    /* Flow control end            */
#define StatAssociation        14    /* Association                 */

/***** M3UA structure definitions *****/
/* Note - some structures areshared with mtp3. */



typedef struct assoc_ind_s {
  U8      code;          /* status indication packet type 0x7A      */
  U8      spare1;        /* Alignment                               */
  S16     sapId;         /* SAP identifier                          */
  U32     associd;       /* Association id                          */
  U32     assoc_state;   /* M3UA_ASSOC_DOWN or M3UA_ASSOC_ACTIVE    */
  S16     status;        /* status type (StatAssociation)           */
  U8      spare2;        /* unused                                  */
  U8      spare3;        /* Alignment                               */
} ASSOC_IND;

/* status confirm */
typedef struct stat_cfm {
  U8      code;          /* status indication                       */
  U8      spare1;        /* Alignment                               */
  S16     suId;          /* SU id                                   */
  U32     dpc;           /* DPC for which status is returned        */
  U16     status;        /* Status of the dpc                       */
  U8      congLevel;     /* Congestion level                        */
  U8      spare2;        /* Spare for future use                    */
} STATUS_CFM;


/****************************************************************************/
/* M3UA API Statistics returned by M3UAGetApiStats()                        */
/****************************************************************************/
typedef struct
{
    U32 qCount;             /* number of API messages currently queued to M3UA layer */
    U32 qPeak;              /* max number of API messages ever queued to M3UA layer */
    U32 txPending;          /* current number of outstanding transmit rqsts to M3UA layer */
    U32 txPendPeak;         /* max number of transmit rqsts ever outstanding to M3UA layer */
    U32 txSuccess;          /* number of successful transmit requests completed */
    U32 txFailed;           /* number of failed transmit requests */
    U32 txLastErr;          /* error code from last failed transmit request */
    U32 rxSuccess;          /* number of events received from M3UA layer */
    U32 rxFailed;           /* number of receive failure events from M3UA layer */
    U32 rxLastErr;          /* error code from last failed receive request */
    U8  apiQCongLvl;        /* current outbound queue congestion level [0..3] */
    U8  spare1;             /* spare for alignment */
    U16 spare2;             /* spare for alignment */
} M3UAAPISTATS;

/* Bind flag states */
#define M3UA_BIND_FLAG_OFF      0
#define M3UA_BIND_FLAG_ON       1
#define M3UA_UNBIND_FLAG_OFF    2
#define M3UA_UNBIND_FLAG_ON     3

/*-----------------------------------------------------------------------------
  M3UA Version Ids
    - These id's are used to identify the revision level of this service.
  ---------------------------------------------------------------------------*/
#define M3UA_MAJORREV                        1
#define M3UA_MINORREV                        0

/*-----------------------------------------------------------------------------
  M3UA API Compatibility Level Id
    - This id is used to determine runtime compatibility between the
      installed M3UA service and clients of the M3UA service. Clients can
      be a CT Access application which accesses M3UA functionality
      via the M3UA API.
    - Clients(CT Access application) #include this file into their code and,
      therefore, insert "hard coded" values for each M3UA compat level
      into their client code.
    - The API_COMPATLEVEL is used by CT Access application developers who
      want to check the "hard coded" value in their application against the
      "hard coded" value in the installed M3UA service (as determined
      via ctaGetServiceVersion() will be available in CT Access Version 2.1).
    - API compatibility level value should be incremented when the associated
      source module changes in a non-backwards compatible way. Refer to the CT
      Access Service Writer's Manual for more info.
  ---------------------------------------------------------------------------*/
#define M3UAAPI_COMPATLEVEL                  1

#define	M3UA_SVCID	0x36

/****************************************************************************/
/* important constants and common data structures                           */
/****************************************************************************/

/* Event generated by M3UA Service and sent back to client for all strings  */
#define M3UAEVN_DATA    ((M3UA_SVCID << 16) | 0x2000 | 0x01)     /* 0x212001   */
#define M3UAEVN_CONGEST ((M3UA_SVCID << 16) | 0x2000 | 0x02)     /* 0x212002   */

/* M3UA - return code definitions */
#define M3UA_SUCCESS    0   /* API completed successfully; the same as CT Access success  */
#define M3UA_NO_MSG     ((M3UA_SVCID << 16) | 0x01)  /*0x210001 no message waiting                */
#define M3UA_NOT_BOUND  ((M3UA_SVCID << 16) | 0x02)  /*0x210002 M3UA primitive called before bind */
#define M3UA_OSERROR    ((M3UA_SVCID << 16) | 0x03)  /*0x210003 OS-specific error                 */
#define M3UA_INVBOARD   ((M3UA_SVCID << 16) | 0x04)  /*0x210004 Board number invalid              */
#define M3UA_RESOURCE   ((M3UA_SVCID << 16) | 0x05)  /*0x210005 Out of resources                  */

/*-----------------------------------------------------------------------------
  M3UA Reason Codes.
    - These codes are constructed using the M3UA_SVCID, the CT Access
      CTA_CODE_REASON id, and a 1-based sequence number. The formula is
        REASONCODE = ((M3UA_SVCID<<16) | (CTA_CODE_REASON | SEQUENCE))
  ---------------------------------------------------------------------------*/

#define M3UA_REASON_UNKNOWN      ((M3UA_SVCID << 16) | 0x1000 | 0xff)     /* 0x2110FF unknown reason */




#ifdef __cplusplus
extern "C" {
#endif


DWORD NMSAPI M3UASendData       (CTAHD ctahd, S16 sapId, U32 opc, U32 dpc, U8 lnkSel, U8 priority,
                                   U8 *data, S16 length, U8 srvInfo );                                  /* Send data to a remote node */
DWORD NMSAPI M3UAFlow           (CTAHD ctahd, S16 sapId, S16 action );                                  /* Flow control the service access point */
DWORD NMSAPI M3UARetrieveMessage(CTAHD ctahd, void *pMsgInd, U16 *Length);                              /* wait for a message to come in */
DWORD NMSAPI M3UAGetApiStats    (CTAHD ctahd, M3UAAPISTATS *pStats, U8 reset);
DWORD NMSAPI M3UAPcStatus       (CTAHD ctahd, S16 sapId, U32 pc);

#ifdef __cplusplus
}
#endif

#endif  /* _M3UAAPI_H_ */
