/*
         Copyright (C) Dialogic Corporation 2008-2013. All Rights Reserved.

 -----  ---------  -----  ---------------------------------------------------
 Issue    Date      By                    Changes
 -----  ---------  -----  ---------------------------------------------------
        02-Jul-13   JLP   CN042TXA - add PSP options and option to turn off RC
        28-Aug-13   IDP   CN044TXA - Replace pointer with fixed size integer to
                                     avoid issues on 64bit machines
 */

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  (c)Copyright 2008-2009 Dialogic Corporation.  All Rights Reserved   **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

#ifndef _M3UAMAPI_H_
#define _M3UAMAPI_H_

#include "ss7type.h"
#include "sigtran.h"
#include "txcpi.h"

/* set all defines that the M3UA management interface was built with */
#ifndef ITASP
#define ITASP       /* ASP function of M3UA is provided */
#endif /* ITASP */

/* define the calling conventions for the API functions for various
 * operating systems - NOTE: API callers should never define the
 * preprocessor constant "_TX_M3UA_SOURCE_"; this is only for use
 * by the M3UA API source code itself.
 */

/* OS2: M3UA API is a DLL */
#ifdef __OS2__
#ifdef _TX_M3UA_SOURCE_
/* MTP3 API source definition */
#define     TXM3UAFUNC               extern _export _syscall
#else
/* MTP3 API caller's definition */
#define     TXM3UAFUNC               _syscall
#endif  /* _TX_M3UA_SOURCE_ */

/* WIN32: M3UA API is a DLL */
#elif  WIN32                        /* WIN32 */
#ifdef _TX_M3UA_SOURCE_
    /* M3UA API source definition */
#define     TXM3UAFUNC               __declspec( dllexport ) WINAPI
#else
    /* M3UA API caller's definition */
#define     TXM3UAFUNC               WINAPI
#endif  /* _TX_M3UA_SOURCE_ */

/* all others: statically linked library */
#else
#define     TXM3UAFUNC
#endif

#ifdef UNIX
#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif
#endif

/* Macro for initializing a timer (value 0 => disable) */
#define SET_TIMER( atimer, avalue )     \
    atimer.val = avalue;                \
    if( atimer.val != 0 )               \
        atimer.enb = 1;                 \
    else                                \
        atimer.enb = 0;

/* Constants */
#define ENTSN       0x15                /* MTP3 (M3UA task & cfg ID) */
#define ENTSD       0x16                /* MTP2 (SCTP task & cfg ID) */
#define ENTSB       0x88                /* SCTP MGR ID*/
#define ENTIT       0x89                /* M3UA MGR ID*/
#define ENTSM       0x0E                /* Stack Manager */

#ifndef M3UA_MAX_PSP
#define M3UA_MAX_PSP            128    /* Was 20.  Needed to support more associations.  This is the max power of 2
                                          to keep the M3UAPsSta structure under 1988 (max msg size host to board) */
#endif

#ifndef M3UA_MAX_SEP
#define M3UA_MAX_SEP            1       /* Maximum number of Self End Points */
#endif

#ifndef  M3UA_MAX_RK_IN_DRKM
#define  M3UA_MAX_RK_IN_DRKM    4       /* Maximum number of RKs in RKM msg */
#endif

#ifndef  M3UA_MAX_INFO
#define  M3UA_MAX_INFO          255     /* Maximum size of INFO string */
#endif

#ifndef M3UA_MAX_PSID
#define M3UA_MAX_PSID           128     /* Maximum number PSID was 60 JK changed to 10. JP - now 128 */
#endif

#ifndef  M3UA_MAX_TRC_LEN
#define  M3UA_MAX_TRC_LEN       200     /* Maximum trace size */
#endif

#ifndef M3UA_MAX_ASSOC
#define M3UA_MAX_ASSOC (M3UA_MAX_SEP)*(M3UA_MAX_PSP)    /* Maximum number of assocs
                                                           will be product of max Self End Points
                                                           and max remote PSP */
#endif

#define SCT_MAX_NET_ADDRS       8       /* Max number of Net addreses in an address list. Make multiple of 4 */

#ifndef  M3UA_MAX_OPC_IN_DRKM
#define  M3UA_MAX_OPC_IN_DRKM   4       /* Maximum no of Originating Point Code allowed in RKM msg */
#endif

#ifndef  M3UA_MAX_STATIC_PSID
#define M3UA_MAX_STATIC_PSID    M3UA_MAX_PSID/2     /* Maximum number of static PS. This value
                                                       should be less than M3UA_MAX_PSID */
#endif

#ifndef  M3UA_MAX_ACT_PS
#define  M3UA_MAX_ACT_PS        M3UA_MAX_PSID       /* Max no. of active PS per PSP */

#endif

#ifndef  M3UA_MAX_SIO_IN_DRKM
#define  M3UA_MAX_SIO_IN_DRKM   4       /* Maximum no of Service Info parameters allowed in RKM msg */
#endif
#ifndef  M3UA_MAX_CIC_IN_DRKM
#define  M3UA_MAX_CIC_IN_DRKM   4       /* Maximum no of CIC ranges allowed in RKM msg */
#endif
#ifndef  M3UA_MAX_RC_IN_DRKM
#define  M3UA_MAX_RC_IN_DRKM    4       /* Maximum number of RCs in RKM msg */
#endif

/* Subservice selections */
#define M3UA_SSF_INTL           0       /* international */
#define M3UA_SSF_SPARE          1       /* spare */
#define M3UA_SSF_NAT            2       /* national */
#define M3UA_SSF_RES            3       /* reserved */

/* DPC LEN Definitions, doesn't go by number 14,24,16 ..*/
#define DPC14                   0       /* ccitt blue book dpc length */
#define DPC24                   1       /* ansi and other national variants */
#define DPC_DEFAULT             2       /* dpc length for management message */
#define DPC16                   3       /* for TTC Japan */

/* network appearance code */
#define M3UA_NWKAPP_ITU         1000000     /* ITU network appearance */
#define M3UA_NWKAPP_ITU1        1100000     /* ITU network appearance */
#define M3UA_NWKAPP_NSS         1000001     /* NSS network appearance */
#define M3UA_NWKAPP_ANSI        2000000     /* ANSI network appearance */
/* suSwtch in nwkCfg */
#define M3UA_SW_ANS             1       /* link type ANSI */
#define M3UA_SW_ITU             2       /* link type CCITT */
#define M3UA_SW_CHINA           4       /* link type CHINA */
#define M3UA_SW_BICI            5       /* link type BICI */
#define M3UA_SW_ANS96           6       /* link type ANSI */
#define M3UA_SW_TTC             10      /* link type TTC */
#define M3UA_SW_NTT             11      /* link type NTT */
/* su2Swtch in nwkCfg */
#define M3UA_SW2_UNUSED         0       /* su2Swtch Unused */
#define M3UA_SW2_ITU            1       /* TCAP type ITU */
#define M3UA_SW2_ETS            4       /* TCAP type ETSI */
#define M3UA_SW2_ANS            7       /* TCAP type ANSI */
#define M3UA_SW2_TTC            8       /* TCAP type TTC */

#define M3UA_NWK_ITU            0       /* ITU network  */
#define M3UA_NWK_ANSI           1       /* ANSI network   */
#define M3UA_NWK_NSS            2       /* NSS network  */

/* suType in nSapCfg */
#define M3UA_SU_SCCP            3       /* SCCP user */
#define M3UA_SU_TUP             4       /* TUP user */
#define M3UA_SU_ISUP            5       /* ISUP user */
#define M3UA_SU_DUP             6       /* DUP user */
#define M3UA_SU_DUPF            7       /* DUPF user */
#define M3UA_SU_MTUP            8       /* MTUP user */
#define M3UA_SU_B_ISUP          9       /* B-ISUP user */
#define M3UA_SU_S_ISUP          10      /* S-ISUP user */
#define M3UA_SU_AAL2            12      /* AAL2 user */
#define M3UA_SU_BICC            13      /* BICC user */
#define M3UA_SP_MTP3            0xf     /* MTP3  */
#define M3UA_SU_MGCP            0xe     /* MGCP  */

/* nodeType in genCfg */
#define M3UA_TYPE_SGP           1       /* Signaling Gateway */
#define M3UA_TYPE_ASP           2       /* Application Server Proc */

/* slsLen in nwkCfg */
#define M3UA_SLS4               4       /* SLS length 4 bits */
#define M3UA_SLS5               5       /* SLS length 5 bits */
#define M3UA_SLS8               8       /* SLS length 8 bits */

/* rtType in rteCfg */
#define M3UA_RTTYPE_NOTFOUND    0       /* No route found */
#define M3UA_RTTYPE_LOCAL       1       /* Route to a local user */
#define M3UA_RTTYPE_MTP3        2       /* Route to SS7 via the MTP3 */
#define M3UA_RTTYPE_PS          3       /* Route to a Peer Server */

/* pspType in pspCfg */
#define M3UA_PSPTYPE_LOCAL      0       /* special case for the local entity */
#define M3UA_PSPTYPE_ASP        1       /* remote PSP is a ASP */
#define M3UA_PSPTYPE_SGP        2       /* remote PSP is a SGP */
#define M3UA_PSPTYPE_IPSP       3       /* remote PSP is a IPSP */

/* ipspType in pspCfg */
#define M3UA_IPSPMODE_SE        1       /* Single ended mode IPSP */
#define M3UA_IPSPMODE_DE        2       /* Double ended mode IPSP */

/* rxTxAspId in pspCfg */
#define M3UA_RX_ASPID           0x01    /* ASP Id needs to be present in ASPUP Ack message */
#define M3UA_TX_ASPID           0x02    /* ASP Id needs to be present in ASPUP message */

/* dpc/opc mask in route filter */
#define M3UA_DPC_SPECIFIED      0xFFFFFF    /* All bits set to 1 */

/* loadShareMode in psCfg */
#define M3UA_LOADSH_RNDROBIN    1       /* Round robin */
#define M3UA_LOADSH_SLS         2       /* By link selector */
#define M3UA_LOADSH_CIC         3       /* By CIC */

 /* mode in psCfg */
#define M3UA_MODE_ACTSTANDBY    1       /* Active/standby mode */
#define M3UA_MODE_LOADSHARE     2       /* Loadsharing mode */
#define M3UA_MODE_BROADCAST     3       /* Broadcast mode */

/* aspSt in aspSta - also used in Notify message */
#define M3UA_ASP_UNSUPP        0     /* unsupported */
#define M3UA_ASP_DOWN          1     /* ASP-DOWN */
#define M3UA_ASP_INACTIVE      2     /* ASP-INACTIVE */
#define M3UA_ASP_ACTIVE        3     /* ASP-ACTIVE */

/* hlSt in assocSta */
#define M3UA_ASSOC_DOWN        1     /* association is down */
#define M3UA_ASSOC_ACTIVE      2     /* association is up */
#define M3UA_ASSOC_CONGSTART   3     /* association has started flow control */
#define M3UA_ASSOC_CONGDROP    4     /* association is dropping data */

 /* SCTP IANA port assignments */
#define SCT_PORT_M2UA           2904    /* MTP2 User Adaptation layer */
#define SCT_PORT_M3UA           2905    /* MTP3 User Adaptation layer */
#define SCT_PORT_NBAP           2914    /* NBAP User Adaptation layer */
#define SCT_PORT_NBAP_PEER      2916    /* NBAP User Adaptation layer */
#define SCT_PORT_IUA            9900    /* ISDN User Adaptation layer */
#define SCT_PORT_SUA            14001   /* SCCP User Adaptation layer */
#define SCT_PORT_V5UA           5675    /* V5.2 User Adoptation layer */

 /* net Addr type */
#define CM_NETADDR_IPV4         4
#define CM_NETADDR_IPV6         6

/* Trace mask */
#define M3UA_TRC_SSNM       0x00000001
#define M3UA_TRC_ASPSM      0x00000002
#define M3UA_TRC_ASPTM      0x00000004
#define M3UA_TRC_M3UA_XFER  0x00000008
#define M3UA_TRC_MGMT       0x00000010
#define M3UA_TRC_RKM        0x00000020
#define M3UA_TRC_ALL        (M3UA_TRC_SSNM | M3UA_TRC_ASPSM| M3UA_TRC_ASPTM | \
                            M3UA_TRC_M3UA_XFER | M3UA_TRC_MGMT | M3UA_TRC_RKM)


/* 
 * cfgFlags defines - added to allow backwards compatibility 
 * Note: must match those defined in lit.x
 */
#define M3UA_PSP_OPTIONS_PRESENT  0x0001  


/* PSP options bits */
#define M3UA_PSP_OPT_RC_DISABLE   0x0001



/*
 * configuration packets
 */

typedef struct systemId         /* system id */
{
    S16 mVer;                   /* main version */
    S16 mRev;                   /* main revision */
    S16 bVer;                   /* branch version */
    S16 bRev;                   /* branch revision */
    U32 _ptNmb;                 /* part number */
} SystemId;

/* Net (ip) address structure */
typedef struct NetAddr
{
    U32     type;               /* type of network address */
    union
    {
        U32 ipv4NetAddr;        /* IP network address */
        U8  ipv6NetAddr[16];    /* IPv6 network address */
    }u;
} NetAddr;

/* Array of Network Addresses */
typedef struct _NetAddrLst
{
    U32         nmb;                        /* Number of Network Addresses */
    NetAddr     nAddr[SCT_MAX_NET_ADDRS];   /* List of Network Addresses */
} NetAddrLst;

/*-----------------------------------------------------------------------------------------------------
                                       Configuration structures
 -----------------------------------------------------------------------------------------------------*/

/* M3UA General timers configuration structure */
typedef struct tmrCfg           /* timer configuration structure */
{
    Bool    enb;                /* enable */
    U8      spare1;             /* alignment */
    U16     val;                /* value */
}TmrCfg;

typedef struct _M3UAGenTimerCfg /* M3UA general timer configuration structure */
{
    U8      nmbAspUp1;          /* initial number of ASPUP */
    U8      maxNmbRkTry;        /* initial number of DRKM msgs */
    U16     spare1;             /* alignment */
    TmrCfg  tmrRestart;         /* restart hold-off time */
    TmrCfg  tmrMtp3Sta;         /* MTP3 status poll time */
    TmrCfg  tmrAsPend;          /* AS-PENDING time */
    TmrCfg  tmrHeartbeat;       /* heartbeat period */
    TmrCfg  tmrAspUp1;          /* initial time between ASPUP */
    TmrCfg  tmrAspUp2;          /* steady-state time between ASPUP */
    TmrCfg  tmrAspDn;           /* time between ASPDN */
    TmrCfg  tmrAspM;            /* timeout value for ASPM messages */
    TmrCfg  tmrDaud;            /* time between DAUD */
    TmrCfg  tmrDrkm;            /* time between DRKM msgs */
    TmrCfg  tmrDunaSettle;      /* time to settle DUNA message */
    TmrCfg  tmrSeqCntrl;        /* Sequence Control timer */
} M3UAGenTimerCfg;

/* M3UA General configuration parameters structure */
typedef struct _M3UAGenCfg          /* M3UA general configuration */
{
    U8              nodeType;       /* type of M3UA - SGP or ASP */
    Bool            dpcLrnFlag;     /* DPC learning mode enable */
    Bool            drkmSupp;       /* DRKM supported */
    Bool            drstSupp;       /* DRST supported */
    U16             maxNmbNSap;     /* number of upper SAPs */
    U16             maxNmbSctSap;   /* number of lower SAPs */
    U16             maxNmbNwk;      /* number of Network Appearances */
    U16             maxNmbRtEnt;    /* number of Routing Entries */
    U16             maxNmbDpcEnt;   /* number of DPC entries */
    U16             maxNmbPs;       /* number of Peer Servers */
    #if (defined(ITASP) && defined(OG_RTE_ON_LPS_STA))
    U16             maxNmbLps;      /* number of Local PS */
    #endif /* ITASP && OG_RTE_ON_LPS_STA */
    U16             maxNmbPsp;      /* number of Peer Signalling Processes */
    U16             maxNmbMsg;      /* max MTP3/M3UA messages in transit */
    U16             maxNmbRndRbnLs; /* number of Round Robin Loadshare contexts */
    U16             maxNmbSlsLs;    /* number of SLS based Loadshare contexts */
    U32             maxNmbSls;      /* number of SLS contexts */
    U32             qSize;          /* congestion queue size in M3UA */
    U32             congLevel1;     /* congestion level 1 in the queue */
    U32             congLevel2;     /* congestion level 2 in the queue */
    U32             congLevel3;     /* congestion level 3 in the queue */
    M3UAGenTimerCfg tmr;            /* gen Timer  Config */
    S16             timeRes;        /* timer resolution */
    U16             spare1;         /* alignment */
    U32             debugMask;      /* Debugging mask */
    U32             traceMask;      /* Tracing mask */
    Pst             smPst;          /* Post stack mgr structure */
} M3UAGenCfg;

/* M3UA Network Configuration */
typedef struct _M3UANwkCfg          /* M3UA network configuration */
{
    U8      nwkId;                  /* network ID */
    U8      ssf;                    /* sub service field */
    U8      dpcLen;                 /* dpc or opc length */
    U8      slsLen;                 /* sls length */
    S16     suSwtch;                /* protocol variant of service user */
    S16     su2Swtch;               /* protocol variant of user of service user */
    U32     nwkApp[M3UA_MAX_PSP];   /* network appearance code */
}M3UANwkCfg;

/* SCT SAP configuration */
typedef struct _M3UASctSapCfg       /* SCT SAP configuration */
{
    S16             suId;           /* service user SAP ID */
    U16             srcPort;        /* source port for listening endpoint */
    NetAddrLst      srcAddrLst;     /* source address list */
    TmrCfg          tmrPrim;        /* lower SAP primitive time */
    TmrCfg          tmrSta;         /* congestion poll time */
    MemoryId        mem;            /* memory region and pool ID */
    U8              selector;       /* selector for SCT */
    U8              spare1;         /* alignment */
    U16             procId;         /* processor ID */
    U8              ent;            /* entity */
    U8              inst;           /* instance */
    U8              prior;          /* priority */
    U8              route;          /* route */
    S16             spId;           /* service provider ID */
    U16             spare2;         /* alignment */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    Bool            remIntfValid;   /* remote interface version is valid */
    CmIntfVer       remIntfVer;     /* remote interface version */
#endif /* TDS_ROLL_UPGRADE_SUPPORT */
} M3UASctSapCfg;

/* NSapCfg structure */
typedef struct _M3UANSapCfg         /* M3UA Network SAP configuration */
{
    S16             sapId;          /* NSAP id */
    U8              nwkId;          /* logical network ID */
    U8              suType;         /* service user protocol type */
    MemoryId        mem;            /* memory region and pool id */
    U8              selector;       /* upper layer selector */
    U8              prior;          /* priority */
    U8              route;          /* route */
    U8              spare1;         /* alignment */
#ifdef ITSG
    TmrCfg          tmrPrim;        /* lower SAP primitive timer */
    U16             procId;         /* processor ID */
    U8              ent;            /* entity */
    U8              inst;           /* instance */
    S16             mtp3SapId;      /* MTP3 service provider ID */
#endif
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    Bool            remIntfValid;   /* remote interface version is valid */
    CmIntfVer       remIntfVer;     /* remote interface version */
#endif /* TDS_ROLL_UPGRADE_SUPPORT */
} M3UANSapCfg;

/* Association configuration structure */
typedef struct _AssocCfg          /* SCTP association configuration */
{
    NetAddr         priDstAddr;     /* primary destination address */
    NetAddrLst      dstAddrLst;     /* destination address list */
    U16             dstPort;        /* destination port */
    U16             locOutStrms;    /* number of streams supported */
    Bool            clientSide;     /* If true, we initiate associations (if IPSP) */
    Bool            disabled;       /* Don't start association automatically */
    U8              tos;            /* type of service */
    U8              spare1;         /* Alignment */
} AssocCfg;

/* Peer Signalling Process configuration */
typedef struct _M3UAPspCfg          /* Peer Signalling Process configuration */
{
    U16             pspId;          /* PSP id */
    U8              pspType;        /* PSP client/server type */
    U8              ipspMode;       /* IPSP single ended/double ended type */
    Bool            dynRegRkallwd;  /* PSP is authorized for DRKM mesg */
    U8              dfltLshareMode; /* Default mode for dynamic cfg of ps */
    Bool            nwkAppIncl;     /* Network Appearance flag */
#ifdef LITV10
    U8              rxTxAspId;      /* ASP Id send/receive flags*/
#else
    Bool            rxTxAspId;      /* ASP Id parameter */
#endif
#ifdef LITV11
    U32             selfAspId[M3UA_MAX_SEP];    /* ASP Id to be sent in ASP UP message from respective SEP */
#else
    U32             selfAspId;      /* ASP Id to be sent in ASP UP message */
#endif
    U8              nwkId;          /* Default network ID */
#ifdef ITASP
    Bool            cfgForAllLps;   /* Configure for all local PSs? */
    U16             spare1;         /* alignment */
#ifdef SGVIEW
    U32             sgId;           /* SG Id to which this PSP belongs */
#endif /* SGVIEW */
#endif /* ITASP */
    AssocCfg        assocCfg;       /* Association configuration */
#ifdef LITV10
    Bool            rcIsMand;       /* This is used only for pspType = ASP/IPSP. Is RC mandatory in the rcvd
                                     * ASP_ACTIVE message or not. */
    Bool            modRegRkallwd;  /* Re-registration is supported or not at this PSP */
#endif
    U32             options;        /* PSP options field */
} M3UAPspCfg;

/* peer server config */
typedef struct _M3UAPsCfg           /* Peer Server configuration */
{
    U8              nwkId;          /* Network ID */
    U8              mode;           /* Active/Standby or load sharing */
    U8              loadShareMode;  /* Round robin, SLS mapping etc. */
    Bool            reqAvail;       /* TRUE if required for SPMC availability */
    U32             psId;           /* Peer Server ID */
    U32             routCtx;        /* Routing Context */
    U16             nmbActPspReqd;  /* number of active PSPs sharing load */
    U16             nmbPsp;         /* number of entries in PSP list */
    U16             psp[M3UA_MAX_PSP]; /* ordered list of PSPs */
    Bool            lclFlag;        /* PS type is local if set */
    U8              spare1;         /* alignment */
    U16             spare2;         /* alignment */
} M3UAPsCfg;

/* routing filter structure */
typedef struct _M3UARtFilter        /* M3UA routing filter */
{
    U32             dpcMask;        /* DPC wildcard mask */
    U32             dpc;            /* Destination Point Code */
    U32             opcMask;        /* OPC wildcard mask */
    U32             opc;            /* Originator Point Code */
    U8              slsMask;        /* Link Selector wildcard mask */
    U8              sls;            /* Link Selector */
    U8              sioMask;        /* SIO wildcard mask */
    U8              sio;            /* Service Identifier Octet */
    U16             cicStart;       /* Start of CIC range */
    U16             cicEnd;         /* End of CIC range */
    Bool            includeCic;     /* TRUE = include ISUP CIC in filter */
    Bool            includeSsn;     /* TRUE = include SCCP SSN in filter */
    U8              ssn;            /* Subsystem Number */
    Bool            includeTrid;    /* TRUE = include TCAP TRID in filter */
    U32             tridStart;      /* Start of TCAP Transaction ID range */
    U32             tridEnd;        /* End of TCAP Transaction ID range */
} M3UARtFilter;

/* route entry configuration */
typedef struct _M3UARteCfg          /* Routing entry */
{
    U8              nwkId;          /* network ID */
    U8              rtType;         /* Route Type */
    U16             spare1;         /* alignment */
    M3UARtFilter    rtFilter;       /* M3UA routing filter */
    U32             psId;           /* Peer Server ID */
    Bool            noStatus;       /* TRUE if status suppressed */
    Bool            nSapIdPres;     /* Upper SAP ID present */
    S16             nSapId;         /* Upper SAP ID */
} M3UARteCfg;

/*-----------------------------------------------------------------------------------------------------
                                           Status structures
 -----------------------------------------------------------------------------------------------------*/

/* Solicited status */
typedef struct _M3UAGenSta          /* General status */
{
    U32         memSize;            /* Reserved memory size */
    U32         memAlloc;           /* Allocated memory size */
    U8          haState;            /* High Availability State */
    U8          isolated;           /* Isolation state.  False = connected to mate. */
    U16         spare2;             /* alignment */
} M3UAGenSta;

/* NSap status */
typedef struct _M3UANSapSta         /* Upper SAP status */
{
    S16         lclSapId;           /* local  SAP Id */
    S16         remSapId;           /* remote (MTP3/M3UA-User) SAP Id */
    U8          hlSt;               /* High level state */
    U8          spare1;             /* alignment */
    U16         spare2;             /* alignment */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    U16         selfIntfVer;        /* self interface version */
    Bool        remIntfValid;       /* remote interface valid ? */
    U16         remIntfVer;         /* remote interface version */
#endif /* TDS_ROLL_UPGRADE_SUPPORT */
} M3UANSapSta;

/* Address translation status */
typedef struct _M3UAAtSta           /* Address Translation status */
{
    U16         nmbDpc;             /* Number of entries in DPC table */
    U16         nmbRout;            /* Number of entries in routing table */
} M3UAAtSta;

/* DPC status */
typedef struct _M3UAUDpcSta         /* DPC Status */
{
    U32         dpc;                /* DPC value  */
    U8          nwkId;              /* Network Id */
    U8          dpcSt;              /* DPC status */
    U8          congLevel;          /* DPC Congestion Level */
    U8          spare1;             /* alignment */
} M3UADpcSta;

/* End point status */
typedef struct _M3UAPsStaEndp
{
    U8          aspSt[M3UA_MAX_PSP];    /* PS's assoc states (currently 20) */
    U16         nmbPspReg;              /* number of registered assocs */
    U16         nmbAct;                 /* number of active assocs */
    U16         actPsp[M3UA_MAX_PSP];   /* list of active assocs */
    struct
    {
        U32     rCtx;                   /* routing Context */
        Bool    rcValid;                /* routing context valid flag */
        U8      mode;                   /* Dynamically/Static */
        U16     spare1;                 /* alignment */
    } rteCtx[M3UA_MAX_PSP];
} M3UAPsStaEndp;

/* Peer server status */
typedef struct _M3UAPsSta           /* Peer Server status */
{
    U32             psId;               /* Peer Server Id */
    U8              asSt;               /* PS's AS state */
    U8              spare1;             /* alignment */
    U16             spare2;             /* alignment */
    M3UAPsStaEndp   psStaEndp[M3UA_MAX_SEP];    /* Status per endpoint */
} M3UAPsSta;

/* Association status */
typedef struct _AssocSta          /* Association status */
{
    U32         spAssocId;              /* Association Id */
    U8          hlSt;                   /* High level state */
    U8          aspSt;                  /* PSP's ASP state */
    Bool        inhibited;              /* Management inhibit status */
    U8          spare1;                 /* alignment */
    U16         nmbAct;                 /* number of active PSs */
    U16         nmbRegPs;               /* number of registered PSs */
    U32         actPs[M3UA_MAX_ACT_PS]; /* list of active PSs */
    U32         regPs[M3UA_MAX_PSID];   /* Registered PSs List */
} AssocSta;

/* Peer signaling process status */
typedef struct _M3UAPspSta              /* Peer Signalling Process status */
{
    U16         pspId;                  /* PSP Id */
    U16         spare1;                 /* alignment */
    AssocSta    assocSta[M3UA_MAX_SEP]; /*status of PSP's all associations*/
} M3UAPspSta;

/* SCT Sap status */
typedef struct _M3UASctSapSta           /* lower SAP status */
{
    S16             suId;               /* Service user SAP Id */
    U8              hlSt;               /* High level state */
    U8              spare1;             /* alignment */
    U32             spEndpId;           /* SP ID of my endpoint */
    U16             nmbActAssoc;        /* Number of active associations */
    U16             spare2;             /* alignment */
#ifdef LITV4
    NetAddrLst      srcAddrLst;         /* source address list */
#endif /* LITV4 */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    U16             selfIntfVer;        /* self interface version */
    Bool            remIntfValid;       /* remote interface valid ? */
    U16             remIntfVer;         /* remote interface version */
#endif /* TDS_ROLL_UPGRADE_SUPPORT */
} M3UASctSapSta;

/* Routing key status */
typedef struct _M3UARkSta               /* Dynamic Routing Key status */
{
    U16             nmbRkReg;           /* Num of RK dynamically registered */
    U16             spare1;             /* alignment */
} M3UARkSta;

/* PSP routing keys status */
typedef struct _M3UAPspRkIdSta          /* PSP's RK-Ids awaiting response */
{
    U16             pspId;              /* PSP Id */
    S16             sctSuId;            /* SCT SAP SU Id */
    U32             nmbRkReqPend;       /* nmb of RKs req pending */
    U32             lclRkId[M3UA_MAX_RK_IN_DRKM];   /* RK-ids */
} M3UAPspRkIdSta;

/*-----------------------------------------------------------------------------------------------------
                                        Statistics structures
 -----------------------------------------------------------------------------------------------------*/

/* MTP3 statistics */
typedef struct _Mtp3Sts
{
    U32         data;           /* MTP3 Data primitives */
    U32         pause;          /* MTP3 Pause status primitives */
    U32         resume;         /* MTP3 Resume status primitives */
    U32         cong;           /* MTP3 Cong status primitives */
    U32         drst;           /* MTP3 Restrict status primitive */
    U32         rstBeg;         /* MTP3 Reset begin status primitives */
    U32         rstEnd;         /* MTP3 Reset end status primitives */
    U32         upu;            /* MTP3 UPU status primitives */
} Mtp3Sts;

/* M3UA statistics */
typedef struct _M3UASts
{
    U32         data;           /* M3UA DATA messages */
    U32         duna;           /* M3UA DUNA messages */
    U32         dava;           /* M3UA DAVA messages */
    U32         daud;           /* M3UA DAUD messages */
    U32         scon;           /* M3UA SCON messages */
    U32         dupu;           /* M3UA DUPU messages */
    U32         drst;           /* M3UA DRST messages */
    U32         regReq;         /* M3UA REG-REQ messages */
    U32         deRegReq;       /* M3UA DEREG-REQ messages */
    U32         regRsp;         /* M3UA REG-RSP messages */
    U32         deRegRsp;       /* M3UA DEREG-RSP messages */
    U32         aspUp;          /* M3UA ASPUP messages */
    U32         aspUpAck;       /* M3UA ASPUP ACK messages */
    U32         aspDn;          /* M3UA ASPDN messages */
    U32         aspDnAck;       /* M3UA ASPDN ACK messages */
    U32         aspAc;          /* M3UA ASPAC messages */
    U32         aspAcAck;       /* M3UA ASPAC ACK messages */
    U32         aspIa;          /* M3UA ASPIA messages */
    U32         aspIaAck;       /* M3UA ASPIA ACK messages */
    U32         hBeat;          /* M3UA HBEAT messages */
    U32         hBeatAck;       /* M3UA HBEAT ACK messages */
    U32         err;            /* M3UA ERR messages */
    U32         notify;         /* M3UA NTFY messages */
} M3UASts;

/* Raw data statistics */
typedef struct _DataSts
{
    U32         nPdus;          /* Number of PDUs */
    U32         pduBytes;       /* Total size of PDUs */
} DataSts;

/* Data error statistics */
typedef struct _DataErrSts
{
    U32         dropNoRoute;        /* Data dropped, no route found */
    U32         dropPcUnavail;      /* Data dropped, point code unavail */
    U32         dropPcCong;         /* Data dropped, point code congested */
    U32         dropNoPspAvail;     /* Data dropped, no PSP avail */
    U32         dropNoNSapAvail;    /* Data dropped, no NSAP avail */
    U32         dropLoadShFail;     /* Data dropped, load-sharing failed */
    U32         dropMmhFail;        /* Data dropped, M3UA message failed */
    U32         dataQCong;          /* Data queued in congestion queue */
    U32         dataQAsPend;        /* Data queued in AS-PENDING queue */
} DataErrSts;

/* M3UA General statistics */
typedef struct _M3UAGenSts            /* General statistics */
{
    DateTime    dt;                 /* Date and time when statistics counters are initialized to zero */
    Mtp3Sts     txMtp3Sts;          /* MTP3 message statistics - TX */
    Mtp3Sts     rxMtp3Sts;          /* MTP3 message statistics - RX */
    M3UASts     txM3uaSts;          /* M3UA message statistics - TX */
    M3UASts     rxM3uaSts;          /* M3UA message statistics - RX */
    DataSts     liTxDataSts;        /* Lower interface data statistics - TX */
    DataSts     liRxDataSts;        /* Lower interface data statistics - RX */
    DataSts     uiTxDataSts;        /* Upper interface data statistics - TX */
    DataSts     uiRxDataSts;        /* Upper interface data statistics - RX */
    DataErrSts  downDataErrSts;     /* Layer data error stats - downward */
    DataErrSts  upDataErrSts;       /* Layer data error statistics - upward */
} M3UAGenSts;

/* M3UA NSap (upper sap) statictics */
typedef struct _M3UANSapSts         /* M3UA Statistics for NSAP */
{
    S16         spId;               /* NSAP Id  */
    U16         spare1;             /* alignment */
    DateTime    dt;                 /* date and time when statistics counters are initialized to zero */
    Mtp3Sts     txMtp3Sts;          /* MTP3 message statistics - TX */
    Mtp3Sts     rxMtp3Sts;          /* MTP3 message statistics - RX */
    DataSts     txDataSts;          /* SAP data statistics - TX */
    DataSts     rxDataSts;          /* SAP data statistics - RX */
    DataErrSts  dataErrSts;         /* SAP data error statistics - downward */
} M3UANSapSts;

/* M3UA SCT Sap (lower sap) statistics */
typedef struct _M3UASctSapSts       /* M3UA Statistics for SCTSAP */
{
    S16         suId;               /* Service user SAP Id */
    U16         spare1;             /* alignment */
    DateTime    dt;                 /* date and time when statistics counters are initialized to zero */
    DataSts     txDataSts;          /* SAP data statistics - TX */
    DataSts     rxDataSts;          /* SAP data statistics - RX */
} M3UASctSapSts;

#if (defined(LITV3) || defined(LITV6))
/* Association unavailable statistics */
typedef struct _UnavSts             /* Unavailable Statistics */
{
    U32         unav;               /* Association unavailable */
    U32         durUnav;            /* Duration of association  unavailability */
} UnavSts;
#endif /* LITV3 || LITV6 */

#ifdef LITV3
/* Association contested statistics */
typedef struct _CongSts
{
    U32         cong;               /* Association congested */
    U32         cong1;              /* Association at congestion level 1 */
    U32         cong2;              /* Association at congestion level 2 */
    U32         cong3;              /* Association at congestion level 3 */
    Ticks       durCong;            /* Duration of association congestion */
} CongSts;
#endif /* LITV3 */

/* M3UA peer signaling process statistics */
typedef struct _M3UAPspSts          /* M3UA Statistics for remote PSP */
{
    U16         pspId;              /* PSP ID */
    U16         spare1;             /* alignment */
    DateTime    dt;                 /* date and time when statistics counters are initialized to zero */
    struct {
        M3UASts         txM3uaSts;      /* M3UA message statistics - TX */
        M3UASts         rxM3uaSts;      /* M3UA message statistics - RX */
        DataSts         txDataSts;      /* PSP data statistics - TX */
        DataSts         rxDataSts;      /* PSP data statistics - RX */
        DataErrSts      dataErrSts;     /* PSP data error statistics - upward */
#ifdef LITV3
        UnavSts         unavSts;        /* Association unavailability statistics */
        CongSts         congSts;        /* Association congestion statistics */
#endif /* LITV3 */
    } assocSts[M3UA_MAX_SEP];
} M3UAPspSts;

/* High Availability states */
#define M3UA_HAST_STANDALONE      0   /* Stand-alone (non-redundant) state */
#define M3UA_HAST_STARTING        1   /* Initial redundant state */
#define M3UA_HAST_PRIMARY         2   /* This MTP3 is the primary */
#define M3UA_HAST_BACKUP          3   /* This MTP3 is the backup */

typedef S16 STATUS;                 /* redefine for portability */

/* Mgmt API return codes */
#define M3UA_SUCCESS    0           /* Successful */
#define M3UA_PARAM      13          /* Invalid parameter to match */
#define M3UA_BOARD      0x1004      /* board number out of range */
#define M3UA_DRIVER     0x1005      /* CPI driver returned error        */
#define M3UA_HANDLE     0x1006      /* Board not previously initialized */
#define M3UA_TIMEOUT    0x1007      /* No response from board */
#define M3UA_RESPONSE   0x1008      /* Wrong response from board */

/* Control Request codes */
#define M3UA_CTRL_ALARM_ENA     101 /* Enable Alarms */
#define M3UA_CTRL_ALARM_DIS     102 /* Disable Alarms */
#define M3UA_CTRL_FLOWCTL_ON    103 /* Start flow control */
#define M3UA_CTRL_FLOWCTL_OFF   104 /* Flow Control off */
#define M3UA_CTRL_DEBUG_ON      105 /* Start trace data */
#define M3UA_CTRL_DEBUG_OFF     106 /* Trace Data off */
#define M3UA_CTRL_SHUTDOWN      107 /* Shutdown M3UA layer */
#define M3UA_CTRL_TRACE_ON      109 /* Start trace data */
#define M3UA_CTRL_TRACE_OFF     110 /* Trace Data off */

#define M3UA_CTRL_SCTSAP_ENA    111 /* Enable SCT sap (upper sap) */
#define M3UA_CTRL_SCTSAP_DIS    112 /* Disable SCT sap (upper sap)*/

#define M3UA_CTRL_ASSOC_EST     121 /* Establish association */
#define M3UA_CTRL_ASSOC_TERM    122 /* Terminate association */
#define M3UA_CTRL_ASSOC_INH     123 /* Inhibit association */
#define M3UA_CTRL_ASSOC_UNI     124 /* Uninhibit association */

#define M3UA_CTRL_ASP_UP        131 /* Send ASPUP (up) message */
#define M3UA_CTRL_ASP_DN        132 /* Send ASPDN (down) message */
#define M3UA_CTRL_ASP_AC        133 /* Send ASPAC (active) message */
#define M3UA_CTRL_ASP_IA        134 /* Send ASPIA (inactive) message */

#define M3UA_CTRL_RK_REG        141 /* Register dynamic routing key */
#define M3UA_CTRL_RK_DEREG      142 /* Deregister dynamic routing key */

#define M3UA_CTRL_SET_TOS       151 /* Set Type Of Service dynamically */

#ifdef __cplusplus
extern "C" {
#endif

STATUS TXM3UAFUNC M3uaMgmtInit      ( U8 board, U8 srcEnt, U8 srcInst );
STATUS TXM3UAFUNC M3uaMgmtTerm      ( U8 board );
STATUS TXM3UAFUNC M3uaMgmtCtrl      ( U8 board, S16 entity, U8 action);
STATUS TXM3UAFUNC M3uaInitGenCfg    ( M3UAGenCfg *pGencfg );
STATUS TXM3UAFUNC M3uaSetGenCfg     ( U8 board, M3UAGenCfg *pGencfg );
STATUS TXM3UAFUNC M3uaGetGenCfg     ( U8 board, M3UAGenCfg *pGenCfg );
STATUS TXM3UAFUNC M3uaInitNwkCfg    ( M3UANwkCfg *pNwkCfg, U8 nwkId );
STATUS TXM3UAFUNC M3uaSetNwkCfg     ( U8 board, M3UANwkCfg *pNwkCfg );
STATUS TXM3UAFUNC M3uaGetNwkCfg     ( U8 board, M3UANwkCfg *pNwkCfg, U8 nwkId );
STATUS TXM3UAFUNC M3uaInitNSapCfg   ( M3UANSapCfg *pNSapCfg, S16 nsapId );
STATUS TXM3UAFUNC M3uaSetNSapCfg    ( U8 board, M3UANSapCfg *pNSapCfg );
STATUS TXM3UAFUNC M3uaGetNSapCfg    ( U8 board, M3UANSapCfg *pNSapCfg, S16 nSapNo );
STATUS TXM3UAFUNC M3uaInitSctSapCfg ( M3UASctSapCfg *pSctSapCfg, S16 sapId );
STATUS TXM3UAFUNC M3uaSetSctSapCfg  ( U8 board, M3UASctSapCfg *pSctSapCfg );
STATUS TXM3UAFUNC M3uaGetSctSapCfg  ( U8 board, M3UASctSapCfg *pSctSapCfg, S16 suId );
STATUS TXM3UAFUNC M3uaInitPspCfg    ( M3UAPspCfg *pPspCfg, U16 pspId, U32 ipAddr );
STATUS TXM3UAFUNC M3uaSetPspCfg     ( U8 board, M3UAPspCfg *pPspCfg );
STATUS TXM3UAFUNC M3uaGetPspCfg     ( U8 board, U16 pspId, M3UAPspCfg *pPspCfg );
STATUS TXM3UAFUNC M3uaInitPsCfg     ( M3UAPsCfg *pPsCfg, U32 psId );
STATUS TXM3UAFUNC M3uaSetPsCfg      ( U8 board, M3UAPsCfg *pPsCfg );
STATUS TXM3UAFUNC M3uaGetPsCfg      ( U8 board, M3UAPsCfg *cfg, U32 psId );
STATUS TXM3UAFUNC M3uaInitRteCfg    ( M3UARteCfg *rte, U32 dpc );
STATUS TXM3UAFUNC M3uaSetRteCfg     ( U8 board, M3UARteCfg *pRteCfg );
STATUS TXM3UAFUNC M3uaGetRteCfg     ( U8 board, M3UARteCfg *pRouteCfg, U32 dpc );
STATUS TXM3UAFUNC M3uaNSapStatus    ( U8 board, S16 nsapNo, M3UANSapSta *pStatus );
STATUS TXM3UAFUNC M3uaAddrTransStatus   ( U8 board, M3UAAtSta *pStatus );
STATUS TXM3UAFUNC M3uaDpcStatus     ( U8 board,U32 pc,U8 nwkId, M3UADpcSta *pStatus );
STATUS TXM3UAFUNC M3uaRteStatus     ( U8 board, M3UARteCfg *pStatus );
STATUS TXM3UAFUNC M3uaGenStatus     ( U8 board, M3UAGenSta *pStatus );
STATUS TXM3UAFUNC M3uaPspStatus     ( U8 board, U16 pspNo, M3UAPspSta *pStatus );
STATUS TXM3UAFUNC M3uaSctSapStatus  ( U8 board, S16 id, M3UASctSapSta *pStatus );
STATUS TXM3UAFUNC M3uaPsStatus      ( U8 board, U32 id, M3UAPsSta *pStatus );
STATUS TXM3UAFUNC M3uaRkStatus      ( U8 board, M3UARkSta *pStatus );
STATUS TXM3UAFUNC M3uaPspRkIdStatus ( U8 board, M3UAPspRkIdSta *pStatus, U16 pspId );
STATUS TXM3UAFUNC M3uaGenStatistics ( U8 board, M3UAGenSts *pSts, Bool isReset );
STATUS TXM3UAFUNC M3uaNSapStatistics( U8 board, M3UANSapSts *pSts, Bool isReset, S16 nsapNo );
STATUS TXM3UAFUNC M3uaSctSapStatistics  ( U8 board, M3UASctSapSts *pSts, Bool isReset, S16 suId );
STATUS TXM3UAFUNC M3uaPspStatistics ( U8 board, M3UAPspSts *pSts, Bool isReset, U16 pspId );
STATUS TXM3UAFUNC M3uaGetSysId      ( U8 board, SystemId *pStatus );
#ifdef __cplusplus
}
#endif

#endif /* _M3UAMAPI_H_ */
