/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifndef _MTP2API_H_
#define _MTP2API_H_

#define MTP2_ENT            0x16
#define MAXMTP2PKT          272
#define MAXMTP2DATA         272
#define MAXMTP2RXDATA       1024
#define FILLER_COUNT        7			/* Changed to fix providerId being on an odd boundary */
#define MAX_LINK_NUM        15

/* The following structures are used as templates laid over the data area of the MTP2PKT */
/*******************************************************************************************/
typedef struct                          /* Bind Request */
{
    S16     userId;
    U8      srcEnt;
    U8      srcInst;
} BindReq;

/*******************************************************************************************/
typedef struct                          /* Unbind Request */
{
    U8      avail;
    U8      spare1[3];                  /* Alignment */
} UnbindReq;

/*******************************************************************************************/
typedef struct                          /* Unbind Confirmation */
{
    U8      avail;
    U8      spare1[3];                  /* Alignment */
} UnbindCfm;

/*******************************************************************************************/
typedef struct                          /* Connect Request */
{
    U8      status;
    U8      spare1[3];                  /* Alignment */
} ConReq;

/* Connect Request Status Field Values */
#define CONREQSTA_LVL_1             0
#define CONREQSTA_LVL_2_EMRG        1
#define CONREQSTA_LVL_1_NORM        2
#define CONREQSTA_LVL_1_2_EMRG      3
#define CONREQSTA_LVL_1_2_NORM      4
#define CONREQSTA_POWER_ON          5
#define CONREQSTA_AFTER_POWER_ON    6

/*******************************************************************************************/
typedef struct                          /* Data Request */
{
    U8      data[MAXMTP2DATA];
} DatReq;

/*******************************************************************************************/
typedef struct                          /* Data Confirmation */
{
    U8      status;
    U8      credit;
    U8      data[MAXMTP2DATA];          /* 272 bytes (ie. multiple of 4 so add alignment bytes) */
    U8      spare1[2];                  /* Alignment to make entire struct a multiple of 4 bytes */
} DatCfm;
/* Data Confirmation Status Field Values */
#define DATCFMSTA_MORE              0
#define DATCFMSTA_LAST              1
#define DATCFMSTA_NUM_ACKED         2

/*******************************************************************************************/
typedef struct                          /* Data Indication */
{
    U8      data[MAXMTP2DATA];
} DatInd;

/*******************************************************************************************/
typedef struct                          /* Disconnect Request */
{
    S16     reason;
    U8      spare1[2];                  /* Alignment */
} DisReq;
/* Disconnect Request Reason Field Values */
#define DISCREQ_MGMT_REQ            0
#define DISCREQ_LVL_1_TOO           10
#define DISCREQ_LVL_2_ONLY          13

/*******************************************************************************************/
typedef struct                          /* Disconnect Indication */
{
    S16     reason;
    U8      spare1[2];                  /* Alignment */
} DisInd;
/* Disconnect Indication Reason Field Values */
#define DISC_IND_SM         0       /* stack manager */
#define DISC_IND_SUERM      1       /* signal unit error rate monitor */
#define DISC_IND_ALIGN      2       /* realignment */
#define DISC_IND_ACK        3       /* delay of acknowledgement */
#define DISC_IND_TE         4       /* failure of TE equipment */
#define DISC_IND_BSN        5       /* 2 out 3 BSN */
#define DISC_IND_FSN        6       /* 2 out 3 FSN */
#define DISC_IND_LSSU       7       /* LSSU condition */
#define DISC_IND_CONG       8       /* too much congestion */
#define DISC_IND_CLROORD    9       /* clear - out of order */
#define DISC_IND_L1         10      /* disconnect and bring down L1 */
#define DISC_IND_TYPE       11      /* pdu type INS or NM (q.2140) */
#define DISC_IND_SYNC       12      /* resync indication (q.2140) */
#define DISC_IND_L2         13      /* disconnect L2 only request */
#define DISC_IND_MAX        13

/*******************************************************************************************/
typedef struct                          /* Flow Control Request */
{
    S16     action;
    U8      flcLevel;
    U8      spare1[1];                  /* Alignment */
} FlowReq;
/* Flow Control Request Action Field Values */
#define FLOWREQ_SETLEVEL            0           /* <3.8>  See below */
/* <3.8>  The MTP2 task only accepts the value zero for action
#define FLOWREQ_START               1
#define FLOWREQ_STOP                1
*/

/*******************************************************************************************/
typedef struct                          /* Flow Control Indication */
{
    S16     action;
    U8      flcLevel;
    U8      spare1[1];                  /* Alignment */
} FlowInd;
/* Flow Control Indication Action Field Values */
#define FLOWIND_STARTED             0
#define FLOWIND_STOPPED             1
#define FLC_LEVEL_MAX               2

/*******************************************************************************************/
typedef struct                          /* Status Request */
{
    S16     action;
    U8      spare1[2];                  /* Alignment */
    U32     status;
} StaReq;
/* Status Request Action Field Values */
#define STAREQ_CHANGE_STATUS        0
#define STAREQ_GET_BSN              1
#define STAREQ_GET_UNACKED          2
#define STAREQ_DROP_UNACKED         3

/* Status Request Status Field Values */
#define STAREQ_STS_LCL_DOWN         0
#define STAREQ_STS_LCL_UP           1
#define STAREQ_STS_START_EMERG      2
#define STAREQ_STS_END_EMERG        3

/*******************************************************************************************/
typedef struct                          /* Status Confirmation */
{
    S16     action;
    U8      spare1[2];                  /* Alignment */
    U32     status;
} StaCfm;
/* Status Confirmation Action Field Values */
#define STACFM_NOACTION             0
#define STACFM_BSN                  1
#define STACFM_NO_UNACKED           3

/* Status Confirm Status Field Values */
#define STACFM_STS_LCL_DOWN         0
#define STACFM_STS_LCL_UP           1

/*******************************************************************************************/
typedef struct                          /* Status Indication */
{
    U8      status;
    U8      value;                      /* Buffer depletion level */
    U8      spare1[2];                  /* Alignment */
} StaInd;

/*******************************************************************************************/
typedef struct                          /* Status Response */
{
    U32     status;
} StaRsp;

/* Status Indication/Response Status Field Values */
#define STAIND_REM_DOWN             0
#define STAIND_REM_UP               1
#define STAIND_BUF_DEPLETION        2


/*******************************************************************************************/
/* -----------------10/2/97 2:47PM-------------------
 * MTP2 Message Types
 * --------------------------------------------------*/
#define MTP2_BIND_REQ       0x04            /* Bind request */
#define MTP2_BIND_CFM       0x05            /* Bind confirm */

#define MTP2_UNBIND_REQ     0x08            /* Unbind request */
#define MTP2_UNBIND_CFM     0x09            /* Unbind confirm */

#define MTP2_CONN_REQ       0x0c            /* Connect request */
#define MTP2_CONN_CFM       0x0d            /* Connect confirm */

#define MTP2_DISC_REQ       0x10            /* Disconnect request */
#define MTP2_DISC_CFM       0x11            /* Disconnect confirm */
#define MTP2_DISC_IND       0x12            /* Disconnect indication */
#define MTP2_DISC_RSP       0x13            /* Disconnect response */

#define MTP2_DATA_REQ       0x14            /* Data request */
#define MTP2_DATA_CFM       0x15            /* Data confirm */
#define MTP2_DATA_IND       0x16            /* Data indication */

#define MTP2_FLOW_REQ       0x28            /* Flow ctrl request */
#define MTP2_FLOW_CFM       0x29            /* Flow ctrl confirm */
#define MTP2_FLOW_IND       0x2a            /* Flow ctrl indication */
#define MTP2_FLOW_RSP       0x2b            /* Flow ctrl response */

#define MTP2_STAT_REQ       0x78            /* Status request */
#define MTP2_STAT_CFM       0x79            /* Status confirm */
#define MTP2_STAT_IND       0x7a            /* Status indication */
#define MTP2_STAT_RSP       0x7b            /* Status response */

typedef struct
{
    CPIHDR              hdr;
    unsigned short      start;
    unsigned short      end;
} MTP2HDR;

typedef struct
{
    MTP2HDR             hdr;
    U8                  filler[FILLER_COUNT];
    U8                  msgType;
    S16                 providerId;
    U8                  spare1[2];                  /* Alignment */
    union
    {
        BindReq         bindReq;
        UnbindReq       unbindReq;          /* Unbind request */
        UnbindCfm       unbindCfm;          /* Unbind confirmation */
        ConReq          conReq;
        DatReq          datReq;
        DisReq          disReq;
        FlowReq         flowReq;
        StaReq          staReq;
        StaRsp          staRsp;
    } data;
} MTP2TXPKT;

typedef struct
{
    MTP2HDR             hdr;
    unsigned char       data[MAXMTP2RXDATA];
} MTP2RXPKT;

#define HDRSIZE             FILLER_COUNT + sizeof(MTP2HDR) + sizeof(U8) + sizeof(S16) + sizeof(U16)
#define ENDSIZE             FILLER_COUNT + sizeof(U8) + sizeof(S16) + sizeof(U16)


#define     SLTM_PKT        0x11
#define     SLTA_PKT        0x21
#define     COO_PKT         0x11
#define     COA_PKT         0x21
#define     SLTM_LEN        3

/* Link States */
#define     LINK_DOWN       0
#define     LINK_UP         1
#define     LINK_CHANGETO   2
#define     LINK_CHANGEFROM 3
#define     LINK_WAITCOA    4
#define     LINK_RCVDCOO    5
#define     LINK_GETUNACKED 6

#endif  /* _MTP2API_H_ */
