/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 2008 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/
#ifndef _SCTPMAPI_H_
#define _SCTPMAPI_H_

#include "ss7type.h"
#include "sigtran.h"
#include "txcpi.h"
/* #include "sctpmspi.h" */

/* define the calling conventions for the API functions for various
 * operating systems - NOTE: API callers should never define the
 * preprocessor constant "_TX_SCTP_SOURCE__"; this is only for use
 * by the SCTP API source code itself.
 */

/* OS2: SCTP API is a DLL */
#ifdef __OS2__
#ifdef _TX_SCTP_SOURCE__
    /* SCTP API source definition */
#define     TXSCTPFUNC               extern _export _syscall
#else
    /* SCTP API caller's definition */
#define     TXSCTPFUNC               _syscall
#endif	/* _TX_SCTP_SOURCE__ */

/* WIN32: SCTP API is a DLL */
#elif  WIN32	                        /* WIN32 */
#ifdef _TX_SCTP_SOURCE__
    /* SCTP API source definition */
#define     TXSCTPFUNC               __declspec( dllexport ) WINAPI
#else
    /* SCTP API caller's definition */
#define     TXSCTPFUNC               WINAPI
#endif	/* _TX_SCTP_SOURCE__ */

/* all others: statically linked library */
#else
#define     TXSCTPFUNC
#endif

#ifdef UNIX
#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif
#endif

#define DEBUGP      /* So user can turn on debugging through mgr */

/* Macro for initializing a timer (value 0 => disable) */
#define SCTP_SET_TIMER( atimer, avalue )                    \
    atimer.value = avalue;                                  \
    if( atimer.value != 0 )                                 \
        atimer.enable = 1;                                  \
    else                                                    \
        atimer.enable = 0;

/* "srvcType" in ServOpenReq and ConReq */
#define  HI_SRVC_UDP                    0       /* UDP */
#define  HI_SRVC_TCP_NO_HDR             1       /* TCP */
#define  HI_SRVC_TCP                    1       /* TCP */
#define  HI_SRVC_TCP_TPKT_HDR           2       /* RFC1006 TPKT TCP */
#define  HI_SRVC_UDP_PRIOR              4       /* priority UDP */
#define  HI_SRVC_UDP_TPKT_HDR           5       /* UDP with TPKT  */
#define  HI_SRVC_RAW_RAW                6
#define  HI_SRVC_RAW_SCTP               7
#define  HI_SRVC_RAW_ICMP               8
#define  HI_SRVC_RAW_SCTP_PRIOR         9
#define  HI_SRVC_RAW_RSVP               10
#define  HI_SRVC_TLS                    11
#define  SCTP_CRC32         0x01
#define  SCTP_ADLER32       0x02

#define ENTSN       0x15        /* MTP3 ID (M3UA task & cfg ID) */
#define ENTSD       0x16        /* MTP2 ID (SCTP task & cfg ID) */
#define ENTSB       0x88        /* SCTP mgr ID */
#define ENTIT       0x89        /* M3UA mgr ID */
#define ENTSM       0x0E              /* Stack Manager */
#define ENTHI       0x75              /* TUCL */

 /* New definition */
#define SCTP_IPV4ADDR_TYPE      4
#define SCTP_IPV6ADDR_TYPE      6
#define SCT_MAX_NET_ADDRS       8       /* Make multiple of 4 */
#define CM_DNS_DNAME_LEN        64      /* JK changed to 64 was 256 */

  /* Trace Indication events */
#define SCTP_MAX_TRC_LEN   512        /* Maximum trace length */
#define SCTP_MSG_RECVD       1        /* Message received */
#define SCTP_MSG_TXED        2        /* Message transmitted */

typedef struct _Ip4TptAddr
{
	U16		port;
    U16     spare1;             /* Alignment */
	U32		address;
} Ip4TptAddr;

typedef U32 Ip4Addr;

typedef U8 Ip6Addr[16];

typedef Ip4TptAddr	CmIpv4TptAddr;
typedef Ip6Addr	CmIpv6TptAddr;

typedef struct cmNetAddr
{
    U32     type;       /* type of network address */
    union
    {
        Ip4Addr ipv4NetAddr;    /* IP network address (U32) */
        Ip6Addr ipv6NetAddr;    /* IPv6 network address (16 bytes) */
    }u;
} CmNetAddr;

/* common alarm */
typedef struct cmAlarm
{
    DateTime    dt;             /* data and time */
    U16         category;       /* alarm category*/
    U16         event;          /* alarm event */
    U16         cause;          /* alarm cause */
    U16         spare1;         /* alignment */
} CmAlarm;

/*-----------------------------------------------------------------------------------------------
 * Configuration Structures
 *----------------------------------------------------------------------------------------------*/

/* SCTP General Reconfigurable Structure */
typedef struct _sctpGenReCfg         /* SCTP General Reconfiguration */
{
   U8              maxInitReTx;    /* Maximum Association Init Retransmits */
   U8              maxAssocReTx;   /* Maximum Retransmissions for an association */
   U8              maxPathReTx;    /* Maximum Retransmission for a destination address */
   Bool            altAcceptFlg;   /* Accept or don't accept additional life
                                      time parameter for init from peer */
   U16             keyTm;          /* Initial value for MD5 key expiry timer */
   U16             alpha;          /* Used in RTT calculations */
   U16         	   beta;           /* Used in RTT calculations */
   U16             spare1;          /* alignment */
} SCTPGenReCfg;

/* SCTP General Configuration Structure */
typedef struct _sctpGenCfg       /* SCTP General Configuration */
{
#ifdef SB_IPV6_SUPPORTED
    U8          ipv6SrvcReqdFlg; /* IPV6 service required for sctp */
#else
    U8          spare1;          /* alignment */
#endif
    U8          serviceType;     /* TUCL transport protocol (IP/UDP) */
    U16         spare2;          /* alignment */
    U16         maxNmbSctSaps;   /* max no. SCT SAPS */
    U16         maxNmbTSaps;     /* max no. Transport SAPS */
    U16         maxNmbEndp;      /* max no. endpoints */
    U16         maxNmbAssoc;     /* max no. associations */
    U16         maxNmbDstAddr;   /* max no. dest. addresses */
    U16         maxNmbSrcAddr;   /* max no. src. addresses */
    U32         maxNmbTxChunks;  /* max no. outgoing chunks */
    U32         maxNmbRxChunks;  /* max no. recv chunks */
    U16         maxNmbInStrms;   /* max no. in streams PER ASSOCIATION */
    U16         maxNmbOutStrms;  /* max no. out streams PER ASSOCIATION */
    U32         initARwnd;       /* max receiver window space */
    U16         mtuInitial;      /* Initial MTU size */
    U16         mtuMinInitial;   /* Initial minimum MTU size */
    U16         mtuMaxInitial;   /* Initial maximum MTU size */
    Bool        performMtu;      /* Perform path MTU discovery */
    Bool        useHstName;      /* Flag whether hostname is to be used in INIT and INITACK msg */
    U16         timeRes;         /* timer resolution */
    U16         spare3;          /* alignment */
    U32         debugMask;       /* Debugging mask */
    U32         traceMask;       /* Tracing mask */
    Pst         smPst;           /* layer manager post structure for alarms */
    U8          hostname[CM_DNS_DNAME_LEN]; /* Own Domain Name */
    SCTPGenReCfg  reConfig;        /* reconfigurable params */
#ifdef LSB5
    U16          srcUdpPort;
    U16          dstUdpPort;
#endif /* LSB5 */
} SCTPGenCfg;

 /* Array of Network Addresses */
 typedef struct _sctptNetAddrLst
 {
    U32           nmb;                        /* Number of Network Addresses */
    CmNetAddr     nAddr[SCT_MAX_NET_ADDRS];   /* List of Network Addresses */
 } SCTPNetAddrLst;

/* Transport address */
typedef struct cmTptAddr
{
    U8   type;                     /* type of transport address */
    U8   spare1;        /* alignment */
    U16  spare2;        /* alignment */
    union
    {
        CmIpv4TptAddr  ipv4TptAddr; /* IP transport address (U32) */
        U8  ipv6TptAddr[20];        /* IPv6 transport address (16 bytes). Changed to 20 to jive w/board version.
                                     * This was messing up useDnsLib which comes after CmTptAddr in TSapCfg */
#ifdef CM_AAL
        AtmVccId       aalTptAddr;  /* AAL transport address */
#endif
    }u;
} CmTptAddr;

/* DNS Configuration  */
typedef struct _sctpDnsCfg
{
   CmTptAddr    dnsAddr;          /* DNS Query Transport Address */
   Bool         useDnsLib;        /* Use DNS library */
   U8           maxRtxCnt;        /* Maximum number of retransmissions */
   U16          dnsTmOut;         /* DNS Timer Value */
} SCTPDnsCfg;

/* SCTP Lower SAP Reconfiguration Structure */
typedef struct _sctpTSapReCfg    /* Transport SAP Reonfiguration */
{
    S16             spId;           /* Service provider SAP ID */
    U8              maxBndRetry;    /* Maximum number of bind retries allowed */
    U8              spare1;         /* alignment */
    U16             tIntTmr;        /* Default time to wait for bind confirm */
    U16             spare2;         /* alignment */
    SCTPDnsCfg      sbDnsCfg;       /* Dns Configuration structure */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    Bool            remIntfValid;   /* remote interface version is valid */
    U8              spare3;         /* alignment */
    U16             remIntfVer;     /* remote interface version */
#endif
} SCTPTSapReCfg;

/* SCTP Lower SAP Configuration Structure */
typedef struct _sctpTSapCfg      /* Transport SAP Configuration */
{
    S16             swtch;          /* Protocol Switch */
    S16             suId;           /* Service User SAP Id */
    U8              sel;            /* Coupling selector for SCTP provider */
    U8              ent;            /* Service Provider Entity */
    U8              inst;           /* Service Provider Instance */
    U8              spare1;         /* alignment */
    U16             procId;         /* Service Provider Processor Id */
    U8              prior;          /* Message priority for SCTP provider
                                        * messages */
    U8              route;          /* Route */
    MemoryId        memId;          /* Memory region and pool id for SCTP
                                        * provider */
    SCTPNetAddrLst  srcNAddrLst;    /* Source Network Address List */
    SCTPTSapReCfg   reConfig;       /* Reconfigurable parameters */
} SCTPTSapCfg;

/* SCTP Upper SAP Reconfiguration Structure */
typedef struct _SctSapReCfg /* SCT SAP Reconfiguration */
{
    U16             maxAckDelayTm;      /* Maximum time delay for generating
                                         * Acks */
    U16             maxAckDelayDg;      /* Maximum # of datagrams after which an
                                         * Ack shall be sent */
    U16             rtoInitial;         /* Initial value of RTO */
    U16             rtoMin;             /* Minimum RTO */
    U16             rtoMax;             /* Maximum RTO */
    U16             freezeTm;           /* Default Freeze timer value */
    U16             bundleTm;           /* Default Bundle timer value */
    U16             spare0;             /* alignment */
    U16             cookieLife;         /* Life time for a Valid Cookie */
    U16             intervalTm;         /* Default Heartbeat interval timer value */
#ifdef LSB8
    U16             maxBurst;           /* new protocol parameter defined */
    U16             maxHbBurst;         /* new protocol parameter defined */
    U16             t5SdownGrdTm;       /* T5 Shutdown Guard Timer value */
    U16             spare1;             /* alignment */
#endif /* LSB8 */
    Bool            handleInitFlg;      /* Flag to indicate whether SCTP should
                                         * handle INIT itself */
    Bool            negAbrtFlg;         /* Negotiate or Abort the init if MIS is
                                         * less than OS */
    Bool            hBeatEnable;        /* Enable HeartBeat by Default */
    U8              spare2;             /* alignment */
    U32             flcUpThr;           /* Flow Control upper threshold */
    U32             flcLowThr;          /* Flow Control lower threshold */
    U32             spare3;             /* alignment */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    Bool            remIntfValid;       /* remote interface version is valid */
    U16             remIntfVer;         /* remote interface version */
#endif
#ifdef SB_ETSI
    U16             maxDataSize;        /* Maximum user data size that can be sent/received on an endpoint */
#endif /* SB_ETSI */
#ifdef SB_CHECKSUM_DUAL
    U8              checksumType;       /* type of checksum to be used in this SAP related Associations */
#endif /* SB_CHECKSUM_DUAL */

} SCTPSctSapReCfg;

/* SCTP Upper SAP Configuration Structure */
typedef struct _SctSapCfg    /* SCT SAP Configuration */
{
    S16             swtch;          /* Protocol Switch */
    S16             spId;           /* Service Provider SAP Id */
    MemoryId        memId;          /* Memory region and pool id for SCTP user */
    U8              sel;            /* Coupling selector for SCTP user */
    U8              prior;          /* Message priority for SCTP user messages */
    U8              route;          /* Route */
    U8              spare1;         /* alignment */
    SCTPSctSapReCfg reConfig;       /* Reconfigurable parameters */
} SCTPSctSapCfg;

/*-----------------------------------------------------------------------------------------------
 * Status Structures
 *----------------------------------------------------------------------------------------------*/

#ifdef DOS
typedef U16 Size;                 /* size (for number of bytes per region or per s-pool) */
#else
typedef U32 Size;                 /* size (for number of bytes per region or per s-pool) */
typedef S32 PtrOff;               /* signed pointer offset */
#endif

/* SCTP General Status */
typedef struct _sctpGenSta       /* General Solicited Status */
{
    U8          status;         /* Status */
    U8          spare1;         /* alignment */
    U16         spare2;         /* alignment */
    Size        memSize;        /* Memory Size Reserved (U32) */
    Size        memAlloc;       /* Memory Size Allocated (U32) */
    U16         nmbAssoc;       /* Number of open associations */
    U16         nmbEndp;        /* Number of open endpoints */
    U16         nmbLocalAddr;   /* Number of local addresses in use */
    U16         nmbPeerAddr;    /* Number of peer addresses in use */
} SCTPGenSta;

/* SCTP SAP status */
typedef struct _sctpSapSta       /* SAP Status */
{
    S16     swtch;              /* Protocol Switch */
    U8      hlSt;               /* SAP State */
    U8      spare1;             /* alignment */
#ifdef TDS_ROLL_UPGRADE_SUPPORT
    Bool    remIntfValid;       /* remote version valid in cfm ?*/
    U16     selfIfVer;          /* self version on this SAP     */
    U16     remIfVer;           /* remote version on this SAP   */
#endif
} SCTPSapSta;

/* Association states */
#define SCTP_ASSOC_STATE_CLOSED             0x0       /* No association active or open */
#define SCTP_ASSOC_STATE_OPEN               0x1       /* Association is capable of receiving associations */
#define SCTP_ASSOC_STATE_COOKIE_WAIT        0x2       /* Awaiting Cookie in INIT_ACK message */
#define SCTP_ASSOC_STATE_ESTABLISHED        0x3       /* Association ready for 2 way communication */
#define SCTP_ASSOC_STATE_COOKIE_SENT        0x4       /* Cookie has been sent. Awaiting COOKIE_ACK message */
#define SCTP_ASSOC_STATE_SHUTDOWN_PEND      0x5       /* Received Shutdown request from service user */
#define SCTP_ASSOC_STATE_SHUTDOWN_SENT      0x6       /* Sent SHUTDOWN message, awaiting SHUTDOWN_ACK */
#define SCTP_ASSOC_STATE_SHUTDOWN_RCVD      0x7       /* Received a SHUTDOWN message awaiting outstanding data to
                                                       * be acknowledged            */
#define SCTP_ASSOC_STATE_SDOWNACK_SENT      0x8       /* Sent Shutdown Ack message and awaiting Shutdown Complete */

typedef struct _sctpAssocSta     /* Association related solicited status */
{
    U32             assocId;        /* association identifier */
    U8              assocState;     /* association state */
    U8              spare1;         /* alignment */
    U16             spare2;         /* alignment */
    SCTPNetAddrLst  dstNAddrLst;    /* destination network address list */
    SCTPNetAddrLst  srcNAddrLst;    /* source network address list */
    CmNetAddr       priNAddr;       /* primary network address */
    U16             dstPort;        /* destination port */
    U16             srcPort;        /* source port */
} SCTPAssocSta;

/* Destination Transport Address Status */
typedef struct _sctpDtaSta       /* Destination transport address status */
{
    CmNetAddr       dstNAddr;       /* destination network address */
    U16             dstPort;        /* destination port */
    U16             mtu;            /* path MTU */
    U32             assocId;        /* association ID */
    U8              state;          /* destination transport address state */
    U8              spare1;         /* alignment */
    U16             rto;            /* retransmission timeout */
} SCTPDtaSta;

/*-----------------------------------------------------------------------------------------------
 * Statistics Structures
 *----------------------------------------------------------------------------------------------*/

/* Statistics counters for incoming and outgoing chunks */
typedef struct _sctpChunkSts     /* Statistics counters for chunks */
{
   U32         noInitTx;       /* number INITs sent */
   U32         noInitReTx;     /* number INITs resent */
   U32         noInitRx;       /* number INITs received */
   U32         noIAckTx;       /* number INIT_ACKs sent */
   U32         noIAckRx;       /* number INIT_ACKs received */
   U32         noShDwnTx;      /* number SHUTDOWNs sent */
   U32         noShDwnReTx;    /* number SHUTDOWNs resent */
   U32         noShDwnRx;      /* number SHUTDOWNs received */
   U32         noShDwnAckTx;   /* number SHUTDOWN_ACKs sent */
   U32         noShDwnAckReTx; /* number SHUTDOWN_ACKSs resent */
   U32         noShDwnAckRx;   /* number SHUTDOWN_ACKs received */
   U32         noCookieTx;     /* number COOKIEs sent */
   U32         noCookieReTx;   /* number COOKIEs resent */
   U32         noCookieRx;     /* number COOKIEs received */
   U32         noCkAckTx;      /* number COOKIE_ACKs sent */
   U32         noCkAckRx;      /* number COOKIE_ACKs received */
   U32         noDataTx;       /* number DATAs sent */
   U32         noDataReTx;     /* number DATAs resent */
   U32         noDataRx;       /* number DATAs received */
   U32         noDAckTx;       /* number SACKs sent */
   U32         noDAckRx;       /* number SACKs received */
   U32         noShDwnCmpltTx; /* number of Shutdown completed sent */
   U32         noShDwnCmpltRx; /* number of Shutdown completed sent */
#ifdef LSB2
   U32         noHBeatTx;      /* number of HEARTBEATs sent */
   U32         noHBeatRx;      /* number of HEARTBEATs received */

   U32         noHBAckTx;      /* number of HBEAT_ACKs sent */
   U32         noHBAckRx;      /* number of HBEAT_ACKs received */
#endif
#ifdef LSB3
   U32         noAbortTx;      /* number of ABORTs sent */
   U32         noAbortRx;      /* number of ABORTs received */
#endif
#ifdef LSB6
   U32         noErrorTx;      /* number of ERRORs sent */
   U32         noErrorRx;      /* number of ERRORs received */
#endif

} SCTPChunkSts;

/* Statistics counters for incoming and outgoing bytes */
typedef struct _sctpByteSts      /*  Statistics counters for bytes */
{
   U32         bytesTx;        /* bytes sent */
   U32         bytesRx;        /* bytes received */
} SCTPByteSts;

/* DNS Stats */
typedef struct _sctpDnsSts
 {
   U32          noQueryTx;     /* Number of DNS Queries Txmitted */
   U32          noQueryReTx;   /* Number of DNS Queries ReTxmitted */
   U32          noQueryRspRx;  /* Number of DNS Query Responses Received  */
 } SCTPDnsSts;

/* SCTP Layer wide statistics */
typedef struct _sctpGenSts
{
    DateTime        dt;             /* Date and time when statistics counters are initialized to zero */
    SCTPChunkSts    sbChunkSts;     /* Statistics counters for incoming and outgoing chunks */
    SCTPByteSts     sbByteSts;      /* Statistics counters for total incoming and outgoing bytes */
    SCTPDnsSts      sbDnsSts;       /* Statistics couters for Dns Transations */
} SCTPGenSts;

/* SCT SAP specific statistics */
typedef struct _sctpSctSapSts    /* SCTP Statistics for SCTSAP */
{
    S16             swtch;          /* Protocol switch */
    U16             spare1;         /* alignment */
    DateTime        dt;             /* date and time when statistics counters are initialized to zero */
    SCTPByteSts     sbByteSts;      /* Statistics counters for total incoming and outgoing bytes */
} SCTPSctSapSts;

/* Transport SAP specific statistics */
typedef struct _sctpTSapSts      /* SCTP Statistics for TSAP */
{
    S16             swtch;          /* Protocol switch */
    U16             spare1;         /* alignment */
    DateTime        dt;             /* date and time when statistics counters are initialized to zero */
    SCTPChunkSts    sbChunkSts;     /* Statistics counters for incoming and outgoing chunks */
    SCTPByteSts     sbByteSts;      /* Statistics counters for total incoming and outgoing bytes */
    U32             nmbBndRetry;    /* number of bind retries on the SAP */
} SCTPTSapSts;

/*-----------------------------------------------------------------------------------------------
 * Unsolicited Status (alarm) Structures
 *----------------------------------------------------------------------------------------------*/

/* SCTP Alarm Structure */
typedef struct _sctpUsta         /* Unsolicited status */
{
    CmAlarm         alarm;          /* Common Alarm Structure */
    S16             sapId;          /* SAP Id */
    S16             swtch;          /* Protocol Switch */
#ifdef LSB7
    S8               info[LSB_MAX_INFO + 1];
#endif
} SCTPUsta;

/*-----------------------------------------------------------------------------------------------
 * Trace Structures
 *----------------------------------------------------------------------------------------------*/

/* SCTP Message Trace Structure */
typedef struct _sctpTrc          /* Trace */
{
    DateTime        dt;             /* date and time */
    S16             sapId;          /* TSAP Id */
    U16             evnt;           /* event */
    U16             len;            /* trace length */
    U16             spare1;         /* alignment */
    U8              evntParm[SCTP_MAX_TRC_LEN];       /* trace data */
} SCTPTrc;

/* Error type returned by API functions */
typedef S16 SCTP_STATUS;   /* redefine for portability */

/* Mgmt API return codes */
#define SCTP_SUCCESS    0           /* Successful */
#define SCTP_PARAM      0x1002      /* Invalid parameter */
#define SCTP_BOARD      0x1004      /* board number out of range */
#define SCTP_DRIVER     0x1005      /* CPI driver returned error        */
#define SCTP_HANDLE     0x1006      /* Board not previously initialized */
#define SCTP_TIMEOUT    0x1007      /* No response from board */
#define SCTP_RESPONSE   0x1008      /* Wrong response from board */

/* Control Request codes */
#define SCTP_CTRL_ALARM_ENA     101 /* Enable Alarms */
#define SCTP_CTRL_ALARM_DIS     102 /* Disable Alarms */
#define SCTP_CTRL_FLOWCTL_ON    103 /* Start flow control */
#define SCTP_CTRL_FLOWCTL_OFF   104 /* Flow Control off */
#define SCTP_CTRL_DEBUG_ON      105 /* Start trace data */
#define SCTP_CTRL_DEBUG_OFF     106 /* Trace Data off */
#define SCTP_CTRL_SHUTDOWN      107 /* Shutdown SCTP layer */
#define SCTP_CTRL_TRACE_ON      109 /* Start trace data */
#define SCTP_CTRL_TRACE_OFF     110 /* Trace Data off */

#define SCTP_CTRL_SCTSAP_ENA    111 /* Enable SCT sap (upper sap) */
#define SCTP_CTRL_SCTSAP_DIS    112 /* Disable SCT sap (upper sap)*/
#define SCTP_CTRL_SCTSAP_DEL    113 /* Delete SCT sap (upper sap)*/

#define SCTP_CTRL_TSAP_ENA      121 /* Enable tsap (lower sap) */
#define SCTP_CTRL_TSAP_DIS      122 /* Disable tsap (lower sap) */
#define SCTP_CTRL_TSAP_DEL      123 /* Delete tsap (lower sap) */

#ifdef __cplusplus
extern "C" {
#endif
SCTP_STATUS TXSCTPFUNC SctpMgmtInit( U8 board, U8 srcEnt, U8 srcInst );
SCTP_STATUS TXSCTPFUNC SctpMgmtTerm( U8 board );
SCTP_STATUS TXSCTPFUNC SctpMgmtCtrl( U8 board, S16 entity, U8 action );
SCTP_STATUS TXSCTPFUNC SctpInitGenCfg( SCTPGenCfg *pCfg );
SCTP_STATUS TXSCTPFUNC SctpSetGenCfg( U8 board, SCTPGenCfg *pGenCfg );
SCTP_STATUS TXSCTPFUNC SctpGetGenCfg( U8 board, SCTPGenCfg *pGenCfg);
SCTP_STATUS TXSCTPFUNC SctpInitSctSapCfg( SCTPSctSapCfg *pCfg, U16 id );
SCTP_STATUS TXSCTPFUNC SctpSetSctSapCfg( U8 board, SCTPSctSapCfg *pSctSapCfg );
SCTP_STATUS TXSCTPFUNC SctpInitTSapCfg( SCTPTSapCfg *pCfg, U16 id );
SCTP_STATUS TXSCTPFUNC SctpSetTSapCfg( U8 board, SCTPTSapCfg *pTSapCfg );
SCTP_STATUS TXSCTPFUNC SctpGetSctSapCfg( U8 board, SCTPSctSapCfg *pSctSapCfg, S16 sapId );
SCTP_STATUS TXSCTPFUNC SctpGetTSapCfg( U8 board, SCTPTSapCfg *pTSapCfg, S16 sapId );
SCTP_STATUS TXSCTPFUNC SctpGenStatistics( U8 board, SCTPGenSts *pStats, Bool isReset);
SCTP_STATUS TXSCTPFUNC SctpSctSapStatistics( U8 board, SCTPSctSapSts *pStats, S16 sapid, Bool isReset);
SCTP_STATUS TXSCTPFUNC SctpTSapStatistics( U8 board, SCTPTSapSts *pStats, S16 sapid, Bool isReset);
SCTP_STATUS TXSCTPFUNC SctpGenStatus(U8 board, SCTPGenSta *sta);
SCTP_STATUS TXSCTPFUNC SctpAssocStatus(U8 board, SCTPAssocSta *sta, U32 assocId);
SCTP_STATUS TXSCTPFUNC SctpDestTransAddrStatus(U8 board, SCTPDtaSta *sta, S16 sapId);
SCTP_STATUS TXSCTPFUNC SctpSapStatus(U8 board, SCTPSapSta *sta, S16 sapId);
#ifdef __cplusplus
}
#endif

#endif  /* _SCTPMAPI_H_  */
