#ifdef WIN32
#include <windows.h>
#include <conio.h>
#include <process.h>
#endif
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "ss7type.h"
#include "hmidef.h"
#include "isupdef.h"
#include "isupmgmt.h"
#include "demodef.h"


/* Globals exported by MAIN to other threads */
extern unsigned char    boardNum;               /* Board number to use on this node (1..8) */
extern CTAHD            isupHndl;
extern char            *promptStr;

static U8 GetSapSwType( U8 board, U8 usap );

/* Service name/manager pair for ctaInitialize */

/* Service list for ctaOpenServices */
CTA_SERVICE_DESC    HmiOpenSvcLst[] = {{{"HMI", "HMIMGR"}, {0}, {0}, {0}}};

/* Service list for ctaCloseServices */
char    *HmiCloseSvcLst[] = { "HMI" };


/*
 * initCtAccess
 *
 * This function initializes CT Access, creates all queues and contexts
 * required by this application, and opens all necessary instances
 * of the HMI service for both the user interface (main thread) and
 * the controller FSM thread.
 *
 * Paramters:   None
 *
 * Returns:     SUCCESS     sucessful
 *              other       CT Access error has occurred
 */
void HMIThread(void *args)
{
    DWORD           ret;
    CTAQUEUEHD      hmiQueue = 0;           /* controller FSM queue handle */
    CTAHD           hmiHndl  = NULL_CTAHD;         /* context handle for async events */
    char            tmpbuf[80];
    CTA_EVENT       ctaEvent = {0};
    char           *buf;
    U8              mtpInstance = 1;

    UNREFERENCED_PARAMETER(args);

    /* create the CT Access queue used to pass all events to the ctlr FSM */
    if((ret = ctaCreateQueue(NULL, 0, &hmiQueue)) != SUCCESS )
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr, "HMIThread terminating, error creating CT Access queue: %s\n", tmpbuf);
        THREAD_EXIT();
    }

    /* create the controller FSM CT Access context for HMI async events
     * and open the HMI async events service on that context
     */
    if ((ret = ctaCreateContext(hmiQueue, 0, "DEMO", &hmiHndl)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr, "HMIThread terminating, error creating CT Access context: %s\n", tmpbuf);
        THREAD_EXIT();
    }

    /* open the specified HMI service on the context just created */
    HmiOpenSvcLst[0].svcargs.args[0] = 0;

    /*In case of host ISUP, get board number (MTP Instance) on which MTP
     * is running*/
    if ( boardNum == 0) {
        mtpInstance = GetSapSwType(boardNum, SAP_ID);
        HmiOpenSvcLst[0].svcargs.args[1] = mtpInstance;
    }
    else
       HmiOpenSvcLst[0].svcargs.args[1] = boardNum;     /* board - use global */

    HmiOpenSvcLst[0].svcargs.args[2] = HMI_RCV_EVENTS;
    if ((ret = ctaOpenServices(hmiHndl, &HmiOpenSvcLst[0], 1)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr, "HMIThread terminating, error opening HMI service: %s\n", tmpbuf);
        THREAD_EXIT();
    }

    /* Wait for service open to complete. */
    if ((ret = ctaWaitEvent(hmiQueue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr, "HMIThread() terminating, ctaWaitEvent() failed: %s\n", tmpbuf);
        THREAD_EXIT();
    }

    if ((ctaEvent.id != CTAEVN_OPEN_SERVICES_DONE) || (ctaEvent.value != CTA_REASON_FINISHED))
    {
        fprintf(stderr, "HMIThread() terminating, ctaOpenServices() failed\n");
        THREAD_EXIT();
    }

    for (;;)
    {
        ctaWaitEvent(hmiQueue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER);

        switch(ctaEvent.id)
        {
           case HMI_EVN_NOWPRIMARY:
                buf = "Now Primary";
                promptStr = "Primary> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = PRIMARY_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_NOWBACKUP:
                buf = "Now Backup";
                promptStr = "Backup> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = BACKUP_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_STARTING:
                buf = "Now Starting";
                promptStr = "Starting> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = STARTED_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_NOWSTANDALONE:
                buf = "Now Standalone";
                promptStr = "Stand Alone> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = STANDALONE_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_STOP:
                buf = "Stop";
                promptStr = "Stopped> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = STOP_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_INSERT:
                buf = "Board Inserted";
                break;

            case HMI_EVN_EXTRACT:
                buf = "Extract Pending";
                break;

            case HMI_EVN_TASKDEAD:
                buf = "Task Failed";
                break;

            case HMI_EVN_BRDDEAD:
                buf = "Board Dead";
                promptStr = "Dead> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = DEAD_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_HALTED:
                buf = "Board Halted";
                promptStr = "Halted> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = DEAD_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_LOADING:
                buf = "Board Loading";
                promptStr = "Loading> ";
                printf("\n%s", promptStr);
                ctaEvent.ctahd  = isupHndl;
                ctaEvent.id     = DEAD_EVT;
                ctaEvent.buffer = NULL;
                ctaEvent.size   = 0;
                ctaQueueEvent(&ctaEvent);
                break;

            case HMI_EVN_CONNECTED:
                buf = "Board Connected";
                break;

            case HMI_EVN_ISOLATED:
                buf = "Board Isolated";
                break;

            case HMI_EVN_CONFLICT:
                buf = "Board Conflict";
                break;

            case HMI_EVN_SERVICE_DOWN:
                buf = "HMI Stopped";
                break;

            default:
                buf = "Unknown";
                break;
        }
/*        fprintf(stderr, "%s received from HMI\n", buf);*/
    }
}                                       /* initCtAccess */

/* GetMtpInstance
 *
 * Determines the board number(MTP Instance) on which MTP is running in case of
 * Host ISUP by opening the ISUP management API for the board zero and retrieving
 * the NSAP configuration.
 *
 * Parameters:
 *  board       board number to open
 *  usap        user SAP number
 *
 * Returns:
 *  MTP Instance read from ISUP config; if couldn't be read, defaults to 1.
 */
static U8 GetSapSwType( U8 board, U8 usap )
{
    U8              mtpInstance  = 1;
    S16             status;
    IsupNSapCfg     nsapcfg;

    /* open the ISUP management API and fetch the switch type from SAP config */
    status = isupInitMgmtAPI( board, MGMT_ENT_ID );
    if( status != ISUP_MGMT_SUCCESS )
    {
        printf( "Error: isupInitMgmtAPI [board %d] failed [%d], defaulting to 1\n", board, status );
        return( mtpInstance );
    }

    /* save entire NSAP config for use by scripts */
    status = isupGetNSapCfg( board, &nsapcfg, usap );
    if( status != ISUP_MGMT_SUCCESS )
    {
        printf( "Error: isupGetNSapCfg [board %d, SAP %d] failed [%d], defaulting to 1\n",
                board, usap, status );
        return( mtpInstance );
    }
    else
    {
        if ( nsapcfg.spId == 0 )        /* For MTP, NSAP is always 0 */
           mtpInstance = nsapcfg.dstInstance;
    }

    isupTermMgmtAPI( board );
    return( mtpInstance );
}                                       /* GetSapSwType */
