/*
 * orig.c - sample call origination application
 *
 */


#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"

#include "dpruser.h"
#include "txcpi.h"

#include "ss7type.h"
#include "isupdef.h"

#include "demodef.h"

/* globals */
U32 lastTrans = 0;
U32 lastIncCnt   = 0;
U32 lastOutCnt   = 0;

extern S16      switchType;
extern U8       boardNum;
extern Circuit *circuits;
extern short    traceFlags;
extern CirId    numIn;
extern CirId    numOut;
extern CirId    firstIn;
extern CirId    lastIn;
extern CirId    firstOut;
extern CirId    lastOut;
extern int      standAlone;

CTAHD               isupHndl         = NULL_CTAHD;
CTAQUEUEHD          isupQueue        = 0;
CTA_SERVICE_DESC    isupOpenSvcLst[] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}}};
char               *isupCloseSvcLst[] = { "ISUP" };

static void DeadState(CTA_EVENT *ctaEvent);
static void StartedState(CTA_EVENT *ctaEvent);
static void BackupState(CTA_EVENT *ctaEvent);
static void PrimaryState(CTA_EVENT *ctaEvent);
static void StandAloneState(CTA_EVENT *ctaEvent);
void (*isupState)(CTA_EVENT *ctaEvent) = DeadState;
static void HandleIsupEvent();
static void OpenISUP(CTA_EVENT *ctaEvent);
static void CloseISUP(CTA_EVENT *ctaEvent);
static void RestoreBackupBoard();
void CheckPoint(Circuit *circuit);
void AppReady();

DWORD EventCircuit(CirId srcId, CirId dstId, DWORD evtId, Event *event)
{
    CTA_EVENT   ctaEvent = {0};

    ctaEvent.id     = evtId;
    ctaEvent.ctahd  = circuits[dstId].ctaHndl;
    ctaEvent.buffer = event;
    ctaEvent.size   = (event == NULL) ? 0 : sizeof(Event);  /* if event is null, set length to 0 */
    ctaEvent.value  = (dstId) ? dstId : srcId;              /* if dstId is test thread, send our own cirId */

    return(ctaQueueEvent(&ctaEvent));
}
void GroupReq(CirId cirId, SiRangStat *rangStat, int evtId)
{
    CirId       idx      =  0;

    do
    {
        if (rangStat->status.pres == PRESENT)
        {
            if (!(rangStat->status.val[idx / 8] & (1 << idx % 8)))
            {
                ++idx;
                continue;
            }
        }
        EventCircuit(0, (cirId + idx), evtId, NULL);

        ++idx;

    } while (idx <= rangStat->range.val);

}


void ISUPThread(void *dummy)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;

    UNREFERENCED_PARAMETER(dummy);

    if ((ret = ctaCreateQueue(NULL, 0, &isupQueue)) != SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ctaCreatQueue() failed: %d\n", ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(isupQueue, 0, NULL, &isupHndl)) != SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ctaCreatContext() failed: %d\n", ret);
        THREAD_EXIT();
    }

    /* if already in standalone mode (i.e., running without HMI), set initial state
     * to standalone.
     */
    if( standAlone )
    {
        OpenISUP(&ctaEvent);
        isupState = StandAloneState;
    }

    for (;;)
    {
        if ((ret = (DWORD)ctaWaitEvent(isupQueue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "isupThread() terminating, ctaWaitEvent() failed\n");
            THREAD_EXIT();
        }
        (*isupState)(&ctaEvent);
    }
}


static void DeadState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "ISUPThread(): 0x%x %d DeadState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case ISUPEVN_DATA:
            fprintf(stderr, "ISUPThread(): Unexpected ISUP event received in Dead state\n");
            break;

        case STARTED_EVT:
            isupState = StartedState;
            break;

        case BACKUP_EVT:
            OpenISUP(ctaEvent);
            RestoreBackupBoard();
            AppReady();
            isupState = BackupState;
            break;

        case PRIMARY_EVT:
            OpenISUP(ctaEvent);
            AppReady();
            isupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            OpenISUP(ctaEvent);
            isupState = StandAloneState;
            break;

        case DEAD_EVT:
            isupState = DeadState;
            return;

        case STOP_EVT:
            isupState = DeadState;
            return;

        case APPREADY_EVT:
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );
            return;
    } /* switch CT Access event */

}

static void StartedState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "ISUPThread(): 0x%x %d StartedState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case ISUPEVN_DATA:
            fprintf(stderr, "ISUPThread(): Unexpected ISUP event received in Started state\n");
            break;

        case DEAD_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        case STARTED_EVT:
            isupState = StartedState;
            break;

        case BACKUP_EVT:
            OpenISUP(ctaEvent);
            RestoreBackupBoard();
            AppReady();
            isupState = BackupState;
            break;

        case PRIMARY_EVT:
            OpenISUP(ctaEvent);
            AppReady();
            isupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            OpenISUP(ctaEvent);
            standAlone = TRUE;
            isupState = StandAloneState;
            break;

        case APPREADY_EVT:
            break;

        case STOP_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void BackupState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "ISUPThread(): 0x%x %d BackupState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case ISUPEVN_DATA:
            HandleIsupEvent();
            break;

        case DEAD_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        case STARTED_EVT:
            isupState = StartedState;
            break;

        case BACKUP_EVT:
            isupState = BackupState;
            break;

        case PRIMARY_EVT:
            isupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            isupState = StandAloneState;
            break;

        case APPREADY_EVT:
            AppReady();
            break;

        case STOP_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void PrimaryState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "ISUPThread(): 0x%x %d PrimaryState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case ISUPEVN_DATA:
            HandleIsupEvent();
            break;

        case DEAD_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        case STARTED_EVT:
            isupState = StartedState;
            break;

        case BACKUP_EVT:
            isupState = BackupState;
            break;

        case PRIMARY_EVT:
            isupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            isupState = StandAloneState;
            break;

        case APPREADY_EVT:
        {
            CirId       cirIdx;

            for (cirIdx = 1; cirIdx <= (numIn + numOut); cirIdx++)
                if ((circuits[cirIdx].callState != CSIDLE)  ||
                    (circuits[cirIdx].blkState != BSNOTBLK) ||
                    (circuits[cirIdx].transient == TRUE))
                     CheckPoint(&circuits[cirIdx]);
        }
        break;

        case STOP_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void StandAloneState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "ISUPThread(): 0x%x %d StandAloneState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case ISUPEVN_DATA:
            HandleIsupEvent();
            break;

        case DEAD_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        case STARTED_EVT:
            isupState = StartedState;
            break;

        case BACKUP_EVT:
            isupState = BackupState;
            break;

        case PRIMARY_EVT:
            isupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            isupState = StandAloneState;
            break;

        case STOP_EVT:
            CloseISUP(ctaEvent);
            isupState = DeadState;
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void HandleIsupEvent()
{
    Event          *evt;
    DWORD           ret;
    CirId           i;

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        fprintf(stderr, "isupThread() terminating, could not allocate required resources\n");
        THREAD_EXIT();
    }
    if ((ret = ISUPRetrieveMessage(isupHndl, &evt->sdu, &evt->info, 0)) != ISUP_SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ISUPRetrieveMessage() failed: %d\n", ret);
        THREAD_EXIT();
    }

    switch( evt->info.indType )
    {
        case EVTSITBNDCFM:              /* connect confirmation */
            free(evt);
            break;

        case EVTSITCONCFM:              /* connect confirmation */
            EventCircuit(0, evt->info.circuit, ANM_MSG, evt);
            break;

        case EVTSITCONIND:              /* connect indication (incoming call) */
            EventCircuit(0, evt->info.circuit, IAM_MSG, evt);
            break;

        case EVTSITSUSPIND:             /* suspend indication */
            EventCircuit(0, evt->info.circuit, SUS_MSG, evt);
            break;

        case EVTSITRESMIND:             /* resume indication */
            EventCircuit(0, evt->info.circuit, RES_MSG, evt);
            break;

        case EVTSITCNSTIND:             /* connect status indication */
            switch (evt->info.evntType)
            {
                case ADDRCMPLT:
                    EventCircuit(0, evt->info.circuit, ACM_MSG, evt);
                    break;

                case PROGRESS:                                                  /* call progress info */
                    EventCircuit(0, evt->info.circuit, CPG_MSG, evt);
                    break;

                case INFORMATION:                                               /* information indication */
                    EventCircuit(0, evt->info.circuit, INF_MSG, evt);
                    break;

                case INFORMATREQ:                                               /* information rqst */
                    EventCircuit(0, evt->info.circuit, INR_MSG, evt);
                    break;

                case SUBSADDR:                                          /* subsequent address */
                    EventCircuit(0, evt->info.circuit, SAM_MSG, evt);
                    break;

                default:
                    /* unrecognized connect status indication */
                    EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
                    free(evt);
                    break;
            } /* switch connect status indication */
            break;

        case EVTSITRELIND:              /* Release indication */
            EventCircuit(0, evt->info.circuit, REL_MSG, evt);
            break;

        case EVTSITRELCFM:              /* Release confirmation (release complete) */
            EventCircuit(0, evt->info.circuit, RLC_MSG, evt);
            break;

        case EVTSITSTAIND:              /* status indication */
            switch (evt->info.evntType)
            {
                case ERRORIND: /* ERROR indication from ISUP layer */
                    EventCircuit(0, evt->info.circuit, ERR_EVT, evt);
                    break;

                case CIRGRPQRYRSP:
                    break;

                case CIRBLKREQ:
                    EventCircuit(0, evt->info.circuit, BLO_MSG, evt);
                    break;

                case CIRBLKRSP:
                    EventCircuit(0, evt->info.circuit, BLA_MSG, evt);
                    break;

                case CIRUNBLKREQ:
                    EventCircuit(0, evt->info.circuit, UBL_MSG, evt);
                    break;

                case CIRUNBLKRSP:
                    EventCircuit(0, evt->info.circuit, UBA_MSG, evt);
                    break;

                case CIRGRPBLKREQ:
                    ret = ISUPStatusReq(isupHndl, circuits[evt->info.circuit].suId, 0, 0, 0, evt->info.circuit, CIRGRPBLKRSP, &evt->sdu.m.siStaEvnt);

                    if (ret != ISUP_SUCCESS)
                        printf("ISUPThread: Error sending Curcuit Group Block response, status = %d\n", ret);
                    GroupReq(evt->info.circuit, &evt->sdu.m.siStaEvnt.rangStat, CGB_MSG);
                    free(evt);
                    break;

                case CIRGRPBLKRSP:
                    free(evt);
                    break;

                case CIRGRPUNBLKREQ:
                    ret = ISUPStatusReq(isupHndl, circuits[evt->info.circuit].suId, 0, 0, 0, evt->info.circuit, CIRGRPUNBLKRSP, &evt->sdu.m.siStaEvnt);

                    if (ret != ISUP_SUCCESS)
                        printf("ISUPThread: Error sending Curcuit Group Unblock response, status = %d\n", ret);
                    GroupReq(evt->info.circuit, &evt->sdu.m.siStaEvnt.rangStat, CGU_MSG);
                    free(evt);
                    break;

                case CIRGRPUNBLKRSP:
                    free(evt);
                    break;

                case CIRGRPRESACK:
                    free(evt);
                    break;

                case CIRRESREQ:
                    EventCircuit(0, evt->info.circuit, RSC_MSG, evt);
                    break;

                case CIRGRPRESREQ:
                    GroupReq(evt->info.circuit, &evt->sdu.m.siStaEvnt.rangStat, RSC_EVT);
                    free(evt);
                    break;

                case REATTEMPT:
                    free(evt);
                    break;

                case CIRUNEQPD:
                    free(evt);
                    break;

                case CIRRESREQLOC:
                    EventCircuit(0, evt->info.circuit, RSC_MSG, evt);
                    break;

                case MTPPAUSE:
                    free(evt);
                    break;

                case MTPRESUME:
                    free(evt);
                    break;

                case MTPCONGEST:
                    free(evt);
                    break;

                case MTPSTOPCONGEST:
                    free(evt);
                    break;

                case RMTUSRUNAVAIL:
                    free(evt);
                    break;

                case RMTUSRAVAIL:
                    free(evt);
                    break;

                case MTPBACKUP:
                    lastTrans = 0;
                    lastIncCnt   = 0;
                    lastOutCnt   = 0;

                    for (i = 1; i <= (numIn + numOut); i++)
                    {
                        if (circuits[i].transient)
                        {
                            ++lastTrans;
                            EventCircuit(0, i, IDLE_EVT, NULL);
                        }
                        if ( circuits[i].callState == CSINCBUSY)
                            ++lastIncCnt;
                        if ( circuits[i].callState == CSOUTBUSY)
                            ++lastOutCnt;
                    }
                    free(evt);
                    break;

                case MTPPRIMARY:
                    lastTrans = 0;
                    lastIncCnt   = 0;
                    lastOutCnt   = 0;

                    for (i = 1; i <= (numIn + numOut); i++)
                    {
                        if (circuits[i].transient)
                        {
                            ++lastTrans;
                            EventCircuit(0, i, RSC_EVT, NULL);
                        }
                        if ( circuits[i].callState == CSINCBUSY)
                            ++lastIncCnt;
                        if ( circuits[i].callState == CSOUTBUSY)
                            ++lastOutCnt;
                    }
                    free(evt);
                    break;

                case MTPSTANDALONE:
                    free(evt);
                    break;

                case INVRUNSTATE:
                    fprintf(stderr, "INVRUNSTATE received\n");
                    free(evt);
                    break;

                default: /* all other status indications */
                    EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
                    free(evt);
                    break;
            }    /* switch status indication */

            break;

        default: /* unrecognized event */
            EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
            free(evt);
            break;

    } /* switch ISUP event */
} /* case ISUPEVN_DATA */

static void CloseISUP(CTA_EVENT *ctaEvent)
{
    DWORD           ret;

    if ((ret = ctaCloseServices(isupHndl, isupCloseSvcLst, 1)) != SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ctaCloseServices() failed: %d\n", ret);
        THREAD_EXIT();
    }

    /* Wait for service open to complete. */
    for (;;)
    {
        if ((ret = (DWORD)ctaWaitEvent(isupQueue, ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "isupThread() terminating, ctaWaitEvent() failed: %d\n", ret);
            THREAD_EXIT();
        }

        if (ctaEvent->id != CTAEVN_CLOSE_SERVICES_DONE)
            continue;
        else if (ctaEvent->value != CTA_REASON_FINISHED)
        {
            fprintf(stderr, "isupThread() terminating, ctaCloseServices() failed\n");
            THREAD_EXIT();
        }
        else
            return;
    }
}

static void RestoreBackupBoard()
{
    CirId       cirId   = 1;
    CirId       cirIdx   = 1;
    SiStaEvnt   staEvnt = {0};
    U8         *status = staEvnt.rangStat.status.val;

    staEvnt.rangStat.eh.pres       = PRESENT;
    staEvnt.rangStat.range.pres    = PRESENT;
    staEvnt.rangStat.status.pres   = PRESENT;

    while (cirIdx <= (numIn + numOut))
    {
        status[(cirIdx - 1) % 128] = circuits[cirIdx].blkState | circuits[cirIdx].callState;
        ++cirIdx;
        ++staEvnt.rangStat.status.len;

        if (!((cirIdx - 1) % 128))
        {
            staEvnt.rangStat.range.val = (U8)127;
            ISUPStatusReq(isupHndl, 0, 0, 0, FALSE, cirId, CIRGRPSET, &staEvnt);
            staEvnt.rangStat.status.len = 0;
            cirId = cirIdx;
        }
    }
    if (staEvnt.rangStat.status.len)
    {
        staEvnt.rangStat.range.val = (staEvnt.rangStat.status.len - 1);
        ISUPStatusReq(isupHndl, 0, 0, 0, FALSE, cirId, CIRGRPSET, &staEvnt);
    }
}

static void OpenISUP(CTA_EVENT *ctaEvent)
{
    DWORD           ret;

    isupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
    isupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
    isupOpenSvcLst[0].svcargs.args[2] = ENT_ID;       /* dprChan */
    isupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
    isupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
    isupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */


    if ((ret = ctaOpenServices(isupHndl, isupOpenSvcLst, 1)) != SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ctaOpenServices() failed: %d\n", ret);
        THREAD_EXIT();
    }

    /* Wait for service open to complete. */
    if ((ret = ctaWaitEvent(isupQueue, ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
    {
        fprintf(stderr, "isupThread() terminating, ctaWaitEvent() failed: %d\n", ret);
        THREAD_EXIT();
    }
    if ((ctaEvent->id != CTAEVN_OPEN_SERVICES_DONE) || (ctaEvent->value != CTA_REASON_FINISHED))
    {
        fprintf(stderr, "isupThread() terminating, ctaOpenServices() failed: id - 0x%x value - %d\n", ctaEvent->id, ctaEvent->value);
        THREAD_EXIT();
    }
}
char *eventNames[] =
{
    "IAM_MSG       ", /* 0x00 */
    "ACM_MSG       ", /* 0x01 */
    "ANM_MSG       ", /* 0x02 */
    "REL_MSG       ", /* 0x03 */
    "RLC_MSG       ", /* 0x04 */
    "RSC_MSG       ", /* 0x05 */
    "BLO_MSG       ", /* 0x06 */
    "BLA_MSG       ", /* 0x07 */
    "UBL_MSG       ", /* 0x08 */
    "UBA_MSG       ", /* 0x09 */
    "INF_MSG       ", /* 0x0a */
    "INR_MSG       ", /* 0x0b */
    "CON_MSG       ", /* 0x0c */
    "CPG_MSG       ", /* 0x0d */
    "SUS_MSG       ", /* 0x0e */
    "RES_MSG       ", /* 0x0f */
    "SAM_MSG       ", /* 0x10 */
    "GRS_MSG       ", /* 0x11 */
    "CGB_MSG       ", /* 0x12 */
    "CGU_MSG       ", /* 0x13 */
    "Undefined     ", /* 0x14 */
    "Undefined     ", /* 0x15 */
    "Undefined     ", /* 0x16 */
    "Undefined     ", /* 0x17 */
    "Undefined     ", /* 0x18 */
    "Undefined     ", /* 0x19 */
    "Undefined     ", /* 0x1a */
    "Undefined     ", /* 0x1b */
    "Undefined     ", /* 0x1c */
    "Undefined     ", /* 0x1d */
    "Undefined     ", /* 0x1e */
    "Undefined     ", /* 0x1f */
    "IAM_EVT       ", /* 0x20 */
    "ACM_EVT       ", /* 0x21 */
    "ANM_EVT       ", /* 0x22 */
    "REL_EVT       ", /* 0x23 */
    "RLC_EVT       ", /* 0x24 */
    "RSC_EVT       ", /* 0x25 */
    "BLO_EVT       ", /* 0x26 */
    "BLA_EVT       ", /* 0x27 */
    "UBL_EVT       ", /* 0x28 */
    "UBA_EVT       ", /* 0x29 */
    "INF_EVT       ", /* 0x2a */
    "INR_EVT       ", /* 0x2b */
    "CON_EVT       ", /* 0x2c */
    "CPG_EVT       ", /* 0x2d */
    "SUS_EVT       ", /* 0x2e */
    "RES_EVT       ", /* 0x2f */
    "SAM_EVT       ", /* 0x30 */
    "GRS_EVT       ", /* 0x31 */
    "CGB_EVT       ", /* 0x32 */
    "CGU_EVT       ", /* 0x33 */
    "Undefined     ", /* 0x34 */
    "Undefined     ", /* 0x35 */
    "Undefined     ", /* 0x36 */
    "Undefined     ", /* 0x37 */
    "Undefined     ", /* 0x38 */
    "Undefined     ", /* 0x39 */
    "Undefined     ", /* 0x3a */
    "Undefined     ", /* 0x3b */
    "Undefined     ", /* 0x3c */
    "Undefined     ", /* 0x3d */
    "Undefined     ", /* 0x3e */
    "Undefined     ", /* 0x3f */
    "ERR_EVT       ", /* 0x40 */
    "DEAD_EVT      ", /* 0x41 */
    "STARTED_EVT   ", /* 0x42 */
    "BACKUP_EVT    ", /* 0x43 */
    "PRIMARY_EVT   ", /* 0x44 */
    "STANDALONE_EVT", /* 0x45 */
    "CHKPNT_EVT    ", /* 0x46 */
    "IDLE_EVT      ", /* 0x47 */
    "Undefined     "
};

