#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"

#include "ss7type.h"
#include "isupdef.h"

#include "demodef.h"

extern CTAHD    isupHndl;
extern short    traceFlags;
extern S16      switchType;
extern Circuit *circuits;

extern char *promptStr;

void CheckPoint(Circuit *circuit);
CirId CircuitSelect(CirId inCirId, U8 testCall);

void TestThread(void *arglist)
{
    Circuit        *circuit = (Circuit *)arglist;
    CTAQUEUEHD      queue;
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    Event          *evt;

    if ((ret = ctaCreateQueue(NULL, 0, &queue)) != SUCCESS)
    {
        fprintf(stderr, "TestThread() terminating, ctaCreateQueue() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(queue, 0, NULL, &circuit->ctaHndl)) != SUCCESS)
    {
        fprintf(stderr, "TestThread() terminating, ctaCreateContext() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }


    for (;;)
    {
        if ((ret = ctaWaitEvent(queue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "TestThread() terminating, ctaWaitEvent() failed: %d\n", circuit->cirId, ret);
            THREAD_EXIT();
        }
        evt = (Event *)ctaEvent.buffer;

        switch (ctaEvent.id)
        {
            case IAM_EVT:
                printf("TestThread: IAM_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case ACM_EVT:
                printf("TestThread: ACM_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case ANM_EVT:
                printf("TestThread: ANM_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case REL_EVT:
                printf("TestThread: REL_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case RSC_EVT:
                printf("TestThread: RSC_EVT received from circuit %d\n", ctaEvent.value);
                break;

            case INF_EVT:
                printf("TestThread: INF_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case INR_EVT:
                printf("TestThread: INR_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case CON_EVT:
                printf("TestThread: CON_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case CPG_EVT:
                printf("TestThread: CPG_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case SUS_EVT:
                printf("TestThread: SUS_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case RES_EVT:
                printf("TestThread: RES_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case SAM_EVT:
                printf("TestThread: SAM_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            default:
                printf("TestThread: unhandled event 0x%x from circuit %d\n", ctaEvent.id, evt->info.circuit);
                free(evt);

        }
        printf(promptStr);
    }
}

