/*
 * orig.c - sample call origination application
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef WIN32
#include <conio.h>
#endif

#ifdef UNIX
#include "sol-dep.h"
#endif /* UNIX */

#include "ss7type.h"
#include "isupdef.h"
#include "hmidef.h"


#ifdef UNIX
#define _getch getchar
#endif

/* globals */

U8       boardNum = 1;
CirId    circuit = 1;
unsigned int timeout = 5000;
U8       primary = 0;
U8       resume = 0;
U8       reset = 0;
U8       loadFlag = 0;
SiInstId suInstId;
SiInstId spInstId;
SuId     suId;
S16 switchType = ST_ANS92;
CTAQUEUEHD      FstQueue = 0;
CTAHD           FstCtaHd = NULL_CTAHD;
U8              *called;
U8              *calling = NULL;      /* default = no calling pty info in IAM */
DWORD    needed;
Bool     bPlacedOutbCall = FALSE;
#define NEED_OPEN_SERVICES_DONE     (1<<0)
#define NEED_RESUME                 (1<<1)
#define NEED_REDUNDANT_TYPE         (1<<2)

static void RebindISUP( CTAHD CtaHd, CTAQUEUEHD CtaQHd );

/* Instance and Entity ID required of any application that
 * utilizes the ISUP API.
 */

#define ENT_ID     0x20     /* Task ID - for host applications this
			     * doubles as the DPR channel, so it should
			     * be in the range 0x20 - 0x3f */

#define INST_ID    0        /* Refers to application's board number *
			     * Host board number is always 0 */

/* This application always uses isup sap 0 */

#define SAP_ID      0


/* internal function prototypes */

void handleStatInd( CTAHD ctahd, SiStaEvnt *event, IsupRcvInfoBlk *infoBlk );
void handleConStaInd( CTAHD ctahd, SiCnStEvnt *event, IsupRcvInfoBlk *infoBlk );
void issueConnectRequests( CTAHD ctahd );


/*This function is called to send the IAM and other connection messages
 * when the open services is complete and the standalone and MTP resume are
 * received
 */
void issueConnectRequests( CTAHD ctahd )
{
      DWORD       status;
      SiConEvnt   siConEvnt;
      U8          cdPty[20];
      U8          cdPtyLen = 0;
      U8          cgPty[20];
      U8          cgPtyLen = 0;
      SiCnStEvnt  siCnStEvnt;

      /* Got the resume, now send the IAM message */
      cdPtyLen = ISUPASCIItoBCD(called, cdPty, 20);
      cgPtyLen = ISUPASCIItoBCD(calling, cgPty, 20);

      printf("cdPtyLen = %d, cgPtyLen = %d\n", cdPtyLen, cgPtyLen);

      ISUPInitIAM(switchType, &siConEvnt, cdPty, cdPtyLen, cgPty, cgPtyLen);
      status = ISUPConnectReq(ctahd, suId, 0, 0, 1, circuit, &siConEvnt);
      if( status != ISUP_SUCCESS )
         printf( "ISUPConnectReq() failed status = %d\n", status );
      else
         printf( "Initial Address Message sent for circuit %ld\n", circuit );

      if (switchType == ST_ITU)           /* do a SGM if this is ITU!! */
      {
         memset(&siCnStEvnt, 0, sizeof(siCnStEvnt));
         siCnStEvnt.accTrnspt.eh.pres          = TRUE;
         siCnStEvnt.accTrnspt.infoElmts.pres   = TRUE;
         siCnStEvnt.accTrnspt.infoElmts.len    = 6;
         siCnStEvnt.accTrnspt.infoElmts.val[0] = 0x31;
         siCnStEvnt.accTrnspt.infoElmts.val[1] = 0x32;
         siCnStEvnt.accTrnspt.infoElmts.val[2] = 0x33;
         siCnStEvnt.accTrnspt.infoElmts.val[3] = 0x34;
         siCnStEvnt.accTrnspt.infoElmts.val[4] = 0x35;
         siCnStEvnt.accTrnspt.infoElmts.val[5] = 0x36;

         siCnStEvnt.genDigits.eh.pres          = TRUE;
         siCnStEvnt.genDigits.typeOfDigits.pres   = TRUE;
         siCnStEvnt.genDigits.typeOfDigits.val    = 1;
         siCnStEvnt.genDigits.encodeScheme.pres   = TRUE;
         siCnStEvnt.genDigits.encodeScheme.val    = 2;
         siCnStEvnt.genDigits.digits.pres   = TRUE;
         siCnStEvnt.genDigits.digits.len    = 5;
         siCnStEvnt.genDigits.digits.val[0] = 0x21;
         siCnStEvnt.genDigits.digits.val[1] = 0x22;
         siCnStEvnt.genDigits.digits.val[2] = 0x23;
         siCnStEvnt.genDigits.digits.val[3] = 0x24;
         siCnStEvnt.genDigits.digits.val[4] = 0x25;

         status = ISUPConnectStatusReq(ctahd, suId, 0, 0, circuit, &siCnStEvnt, SEGMENTATION);
         if( status != ISUP_SUCCESS )
           printf( "Segmentation failed status = %d\n", status );
         else
           printf( "Segmentation sent for circuit %ld\n", circuit );
      }

      bPlacedOutbCall = TRUE;
}

/* handleIsupMessage
 *
 * This function is called to handle all incoming ISUP messages.
 * It just calls the proper message handling function based on the
 * type of message received.
 */
DWORD handleIsupMessage(CTAHD ctahd)
{
    DWORD status;
    SiAllSdus       rcvEvent;
    SiAllSdus   sendEvent;
    IsupRcvInfoBlk      rcvInfo;

    status = ISUPRetrieveMessage(ctahd, &rcvEvent, &rcvInfo, 1);
    if (status == ISUP_NOMSG)
    {
      fprintf(stderr, "ISUPRetrieveMessage() did not get a message, probably a congestion event \n");
      return(ISUP_SUCCESS);
    }
    if (status != ISUP_SUCCESS)
    {
      fprintf(stderr, "ISUPRetrieveMessage() failed, returned %d\n", status);
      return(status);
    }
  /* determine indication/confirmation type received */
    switch( rcvInfo.indType )
    {

    case EVTSITBNDCFM:              /* bind confirmation */
      printf("orig: Bind Confirmation received\n");
	    break;

    case EVTSITCONCFM:              /* connect confirmation */
	    printf("orig: Connect Confirmation received for circuit %ld\n",
		    rcvInfo.circuit );
        ISUPInitREL(switchType, &sendEvent.m.siRelEvnt, CCCALLCLR);
        status = ISUPReleaseReq(ctahd, rcvInfo.suId, rcvInfo.suInstId,
				    rcvInfo.spInstId, rcvInfo.circuit, &sendEvent.m.siRelEvnt);
	    if (status != ISUP_SUCCESS)
           printf("orig: ISUPReleaseReq() failed status = %d\n", status);
	    else
           printf("orig: Release sent for circuit %ld\n", rcvInfo.circuit);
      break;

    case EVTSITCONIND:              /* connect indication (incoming call) */
	    printf("orig: Unexpected Connect Indication for circuit %ld\n", rcvInfo.circuit );

	    ISUPInitREL(switchType, &sendEvent.m.siRelEvnt, CCCALLRJT);
      status = ISUPReleaseReq(ctahd, rcvInfo.suId, rcvInfo.suInstId,
				    rcvInfo.spInstId, rcvInfo.circuit, &sendEvent.m.siRelEvnt);
	    if (status != ISUP_SUCCESS)
        printf("orig: ISUPReleaseReq() failed status = %d\n", status);
	    else
        printf("orig: Release sent for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITDATIND:              /* data indication (user-to-user data) */
	    printf("orig: Unexpected Data Indication for circuit %ld\n",
		    rcvInfo.circuit);
	    break;

    case EVTSITSUSPIND:             /* suspend indication */
	    printf("orig: Unexpected Suspend Indication for circuit %ld\n",
		    rcvInfo.circuit);
	    break;

    case EVTSITRESMIND:             /* resume indication */
	    printf("orig: Unexpected Resume Indication for circuit %ld\n",
		    rcvInfo.circuit);
	    break;

    case EVTSITCNSTIND:             /* connect status indication */
	    printf("orig: Connect Status Indication received for circuit %ld\n",
		    rcvInfo.circuit);
      handleConStaInd(ctahd, &rcvEvent.m.siCnStEvnt, &rcvInfo);
	    break;

    case EVTSITRELIND:              /* Release indication */
	    printf("orig: Unexpected Release Indication for circuit %ld\n",
		    rcvInfo.circuit);

      status = ISUPReleaseResp(ctahd, rcvInfo.suId, rcvInfo.suInstId,
				    rcvInfo.spInstId, rcvInfo.circuit, &rcvEvent.m.siRelEvnt);
	    if (status != ISUP_SUCCESS)
        printf("orig: ISUPReleaseResp() failed status = %d\n", status);
	    else
        printf("orig: Release Complete sent for circuit %ld\n", rcvInfo.circuit);
	    break;

    case EVTSITRELCFM:              /* Release confirmation (release complt) */
	    printf("orig: Release Complete received for circuit %ld\n",
		    rcvInfo.circuit);
      reset = 1;

      /*ISUPInitIAM(switchType, &rcvEvent.m.siConEvnt, (U8 *)"111", 6, (U8 *)"222", 6);
      status = ISUPConnectReq(ctahd, 0, 0, 0, 1, circuit, &rcvEvent.m.siConEvnt);
      if( status != ISUP_SUCCESS )
        printf( "ISUPConnectReq() failed status = %d\n", status );
      else
        printf( "Initial Address Message sent for circuit %ld\n", circuit );*/
      break;

    case EVTSITFACIND:              /* Facility indication */
	    printf("orig: Unexpected Facility Indication for circuit %ld\n",
		    rcvInfo.circuit);
	    break;

    case EVTSITSTAIND:              /* status indication */
      handleStatInd(ctahd, &rcvEvent.m.siStaEvnt, &rcvInfo);
	    break;

    default:
	    /* unrecognized event */
	    printf("orig: Unrecognized indication [%d] for circuit %ld\n",
		    (int) rcvInfo.indType, rcvInfo.circuit );
	    break;

    }
    return(ISUP_SUCCESS);

}       /* end handleIsupMessage */

/* handleStatInd
*
* This function handles a status indication from the far exchange
* or ISUP layer. A ERRORIND indicates that the call has failed, so
* the application terminates. Other indications are just logged (a real
* application would need to respond to some of these indications, such
* as circuit/group blocking/unblocking requests).
*/
void handleStatInd( CTAHD ctahd, SiStaEvnt *event, IsupRcvInfoBlk *infoBlk )
{
    DWORD       status;

	switch (infoBlk->evntType)
	{

    case ERRORIND:   /* ERROR indication from ISUP layer */
      if (event->causeDgn.eh.pres     && event->causeDgn.causeVal.pres)
      {
        printf("orig: ERROR Status, circ=%d, suId=%d, spId=%d, cause=%d\n",
            infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId,
            (int) event->causeDgn.causeVal.val );
      }
      else
      {
        printf("orig: ERROR Status, circ=%d, suId=%d, spId=%d, no cause value\n",
            infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId );
      }
          /*exit(150);*/
      break;

    case CONTCHK:           /* continuity check requested */
      printf("orig: Unexpected Continuity Check Request received for circuit %ld\n", infoBlk->circuit);
      break;

    case CONTREP:
      printf("orig: Unexpected Continuity Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPQRYRSP:
      printf("orig: Unexpected Circuit Query Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRBLKREQ:
      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRBLKRSP, event);

      if (status != ISUP_SUCCESS)
        printf("orig: Block Response failed for circuit %ld, status = %d\n",
            infoBlk->circuit, status);
      else
        printf("orig: Block response sent for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRBLKRSP:
      printf("orig: Unexpaected Block Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRUNBLKREQ:
      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRUNBLKRSP, event);

      if (status != ISUP_SUCCESS)
        printf("orig: Unblock Response failed for circuit %ld, status = %d\n",
            infoBlk->circuit, status);
      else
        printf("orig: Unblock response sent for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRUNBLKRSP:
      printf("orig: Unecpected Unblock Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPBLKREQ:
      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRGRPBLKRSP, event);

      if (status != ISUP_SUCCESS)
        printf("orig: Group Block Response failed for circuit %ld, status = %d\n",
            infoBlk->circuit, status);
      else
        printf("orig: Group Block response sent for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPBLKRSP:
      printf("orig: Unecpected Group Block Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPUNBLKREQ:
      status = ISUPStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId,
			    infoBlk->spInstId, 0, infoBlk->circuit, CIRGRPUNBLKRSP, event);

      if (status != ISUP_SUCCESS)
          printf("orig: Group Unblock Response failed for circuit %ld, status = %d\n",
            infoBlk->circuit, status);
      else
        printf("orig: Group Unblock response sent for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRGRPUNBLKRSP:
      printf("orig: Unecpected Group Unblock Response received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRRESERVE:
      printf("orig: Unexpected Circuit Reservation received for circuit %ld\n", infoBlk->circuit);
      break;

    case CIRRESREQ:
      printf("orig: Unexpected Circuit Reset circuit received for %ld\n", infoBlk->circuit);
      break;

    case CIRGRPRESREQ:
      printf("orig: Unexpected Circuit Group Reset received for circuit %ld\n", infoBlk->circuit);
      break;

    case MTPPAUSE:
      printf("orig: mtp pause\n");
      resume = 0;
      break;

    case MTPRESUME:
      printf("orig: mtp resume\n");
      if ( primary && (!resume) )
      {
         ISUPStatusReq(FstCtaHd, suId,suInstId,spInstId,FALSE,circuit,CIRRESREQ,NULL);
         reset  = 0;
      }
      resume = 1;

      needed &= ~NEED_RESUME;

      /* If the redundant type and resume received, then send IAM and other connection
       * requests */
      if ( (needed == 0) && (!bPlacedOutbCall) )
          issueConnectRequests( FstCtaHd );
      break;

    case MTPBACKUP:
      printf("orig: mtp backup\n");
      if ( !loadFlag)
          resume = 1;
      primary = 0;
      loadFlag = 0;
      break;

    case MTPPRIMARY:
      printf("orig: mtp primary\n");
      if ( !loadFlag)
          resume = 1;
      if ( resume && (!primary))
      {
          ISUPStatusReq(FstCtaHd, suId,suInstId,spInstId,FALSE,circuit,CIRRESREQ,NULL);
          reset = 0;
      }
      primary = 1;
      loadFlag = 0;
      needed &= ~NEED_REDUNDANT_TYPE;
      break;

    case MTPSTANDALONE:
      printf("orig: mtp stand alone\n");
      needed &= ~NEED_REDUNDANT_TYPE;
    break;

    default:        /* all other status indications */
      printf("orig: Unexpected Status received, circ=%d, suId=%d, spId=%d, type=%d\n",
				infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId,
				(int) infoBlk->evntType );
      break;

  }

} /* end of handleStatInd */

/* handleConStaInd
*
* This function handles a connection status indication event; if this
* is a info request for calling party number, an INFormation message is
* generated containing calling party number (only).
*
*/
void handleConStaInd( CTAHD ctahd, SiCnStEvnt *event, IsupRcvInfoBlk *infoBlk )
{
	int             circuit;

	circuit = (int) infoBlk->circuit;

	switch (infoBlk->evntType) {

	case ADDRCMPLT:
    printf( "Addr Complete received for circuit %ld\n", infoBlk->circuit );
      /* Service provider ID (spId) MUST be saved for use in all subsequent
      * messages associated with this connection
      */
    circuit  = infoBlk->circuit;
    suInstId = infoBlk->suInstId;
    spInstId = infoBlk->spInstId;
    suId     = infoBlk->suId;
		break;

	case MODIFY:                                                    /* modify request */
    printf( "Modify request received for circuit %ld\n", infoBlk->circuit );
    break;

	case MODCMPLT:                                                  /* modify complete */
    printf( "Modify Complete indication received for circuit %ld\n", infoBlk->circuit );
    break;

	case MODREJ:                                                    /* modify rejected */
    printf( "Modify Reject indication received for circuit %ld\n", infoBlk->circuit );
    break;

	case PROGRESS:                                                  /* call progress info */
		/* assumes mandatory event info is present */
    printf( "Progress (%d) indication received for circuit %ld\n",
      (int) event->evntInfo.evntInd.val, infoBlk->circuit );
    break;

	case FRWDTRSFR:                                                 /* forward transfer */
    printf( "Forward Transfer indication received for circuit %ld\n", infoBlk->circuit );
    break;

	case INFORMATION:                                               /* information indication */
    printf( "Information indication received for circuit %ld\n",
          (int) infoBlk->circuit );
    break;

	case INFORMATREQ:                                               /* information rqst */
    {
    U8      cgPtyLen;
    DWORD     status;

    printf( "Information Request received for circuit %ld\n", infoBlk->circuit );
    ISUPInitINF(switchType, event);

    event->infoInd.cgPtyAddrRespInd.val = CGPRTYADDRESP_INCL;

    event->cgPtyNum.eh.pres      = PRESENT;
    event->cgPtyNum.addrSig.pres = PRESENT;

    cgPtyLen = ISUPASCIItoBCD((U8 *)"8477069700", event->cgPtyNum.addrSig.val, MF_SIZE_TKNSTR);

    event->cgPtyNum.addrSig.len  = cgPtyLen/(U8)2;

    if (cgPtyLen % 2)
    {
	    event->cgPtyNum.addrSig.len += 1;
	    event->cgPtyNum.oddEven.val = NMB_ODD;
    }
    else
	    event->cgPtyNum.oddEven.val = NMB_EVEN;

    event->cgPtyNum.oddEven.pres     = PRESENT;
    event->cgPtyNum.natAddrInd.pres  = PRESENT;
    event->cgPtyNum.natAddrInd.val   = NATNUM;           /* national number */
    event->cgPtyNum.numPlan.pres     = PRESENT;
    event->cgPtyNum.numPlan.val      = NP_ISDN;          /* ISDN numbering plan */
    event->cgPtyNum.niInd.pres       = PRESENT;
    event->cgPtyNum.niInd.val        = NBMCMLTE;         /* number complete */

    status = ISUPConnectStatusReq(ctahd, infoBlk->suId, infoBlk->suInstId, infoBlk->spInstId,
        infoBlk->circuit, event, INFORMATION);
    if (status != ISUP_SUCCESS)
        printf("orig: ISUPConnectStatusReq() failed status = %d\n", status);
    else
        printf("orig: Information Response sent for circuit %ld\n", infoBlk->circuit);
    break;
    }
	case SUBSADDR:                                                  /* subsequent address */
    printf( "Subsequent Address received for circuit %ld\n", infoBlk->circuit );
    break;

	default:
		/* unrecognized connect status indication */
    printf( "Unrecognized connect status ind., circ=%ld, suId=%d, spId=%d\n",
				(int) infoBlk->circuit, (int) infoBlk->suInstId,
				(int) infoBlk->spInstId );
		break;

	}       /* of switch */

} /* end of handleCnStInd */


static void ErrorAndExit__( DWORD ErrId, char *Fcn, int line )
{

   printf( "Line(%d) error : (%d) error returned in (%s) call.\n", line, ErrId, Fcn );
   exit( 1 );
}
#define ErrorAndExit(e,f) ErrorAndExit__(e,f,__LINE__)

#if defined(HEALTH_MANAGEMENT)
#define num_isupServices 2
#else
#define num_isupServices 1
#endif

static void RebindISUP( CTAHD CtaHd, CTAQUEUEHD CtaQHd )
{
#if defined(HEALTH_MANAGEMENT)
    CTA_SERVICE_DESC isupOpenSvcLst[num_isupServices] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}}, {{"HMI", "HMIMGR"}, {0}, {0}, {0}}};
    char *isupCloseSvcLst[num_isupServices] = { "ISUP" , "HMI" };
#else
	CTA_SERVICE_DESC isupOpenSvcLst[num_isupServices] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}} };
    char *isupCloseSvcLst[num_isupServices] = { "ISUP" };
#endif

    CTA_EVENT       Event;
    DWORD           Ret;

   isupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
   isupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
   isupOpenSvcLst[0].svcargs.args[2] = ENT_ID;       /* dprChan */
   isupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
   isupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
   isupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */

    Ret = ctaCloseServices( CtaHd, isupCloseSvcLst, num_isupServices );
    if ( Ret != SUCCESS )
    {
        ErrorAndExit( Ret, "ctaCloseServices" );
    }

    /* Wait for service close to complete. */
    Ret = ctaWaitEvent( CtaQHd, &Event, (unsigned)CTA_WAIT_FOREVER );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }


    Ret = ctaOpenServices( CtaHd, isupOpenSvcLst, num_isupServices );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete. */
    Ret = ctaWaitEvent( CtaQHd, &Event, (unsigned)CTA_WAIT_FOREVER );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    if (( Event.id != CTAEVN_OPEN_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }
}

void DemoShowEvent( CTA_EVENT *event )
{
    char format_buffer[CTA_MAX_FORMAT_SIZE];
    char *prefix = "\t\t\t";  /* default demo indent */
    format_buffer[0] = '\0';
    ctaFormatEvent( prefix, event, format_buffer, CTA_MAX_FORMAT_SIZE );
    printf( "%s", format_buffer );
}

/* main()
 *
 * This is the main routine for the ISUP incoming test application.
 * It initializes the application's local circuit state database,
 * initializes the ISUP API (by binding to ISUP), and then just loops
 * waiting for messages from ISUP and dispatching the appropriate
 * routine to handle the message.
 */

int main( int argc, char *argv[] )
{
    short       optind = 1;
    char       *str;
#if defined(HEALTH_MANAGEMENT)
    CTA_SERVICE_DESC isupOpenSvcLst[num_isupServices] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}}, {{"HMI", "HMIMGR"}, {0}, {0}, {0}}};
    char *isupCloseSvcLst[num_isupServices] = { "ISUP" , "HMI" };
#else
	CTA_SERVICE_DESC isupOpenSvcLst[num_isupServices] = {{{"ISUP", "ISUPMGR"}, {0}, {0}, {0}} };
    char *isupCloseSvcLst[num_isupServices] = { "ISUP" };
#endif

    CTA_EVENT         Event;
    DWORD           Ret;
    CTA_INIT_PARMS      isupInitparms =                 { 0 };
#if defined(HEALTH_MANAGEMENT)
    CTA_SERVICE_NAME    isupServiceNames[num_isupServices] =            { { "ISUP", "ISUPMGR" }, { "HMI", "HMIMGR" }};
#else
	CTA_SERVICE_NAME    isupServiceNames[num_isupServices] =            { { "ISUP", "ISUPMGR" } };
#endif
    U8         inloop = 1;
    int        c;
    SiRelEvnt   relEvnt;
    /*SiCnStEvnt  siCnStEvnt;*/
    DWORD       status;
    ISUPAPISTATS pStats;
    SiStaEvnt siEvent = {0};

	/* check for required and optional parameters */
    if( argc < 3 )
    {
      printf( "Usage: %s [-b <board>] <called> <circuit> [<calling>]\n", argv[0] );
      exit(1);
    }

    str = argv[optind];

    while (str)
    {
      if (*str == '-')
      {
        optind++;
        switch (*++str)
        {
        case 'b':   /* which board should we speak to ? */
          if (*++str)         /* was there no space ? */
              boardNum = (U8)atoi(str);
          else
              boardNum = (U8)atoi(argv[optind++]);
          break;

        case 'i':   /* ITU ? */
          switchType = ST_ITU;
          break;

        case 'a':   /* ANSI BICC ? */
          switchType = ST_ANSIBICC;
          break;

        case 's':   /* ITU BICC ? */
          switchType = ST_ITUBICC;
          break;

        case 'd':   /* delay in seconds between origs ? */
          if (*++str)         /* was there no space ? */
            timeout = atoi(str);
          else
            timeout = atoi(argv[optind++]);
          break;

        default:
usage_and_exit:
		    printf( "Usage: %s [-b <board>] <called> <circuit> [<calling>]\n", argv[0] );
		    exit(1);
        }
      }
  else
    break;

    str = argv[optind];
  }/* first parameter is always called party, 2nd is always circuit,
	 * 3rd (optional) is calling
	 */
    if (argv[optind+1] == NULL)
        goto usage_and_exit;

    called = (U8 *)argv[optind++];
    circuit = atoi( argv[optind++] );

    if( argc > optind )
    {
      calling = (U8 *)argv[optind];   /* user specified it, so include it in IAM */
    }

   needed = NEED_RESUME | NEED_REDUNDANT_TYPE;     /* always need these before connects */
   needed |= NEED_OPEN_SERVICES_DONE;          /* also need open services done */

   /* bind this application as a user of the ISUP layer */
   isupInitparms.size = sizeof(CTA_INIT_PARMS);
   isupInitparms.traceflags = 0;
   isupInitparms.parmflags = 0;
   isupInitparms.ctacompatlevel = CTA_COMPATLEVEL;
/*   isupInitparms.traceflags = CTA_TRACE_ENABLE;
   isupInitparms.parmflags = CTA_PARM_MGMT_SHARED;
   isupInitparms.ctaflags = CTA_NOTIFY_UPDATE_WAITOBJS;*/

   Ret = ctaInitialize(isupServiceNames, num_isupServices, &isupInitparms);
   if (Ret != SUCCESS)
   {
           ErrorAndExit( Ret, "ctaInitialize" );
   }

   Ret = ctaCreateQueue( NULL, 0, &FstQueue );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaCreateQueue" );
   }

   /* Open FstCtaHd on FstQueue                   */
   Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaCreateContext" );
   }

   isupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
   isupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
   isupOpenSvcLst[0].svcargs.args[2] = ENT_ID;       /* dprChan */
   isupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
   isupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
   isupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */

   isupOpenSvcLst[1].svcargs.args[0] = 1;     /* board number */
   isupOpenSvcLst[1].svcargs.args[1] = boardNum;      /* srcInst */
   isupOpenSvcLst[1].svcargs.args[2] = HMI_RCV_EVENTS;       /* dprChan */

   Ret = ctaOpenServices( FstCtaHd, &isupOpenSvcLst[0], num_isupServices);
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaOpenServices" );
   }

   /* Wait for service open to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   /*DemoShowEvent(&Event);*/
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_OPEN_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   else
   {
      needed &= ~NEED_OPEN_SERVICES_DONE;
   }

	/* loop here receiving ISUP messages and calling the appropriate
	 * message handling function
     */
    while ( inloop)
    {
        /* cta wait event */
        Ret = ctaWaitEvent( FstQueue, &Event, timeout );
		/*DemoShowEvent(&Event);*/
        if ( Ret != SUCCESS )
        {
                ErrorAndExit( Ret, "ctaWaitEvent" );
        }
        switch ( Event.id )
        {
            case ISUPEVN_DATA:
            /* an ISUP event has occured, call ISUPRetrieveMessage() to get the event */
            handleIsupMessage(FstCtaHd);
            break;

            case CTAEVN_WAIT_TIMEOUT:
            /* 5 seconds passed, check for keyboard input */
                /* make sure a key was hit */
                if (_kbhit())
                {
                  c = _getch();
                  if (c == 'q')
                  {
                      inloop = 0;
                      break;
                  }
                  else if (c == 'r')
                  {
                      ISUPInitREL(switchType, &relEvnt, CCCALLCLR);
                      status = ISUPReleaseReq(FstCtaHd, suId, suInstId,
                                  spInstId, circuit, &relEvnt);
                      if (status != ISUP_SUCCESS)
                      printf("orig: ISUPReleaseReq() failed status = %d\n", status);
                      else
                      printf("orig: Release sent\n");
                      break;
                  }
                  else if ( c == 'b')
                  {
                      status = ISUPStatusReq (FstCtaHd, suId, suInstId, spInstId, FALSE, circuit, CIRBLKREQ, &siEvent);
                      if ( status != ISUP_SUCCESS)
                        printf("orig: Request for BLOCK failed. %d\n", status);
                      else
                        printf("orig: Success..Request to BLOCK sent to the task. \n");
                      break;
                  }
                  else if ( c == 'u')
                  {
                      status = ISUPStatusReq (FstCtaHd, suId, suInstId, spInstId, FALSE, circuit, CIRUNBLKREQ, &siEvent);
                      if ( status != ISUP_SUCCESS)
                        printf("orig: Request for UNBLOCK failed. %d\n", status);
                      else
                        printf("orig: Success..Request to UNBLOCK sent to the task. \n");
                      break;
                  }

                  else if (c == 's')
                  {
                      status = ISUPGetApiStats(FstCtaHd, &pStats, 0);
                      if (status != ISUP_SUCCESS)
                        printf("orig: ISUPGetApiStats() failed status = %d\n", status);
                      else
                      {
                        printf("qCount = %x\n", pStats.qCount);
                        printf("qPeak = %x\n", pStats.qPeak);
                        printf("txPending = %x\n", pStats.txPending);
                        printf("txPendPeak = %x\n", pStats.txPendPeak);
                        printf("txSuccess = %x\n", pStats.txSuccess);
                        printf("txFailed = %x\n", pStats.txFailed);
                        printf("txLastErr = %x\n", pStats.txLastErr);
                        printf("rxSuccess = %x\n", pStats.rxSuccess);
                        printf("apiQCongLvl = %x\n", pStats.apiQCongLvl);
                        printf("isupCongLvl = %x\n", pStats.isupCongLvl);
                        printf("isupCongSrc = %x\n", pStats.isupCongSrc);
                      }
                      break;
                  }
                }
            break;

            case HMI_EVN_NOWPRIMARY:
                printf("now primary\n");
                /*(primary = 1;*/
                /* wait until MTP Resume occurs before attempting to rebind and send requests */
                /* reset circuit */
                /*ISUPStatusReq(FstCtaHd, suId,suInstId,spInstId,FALSE,circuit,CIRRESREQ,NULL); */
                if ( loadFlag)
                {
                    RebindISUP(FstCtaHd, FstQueue);
                }
            break;

            case HMI_EVN_NOWBACKUP :

                printf("now backup\n");
                /*primary = 0;
                resume = 0;*/
                /* reset circuit */
                /*ISUPStatusReq(FstCtaHd, suId,suInstId,spInstId,FALSE,circuit,CIRRESREQ,NULL);*/
                if ( loadFlag)
                {
                    RebindISUP(FstCtaHd, FstQueue);
                }

            break;

            case HMI_EVN_LOADING:
                printf("loading\n");
                loadFlag = 1;
                resume = 0;
                primary = 0;
            break;

            case HMI_EVN_TASKDEAD :
            case HMI_EVN_BRDDEAD :
            case HMI_EVN_HALTED :
                printf("something died\n");
                primary = 0;

            break;

            case HMI_EVN_STARTING :

            break;

            case HMI_EVN_CONNECTED :
            case HMI_EVN_ISOLATED :
            case HMI_EVN_CONFLICT :
            case HMI_EVN_NOWSTANDALONE :
            case HMI_EVN_SERVICE_DOWN :
            case HMI_EVN_NO_BOARD :

            break;

            default:
                printf( "CTAHD(0x%08X) : Unhandled event id (0x%x) during "
                        "wait.\nExiting...\n", Event.ctahd,
                        Event.id );
        }


    }           /* end while */

   /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, isupCloseSvcLst, num_isupServices );
   if ( Ret != SUCCESS )
   {
      ErrorAndExit( Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != SUCCESS )
   {
      ErrorAndExit( Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != SUCCESS )
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
           ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != SUCCESS )
   {
      ErrorAndExit( Ret, "ctaDestroyQueue" );
   }


} /* end of main */

#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;

   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;

   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }

   return (RetCode);
}
#endif

