/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1996 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/*
 * mtp3cfg.c
 *
 * Scans ascii text configuration file and downloads configuration to MTP3
 * task on specified board.
 */

/* header include files -- defines (.h) */
#include <sys/types.h>
/* #include <stropts.h> */
/* #include <poll.h> */
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

/*
#define IO
    #include "cpkuser.h"
    #include "dpruser.h"
#undef IO
*/

#include "ss7type.h"
#include "mtp3mapi.h"
#include "cfgscan.h"
#include "mtperr.h"
#include "mtp3ver.h"

/* global variables */
short   bDebug = 0;                  /* prints each packet sent/received */

short   timerRes = RES_SECONDS;
MTP3GenCfg gencfg;                  /* Make global so subsequent sections can use the info */

char configFile[256] = "MTP3.cfg";
static char *usage = "Usage: mtp3cfg [ -b <board number> ] [-f <filename>]\n";
char buf[80];

/* macro to extract port type from port id (hi byte) */
#define TXPORTTYPE( portid )    ((portid >> 8) & 0xFF)

/* macro to extract port number from port id (low byte) */
#define TXPORTNUM( portid )     (portid & 0xff)

/* macro to extract port type from port id (hi byte) & convert to printable char */
#define TXASCPORTTYPE( portid ) ((((portid >> 8) & 0xFF) == PORT_TYPE_TDM) ? "T" : \
                                 (((portid >> 8) & 0xFF) == PORT_TYPE_REMOTE) ? "R" : "S")


/*
* doMtp3GenCfg
*
* Configure general parameters for mtp3
*/
MTP3_STATUS doMtp3GenCfg( U8 board )
{
   U32              opc;
   MTP3_STATUS      status;
   U32              temp;
   U32              line;
   U8               i;

   /* first fetch this node's point code - this is required, all others can be defaulted */
    if (getcfgval("POINT_CODE", CFGT_PC, (void *) &opc, 0) <= 0)
    {
        printf( "MTP3 Gen Config: Missing required POINT_CODE, configuration load failed\n" );
        return( MTP3_PARAM );
    }

    status = Mtp3InitGenCfg( &gencfg, opc );
    if ( status != MTP3_SUCCESS )
    {
        /* should never happen, mtp3InitGenCfg always returns success */
        printf( "MTP3 Gen Config: mtp3InitCfg error %d, configuration load failed\n", status );
        return( status );
    }

    /* scan for remainder of optional parameters */
    if (getcfgval("MTP3_TIMER_RES", CFGT_STRING, (void *) buf, 0) > 0)
    {
        if (strcmp(buf, "TENTHS") == 0)
            timerRes = RES_TENTHS;
        else if (strcmp(buf, "SECONDS") == 0)
            timerRes = RES_SECONDS;
        else
            printf( "MTP3 Gen Config: Invalid MTP3_TIMER_RES [%s], defaulting to SECONDS\n", buf );
    }

    if (getcfgval("NODE_TYPE", CFGT_STRING, (void *) buf, 0) > 0)
    {
        if (strcmp(buf, "STP") == 0)
            gencfg.typeSP = MTP3TYPE_STP;
        else if (strcmp(buf, "SP") == 0)
            gencfg.typeSP  = MTP3TYPE_SP;
        else
            printf( "MTP3 Gen Config: Invalid NODE_TYPE [%s], defaulting to STP\n", buf );
    }

    if (getcfgval("POINT_CODE2", CFGT_PC, (void *) &temp, 0) > 0)       /* optional alternate point code */
        gencfg.spCode2 = (U32) temp;

    if (getcfgval("DISABLE_UPU", CFGT_BOOL, (void *) &temp, 0) > 0)    /* Disable sending of user part unavailable */
        gencfg.disableUPU = (Bool) temp;

    if (getcfgval("VALIDATE_SSF", CFGT_BOOL, (void *) &temp, 0) > 0)    /* SSF Validation */
        gencfg.ssfValid = (Bool) temp;

    if (getcfgval("RESTART_REQUIRED", CFGT_BOOL, (void *) &temp, 0) > 0)    /* restarting procedure required */
        gencfg.rstReq = (Bool) temp;

    if (getcfgval("MAX_LINKS", CFGT_INT, (void *) &temp, 0) > 0)        /* number of physical links */
        gencfg.nmbLinks = (U16) temp;

    if (getcfgval("MAX_USERS", CFGT_INT, (void *) &temp, 0) > 0)        /* number of SAPs */
        gencfg.nmbNSap = (U16) temp;

    if (getcfgval("MAX_LINK_SETS", CFGT_INT, (void *) &temp, 0) > 0)    /* number of link sets */
        gencfg.nmbLnkSets = (U16) temp;

    if (getcfgval("MAX_ROUTES", CFGT_INT, (void *) &temp, 0) > 0)       /* number of routes */
        gencfg.nmbRouts = (U16) temp;

    if (getcfgval("MAX_ROUTE_ENTRIES", CFGT_INT, (void *) &temp, 0) > 0)    /* number of route instances */
        gencfg.nmbRteInst = (U16) temp;

    if (getcfgval("MTP3_TRACE_DATA", CFGT_BOOL, (void *) &temp, 0) > 0)       /* trace data flag */
        gencfg.traceData = (U8) temp;

    if (getcfgval("OPC_ROUTING", CFGT_BOOL, (void *) &temp, 0) > 0)       /* use OPC in routing or not */
        gencfg.opcRouting = (U8) temp;

    if (getcfgval("TRANSPARENT_MODE", CFGT_BOOL, (void *) &temp, 0) > 0)       /* operate in transparent mode? */
        gencfg.transparentMode = (U8) temp;

    /* routing masks */
    if ((line = getcfgval("MAX_ROUTE_MASKS", CFGT_INT, (void *) &temp, 0)) > 0)
    {
        if (temp > MTP3MAXMASKS)
            temp = MTP3MAXMASKS;
        gencfg.nmbMasks = (U8)temp;
        for (i = 0; i < gencfg.nmbMasks; i++)
        {
            if ((line = getcfgval("ROUTE_MASK", CFGT_INT, (void *) &temp, line)) > 0)
            gencfg.masks[i] = (U32) temp;
            else
            break;
        }
    }

    /* scan for timer overrides */
    if (getcfgval("TIMER_T15", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t15, (U16)temp * timerRes);

    if (getcfgval("TIMER_T16", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t16, (U16)temp * timerRes);

    if (getcfgval("TIMER_T18_ITU", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t18_itu, (U16)temp * timerRes);

    if (getcfgval("TIMER_T20_ITU", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t20_itu, (U16)temp * timerRes);

    if (getcfgval("TIMER_T22_ANSI", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t22_ansi, (U16)temp * timerRes);

    if (getcfgval("TIMER_T23_ANSI", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t23_ansi, (U16)temp * timerRes);

    if (getcfgval("TIMER_T24_ANSI", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t24_ansi, (U16)temp * timerRes);

    if (getcfgval("TIMER_T26_ANSI", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t26_ansi, (U16)temp * timerRes);

    if (getcfgval("TIMER_T27_ANSI", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.t27_ansi, (U16)temp * timerRes);

    if (getcfgval("TIMER_TRTEINST", CFGT_INT, (void *) &temp, 0) > 0)
        MTP3_SET_TIMER( gencfg.tmr.tRteInst, (U16)temp * timerRes);


    /* get the point code display format from the config scanner global setting */
    gencfg.pcDispFmt = CfgPcDispFmt;

   /* generate the config request */
    status = Mtp3SetGenCfg( board, &gencfg );
    switch( status )
    {
	case MTP3_SUCCESS:
	    break;

	case MTP3_LNKTYPE:
	    printf( "MTP3 Gen Config: Invalid link type from board %d\n", board );
	    break;

	case MTP3_PARAM:
	    printf( "MTP3 Gen Config: Invalid parameter from board %d\n", board );
	    break;

	case MTP3_TIMEOUT:
	    printf( "MTP3 Gen Config: No response from board %d\n", board );
	    break;

	case MTP3_BOARD:
	    printf( "MTP3 Gen Config: Invalid board %d\n", board );
	    break;

	case MTP3_HANDLE:
	    printf( "MTP3 Gen Config: Board %d not initialized\n", board );
	    break;

    case MTP3_RESPONSE:
        printf( "MTP3 Gen Config: Board %d returned unexpected response\n", board );
	    break;

	default:
        printf( "MTP3 Gen Config: Error %x sending config request to board %d\n", status, board );
	    break;
    }

    return( status );

} /* doMtp3GenCfg */

/*
* mtp3LinkCfg
*
* Configures MTP3 data link saps
*/
void doMtp3LinkCfg( U8 board )
{
    MTP3LinkCfg         linkcfg;
    MTP3_STATUS         status;
    S32                 temp, last, next;
    S32                 maxlinks;
    short               linkno = 0;
    U32                 dpc, opc;
    short               portid;
    U8                  lnktype;
    short               i;
    U8                  oldmethod = FALSE, newmethod = FALSE;

    maxlinks = 4;
    getcfgval("MAX_LINKS", CFGT_INT, &maxlinks, 0);

    last = 0;           /* Last line we read */
    for(i = 0; ; i++ )
    {
        /* Check old link/port config method first */
        if ((next = getcfgval("LINK", CFGT_LINK, (void *) &temp, last)) <= 0)
            break;          /* No more links */

        if (temp & NEW_LINK_TYPE)
        {
            last = next;                        /* Save line number in file in case of error below */

            /* New type of link specification of link number only */
            if (oldmethod)
            {
                printf ("Cannot mix old and new style LINK specifications\n");
                continue;
            }
            newmethod = TRUE;

            linkno = (short)temp & 0xFF;        /* Remove NEW_LINK_TYPE mask */
            if ((next = getcfgval("PORT", CFGT_PORT, (void *) &temp, last)) <= 0)
            {
                printf ("You must specify PORT T<n>, S<n> or R after LINK <n>\n");
                continue;
            }
        }
        else
        {
            /* Old method of link type specified, ie. LINK T1 */
            if (newmethod)
            {
                printf ("Cannot mix old and new style LINK specifications\n");
                last = next;
                continue;
            }
            oldmethod = TRUE;
            linkno = i;
        }

        portid = (short) temp;

        last = next;                        /* save starting line in config file */
        if (linkno >= maxlinks)
        {
            printf( "MTP3 DLSAP Config:Link number %s%d exceeds maximum (%d) - not configured\n",
                TXASCPORTTYPE( portid ), TXPORTNUM( portid ), maxlinks );
            continue;
        }

        /* retrieve required adjacent point code - can't be defaulted */
        if (getcfgval("ADJACENT_DPC", CFGT_PC, (void *) &dpc, last) <= 0)
        {
            printf( "MTP3 DLSAP Config: Link %s%d Missing required ADJACENT_DPC - not configured\n",
                TXASCPORTTYPE( portid ), TXPORTNUM( portid ) );
            continue;
        }

        /* retrieve link type (CCITT, ANSI88, ANSI92) - default is ANSI(92) */
        lnktype = MTP3LNK_ANSI;                             /* default */
        if (getcfgval("LINK_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if ((strcmp(buf, "CCITT") == 0) || (strcmp(buf, "ITU") == 0))
                lnktype = MTP3LNK_CCITT;
            else if (strcmp(buf, "ANSI88") == 0)
                lnktype = MTP3LNK_ANSI88;
            else if (strcmp(buf, "ANSI") == 0)
                lnktype = MTP3LNK_ANSI;
            else if (strcmp(buf, "JNTT") == 0)
                lnktype = MTP3LNK_JNTT;
            else if (strcmp(buf, "JTTC") == 0)
                lnktype = MTP3LNK_JTTC;
            else
            {
                printf( "MTP3 DLSAP Config: Link %s%d LINK_TYPE [%s] unknown - defaulted to ANSI\n",
                    TXASCPORTTYPE( portid ), TXPORTNUM( portid ), buf );
            }
        }

        /* initialize link params block to default values */
        status = Mtp3InitLinkCfg( board, &linkcfg, linkno, lnktype, dpc );
        if( status != MTP3_SUCCESS )
        {
            /* should never happen, we've already validated link type */
            printf( "MTP3 DlSap Config: invalid link type, link %s%d configuration failed\n",
                TXASCPORTTYPE( portid ), TXPORTNUM( portid ), buf );
            continue;
        }

        /* store the port number and type.  MUST be after InitLinkCfg to avoid resetting to defaults */
        linkcfg.spId     = linkno;
        linkcfg.portType = TXPORTTYPE( portid );

        /* retrieve optional default overrides */

        /* Alternate OPC */
        if (getcfgval("OPC", CFGT_PC, (void *) &opc, last) > 0)
            linkcfg.opc = opc;

        if (getcfgval("LINK_SET", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.lnkSetId = (U16) temp;

        if (getcfgval("DISABLED", CFGT_BOOL, (void *) &temp, last) > 0 )
            linkcfg.dis = (U8) temp;

        if (getcfgval("HS_LINK", CFGT_BOOL, (void *) &temp, last) > 0 )
        {
            // Set EXT_SEQ to be the same as HS_LINK.  EXT_SEQ can be overridden to false below but cannot be true
            // if HS_LINK is false.
            if (temp)
            {
                linkcfg.hsBits |= HS_LINK;
                linkcfg.hsBits |= HS_EXT_SEQ;
            }
            else
            {
                linkcfg.hsBits &= ~HS_LINK;
                linkcfg.hsBits &= ~HS_EXT_SEQ;
            }
        }

        if (getcfgval("HS_EXT_SEQ", CFGT_BOOL, (void *) &temp, last) > 0 )
        {
            // If EXT_SEQ is true, HS_LINK must be true but if EXT_SEQ is false, HS_LINK can be true or false.
            if (temp)
            {
                linkcfg.hsBits |= HS_LINK;
                linkcfg.hsBits |= HS_EXT_SEQ;
            }
            else
            {
                linkcfg.hsBits &= ~HS_EXT_SEQ;
            }
        }

        if (getcfgval("SSF", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if (strcmp(buf, "INTERNATIONAL") == 0)
                linkcfg.ssf = MTP3SSF_INTL;
            else if (strcmp(buf, "NATIONAL") == 0)
                linkcfg.ssf = MTP3SSF_NAT;
            else
                printf( "MTP3 DLSAP Config: Link %s%d SSF unknown value [%s] ignored\n",
                    TXASCPORTTYPE( portid ), TXPORTNUM( portid ), buf );
        }

        /* can put in any value this way, overrides SSF above */
        if (getcfgval("SUB_SERVICE", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.ssf = (U8) temp;

        if (getcfgval("DPC_LENGTH", CFGT_INT, (void *) &temp, last) > 0 )
        {
            if ( (temp == 14) && ( (lnktype != MTP3LNK_JNTT) && (lnktype != MTP3LNK_JTTC) ))
                linkcfg.dpcLen = PCLEN_ITU;
            else if( (temp == 24) && ( (lnktype != MTP3LNK_JNTT) && (lnktype != MTP3LNK_JTTC) ))
                linkcfg.dpcLen = PCLEN_ANSI;
            else if( (temp == 16) && ( (lnktype == MTP3LNK_JNTT) || (lnktype == MTP3LNK_JTTC) ))  /* 16 bits for type JNTT/JTTC only */
                linkcfg.dpcLen = PCLEN_JAPAN;
            else
                fprintf( stderr, "MTP3CFG: Link %s%d DPC_LENGTH (%d) Invalid, defaulted to %d\n",
                    TXASCPORTTYPE( portid ), TXPORTNUM( portid ), temp,
                    (linkcfg.dpcLen == PCLEN_ITU) ? 14 : ((linkcfg.dpcLen == PCLEN_JAPAN) ? 16 : 24) );
        }

        if (getcfgval("LINK_PRIORITY", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.lnkPrior = (U8) temp;

        if (getcfgval("MESSAGE_SIZE", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.msgSize = (U16) temp;

        if (getcfgval("USE_PRIORITY", CFGT_BOOL, (void *) &temp, last) > 0 )
            linkcfg.usePrior = (U8) temp;

        if (getcfgval("MGNT_MSG_PRIORITY", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.msgPrior = (U8) temp;

        if (getcfgval("MAX_SLTM_RETRY", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.maxSLTtry = (U8) temp;

        if (getcfgval("P0QUE_LENGTH", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.p0QLen = (S16) temp;

        if (getcfgval("P1QUE_LENGTH", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.p1QLen = (S16) temp;

        if (getcfgval("P2QUE_LENGTH", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.p2QLen = (S16) temp;

        if (getcfgval("P3QUE_LENGTH", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.p3QLen = (S16) temp;

        if (getcfgval("DISCARD_PRIORITY", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.discPrior = (U8) temp;

        if (getcfgval("CHANGEBACK_LINKID", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.lnkId = (U16) temp;

        if (getcfgval("LINK_TEST_SELECT_CODE", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.lnkTstSLC = (U8) temp;
        if (getcfgval("LINK_SLC", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.lnkTstSLC = (U8) temp;
        if (getcfgval("LINK_TRACE_DATA", CFGT_BOOL, (void *) &temp, last) > 0)       /* trace data flag */
            linkcfg.traceData = (U8) temp;

        if( getcfgval("LINK_TEST_PATTERN", CFGT_STRING, (void *) buf, last ) > 0 )
        {
            strncpy( (char *)linkcfg.tst, buf, sizeof( linkcfg.tst ) - 1 );
            linkcfg.tst[ sizeof( linkcfg.tst ) - 1] = '\0';
            linkcfg.tstLen = (U8)strlen((const char *)linkcfg.tst);     /* link test pattern length */
        }

        /* scan for timer definitions */
        if (getcfgval("TIMER_T1", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t1, (U16)temp * timerRes);

        if (getcfgval("TIMER_T2", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t2, (U16)temp * timerRes);

        if (getcfgval("TIMER_T3", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t3, (U16)temp * timerRes);

        if (getcfgval("TIMER_T4", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t4, (U16)temp * timerRes);

        if (getcfgval("TIMER_T5", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t5, (U16)temp * timerRes);

        if (getcfgval("TIMER_T6", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t6, (U16)temp * timerRes);

        if (getcfgval("TIMER_T7", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t7, (U16)temp * timerRes);

        if (getcfgval("TIMER_T11", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t11, (U16)temp * timerRes);

        if (getcfgval("TIMER_T12", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t12, (U16)temp * timerRes);

        if (getcfgval("TIMER_T13", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t13, (U16)temp * timerRes);

        if (getcfgval("TIMER_T14", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t14, (U16)temp * timerRes);

        if (getcfgval("TIMER_T17", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t17, (U16)temp * timerRes);

        if (getcfgval("TIMER_T22", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t22, (U16)temp * timerRes);

        if (getcfgval("TIMER_T23", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t23, (U16)temp * timerRes);

        if (getcfgval("TIMER_T24", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t24, (U16)temp * timerRes);

        if (getcfgval("TIMER_T31", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t31, (U16)temp * timerRes);

        if (getcfgval("TIMER_T32", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t32, (U16)temp * timerRes);

        if (getcfgval("TIMER_T33", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t33, (U16)temp * timerRes);

        if (getcfgval("TIMER_T34", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t34, (U16)temp * timerRes);

        if (getcfgval("TIMER_T40", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t40, (U16)temp * timerRes);

        if (getcfgval("TIMER_T41", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t41, (U16)temp * timerRes);

        if (getcfgval("TIMER_T42", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t42, (U16)temp * timerRes);

        if (getcfgval("TIMER_T43", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t43, (U16)temp * timerRes);

        if (getcfgval("TIMER_T44", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( linkcfg.tmr.t44, (U16)temp * timerRes);

        /* generate the config request */
        status = Mtp3SetLinkCfg( board, &linkcfg );
        switch( status )
        {
            case MTP3_SUCCESS:
                break;

            case MTP3_TIMEOUT:
                printf( "MTP3 DLSAP Config: No response from board %d\n", board );
                break;

            case MTP3_BOARD:
                printf( "MTP3 DLSAP Config: Invalid board %d\n", board );
                break;

            case MTP3_HANDLE:
                printf( "MTP3 DLSAP Config: Board %d not initialized\n", board );
                break;

            case MTP3_RESPONSE:
                printf( "MTP3 DLSAP Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "MTP3 DLSAP Config: Error %d sending config request to board %d\n", status, board );
                break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;

    }   /* end for loop */

} /* doMtp3DlSapCfg */

/*
* doMtp3NSapCfg
*
* Configure mtp3 user saps
*/
void doMtp3NSapCfg( U8 board )
{
    MTP3NSapCfg         nsapcfg;
    MTP3_STATUS         status;
    short               i;
    S32                 temp, last, next;
    S32                 maxNSAP;
    S16                 nsapno;
    U8                  lnktype;

    maxNSAP = 1;
    if (getcfgval("MAX_USERS", CFGT_INT, &temp, 0) > 0)
        maxNSAP = temp;

    for( last=0, i=0; i<maxNSAP; i++ )
    {
        if ((next = getcfgval("NSAP", CFGT_INT, &temp, last)) == 0)
            break;
        last = next;
        nsapno = (S16) temp;

        /* retrieve NSAP protocol variant (CCITT, ANSI88, ANSI92) - default is ANSI(92) */
        lnktype = MTP3LNK_ANSI;                             /* default */
        if (getcfgval("LINK_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if ((strcmp(buf, "CCITT") == 0) || (strcmp(buf, "ITU") == 0))
                lnktype = MTP3LNK_CCITT;
            else if (strcmp(buf, "ANSI88") == 0)
                lnktype = MTP3LNK_ANSI88;
            else if (strcmp(buf, "ANSI") == 0)
                lnktype = MTP3LNK_ANSI;
            else if (strcmp(buf, "JNTT") == 0)
                lnktype = MTP3LNK_JNTT;
            else if (strcmp(buf, "JTTC") == 0)
                lnktype = MTP3LNK_JTTC;
            else
            {
                printf( "MTP3 NSAP Config: NSAP %d LINK_TYPE [%s] unknown - defaulted to ANSI\n",
                    nsapno, buf );
            }
        }

        /* initialize link params block to default values */
        status = Mtp3InitNSapCfg( &nsapcfg, lnktype );
        if( status != MTP3_SUCCESS )
        {
            /* should never happen, we've already validated link type */
            printf( "MTP3 NSap Config: invalid link type, NSap %d configuration failed\n", nsapno );
            continue;
        }

        /* scan for optional parameters */
        if (getcfgval("P0QUE_LENGTH", CFGT_INT, &temp, last) > 0 )
            nsapcfg.p0QLen = (S16) temp;
        if (getcfgval("P1QUE_LENGTH", CFGT_INT, &temp, last) > 0 )
            nsapcfg.p1QLen = (S16) temp;
        if (getcfgval("P2QUE_LENGTH", CFGT_INT, &temp, last) > 0 )
            nsapcfg.p2QLen = (S16) temp;
        if (getcfgval("P3QUE_LENGTH", CFGT_INT, &temp, last) > 0 )
            nsapcfg.p3QLen = (S16) temp;

        if (getcfgval("DPC_LENGTH", CFGT_INT, &temp, last))
        {
            if ( (temp == 14) && ( (lnktype != MTP3LNK_JNTT) && (lnktype != MTP3LNK_JTTC) ))
                nsapcfg.dpcLen = PCLEN_ITU;
            else if( (temp == 24) && ( (lnktype != MTP3LNK_JNTT) && (lnktype != MTP3LNK_JTTC) ))
                nsapcfg.dpcLen = PCLEN_ANSI;
            else if( (temp == 16) && ( (lnktype == MTP3LNK_JNTT) || (lnktype == MTP3LNK_JTTC) ))  /* 16 bits for type JNTT/JTTC only */
                nsapcfg.dpcLen = PCLEN_JAPAN;
            else
                fprintf( stderr, "MTP3 NSAP Config: NSAP %d DPC_LENGTH (%d) Invalid, defaulted to %d\n",
                    nsapno, temp,
                    (nsapcfg.dpcLen == PCLEN_ITU) ? 14 : ((nsapcfg.dpcLen == PCLEN_JAPAN) ? 16 : 24) );
        }

        if (getcfgval("OPC", CFGT_PC, &temp, last))
            nsapcfg.opc = (U32) temp;

        /* generate the config request */
        status = Mtp3SetNSapCfg( board, &nsapcfg, nsapno );
        switch( status )
        {
            case MTP3_SUCCESS:
                break;

            case MTP3_TIMEOUT:
                printf( "MTP3 NSAP Config: No response from board %d\n", board );
                break;

            case MTP3_BOARD:
                printf( "MTP3 NSAP Config: Invalid board %d\n", board );
                break;

            case MTP3_HANDLE:
                printf( "MTP3 NSAP Config: Board %d not initialized\n", board );
                break;

            case MTP3_RESPONSE:
                printf( "MTP3 NSAP Config: Board %d returned unexpected response\n", board );
                break;

            default:
                printf( "MTP3 NSAP Config: Error %d sending config request to board %d\n", status, board );
            break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;

    }   /*end for loop */

}  /* spNSapCfg */

/*
*
* doMtp3LinkSetCfg
*
* Configures MTP3 link sets
*
*/
void doMtp3LinkSetCfg( U8 board )
{
    MTP3LinkSetCfg      lnksetcfg;
    MTP3_STATUS         status;
    S32                 tline, temp, last, next;
    S32                 route[2];
    U32                 dpc, opc;
    short               rtecnt;
    S16                 lnksetid;

    last = 0;
    while( (next=getcfgval("LINK_SET_DESCRIPTOR", CFGT_INT, &temp, last)) > 0 )
    {
        last = next;
        lnksetid = (U16) temp;        /* link set ID */

        /* retrieve required adjacent point code - can't be defaulted */
        if (getcfgval("ADJACENT_DPC", CFGT_PC, (void *) &dpc, last) <= 0)
        {
            printf( "MTP3 Linkset Config: Linkset %d Missing required ADJACENT_DPC - not configured\n",
                lnksetid );
            continue;
        }

        /* initialize link params block to default values */
        status = Mtp3InitLinkSetCfg( &lnksetcfg, lnksetid, dpc );
        if( status != MTP3_SUCCESS )
        {
            /* should never happen, mtp3InitGenCfg always returns success */
            printf( "MTP3 Linkset Config: mtp3InitLinkSetCfg error %d, linkset %d not configured\n",
                status, lnksetid );
            continue;
        }

        if (getcfgval("OPC", CFGT_PC, (void *) &opc, last) )
            lnksetcfg.opc = opc;                 /* alternative Origination Point Code */

        if( getcfgval("MAX_ACTIVE_LINKS", CFGT_INT, &temp, last) )
            lnksetcfg.minNmbActLnk = (U16) temp;    /* MAXIMUM number of active links */

        /* scan for all routes assigned to this linkset */
        route[0] = route[1] = 0;
        rtecnt = 0;
        while( (tline = getcfgval("ROUTE_NUMBER", CFGT_INTPAIR, route, last)) > 0 )
        {
            last = tline;
            if( rtecnt >= MTP3MAXCMBLNK )
            {   /* Too Many! */
                printf( "MTP3 Linkset Config: linkset %d has too many routes: route %d ignored\n",
                    lnksetid, route[0] );
            }
            else
            {
                lnksetcfg.cmbLnkSet[rtecnt].cmbLnkSetId = (U16) route[0];    /* combined link set ID */
                lnksetcfg.cmbLnkSet[rtecnt].lnkSetPrior = (U8) route[1];     /* link set priority */
                rtecnt++;
            }
            route[0] = route[1] = 0;
        }

        /* set the number of combined link sets */
        lnksetcfg.nmbCmbLnkSet = rtecnt;

        /* generate the config request */
        status = Mtp3SetLinkSetCfg( board, &lnksetcfg );
        switch( status )
        {
            case MTP3_SUCCESS:
                break;

            case MTP3_TIMEOUT:
                printf( "MTP3 Linkset Config: No response from board %d\n", board );
                break;

            case MTP3_BOARD:
                printf( "MTP3 Linkset Config: Invalid board %d\n", board );
                break;

            case MTP3_HANDLE:
                printf( "MTP3 Linkset Config: Board %d not initialized\n", board );
                break;

            case MTP3_RESPONSE:
                printf( "MTP3 Linkset Config: Board %d returned unexpected response\n", board );
                break;

            default:
                printf( "MTP3 Linkset Config: Error %d sending config request to board %d\n", status, board );
                break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }   /* of while loop */

} /* doMtp3LinkSetCfg */

/*
* doMtp3RouteCfg
*
* Configures Routing Table
*/
void doMtp3RouteCfg( U8 board )
{
    MTP3RouteCfg        routecfg;
    MTP3_STATUS         status;
    S32                 temp, last, next;
    U32                 dpc, opc;
    U16                 cmblnksetid;
    U8                  swtype;

    last = 0;
    while( (next=getcfgval( "ROUTE", CFGT_INT, &temp, last )) > 0 )
    {
        last = next;
        cmblnksetid = (U16) temp;     /* combined link set ID */

        /* retrieve required destination point code - can't be defaulted */
        if (getcfgval("DPC", CFGT_PC, (void *) &dpc, last) <= 0)
        {
            printf( "MTP3 ROUTE Config: Route %d Missing required DPC - not configured\n",
                cmblnksetid );
            continue;
        }

        /* retrieve destination switch type - default is ANSI(92) */
        swtype = MTP3LNK_ANSI;                             /* default */
        if (getcfgval("LINK_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if ((strcmp(buf, "CCITT") == 0) || (strcmp(buf, "ITU") == 0))
                swtype = MTP3LNK_CCITT;
            else if (strcmp(buf, "ANSI88") == 0)
                swtype = MTP3LNK_ANSI88;
            else if (strcmp(buf, "ANSI") == 0)
                swtype = MTP3LNK_ANSI;
            else if (strcmp(buf, "JNTT") == 0)
                swtype = MTP3LNK_JNTT;
            else if (strcmp(buf, "JTTC") == 0)
                swtype = MTP3LNK_JTTC;
            else
            {
                printf( "MTP3 Route Config: Route %d LINK_TYPE [%s] unknown - defaulted to ANSI\n",
                    cmblnksetid, buf );
            }
        }

        /* initialize link params block to default values */
        status = Mtp3InitRouteCfg( &routecfg, cmblnksetid, dpc, swtype );
        if( status != MTP3_SUCCESS )
        {
            /* should never happen, we've already validated switch type */
            printf( "MTP3 Route Config: invalid link type, route %d configuration failed\n",
                cmblnksetid );
            continue;
        }

        /* scan for optional parameters */
        if( getcfgval("SPTYPE", CFGT_STRING, buf, last) > 0 )
        {
            if (strcmp( buf,"STP" ) == 0)
                routecfg.spType = MTP3TYPE_STP;   /* signalling point type (TRANSFER) */
            else if (strcmp( buf,"SP" ) == 0)
                routecfg.spType = MTP3TYPE_SP;    /* signalling point type (END POINT) */
            else
                printf( "MTP3 Route Config: Route %d SPTYPE [%s] unknown - defaulted to STP\n",
                    cmblnksetid, buf );
        }

        if (getcfgval("SSF", CFGT_STRING, buf, last) > 0 )
        {
            if (strcmp(buf, "INTERNATIONAL") == 0)
                routecfg.ssf = MTP3SSF_INTL;
            else if (strcmp(buf, "NATIONAL") == 0)
                routecfg.ssf = MTP3SSF_NAT;
            else
                printf( "MTP3 Route Config: Route %d SSF unknown value [%s] ignored\n",
                    cmblnksetid, buf );
        }

        /* can put in any value this way, overrides SSF above */
        if (getcfgval("SUB_SERVICE", CFGT_INT, (void *) &temp, last) > 0 )
            routecfg.ssf = (U8) temp;

        if (getcfgval("DIRECTION", CFGT_STRING, buf, last) > 0 )
        {
            if (strcmp(buf, "DOWN") == 0)
                routecfg.dir = MTP3_RTE_DN;
            else if (strcmp(buf, "UP") == 0)
                routecfg.dir = MTP3_RTE_UP;
            else
                printf( "MTP3 Route Config: Route %d DIRECTION unknown value [%s] - defaulted to DOWN\n",
                    cmblnksetid, buf );
        }

        if( getcfgval("ADJACENT_ROUTE", CFGT_BOOL, &temp, last) > 0 )
            routecfg.rteToAdjSp = (Bool) temp;       /* route to adjacent SP ?*/

        if( getcfgval("ADJACENT_CLUSTER", CFGT_BOOL, &temp, last) > 0 )
            routecfg.rteToCluster = (Bool) temp;    /* route to cluster? */

        if (getcfgval("OPC", CFGT_PC, (void *) &opc, last) > 0)
        {
            routecfg.opc = opc;                 /* Set this if OPC specified and we are using OPC routing */
        }

        /* scan for timer definitions */
        if (getcfgval("TIMER_T8", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t8, (U16)temp * timerRes);

        if (getcfgval("TIMER_T10", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t10, (U16)temp * timerRes);

        if (getcfgval("TIMER_T19_ITU", CFGT_INT, (void *) &temp, 0) > 0)
            MTP3_SET_TIMER( routecfg.tmr.t19_itu, (U16)temp * timerRes);

        if (getcfgval("TIMER_T21_ITU", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t21_itu, (U16)temp * timerRes);

        if (getcfgval("TIMER_T25_ANSI", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t25_ansi, (U16)temp * timerRes);

        if (getcfgval("TIMER_T28_ANSI", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t28_ansi, (U16)temp * timerRes);

        if (getcfgval("TIMER_T29_ANSI", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t29_ansi, (U16)temp * timerRes);

        if (getcfgval("TIMER_T30_ANSI", CFGT_INT, (void *) &temp, last) > 0 )
            MTP3_SET_TIMER( routecfg.tmr.t30_ansi, (U16)temp * timerRes);

        /* generate the config request */
        status = Mtp3SetRouteCfg( board, &routecfg );
        switch( status )
        {
            case MTP3_SUCCESS:
                break;

            case MTP3_TIMEOUT:
                printf( "MTP3 Route Config: No response from board %d\n", board );
                break;

            case MTP3_BOARD:
                printf( "MTP3 Route Config: Invalid board %d\n", board );
                break;

            case MTP3_HANDLE:
                printf( "MTP3 Route Config: Board %d not initialized\n", board );
                break;

            case MTP3_RESPONSE:
                printf( "MTP3 Route Config: Board %d returned unexpected response\n", board );
                break;

            default:
                printf( "MTP3 Route Config: Error %d sending config request to board %d\n", status, board );
                break;
        }

        /* move up to end of ROUTE info... */
        next=getcfgval("END",CFGT_INT,&temp,last);
        last = next;
    }   /* end while loop */

} /* doMtp3RouteCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    unsigned short      board = 1;                  /* default board is 1 */
    MTP3_STATUS         status;
    int                 i;
    char                *p;

    printf("\n%s: sample MTP3 configuration application version %s %s\n",
            argv[0], MTP3CFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++)
    {
		p = argv[i];
        if (*p == '-')
        {
            switch (*++p)
            {
            case 'f' :     /* configuration file name */
				i++;

                if (argv[i] == NULL)
                {
                  printf( usage );
                  exit ( 1 );
                }
                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
				break;

			case 'b' :    /* Set Board number */
				i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%hu", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
				break;

            case 'd' :
                bDebug = 1;              /* i/o packet dumps */
                CDebug = 1;             /* config scanner debugging */
                break;

			default:
                printf( usage );
                exit( 1 );
                break;
			}       /* end switch */
        }
        else
        {
            printf( usage );
            exit( 1 );
		}

	}       /* end for */

    /* Initialize the mtp3 management library */
    status = Mtp3MgmtInit( (U8)board, MTP3_ENT, 0 );
    if( status != MTP3_SUCCESS )
    {
        printf( "Unable to open management library, error %d\n", status );
		exit(1);
	}

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if( initcfg( configFile ) != 0 )
    {
        /* initcfg() prints error message if it fails */
        exit( 1 );
    }

    /* do the mtp configuration */
    status = doMtp3GenCfg( (U8)board );
    if ( status != MTP3_SUCCESS )
    {
        /* gen config failure is fatal; all others non-fatal */
        endcfg();
        (void) Mtp3MgmtTerm( (U8)board );
        exit( 1 );
    }
    doMtp3LinkCfg( (U8)board );
    doMtp3NSapCfg( (U8)board );
    doMtp3LinkSetCfg( (U8)board );
    doMtp3RouteCfg( (U8)board );

    /* allow libraries to clean up and terminate */
    endcfg();
    (void) Mtp3MgmtTerm( (U8)board );               /* ignore return code */

    exit (0);           /* Exit code success */
}
