/*
 * THIS IS THE ITU VERSION OF THIS PROGRAM
 *
 * Test app to send and receive MTP2 packets and generate other MTP2 calls.
 * Always displays any MTP packets (or other events) received. User hits <CR>
 * and can get a prompt to enter MTP2 primitive command.
 *
 * To run:
 *
 *  MTP2itu  [-o <opc>] [-d <dpc>] [-s <sio>] [-b <board>]  -?
 *
 *      <opc>       originating point code to use for outgoing packets
 *      <dpc>       destination point code to use for outgoing packets
 *
 *      <sio>       Service Information Octet value; default: 0x01
 *
 *      <board>     Board number to use (defaults to 1)
 *
 * User Commands:
 *
 *      CONNECT <link>          # starts link alignment for <link>
 *      DISCONN <link>          # disconnects the link
 *      SEND <link>             # sends an SLTM to the link
 *      STATUS <link>  [ START | END | UP | DOWN]
 *      FLOW [OFF | ON]         # flow control on and off
 */

#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <ctype.h>
#include <string.h>
#endif	                                /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#endif	                                /* __OS2__ */

#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#endif	                                /* WIN32 */

#include "ss7type.h"
#include "tx_core.h"
#include "txcpi.h"
#include "mtp2api.h"
#include "mtp2itu.h"
#include "osdep.h"

#define DFLT_ENT   0x20                 /* Default application entity ID */

/* globals (with defaults) for parameter values */
U8              Sio     = 0x01;         /* SIO value to send */
int             Board   = 1;            /* MTP2's Board Number */
U16             Dpc = 2;
U16             Opc = 1;
U8              Sls = 0;

/* other globals */
TX_HANDLE       txHandle;
MTP2TXPKT       TxBuf;
MTP2RXPKT       RxBuf;                  /* packet buffer */
short           changeToLink;
short           changeFromLink;
short           currentState[2];
U32             PktXmitCnt  = 0;        /* packet xmit count */
U32             ByteXmitCnt = 0;        /* byte xmit count */
U32             fsn, bsn;
void           *mtp2ChanKey;    /* key value, passed to CPIA callback routines */
void           *myTxKey = NULL;     /* per-transmit key under monapp's control - not currently needed so set to NULL */

#define MTP2APP_STATE_NOBIND    1
#define MTP2APP_STATE_BIND1     2
#define MTP2APP_STATE_BIND2     3
#define MTP2APP_STATE_BIND3     4
#define MTP2APP_STATE_BIND4     5

#define MAX_PARMS	10

U32             mtp2State = MTP2APP_STATE_NOBIND;

extern CPI_WAIT_TYPE stdInput;
void showHex(unsigned char *Buf, unsigned short ByteCnt);

/*
 *  doFlow - This routine sends a Flow Request
 *
 * Parameters
 *      link    -   Which link to send on
 *      action  -   Should we turn flow on or off
                    FLOWREQ_START or FLOWREQ_STOP
 *
 *
 */
void doFlow(short link, short action, char flcLevel)
{
    short status;

    printf("Link %d:Sending a Flow Request\n",link);

    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(FlowReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(FlowReq));
    TxBuf.msgType                   = MTP2_FLOW_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.flowReq.action       = cpi_htocp_s(action);
    TxBuf.data.flowReq.flcLevel     = flcLevel;
    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Flow send failed, status = %d\n", status );
    }
}

/*
 *  doStatusReq - This routine sends a Status Request
 *
 * Parameters
 *      link    -   Which link to send on
 *      action  -   Which action should we take: STAREQ_CHANGE_STATUS (if this value, then status parameter
 *                  is also filled), STAREQ_GET_BSN, STAREQ_GET_UNACKED or STAREQ_DROP_UNACKED
*      status  -   Which status change should we report, processor up or down, emergency start or end
 *                  STAREQ_STS_LCL_DOWN, STAREQ_STS_LCL_UP, STAREQ_STS_START_EMERG, STAREQ_STS_STOP_EMERG
 */
void doStatusReq(short link, S16 action, U32 status )
{
    short ret;

    printf("Link %d:Sending a Status Request\n",link);

    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(StaReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(StaReq));
    TxBuf.msgType                   = MTP2_STAT_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.staReq.action        = cpi_htocp_s(action);
    TxBuf.data.staReq.status        = cpi_htocp_l(status);
    ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( ret != CPI_SUCCESS )
    {
        printf( "Status send failed, status = %d\n", ret);
    }
}

/*
 *  doStatusRsp - This routine sends a Status Request
 *
 * Parameters
 *      link    -   Which link to send on
 *      status  -   Which status indication to respond to, processor up or down
 */
void doStatusRsp(short link, U32 status )
{
    short ret;

    printf("Link %d:Sending a Status Response\n",link);

    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(StaReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(StaReq));
    TxBuf.msgType                   = MTP2_STAT_RSP;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.staRsp.status        = status;           /* Back to a U32 instead of 2 U16s */

    ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( ret != CPI_SUCCESS )
    {
        printf( "StatusRsp send failed, status = %d\n", ret);
    }
}

/*
 * startChangeOver - This routine does the first step of a change over
 *
 * Parameters
 *       userId    -  Which link to change FROM
 *
 */
void startChangeOver(short userId)
{
    /* To do a change over, we first need to retrieve the BSN */
    printf("Link %d:Retrieving BSN\n",userId);
    doStatusReq( userId, STAREQ_GET_BSN, 0);
}

/*
 *  doDisconn - This routine sends a Disconnect Request
 *
 * Parameters
 *      link    -   Which link to send on
 *
*/
void doDisconn(short link)
{
    short status;

    printf( "Link %d:Disconnecting\n", link);

    /* send the data */
    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(DisReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(DisReq));
    TxBuf.msgType                   = MTP2_DISC_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.disReq.reason        = cpi_htocp_s(DISCREQ_LVL_2_ONLY);
    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Disconnect failed, status = %d\n", status );
    }
}

/*
 *  doConnect - This routine sends a Connect Request
 *
 * Parameters
 *      link    -   Which link to send on
 *
*/
void doConnect(short link)
{
    short status;

    printf( "Link %d:Connecting\n", link);

    /* send the data */
    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(ConReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(ConReq));
    TxBuf.msgType                   = MTP2_CONN_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.conReq.status        = CONREQSTA_LVL_1_2_NORM;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Connect failed, status = %d\n", status );
    }
}

/*
 *  doEConn - This routine sends an Emergency Connect Request
 *
 * Parameters
 *      link    -   Which link to send on
 *
*/
void doEConn(short link)
{
    short status;

    printf( "Link %d:Connecting\n", link);

    /* send the data */
    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(ConReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(ConReq));
    TxBuf.msgType                   = MTP2_CONN_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.conReq.status        = CONREQSTA_LVL_1_2_EMRG;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Connect failed, status = %d\n", status );
    }
}
/*
 *  doPowerOn - This routine sends a Connect Req Power On State
 *
 * Parameters
 *      link    -   Which link to send on
 *
*/
void doPowerOn(short link)
{
    short status;

    printf( "Link %d:Connecting\n", link);

    /* send the data */
    TxBuf.hdr.hdr.src               = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst               = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length            = HDRSIZE + sizeof(ConReq);
    TxBuf.hdr.start                 = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end                   = cpi_htocp_s(ENDSIZE + sizeof(ConReq));
    TxBuf.msgType                   = MTP2_CONN_REQ;
    TxBuf.providerId                = cpi_htocp_s(link);
    TxBuf.data.conReq.status        = CONREQSTA_POWER_ON;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Connect failed, status = %d\n", status );
    }
}
/*
 *  doSLTM - This routine sends a Data Request containing an SLTM
 *
 * Parameters
 *      link    -   Which link to send on
 *
 */
void doSLTM(short link)
{
    U8      i;
    short   status;
    U32     label;
    char   *label_ch = (char *)&label;            /* Play games because buf->label is a char array for alignment reasons */
    pktSLT *buf = (pktSLT *)&(TxBuf.data);

    printf("Link %d:Sending an SLTM\n",link);

    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);
    TxBuf.msgType       = MTP2_DATA_REQ;
    TxBuf.providerId    = cpi_htocp_s(link);

    buf->sio    = Sio;                  /* SIO */
    label       = (Dpc & 0x3fff) | ( (Opc & 0x3fff) << 14) | ((link & 0x0f) << 28);
    buf->label[0] = label_ch[0];
    buf->label[1] = label_ch[1];
    buf->label[2] = label_ch[2];
    buf->label[3] = label_ch[3];

    buf->sltype  = SLTM_PKT;
	buf->length  = (SLTM_LEN << 4);

    for(i = 0; i < 15; ++i)
    {
        buf->pattern[i] = 0x30 + i;
    }

	TxBuf.hdr.end           = cpi_htocp_s(ENDSIZE + SLTM_LEN + 7);
	TxBuf.hdr.hdr.length    = HDRSIZE + SLTM_LEN + 7;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Data failed, status = %d\n", status );
    }
}

/*
 *  doSLTA - This routine sends a Data Request with an SLTA in it, called after an SLTM is received
 *
 * Parameters
 *       inbuf   -  A pointer to the data from the received SLTM
 *       link    -  Which link to send on
 *
 */
void doSLTA(pktSLT *inbuf, short link)
{
    unsigned short i, length;
    short status;
	U16 dpc,opc;
    U32 label;
    char *label_ch = (char *)&label;            /* Play games because buf->label is a char array for alignment reasons */
    pktSLT *outbuf = (pktSLT *)&(TxBuf.data);

    printf("Link %d:Received SLTM, sending SLTA in response\n", link);

    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);
    TxBuf.msgType       = MTP2_DATA_REQ;
    TxBuf.providerId    = cpi_htocp_s(link);

	outbuf->sio     = inbuf->sio;

    memcpy  (label_ch, inbuf->label, sizeof(inbuf->label));
    dpc             = (U16)(label & 0x3fff);
    opc             = (U16)((label >> 14) & 0x3fff);
    label           = (opc & 0x3fff) | ( (dpc & 0x3fff) << 14) | ((link & 0x0f) << 28);
    outbuf->label[0] = label_ch[0];
    outbuf->label[1] = label_ch[1];
    outbuf->label[2] = label_ch[2];
    outbuf->label[3] = label_ch[3];

    outbuf->sltype  = SLTA_PKT;
    outbuf->length  = inbuf->length;
    length = outbuf->length >> 4;
    for ( i=0; i < length; ++i )
    {
        outbuf->pattern[i] = inbuf->pattern[i];
    }

	TxBuf.hdr.end           = cpi_htocp_s((unsigned short)(ENDSIZE + length + 7));
	TxBuf.hdr.hdr.length    = HDRSIZE + length + 7;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send SLTA failed, status = %d\n", status );
    }

}

/*
 *  doCOO - This routine sends a Data Request containing an COO
 *
 * Parameters
 *      link    -   Which link to send on
 *
 */
void doCOO(short link, U32 bsn_in)
{
    short status;
    U32 label;
    char *label_ch = (char *)&label;            /* Play games because buf->label is a char array for alignment reasons */
    pktCOO *buf = (pktCOO *)&(TxBuf.data);
    U8 bsn = (U8)(bsn_in & 0x00ff);

    printf("Link %d:Sending an COO with bsn = 0x%x\n",link, bsn);

    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);
    TxBuf.msgType       = MTP2_DATA_REQ;
    TxBuf.providerId    = cpi_htocp_s(link);

    buf->sio    = SIO_MGMT_MSG;         /* SIO */
    label       = (Dpc & 0x3fff) | ( (Opc & 0x3fff) << 14) | ((link & 0x0f) << 28);
    buf->label[0] = label_ch[0];
    buf->label[1] = label_ch[1];
    buf->label[2] = label_ch[2];
    buf->label[3] = label_ch[3];

    buf->sltype  = COO_PKT;
	buf->fsn = bsn;

    TxBuf.hdr.end           = cpi_htocp_s(ENDSIZE + sizeof(pktCOO));
    TxBuf.hdr.hdr.length    = HDRSIZE + sizeof(pktCOO);

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Data failed, status = %d\n", status );
    }
}
/*
 *  doCOA - This routine sends a Data Request containing an COA
 *
 * Parameters
 *      link    -   Which link to send on
 *
 */
void doCOA(short link, U32 bsn_in)
{
    short status;
    U32 label;
    char *label_ch = (char *)&label;            /* Play games because buf->label is a char array for alignment reasons */
    pktCOO *buf = (pktCOO *)&(TxBuf.data);
    U8 bsn = (U8)(bsn_in & 0x00ff);

    printf("Link %d:Sending an COA with bsn = 0x%x\n",link, bsn);

    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);
    TxBuf.msgType       = MTP2_DATA_REQ;
    TxBuf.providerId    = cpi_htocp_s(link);

    buf->sio    = SIO_MGMT_MSG;         /* SIO */
    label       = (Dpc & 0x3fff) | ( (Opc & 0x3fff) << 14) | ((link & 0x0f) << 28);
    buf->label[0] = label_ch[0];
    buf->label[1] = label_ch[1];
    buf->label[2] = label_ch[2];
    buf->label[3] = label_ch[3];

    buf->sltype  = COA_PKT;
	buf->fsn = bsn;

    TxBuf.hdr.end           = cpi_htocp_s(ENDSIZE + sizeof(pktCOO));
    TxBuf.hdr.hdr.length    = HDRSIZE + sizeof(pktCOO);

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Data failed, status = %d\n", status );
    }
}
/*
 *  doResend - This routine sends a Data Request containing a resend of a data confirmation
 *
 * Parameters
 *      link    -   Which link to send on
 *
 */
void doResend(short link, MTP2RXPKT *RxBuf)
{
    short i, status, length, start = cpi_htocp_s(RxBuf->hdr.start);
    U32 label_U32;                      /* Play games because buf->label is a char array for alignment reasons */
    char *label_ch = (char *)&label_U32;
    char *ptr = (char *)&(TxBuf.data);
    char *dataPtr = (char *)&RxBuf->data[start+5];
    pktSLT *buf = (pktSLT *)ptr;

    printf("Link %d:Resending a data packet\n",link);

    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);
    TxBuf.msgType       = MTP2_DATA_REQ;
    TxBuf.providerId    = cpi_htocp_s(link);

    length = cpi_htocp_s(RxBuf->hdr.end) - start + 1;
    for ( i = 0; i < length; ++i)
    {
        ptr[i] = dataPtr[i];
    }


    /* If the message is an SLT message, then change the link in the msg.
     * Ordinarily, we would not necessarily resend a SLT packet.  However,
     * for this example program we do and in order for the SLT to succeed, we need
     * to change this piece of data.  Usually, the above code should be enough
     * to successfully re-send an unacked packet*/
    if ( (buf->sio == Sio) && ( (buf->sltype == SLTM_PKT)||(buf->sltype == SLTA_PKT) ) )
    {
        memcpy (label_ch, buf->label, sizeof(buf->label));
        label_U32  = (label_U32 & 0x0fffffff) | ( (link & 0x0f) << 28);
    }

    TxBuf.hdr.end           = cpi_htocp_s((unsigned short)(ENDSIZE + length));
    TxBuf.hdr.hdr.length    = HDRSIZE + length;

    status = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
    if( status != CPI_SUCCESS )
    {
        printf( "Send Data failed, status = %d\n", status );
    }
}
/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
	 ByteCnt    #Bytes in Buf
OUT: <file>      gets multiple lines
*/
void showHex(unsigned char *Buf, unsigned short ByteCnt)
{
    unsigned short Li, LMax, ToDo, Bi, Ai;
    char Ascii[80];
    unsigned char B;

    /* Dump to std out in hex followed by ascii */

    for (ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax)
    {
        LMax = (ToDo > 16) ? 16 : ToDo;         /* max bytes this line */
        printf("%04X| ", Bi);
        Ai = 0;
        for (Li = 0; Li < LMax; Li++)           /* each byte this line */
        {
            B = *Buf++;                         /* next byte to show */
            printf("%02X ", B);                 /* as hex */
            if (!isprint(B))                    /* printable ascii? */
            {
                B = '.';                        /* no, use this */
            }
            Ascii[Ai++] = B;                    /* into ascii portiob */
        }  /* for Li */
        for (Li = LMax; Li < 16; Li++)          /* pad short ln if necessary */
        {
            printf("   ");                      /* so asci lines up */
        }
        Ascii[Ai] = 0;                          /* terminate the ascii */
        printf("    ");
        printf("%s\n", Ascii);                  /* show the ascii */
        Bi += LMax;
    }   /* for ToDo */
}	/* ShowHex */

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
    {
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
        {
			p++;
        }

		/* beginning of token (or end of line) */
		if( *p != 0 )
        {
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
            {
				if( islower( *endp ) )
                {
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
                }
				endp++;
            }

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
            {
				p = endp;	/* end of input - terminate scan */
            }
			else
            {
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
            }
        }   /* end if *p != 0 */
    }   /* end while *p != 0 .... */

	return( cnt );
}

/* cmdPrompt
 *
 * Prompts user to enter command, collects parameters, and issues appropriate
 * MTP2 commands
 */
short cmdPrompt( void )
{
    char            buffer[256];
    char           *tokens[MAX_PARMS +1];
    U16             cnt, i, numPkt;
    U16             action, link;
    char            flcLevel;
    unsigned int    scan_uint;

    /* prompt for a command */
    printf( "Command?>" );
    tx_core_gets(buffer, 255);
    printf( "\n" );
    cnt = tokenize( buffer, tokens );
    if( cnt == 0 )
    {
        return(TX_CORE_SUCCESS);
    }

    if (strcmp( tokens[0], "SEND" ) == 0)
    {
        if( cnt < 2 )
        {
            printf( "The SEND command expects link number, e.g., send 0\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        /* send the data */
        doSLTM(link);
        }
	else if (strcmp( tokens[0], "MSEND" ) == 0)
    {
		if( cnt < 3 )
        {
			printf( "The MSEND command expects link number and number of packets, e.g., send 0 10\n" );
			return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
			printf( "Parameter error [%s] - link expected\n", tokens[1]);
			return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        if (( (sscanf( tokens[2], "%u", &scan_uint )) != 1 ) || ((scan_uint <= 0) && (scan_uint >= 200)))
        {
			printf( "Parameter error [%s] - number of packets expected\n", tokens[1]);
			return(TX_CORE_SUCCESS);
        }
        numPkt = (U16)scan_uint;
		/* send the data */
		for( i = 0; i < numPkt; ++i)
        {
			doSLTM(link);
        }
    }
    else if (strcmp( tokens[0], "CONNECT" ) == 0)
    {
        if( cnt < 2 )
        {
            printf( "The CONNECT command expects link number, e.g., connect 0\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        doConnect(link);
    }
    else if (strcmp( tokens[0], "ECONN" ) == 0)
    {
        if( cnt < 2 )
        {
            printf( "The ECONN command expects link number, e.g., econn 0\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        doEConn(link);
    }
    else if (strcmp( tokens[0], "POWCONN" ) == 0)
    {
        if( cnt < 2 )
        {
            printf( "The POWCONN command expects link number, e.g., powconn 0\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        doPowerOn(link);
    }
    else if (strcmp( tokens[0], "DISCONN" ) == 0)
    {
        if( cnt < 2 )
        {
            printf( "The DISCONN command expects link number, e.g., disconn 0\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        doDisconn(link);
    }
    else if (strcmp( tokens[0], "FLOW" ) == 0)
    {
        if( cnt < 3 )
        {
            printf( "The FLOW command expects link number and ON or OFF, e.g., flow 0 OFF\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        if( strncmp( tokens[2], "ON", 2 ) == 0 )
        {
            flcLevel = 1;
        }
        else if( strncmp( tokens[2], "OFF", 3 ) == 0 )
        {
            flcLevel = 0;
        }
        else
        {
            printf( "Parameter error [%s] - ON or OFF expected\n", tokens[2]);
            return(TX_CORE_SUCCESS);
        }
        doFlow(link, FLOWREQ_SETLEVEL, flcLevel);
    }
    else if (strcmp( tokens[0], "STATUS" ) == 0)
    {
        if( cnt < 3 )
        {
            printf( "The STATUS command expects link number and an action, e.g., status 0 UP\n" );
            printf( "Actions consist of: 1) UP    - processor up action\n" );
            printf( "                    2) DOWN  - processor down action\n" );
            printf( "                    3) START - start emergency condition\n" );
            printf( "                    3) END   - end emergency condition\n" );
            return(TX_CORE_SUCCESS);
        }
        if (( (sscanf( tokens[1], "%u", &scan_uint )) != 1 ) || (scan_uint > MAX_LINK_NUM))
        {
            printf( "Parameter error [%s] - link expected\n", tokens[1]);
            return(TX_CORE_SUCCESS);
        }
        link = (U16)scan_uint;
        if( strncmp( tokens[2], "DOWN", 4 ) == 0 )
        {
            action = STAREQ_STS_LCL_DOWN;
        }
        else if( strncmp( tokens[2], "UP", 2 ) == 0 )
        {
            action = STAREQ_STS_LCL_UP;
        }
        else if( strncmp( tokens[2], "START", 5 ) == 0 )
        {
            action = STAREQ_STS_START_EMERG;
        }
        else if( strncmp( tokens[2], "END", 3 ) == 0 )
        {
            action = STAREQ_STS_END_EMERG;
        }
        else
        {
            printf( "Parameter error [%s] - UP, DOWN, START or END expected\n", tokens[2]);
            return(TX_CORE_SUCCESS);
        }
        doStatusReq(link, STAREQ_CHANGE_STATUS, action);
    }
    else if (strncmp( tokens[0], "Q", 1 ) == 0)
    {
        return (TX_CORE_EXIT_POLL_LOOP);
    }
    else
    {
        printf( "Valid Commands:\n" );
        printf( "Connect <link>                             Starts Normal Alignment\n" );
        printf( "Econn <link>                               Starts Emergency Alignment\n" );
        printf( "Powconn <link>                             Power On (link sends SIOS)\n" );
        printf( "Disconn <link>                             Disconnects\n" );
        printf( "Flow <link> [ON | OFF]                     Turns Flow ON or OFF\n" );
        printf( "Send <link>                                Sends a SLTM message\n" );
		printf( "MSend <link> <# packets>                   Sends many SLTM messages\n" );
        printf( "Status <link> [UP | DOWN | START | END]    Sends a status request\n" );
    }
    return(TX_CORE_SUCCESS);
}
/* kbHandler
 *
 * Keyboard event handler routine, to be called by "tx_core" whenever an asynchronous
 * keyboard input is detected.
 */
short kbHandler(TX_HANDLE handle, short events)
{
    int     c;

    UNREFERENCED_PARAMETER(handle);
    /* check if input event really did occur */
    if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
    {
        return (TX_CORE_SUCCESS);
    }

	/* make sure a key was hit */
    if (tx_core_kbhit())
    {
        c = getchar();
        if( c == 'q' || c == 'Q' )
        {
            return (TX_CORE_EXIT_POLL_LOOP);
        }
        else
        {
            return(cmdPrompt());
        }
    }

    return (TX_CORE_SUCCESS);
}

/****************************************************************************
 * Mtp2RxNotify - handle notification of incoming receive (from the TX board)
 *
 * INPUT:   handle - handle to CPI channel that message was received over
 *          ckhey - any control key associated with given CPI channel
 ****************************************************************************/
void Mtp2RxNotify( TX_HANDLE handle, void *chkey )
{
    S16             ret;
    short           status, i;
    short           len = sizeof(MTP2RXPKT);
    unsigned short  start, end;
    U8              msgType;
    S16             userId;
    U8             *dataPtr;
    pktSLT          buf;
    pktCOO         *cooBuf;
    DatCfm          datCfm;
    DisInd          disInd;
    StaCfm          staCfm;
    FlowInd         flowInd;
    StaInd          staInd;

    UNREFERENCED_PARAMETER(handle);
    UNREFERENCED_PARAMETER(chkey);

    status = cpia_get_data( txHandle, (void *) &RxBuf, &len );
    if( status != CPI_SUCCESS )
    {
        printf( "Retrieve Message failed, status %d\n", cpi_get_last_error());
        return;  /* fatal */
    }

    start = cpi_cptoh_s(RxBuf.hdr.start);
    end = cpi_cptoh_s(RxBuf.hdr.end);
    msgType = RxBuf.data[start];
    userId = (RxBuf.data[start+1] << 8) | RxBuf.data[start+2];
    dataPtr = (U8 *)&RxBuf.data[start+3];

    switch(mtp2State)
    {
        case MTP2APP_STATE_BIND1 :
            if (msgType == MTP2_BIND_CFM)
            {
                printf( "Link %d:Bind Confirmation received\n", userId);
            }
            else
            {
                printf( "Link %d: Rcvd msg type %d while waiting for Bind Confirm", userId, msgType);
                exit(1);
            }

            TxBuf.providerId            = cpi_htocp_s(1);
            TxBuf.data.bindReq.userId   = cpi_htocp_s(1);
            ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey);
            if (ret != CPI_SUCCESS)
            {
                printf("The second Bind cpia_send failed: %d %d\n", ret, cpi_get_last_error());
                exit(1);
            }
            mtp2State = MTP2APP_STATE_BIND2;
            break;

        case MTP2APP_STATE_BIND2 :
            if (msgType == MTP2_BIND_CFM)
            {
                printf( "Link %d:Bind Confirmation received\n", userId);
            }
            else
            {
                printf( "Link %d: Rcvd msg type %d while waiting for Bind Confirm", userId, msgType);
                exit(1);
            }

            TxBuf.providerId            = cpi_htocp_s(2);
            TxBuf.data.bindReq.userId   = cpi_htocp_s(2);
            ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey);
            if (ret != CPI_SUCCESS)
            {
                printf("The second Bind cpia_send failed: %d %d\n", ret, cpi_get_last_error());
                exit(1);
            }
            mtp2State = MTP2APP_STATE_BIND3;
            break;

        case MTP2APP_STATE_BIND3 :
            if (msgType == MTP2_BIND_CFM)
            {
                printf( "Link %d:Bind Confirmation received\n", userId);
            }
            else
            {
                printf( "Link %d: Rcvd msg type %d while waiting for Bind Confirm", userId, msgType);
                exit(1);
            }

            TxBuf.providerId            = cpi_htocp_s(3);
            TxBuf.data.bindReq.userId   = cpi_htocp_s(3);
            ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey);
            if (ret != CPI_SUCCESS)
            {
                printf("The second Bind cpia_send failed: %d %d\n", ret, cpi_get_last_error());
                exit(1);
            }
            mtp2State = MTP2APP_STATE_BIND4;
            break;

        case MTP2APP_STATE_BIND4 :
            /* got an incoming message, decode it */
            switch( msgType )
            {
                case MTP2_BIND_CFM:
                    printf( "Link %d:Bind Confirmation received\n", userId);
                    break;

                case MTP2_UNBIND_CFM:
                    printf( "Link %d:Unbind Confirmation received\n", userId);
                    break;

                case MTP2_DISC_CFM:
                    printf( "Link %d:Disconnect Confirmation received\n", userId);
                    break;

                case MTP2_FLOW_CFM:
                    printf( "Link %d:Flow Control Confirmation received\n", userId);
                    break;

                case MTP2_CONN_CFM:
                    printf( "Link %d:Connect Confirmation received\n", userId);
                    currentState[userId] = LINK_UP;
                    break;

                case MTP2_DATA_CFM:
                    memcpy( &datCfm, dataPtr, sizeof(DatCfm) );
                    printf( "Link %d:Data Confirmation received ", userId);

                    switch ( datCfm.status )
                    {
                        case DATCFMSTA_MORE :
                            printf("with a packet\n");
                            if (currentState[userId] == LINK_GETUNACKED)
                            {
                                doResend( changeToLink, &RxBuf);
                            }
                            break;
                        case DATCFMSTA_LAST :
                            printf("with the last packet\n");
                            if (currentState[userId] == LINK_GETUNACKED)
                            {
                                doResend( changeToLink, &RxBuf);
                                currentState[userId] = LINK_DOWN;
                                currentState[changeToLink] = LINK_UP;
                                printf("Link %d:Changeover complete\n", userId);
                            }
                            break;
                        case DATCFMSTA_NUM_ACKED :
                            printf("acking %d messages\n", datCfm.credit);
                            break;
                        default:
                            printf("with an illegal status field [%d]\n", datCfm.status);
                            break;
                    }
                    break;

                case MTP2_DISC_IND:
                    memcpy( &disInd, dataPtr, sizeof(DisInd) );
                    printf( "Link %d:Disconnect Indication received - reason %d\n", userId,
                            cpi_cptoh_s(disInd.reason));

                    if ( currentState[userId] == LINK_UP)
                    {
                        currentState[userId] = LINK_CHANGEFROM;
                        changeFromLink = userId;
                        if ( userId == 0 )
                        {
                            changeToLink = 1;
                        }
                        else changeToLink = 0;
                        currentState[changeToLink] = LINK_CHANGETO;
                        startChangeOver(userId);
                    }
                    break;

                case MTP2_FLOW_IND:
                    memcpy( &flowInd, dataPtr, sizeof(FlowInd) );
                    printf( "Link %d:Flow Indication received - congestion ", userId);

                    if (cpi_cptoh_s(flowInd.action) == FLOWIND_STARTED)
                    {
                        printf("started\n");
                    }
                    else if (cpi_cptoh_s(flowInd.action) == FLOWIND_STOPPED)
                    {
                        printf("ended\n");
                    }
                    else
                    {
                        printf("UNKNOWN ACTION\n");
                    }
                    break;

                case MTP2_STAT_CFM:
                    memcpy( &staCfm, dataPtr, sizeof(StaCfm) );
                    printf( "Link %d:Status Confirmation received ", userId);

                    switch ( cpi_cptoh_s(staCfm.action) )
                    {
                        case STACFM_BSN :
                            bsn = cpi_cptoh_l(staCfm.status);
                            printf("with a BSN = %d\n", bsn);
                            if (currentState[userId] == LINK_CHANGEFROM)
                            {
                                doCOO(changeToLink, bsn);
                                currentState[changeToLink] = LINK_WAITCOA;
                            }
                            else if (currentState[userId] == LINK_RCVDCOO)
                            {
                                doCOA(changeToLink, bsn);
                                currentState[userId] = LINK_CHANGEFROM;
                                printf("Link %d:Sent COA, retrieving unacked packets from downed link fsn = %d\n", userId, fsn);
                                doStatusReq( changeFromLink, STAREQ_GET_UNACKED, fsn );
                                currentState[changeFromLink] = LINK_GETUNACKED;
                            }
                            break;
                        case STACFM_NO_UNACKED :
                            printf("with no unacknowledged messages\n");
                            if (currentState[userId] == LINK_GETUNACKED)
                            {
                                currentState[userId] = LINK_DOWN;
                                currentState[changeToLink] = LINK_UP;
                                printf("Link %d:Changeover complete\n", userId);
                            }
                            break;
                        case STACFM_NOACTION :
                            switch (cpi_cptoh_l(staCfm.status))
                            {
                                case STACFM_STS_LCL_DOWN:
                                    printf (" - Local Processor Down\n");
                                    break;
                                case STACFM_STS_LCL_UP:
                                    printf (" - Local Processor Up\n");
                                    break;
                                default:
                                    printf ("Status confirm with illegal status field [%d]\n",cpi_cptoh_l(staCfm.status));
                                    break;
                            }
                            break;
                        default:
                            printf("with an illegal action field [%d]\n",cpi_cptoh_s(staCfm.action));
                            break;
                    }
                    break;

                case MTP2_DATA_IND:
                    memcpy( &buf, dataPtr, sizeof(pktSLT) );

                    if ( (buf.sltype == SLTM_PKT) && (buf.sio == Sio) )  /* answer an SLTM with an SLTA */
                    {
                        doSLTA( &buf, userId);
                    }
                    else if ( (buf.sltype == SLTA_PKT) && (buf.sio == Sio) ) /* check the SLTA */
                    {
                        status = 0;
                        for ( i=0; i < SLTM_LEN; ++i)
                        {
                            if ( buf.pattern[i] != 0x30+i)
                            {
                                status = 1;
                            }
                        }
                        if (status)
                        {
                            printf("Link %d:Received an incorrect SLTA\n", userId);
                        }
                        else
                        {
                            printf("Link %d:Received a correct SLTA\n", userId);
                        }
                    }
                    else if ( (buf.sltype == COA_PKT) && (buf.sio == SIO_MGMT_MSG) )  /* we recvd a COA */
                    {
                        cooBuf = (pktCOO *)&buf;  /* get the unacked packets to forward */
                        fsn = (U32)cooBuf->fsn;
                        printf("Link %d:Received COA, retrieving unacked packets from downed link fsn = %d\n", userId, fsn);
                        doStatusReq( changeFromLink, STAREQ_GET_UNACKED, fsn );
                        currentState[changeFromLink] = LINK_GETUNACKED;
                    }
                    else if ( (buf.sltype == COO_PKT) && (buf.sio == SIO_MGMT_MSG) )
                    {
                        cooBuf = (pktCOO *)&buf;
                        fsn = (U32)cooBuf->fsn;
                        printf("Link %d:Received COO\n", userId);

                        /* if we are not already waiting for a COA or the BSN */
                        if ( currentState[userId] != LINK_WAITCOA )
                        {
                            U32 label_U32;
                            char *label_ch = (char *)&label_U32;
                            memcpy (label_ch, cooBuf->label, sizeof(cooBuf->label));
                            changeToLink = userId;
                            changeFromLink = (short)((label_U32 & 0xf0000000) >> 28);
                            currentState[changeFromLink] = LINK_RCVDCOO;
                            if ( currentState[userId] != LINK_CHANGETO )  /* have we started at all yet? */
                            {
                                startChangeOver(changeFromLink);
                            }
                            currentState[userId] = LINK_CHANGETO;  /* start the changeover process */
                        }
                        else
                        {
                            doCOA(userId, bsn);  /* send a COA if we are not waiting for one */
                        }
                    }
                    else
                    {
                        printf( "Link %d:Data Packet received length %d\n", userId, end - start + 1);
                        showHex( dataPtr, (unsigned short) (end - start + 1) );
                        printf( "\n" );
                    }
                    break;

                case MTP2_STAT_IND:
                    memcpy( &staInd, dataPtr, sizeof(StaInd) );
                    /* print out status indication message */
                    printf( "Link %d:Status indication received - status: Remote Processor ", userId);

                    if (staInd.status == STAIND_REM_DOWN)
                    {
                        printf("Down\n");
                    }
                    else if (staInd.status == STAIND_REM_UP)
                    {
                        printf("Up\n");
                    }
                    else
                    {
                        printf("UNKNOWN\n");
                    }

                    doStatusRsp (userId, staInd.status);        /* Respond to status indication */
                    break;

                default:
                    printf( "Link %d:Unknown MTP2 message received, message type: %d\n", userId, msgType);
            }                           /* switch( msg.msgcode ) */

            break;
            default:
                printf("Unknown state %d\n", mtp2State);
                exit(1);
    }
}

/****************************************************************************
 * Mtp2TxNotify - handle notification of transmit (to TX board) completed
 *
 * For the MONapp program, a transmit completes whenever a BIND (or UNBIND)
 * sent by MONapp has reached the monitor task on the board.
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          chkey  - Mtp2app's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - Mtp2app's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void Mtp2TxNotify( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                  void *tx_key, CPI_ERR_TYPE ccode )
{
    UNREFERENCED_PARAMETER(handle);
    UNREFERENCED_PARAMETER(chkey);
    UNREFERENCED_PARAMETER(msg);
    UNREFERENCED_PARAMETER(tx_key);
    UNREFERENCED_PARAMETER(ccode);
    /* Nothing to do here since all messages transmitted to the board are
     * built using the global TxBuf (so no memory to deallocate)
     */
}

/****************************************************************************
 * MTP2Handler
 *
 * MTP2 event handler routine, to be called by "tx_core"
 * whenever an asynchronous event message for MTP2 is pending.
 ****************************************************************************/
short MTP2Handler( TX_HANDLE handle, short events )
{
    U32     ret;
    U32     status;

    /* check if input event really did occur */
    if (tx_core_process_events( TX_CORE_INPUT, events ) == FALSE)
    {
        return( TX_CORE_SUCCESS );
    }

    /* call into the CPI library to service any asynchronous IO over the CPI channel */
    status = cpia_intr( handle );
    if( status != CPI_SUCCESS )
    {
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpia_intr\n", ret, cpi_get_error_str( ret ) );

        return( TX_CORE_EXIT_POLL_LOOP );   /* fatal */
    }

    return( TX_CORE_SUCCESS );
}

/*
 * Main routine
 *
 * Parse command line arguments, start main processing loop
 */
int main( int argc, char *argv[] )
{
    unsigned int    scan_uint;
    S16             n;
    S16             ret = 0;
    char            sbuf[CPSTRSIZE], *ptr = sbuf; /* dummy for old cpi_init */
    char            *ap;
    CPI_WAIT_TYPE   hSync;      /* synchronization handle for MTP2 API */
    CPI_OPEN_CONFIG openCfg;
    MULTI_TYPE      key;

    /* set up key */
    key.as_ptr = NULL;
    key.as_u32 = 0xAABBCCDD; /* any key value, passed to CPIA callback routines */
    mtp2ChanKey = key.as_ptr;


    currentState[0]     = LINK_DOWN;
    currentState[1]     = LINK_DOWN;

    /*These values never change !! */
    TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst   = PORT(Board, MTP2_ENT);
    TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);

	/* process arguments */
    for (n=1; n < argc; n++)
    {
		ap = argv[n];
        if (*ap == '-')
        {
            switch (*++ap)
            {
                case 's' :     /* SIO */
                    if (*++ap == 0)
                    {
                        n++;
                        sscanf(argv[n], "%u", &scan_uint);
                    }
                    else
                    {
                        sscanf(ap, "%u", &scan_uint);
                    }

                    if (scan_uint > 255)
                    {
                        printf("SIO must be in range 0 - 0xFF\n");
                        exit( 1 );
                    }
                    Sio = (unsigned char)scan_uint;
                    break;

				case 'd' :     /* DPC */
                    if (*++ap == 0)
                    {
                        n++;
                        sscanf(argv[n], "%u", &scan_uint);
                        Dpc = (U16)scan_uint;
                    }
                    else
                    {
                        sscanf(ap, "%u", &scan_uint);
                        Dpc = (U16)scan_uint;
                    }
                    if ( Dpc > 0x3fff )
                    {
                        printf("Point Code fields must be in range 0 - 0x3FFF\n");
						exit( 1 );
                    }
					break;

				case 'o' :     /* OPC */
                    if (*++ap == 0)
                    {
                        n++;
                        sscanf(argv[n], "%u", &scan_uint);
                        Opc = (U16)scan_uint;
                    }
                    else
                    {
                        sscanf(ap, "%u", &scan_uint);
                        Opc = (U16)scan_uint;
                    }
                    if ( Opc > 0x3fff )
                    {
                        printf("Point Code fields must be in range 0 - 0x3FFF\n");
						exit( 1 );
                    }
					break;

                case 'b' :     /* board number */
                    if (*++ap == 0)
                    {
                        n++;
                        Board = atoi(argv[n]);
                    }
                    else
                    {
                        Board = atoi(ap);
                    }

                    if( Board < 1 || Board > 8 )
                    {
                        printf("Board must be in range 1 - 8\n");
                        exit(1);
                    }
                    break;

				case '?' :      /* Help */
                    printf ("MTP2itu  [-s <sio>] [-b <board>] [-d n] [-o n] [-?]\n");
                    printf ("    -s <sio>      Service Information Octet value; default: 0x01 (International SNT msg)\n");
					printf ("    -b <board>    Board number to use; default: 1\n");
                    printf ("    -d n          Destination Point Code; default: 2\n");
                    printf ("    -o n          Originating Point Code; default: 1\n");
					exit (1);
					break;

				default:
					printf("Unrecogized Parameter: %s\n", argv[n]);
                    printf ("MTP2itu  [-s <sio>] [-b <board>] [-d n.n.n] [-o n.n.n] [-?]\n");
                    printf ("    -s <sio>      Service Information Octet value; default: 0x01 (International SNT msg)\n");
					printf ("    -b <board>    Board number to use; default: 1\n");
                    printf ("    -d n          Destination Point Code; default: 2\n");
                    printf ("    -o n          Originating Point Code; default: 1\n");
					exit(1);

            }   /* end switch */
        }
        else
        {
			printf("Unrecogized Parameter: %s\n", argv[n]);
			exit(1);
        }
    }   /* end for loop */


    /*open the cpi channel */
    cpi_init(ret, &ptr);

    memset( (void *)&openCfg, 0, sizeof(CPI_OPEN_CONFIG) );
    openCfg.board = (U16)Board;                  /* TX board number */
    openCfg.channel = DFLT_ENT;             /* channel number to register for receives on */
    openCfg.maxInProg = TXCPI_MAX_IN_PROG;  /* maximum number of transmits or receives in progress for this channel (set to max allowed) */
    openCfg.rxnotify = Mtp2RxNotify;         /* receive notification - trigger to call cpia_get_data */
    openCfg.txnotify = Mtp2TxNotify;         /* transmit complete notification */
    /* NOTE: openCfg.evnotify is not used so leave as NULL */

    txHandle = cpix_open( mtp2ChanKey, &openCfg );
    if (txHandle == CPI_INVALID_HANDLE)
    {
        printf("The cpi_open failed: %d\n", cpi_get_last_error());
        exit(1);
    }
    hSync = cpi_wait_obj(txHandle);
    if (hSync == CPI_INVALID_WAIT_HANDLE)
    {
        printf("The cpi_wait_obj failed: %d\n", cpi_get_last_error());
        exit(1);
    }

    /* Bind this application as a user of the MTP2 layer */
    TxBuf.hdr.hdr.src           = PORT(0, DFLT_ENT);
    TxBuf.hdr.hdr.dst           = PORT(Board, MTP2_ENT);
    TxBuf.hdr.hdr.length        = HDRSIZE + sizeof(BindReq);
    TxBuf.hdr.start             = cpi_htocp_s(FILLER_COUNT);
    TxBuf.hdr.end               = cpi_htocp_s(ENDSIZE + sizeof(BindReq));
    TxBuf.msgType               = MTP2_BIND_REQ;
    TxBuf.providerId            = cpi_htocp_s(0);
    TxBuf.data.bindReq.userId   = cpi_htocp_s(0);
    TxBuf.data.bindReq.srcEnt   = DFLT_ENT;
    TxBuf.data.bindReq.srcInst  = 0;
    ret = cpia_send(txHandle, (CPIPKT *)&TxBuf, myTxKey);
    if (ret != CPI_SUCCESS)
    {
        printf("The first Bind cpia_send failed: %d %d\n", ret, cpi_get_last_error());
        exit(1);
    }
    mtp2State = MTP2APP_STATE_BIND1;

    /* install keyboard and MTP2 event handlers with "tx_core" and let it
     * perform the main "wait for event" loop
     */
    tx_core_init();
    tx_core_install_handler( stdInput, TX_CORE_UNUSED, kbHandler );
    tx_core_install_handler( hSync, txHandle, MTP2Handler );
    tx_core_set_poll(stdInput, TX_CORE_INPUT);
    tx_core_set_poll(hSync, TX_CORE_INPUT);

    /**** let her rip ****/
    tx_core_poll_loop( TX_CORE_INFINITE );

    /* when tx_core_poll_loop returns, user has selected "quit" */
    cpi_close(txHandle);
} /* end of main */
