/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1996 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/*
 * mtp2cfg.c
 *
 * Scans ascii text configuration file and downloads configuration to MTP2
 * task on specified board.
 */

/* header include files -- defines (.h) */
#include <sys/types.h>
/* #include <stropts.h> */
/* #include <poll.h> */
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>

/*
#define IO
    #include "cpkuser.h"
    #include "dpruser.h"
#undef IO
*/

#include "ss7type.h"
#include "mtp2mapi.h"
#include "cfgscan.h"
#include "mtperr.h"
#include "mtp2ver.h"

/* global variables */
U8      myInst;
U8      myEntId;
short   bDebug = 0;                  /* prints each packet sent/received */

char configFile[256] = "MTP2.cfg";
static char *usage = "Usage: mtp2cfg [ -b <board number> ] [-f <filename>]\n";
char buf[80];

/* macro to extract port type from port id (hi byte) */
#define TXPORTTYPE( portid )    ((portid >> 8) & 0xFF)

/* macro to extract port number from port id (low byte) */
#define TXPORTNUM( portid )     (portid & 0xff)

/* macro to extract port type from port id (hi byte) & convert to printable char */
#define TXASCPORTTYPE( portid ) ((((portid >> 8) & 0xFF) == PORT_TYPE_TDM) ? "T" : \
                                 (((portid >> 8) & 0xFF) == PORT_TYPE_REMOTE) ? "R" : "S")

/* Handy macro to get & range check a long, print a warning if its out
 * of range, and copy the value (possibly cast to a smaller type)
 * to "dest" if its in range; assumes "dest" is previously set to
 * default value. "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_INT( name, min, max, dest, cast )         \
{                                                           \
    if (getcfgval(name, CFGT_INT, &temp, 0) > 0) {          \
        if ( temp < min || temp > max ) {                   \
        printf( "Warning: %s [%d] out of range, setting to default [%d]\n",\
                name, temp, dest );                         \
        } else {                                            \
            dest = (cast) temp;                             \
        }                                                   \
    }                                                       \
}

#define GET_CHECK_INT_FROM( name, min, max, dest, cast, from, next )    \
{                                                                       \
    if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0) {          \
        if ( temp < min || temp > max ) {                               \
            printf( "Warning: %s [%d] out of range, setting to default [%d]\n",\
                name, temp, dest );                                     \
        } else {                                                        \
            dest = (cast) temp;                                         \
        }                                                               \
    }                                                                   \
}

/* Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER( name, timer )                            \
{                                                           \
  if (getcfgval(name, CFGT_INT, &temp, 0) > 0)              \
    timer.value = (U16) temp;                               \
  timer.enable = (timer.value != 0) ? 1 : 0;                \
}

#define GET_TIMER_FROM( name, timer, from, next )               \
{                                                               \
  if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0)      \
    timer.value = (U16) temp;                                   \
  timer.enable = (timer.value != 0) ? 1 : 0;                    \
}

/*
* mtp2LinkCfg
*
* Configures MTP2 data link saps
*/
void doMtp2LinkCfg( U8 board )
{
    MTP2LinkCfg         linkcfg;
    MTP2_STATUS         status;
    U32                 temp, last, next;
    U32                 maxlinks;
    U16                 linkno;
    U16                 portid;
    U8                  lnktype, i;
    char *              pErrstr;
    U8                  oldmethod = FALSE, newmethod = FALSE;

    maxlinks = 4;
    getcfgval("MAX_LINKS", CFGT_INT, (void *) &maxlinks, 0);
    last = 0;                   /* Last line we read */
    for( i = 0; ; i++ )
    {
        /* Check old link/port config method first */
        if ((next = getcfgval("LINK", CFGT_LINK, (void *) &temp, last)) <= 0)
        {
            break;                      /* no more links */
        }

        if (temp & NEW_LINK_TYPE)
        {
            last = next;                        /* Save line number in file in case of error below */

            /* New type of link specification of link number only */
            if (oldmethod)
            {
                printf ("Cannot mix old and new style LINK specifications\n");
                continue;
            }
            newmethod = TRUE;

            linkno = (short)temp & 0xFF;        /* Remove NEW_LINK_TYPE mask */
            if ((next = getcfgval("PORT", CFGT_PORT, (void *) &temp, last)) <= 0)
            {
                printf ("You must specify PORT T<n>, S<n> or R after LINK <n>\n");
                continue;
            }
        }
        else
        {
            /* Old method of link type specified, ie. LINK T1 */
            if (newmethod)
            {
                printf ("Cannot mix old and new style LINK specifications\n");
                last = next;
                continue;
            }
            oldmethod = TRUE;
            linkno = i;
        }


        portid = (short) temp;

        last = next;                        /* save starting line in config file */

        if (linkno >= maxlinks)
        {
            printf( "MTP2 Link Config:Link number %s%d exceeds maximum (%d) - not configured\n",
                TXASCPORTTYPE( portid ), TXPORTNUM( portid ), maxlinks );
            break;
        }

        /* retrieve link type (CCITT, ANSI88, ANSI92) - default is ANSI(92) */
        lnktype = MTP2LNK_ANSI;                             /* default */
        if (getcfgval("LINK_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if ((strcmp(buf, "CCITT") == 0) || (strcmp(buf, "ITU") == 0))
                lnktype = MTP2LNK_CCITT;
            else if (strcmp(buf, "JNTT") == 0)
                lnktype = MTP2LNK_JNTT;
            else if (strcmp(buf, "JTTC") == 0)
                lnktype = MTP2LNK_JTTC;
            else if (strcmp(buf, "ANSI88") == 0)
                lnktype = MTP2LNK_ANSI88;
            else if (strcmp(buf, "ANSI") == 0)
                lnktype = MTP2LNK_ANSI;
            else
            {
                printf( "MTP2 Link Config: Link %s%d LINK_TYPE [%s] unknown - defaulted to ANSI\n",
                    TXASCPORTTYPE( portid ), TXPORTNUM( portid ), buf );
            }
        }

        /* We must know high speed links or not when defaulting timer values in Mtp2InitLinkCfg */
        linkcfg.hsBits = 0;
        if (getcfgval("HS_LINK", CFGT_BOOL, (void *) &temp, last) > 0 )
        {
            // Set EXT_SEQ to be the same as HS_LINK.  EXT_SEQ can be overridden to false below but cannot be true
            // if HS_LINK is false.
            if (temp)
            {
                linkcfg.hsBits |= HS_LINK;
                linkcfg.hsBits |= HS_EXT_SEQ;
            }
            else
            {
                linkcfg.hsBits &= ~HS_LINK;
                linkcfg.hsBits &= ~HS_EXT_SEQ;
            }
        }

        if (getcfgval("HS_EXT_SEQ", CFGT_BOOL, (void *) &temp, last) > 0 )
        {
            // If EXT_SEQ is true, HS_LINK must be true but if EXT_SEQ is false, HS_LINK can be true or false.
            if (temp)
            {
                linkcfg.hsBits |= HS_LINK;
                linkcfg.hsBits |= HS_EXT_SEQ;
            }
            else
            {
                linkcfg.hsBits &= ~HS_EXT_SEQ;
            }
        }
        /* initialize link params block to default values */
        status = Mtp2InitLinkCfg( board, &linkcfg, linkno, lnktype );
        if( status != MTP2_SUCCESS )
        {
            /* should never happen, we've already validated link type */
            printf( "MTP2 DlSap Config: invalid link type, link %s%d configuration failed\n",
                TXASCPORTTYPE( portid ), TXPORTNUM( portid ), buf );
            continue;
        }
        /* store the port number and type.  MUST be after InitLinkCfg to avoid resetting to defaults there */
        linkcfg.portType = TXPORTTYPE( portid );
        linkcfg.portNum = TXPORTNUM ( portid );

        if (linkcfg.portType == PORT_TYPE_REMOTE)
        {
            /* skip up to end of section */
            next = getcfgval("END", CFGT_INT, (void *)&temp, last);
            last = next;
            continue;                       /* Skip remote links for MTP2 */
        }

        /* retrieve optional default overrides */
        GET_CHECK_INT_FROM( "LSSU_LEN",          1,     2, linkcfg.lssuLen,     U8, last, next );
        GET_CHECK_INT_FROM( "MAX_FRAME",        64,  1024, linkcfg.maxFrmLen,  U16, last, next );
        GET_CHECK_INT_FROM( "SUERM_THRESH",      1,   255, linkcfg.sdT,         U8, last, next );
        GET_CHECK_INT_FROM( "SUERM_D_RATE",      1, 65535, linkcfg.sdD,        U16, last, next );
        GET_CHECK_INT_FROM( "AERM_THRESH_N",     1,   255, linkcfg.sdTin,       U8, last, next );
        GET_CHECK_INT_FROM( "AERM_THRESH_E",     1,   255, linkcfg.sdTie,       U8, last, next );
        GET_CHECK_INT_FROM( "MAX_RTB_MSGS",      1,   255, linkcfg.sdN1,        U8, last, next );
        GET_CHECK_INT_FROM( "MAX_RTB_OCTETS",    1, 65535, linkcfg.sdN2,       U16, last, next );
        GET_CHECK_INT_FROM( "MAX_PROV_ABORT",    1,   255, linkcfg.sdCp,        U8, last, next );

        /* Link Related thresholds on queues */
        GET_CHECK_INT_FROM( "ISO_THRESH",        1,  1000, linkcfg.isoThresh,  U16, last, next );
        GET_CHECK_INT_FROM( "L2_TXQ_THRESH1",    1,  1000, linkcfg.txqThresh1, U16, last, next );
        GET_CHECK_INT_FROM( "L2_TXQ_THRESH1_A",  1,  1000, linkcfg.txqThresh1Abate, U16, last, next );
        GET_CHECK_INT_FROM( "L2_TXQ_THRESH2",    1,  1000, linkcfg.txqThresh2, U16, last, next );
        GET_CHECK_INT_FROM( "L2_TXQ_THRESH2_A",  1,  1000, linkcfg.txqThresh2Abate, U16, last, next );
        GET_CHECK_INT_FROM( "L2_SAP_THRESH",     1,  1000, linkcfg.sapThresh,  U16, last, next );
        GET_CHECK_INT_FROM( "L2_SAP_THRESH_A",   1,  1000, linkcfg.sapThreshAbate,  U16, last, next );


        if (getcfgval("CONG_DISCARD", CFGT_BOOL, (void *) &temp, last) > 0 )
            linkcfg.congDisc = (Bool) temp;

        if (getcfgval("ERR_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
            {
            if (strcmp(buf, "NORMAL") == 0)
                linkcfg.errType = (Bool) 0;         /* Normal check */
            else if (strcmp(buf, "PCR") == 0)
                linkcfg.errType = (Bool) 1;         /* Cyclic redundancy check */
            else
                {
                printf( "MTP2 Link Config: ERR_TYPE [%s] unknown - defaulted to NORMAL\n",
                    buf );
                linkcfg.errType = (Bool) 0;         /* Normal check */
                }
            }

        /* scan for timer definitions */
        GET_TIMER_FROM( "L2_T1", linkcfg.t1, last, next );
        GET_TIMER_FROM( "L2_T2", linkcfg.t2, last, next );
        GET_TIMER_FROM( "L2_T3", linkcfg.t3, last, next );
        /* NOTE: linkcfg.t4 was removed in the Trillium merge, provNormal & provEmrgcy are used */
        GET_TIMER_FROM( "L2_T5", linkcfg.t5, last, next );
        GET_TIMER_FROM( "L2_T6", linkcfg.t6, last, next );
        GET_TIMER_FROM( "L2_T7", linkcfg.t7, last, next );
        GET_TIMER_FROM( "L2_T10", linkcfg.t10, last, next );
        GET_TIMER_FROM( "L2_T11", linkcfg.t11, last, next );
        GET_TIMER_FROM( "L2_T12", linkcfg.t12, last, next );
        GET_TIMER_FROM( "L2_T13", linkcfg.t13, last, next );

        /* Set proving times, normal and emergency */
        if (getcfgval("L2_T4_N", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.provNormal = (U16)temp;
        if (getcfgval("L2_T4_E", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.provEmrgcy = (U16)temp;


        /**** Get MTP level 1 parameters ****/
        if (getcfgval("BAUD", CFGT_INT, (void *) &temp, last) > 0 )
            {
            /* Must convert numeric baud to define */
            switch (temp)
                {
                case 4800:
                    linkcfg.baud = BD_4800;
                    break;

                case 9600:
                    linkcfg.baud = BD_9600;
                    break;

                case 14440:
                    linkcfg.baud = BD_14440;
                    break;

                case 19200:
                    linkcfg.baud = BD_19200;
                    break;

                case 38400:
                    linkcfg.baud = BD_38400;
                    break;

                case 48000:
                    linkcfg.baud = BD_48000;
                    break;

                case 56000:
                    linkcfg.baud = BD_56000;
                    break;

                case 64000:
                    linkcfg.baud = BD_64000;
                    break;

                case 512000:
                    linkcfg.baud = BD_512000;
                    break;

                default:
                    printf( "MTP1 Link Config: Baud [%d] invalid - defaulted to 56000\n",
                        (lnktype == MTP2LNK_CCITT) ? 64000 : 56000 );
                    break;
                }
            }

        linkcfg.physIntType = INT_DTE;      /* DTE */
        if (getcfgval("INT_TYPE", CFGT_STRING, (void *) &buf, last) > 0 )
            {
            if (strcmp(buf, "DCE") == 0)
                linkcfg.physIntType = INT_DCE;      /* DCE */
            else if (strcmp(buf, "DTE") == 0)
                linkcfg.physIntType = INT_DTE;      /* DTE */
            else if (strcmp(buf, "DCE_MODEM") == 0)
                linkcfg.physIntType = INT_DCE_MODEM;      /* DCE with modem signals */
            else if (strcmp(buf, "DTE_MODEM") == 0)
                linkcfg.physIntType = INT_DTE_MODEM;      /* DTE with modem signals */
            else
                {
                printf( "MTP1 Link Config: INT_TYPE [%s] unknown - defaulted to DTE\n",
                    buf );
                }
            }


        linkcfg.dataEncoding = DAT_NRZ;
        if (getcfgval("DATA_ENC", CFGT_STRING, (void *) &buf, last) > 0 )
            {
            if (strcmp(buf, "NRZ") == 0)
                linkcfg.dataEncoding = DAT_NRZ;      /* NRZ */
            else if (strcmp(buf, "NRZI") == 0)
                linkcfg.dataEncoding = DAT_NRZI;     /* NRZI */
            else
                {
                printf( "MTP1 Link Config: DATA_ENC [%s] unknown - defaulted to NRZ\n",
                    buf );
                }
            }

        if (getcfgval("USE_FLAGS", CFGT_BOOL, (void *) &temp, last) > 0 )
            linkcfg.flgIdle = (Bool) !temp;     /* Note reversal.  PC & task use opposite field */

        if (getcfgval("SHARE_FLAGS", CFGT_BOOL, (void *) &temp, last) > 0 )
            linkcfg.flgShare = (Bool) temp;

        GET_CHECK_INT_FROM( "MIN_FLAGS",    0,    15, linkcfg.numFlg,        U8, last, next );

        if (getcfgval("IDLE_FREQ", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.idleFreq = temp;

        if (getcfgval("RT_FREQ", CFGT_INT, (void *) &temp, last) > 0 )
            linkcfg.rtFreq = temp;

        /* generate the config request */
        status = Mtp2SetLinkCfg( board, &linkcfg );
        switch( status )
        {
            case MTP2_SUCCESS:
                break;

            case MTP2_TIMEOUT:
                printf( "MTP2 Link (%d) Cfg: No response from board %d\n", linkno, board );
                break;

            case MTP2_BOARD:
                printf( "MTP2 Link Cfg: Invalid board %d\n", board );
                break;

            case MTP2_HANDLE:
                printf( "MTP2 Link Cfg: Board %d not initialized\n", board );
                break;

            case MTP2_RESPONSE:
                printf( "MTP2 Link Cfg: Board %d returned unexpected response.  Line %d\n", board, linkcfg.spIdSE );
                break;

            default:
                Mtp2Geterror( status, &pErrstr );
                printf( "MTP2 Link Cfg: Link %d (%c%d) caused error [%s]\n",
                    linkcfg.spIdSE, linkcfg.portType == 0 ? 'S' : 'T',
                    linkcfg.portNum, pErrstr );
                break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, (void *)&temp, last);
        last = next;

    }   /* end for loop */

} /* doMtp2LinkCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    MTP2_STATUS         status;
    U16                 board = 1;                  /* default board is 1 */
    S16                 i;
    char                *p;

    printf("\n%s: sample MTP2 configuration application version %s %s\n",
            argv[0], MTP2CFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++)
    {
		p = argv[i];
        if (*p == '-')
        {
            switch (*++p)
            {
            case 'f' :     /* configuration file name */
		i++;
                if (argv[i] == NULL)
                {
                	printf( usage );
                	exit ( 1 );
                }

                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
				break;

			case 'b' :    /* Set Board number */
				i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%hu", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
				break;

            case 'd' :
                bDebug = 1;              /* i/o packet dumps */
                CDebug = 1;             /* config scanner debugging */
                break;

			default:
                printf( usage );
                exit( 1 );
                break;
			}       /* end switch */
        }
        else
        {
            printf( usage );
            exit( 1 );
		}

	}       /* end for */


    /* Initialize the mtp2 management library */
    status = Mtp2MgmtInit( (U8)board, MTP2_ENT, 0 );
    if( status != MTP2_SUCCESS )
    {
        printf( "Unable to open management library, error %d\n", status );
		exit(1);
	}

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if( initcfg( configFile ) != 0 )
    {
        /* initcfg() prints error message if it fails */
        exit( 1 );
    }

    doMtp2LinkCfg( (U8)board );

    /* allow libraries to clean up and terminate */
    endcfg();
    (void) Mtp2MgmtTerm( (U8)board );               /* ignore return code */

    exit (0);           /* Exit code success */
}
