/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifndef _MTP2CFG_H_
#define _MTP2CFG_H_

/* Request codes */
#define MTP3_GEN_CFG     1           /* general Parameters */
#define MTP3_NSAP_CFG    3           /* network SAP */
#define MTP3_DLSAP_CFG   5           /* data link SAP */
#define MTP3_ROUT_CFG    10          /* route configuration*/
#define MTP3_ROUT_DEL    30          /* delete route */
#define MTP3_LNKSET_CFG  33          /* link set configuration */

/* constants */
#define MTP3LNKTSTMAX   16      /* Max size of link test pattern */
#define MTP3MAXMASKS    8       /* Max number of routing masks */
#define MTP3MAXCMBLNK   256     /* Max number of combined link sets */

/* Selector value for MTP3 upper (user) interface */
#define MTP3_SEL_LC     0               /* loose coupling */

/* Signalling point types */
#define MTP3TYPE_SP     0       /* signalling point without transfer function */
#define MTP3TYPE_STP    1       /* signalling transfer point */

/* link types */
#define MTP3LNK_DEFAULT           0   /* link type for management message */
#define MTP3LNK_ANSI              1   /* link type ANSI */
#define MTP3LNK_CCITT             2   /* link type CCITT */
#define MTP3LNK_ANSI88            3   /* link type ANSI 88 */
#define MTP3LNK_CHINA             4   /* link type CHINA */
#define MTP3LNK_BICI              5   /* link type BICI */
#define MTP3LNK_SINGAPORE         6   /* link type SINGAPORE */

/* subservice selections */
#define MTP3SSF_INTL              0       /* international */
#define MTP3SSF_SPARE             1       /* spare */
#define MTP3SSF_NAT               2       /* national */
#define MTP3SSF_RES               3       /* reserved */

/* routing directions */
#define MTP3_RTE_DN             1   /* down / outbound route */
#define MTP3_RTE_UP             2   /* up   / inbound route  */


#define MTP3_MGMPRI               3   /* mtp 3 management priority highest */

/*
 * configuration packets
 */

/* MTP3 General timers configuration structure */

typedef struct _Mtp3GenTimerCfg
{
   TimerCfg t15;        /* t15 - waiting to start route set congestion test */
   TimerCfg t16;        /* t16 - waiting for route set congestion status update */
   TimerCfg t18;        /* t18 - waiting for links to become available */
   TimerCfg t19;        /* t19 - waiting to receive all traffic restart allowed */
   TimerCfg t20;        /* t20 - waiting to broadcast traffic restart allowed */
   TimerCfg t21;        /* t21 - waiting to restart traffic routed through adjacent SP */
   TimerCfg t26;        /* t26 - waiting to repeat traffic restart waiting message for ANSI */
   TimerCfg t29;        /* t29 - timer started when TRA sent in response to unexpected TRA or TRW */
   TimerCfg t30;        /* t30 - route instance timer */
   TimerCfg tResync;    /* internal timer for re-syncing checkpointing */
} Mtp3GenTimerCfg;

/* Mtp3 General configuration parameters structure */

typedef struct _MTP3GenCfg
{
   U32 spCode1;                 /* our DPC for CCITT version */
   U32 spCode2;                 /* our DPC for ANSI or CHINA version */
   U8 typeSP;                   /* type of signalling point */
   U8 spare1;                   /* alignment */
   Bool ssfValid;               /* ssf validation required */
   Bool rstReq;                 /* restarting procedure required */
   U16 nmbLinks;                /* number of MTP Data Links */
   U16 nmbNSap;                 /* number of Upper Layer Saps */
   U16 nmbRouts;                /* maximum number of routing entries */
   U16 nmbLnkSets;              /* number of link sets */
   U16 nmbRteInst;              /* number of simultaneous Rte instances */
   U8 nmbMasks;                 /* number of masks */
   U8 pcDispFmt;                /* point code display format */
   U32 masks[MTP3MAXMASKS];     /* route masks */
   S16 cbTimeRes;               /* time resolution */
   S16 icbTimeRes;              /* time resolution */
   S16 spTimeRes;               /* time resolution */
   S16 rteTimeRes;              /* time resolution */
   MTP3GenTimerCfg tmr;         /* general timer configuration */
   PDesc stkmgr;                /* stack manager */
   U16 memThresh;               /* memory threshold at which to go into congestion */
   U16 memThreshAbate;          /* memory threshold at which to abate congestion */
   U8 memUnit;                  /* Units in which the memory thresholds are given */
   U8 traceData;                /* trace data flag */
   U16 svqThresh;               /* SVQ Threshold at which MTP3 goes into congestion */
   U16 svqThreshAbate;          /* SVQ Threshold at which MTP3 abates congestion */
   U16 spare3;                  /* Make an even number of longs */
} MTP3GenCfg;


/* Network SAP definition structure */

typedef struct _Mtp3NSAPCfg
{
   S16 p0QLen;                  /* size of the priority 0 Q */
   S16 p1QLen;                  /* size of the priority 1 Q */
   S16 p2QLen;                  /* size of the priority 2 Q */
   S16 p3QLen;                  /* size of the priority 3 Q */
   U8 discPrior;                /* discard priority */
   U8 lnkType;                  /* link type ANSI or CCITT */
   U8 selector;                 /* upper layer selector */
   U8 spare1;                   /* alignment */
   MemoryId mem;                /* memory region and pool id */
   U8 dpcLen;                   /* dpc or opc length */
   U8 prior;                    /* priority */
   U8 route;                    /* route */
   U8 spare2;                   /* alignment */
   U32 opc;                     /* default opc for UDatReq from user layer */
} Mtp3NSAPCfg;

/* Data Link Sap Timers configuration structure */

typedef struct _Mtp3DLSapTimerCfg
{
   TimerCfg t1;                 /* t1 - delay to avoid missequencing on changeover */
   TimerCfg t2;                 /* t2 - waiting for changeover ack */
   TimerCfg t3;                 /* t3 - delay to avoid missequencing on changeback */
   TimerCfg t4;                 /* t4 - waiting for first changeback ack */
   TimerCfg t5;                 /* t5 - waiting for second changeback ack */
   TimerCfg t6;                 /* t6 - delay to avoid missequencing on rerouting */
   TimerCfg t7;                 /* t7 - waiting for link connection ack */
   TimerCfg t11;                /* t11 - transfer restricted timer */
   TimerCfg t12;                /* t12 - waiting for uninhibit ack */
   TimerCfg t13;                /* t13 - waiting for forced uninhibit */
   TimerCfg t14;                /* t14 - waiting for inhibition ack */
   TimerCfg t17;                /* t17 - delay to avoid oscillation of initial alignment failure */
   TimerCfg t22;                /* t22 - local inhibit test timer */
   TimerCfg t23;                /* t23 - remote inhibit test timer */
   TimerCfg t24;                /* t24 - stabilizing timer */
   TimerCfg t31;                /* t31 - BSN requested timer */
   TimerCfg t32;                /* t32 - SLT timer */
   TimerCfg t33;                /* t33 - connecting timer */
   TimerCfg t34;                /* t34 - periodic signalling link test timer */
   TimerCfg t40;                /* Redundancy bind timer */
   TimerCfg t41;                /* Redundancy disconnect request timer */
   TimerCfg t42;                /* Redundancy flow control request timer */
   TimerCfg t43;                /* Redundancy local processor status timer */
   TimerCfg t44;                /* Redundancy unbind timer */
} Mtp3DLSapTimerCfg;

/* Data Link SAP configuration structure */

typedef struct _Mtp3DLSapCfg
{
   U16 lnkSetId;                /* link set ID */
   U16 lnkId;                   /* signalling link identity */
   U32 adjDpc;                  /* Adlacent Destination Point Code */
   U8 lnkPrior;                 /* link priority within the link set */
   U8 usePrior;                 /* use message priority, or blast zero for all priorities */
   U16 msgSize;                 /* message length */
   U8 msgPrior;                 /* management message priority */
   U8 lnkType;                  /* link type ANSI or CCITT */
   U8 maxSLTtry;                /* maximun times to retry SLTM */
   U8 spare1;                   /* alignment */
   S16 p0QLen;                  /* size of the priority 0 Q */
   S16 p1QLen;                  /* size of the priority 1 Q */
   S16 p2QLen;                  /* size of the priority 2 Q */
   S16 p3QLen;                  /* size of the priority 3 Q */
   U8 discPrior;                /* discard priority */
   U8 ssf;                      /* sub service field */
   U8 lnkTstSLC;                /* link selection code for link test */
   U8 tstLen;                   /* link test pattern length */
   U8 tst[MTP3LNKTSTMAX];       /* link test pattern */
   MTP3LinkTimerCfg tmr;        /* timer configuration */
   U16 dstProcId;               /* destination processor id */
   U8 dstEnt;                   /* entity */
   U8 dstInst;                  /* instance */
   U8 prior;                    /* priority */
   U8 route;                    /* route */
   U8 selector;                 /* lower layer selector */
   U8 spare2;                   /* alignment */
   MemoryId mem;                /* memory region and pool id */
   U8 dpcLen;                   /* dpc or opc length */
   U8 lnkIndex;                 /* Index into link array.  0 to n-1 in order defined in config */
   S16 spId;                    /* service provider id */
   U8 dis;                      /* initial link state (enabled/disabled) JMK */
   U8 portType;                 /* port type - T1/E1/MVIP or Serial - TEK */
   U16 spare3;                  /* End structure on even 4-byte boundardy */
} Mtp3DLSapCfg;

typedef struct _Mtp3CmbLnkSet   /* MTP Level 3 combined link set cfg */
{
   U16 cmbLnkSetId;             /* combined link set ID */
   U8 lnkSetPrior;              /* link set priority */
   U8 spare1;                   /* alignment */
} Mtp3CmbLnkSet;

typedef struct _Mtp3LnkSetCfg   /* MTP Level 3 link set configuration */
{
   U16 lnkSetId;                /* link set ID */
   U16 spare1;                  /* alignment */
   U32 altOpc;                  /* alternative Origination Point Code */
   U32 adjDpc;                  /* adjacent Destination Point Code */
   Bool loadShar;               /* load sharing indication between links */
   U8 spare2;                   /* alignment */
   U16 minNmbActLnk;            /* MAXIMUM number of active links */
   U16 spare3;                  /* alignment */
   U16 nmbCmbLnkSet;            /* number of combined link sets */
   MTP3CmbLnkSet cmbLnkSet[MTP3MAXCMBLNK];   /* combined link sets */
} Mtp3LnkSetCfg;

/* Route timers structure */

typedef struct _Mtp3RteTimerCfg
{
   TimerCfg t8;                 /* t8 - transfer prohibited inhibition timer */
   TimerCfg t10;                /* t10 - waiting to repeat route set test */
   TimerCfg t21;                /* t21 - waiting to restart traffic routed through adjacent SP */
   TimerCfg t25;                /* t25 - waiting to traffic resatrt allowed message for ANSI */
} Mtp3RteTimerCfg;

typedef struct _Mtp3RoutCfg     /* MTP Level 3 routing configuration */
{
   U32 dpc;                     /* destination point code */
   U8 spType;                   /* signalling point type */
   U8 swtchType;                /* switch type */
   U16 cmbLnkSetId;             /* combined link set ID */
   Bool loadShar;               /* load sharing indication between link sets */
   U8 dir;                      /* direction */
   Bool rteToAdjSp;             /* flag indicating this route to adjacent SP */
   Bool rteToCluster;           /* flag indicating this route to a cluster */
   U8 ssf;                      /* sub service field */
   U8 spare1;                   /* Alignment */
   U16 spare2;                  /* Alignment */
   MTP3RteTimerCfg tmr;         /* route timer configuration */
} Mtp3RoutCfg;


typedef struct _Mtp3GlbStats   /* MTP Level 3 singalling point statistics */
{
   U32 usrUnavailRx;           /* user part unavailable received */
   U32 usrUnavailTx;           /* user part unavailable transmitted */
   U32 traTx;                  /* traffic restrt allowed transmitted */
   U32 traRx;                  /* traffic restrt allowed received */
   U32 trwTx;                  /* traffic restrt waiting transmitted */
   U32 trwRx;                  /* traffic restrt waiting received */
   U32 msuDropRteErr;          /* MSU dropped due to a routing data error */
} Mtp3GlbStats;

typedef struct _Mtp3DLSAPStats  /* MTP Level 3 link statistics */
{
   U32 changeOverTx;           /* changeover, order transmitted */
   U32 changeOverRx;           /* changeover, order received */
   U32 changeOverAckTx;        /* changeover, ack transmitted */
   U32 changeOverAckRx;        /* changeover, ack transmitted */
   U32 changeBackTx;           /* changeback, declaration transmitted */
   U32 changeBackRx;           /* changeback, declaration received */
   U32 changeBackAckTx;        /* changeback, ack transmitted */
   U32 changeBackAckRx;        /* changeback, ack received */
   U32 emChangeOverTx;         /* emergency changeover transmitted */
   U32 emChangeOverRx;         /* emergency changeover received */
   U32 emChangeOverAckTx;      /* emergency changeover ack transmitted */
   U32 emChangeOverAckRx;      /* emergency changeover ack received */
   U32 lnkInhTx;               /* link inhibit transmitted */
   U32 lnkInhRx;               /* link inhibit received */
   U32 lnkUninhTx;             /* link uninhibit transmitted */
   U32 lnkUninhRx;             /* link uninhibit received */
   U32 lnkInhAckTx;            /* link inhibited ack transmitted */
   U32 lnkInhAckRx;            /* link inhibited ack received */
   U32 lnkUninhAckTx;          /* link uninhibited ack transmitted */
   U32 lnkUninhAckRx;          /* link uninhibited ack received */
   U32 lnkInhDenTx;            /* link inhibit denied transmitted */
   U32 lnkInhDenRx;            /* link inhibit denied received */
   U32 lnkForceUninhTx;        /* force link uninhibit transmitted */
   U32 lnkForceUninhRx;        /* force link uninhibit received */
   U32 lnkLocInhTstTx;         /* local link inhibit test transmitted */
   U32 lnkLocInhTstRx;         /* local link inhibit test received */
   U32 lnkRmtInhTstTx;         /* remote link inhibit test transmitted */
   U32 lnkRmtInhTstRx;         /* remote link inhibit test received */
   U32 lnkConOrdTx;            /* link connection order transmitted */
   U32 lnkConOrdRx;            /* link connection order received */
   U32 lnkConAckTx;            /* link connection ack transmitted */
   U32 lnkConAckRx;            /* link connection ack received */
   U32 txCntrlRx;              /* transfer controlled received */
   U32 txCntrlTx;              /* transfer controlled transmitted */
   U32 lnkTstRx;               /* link test received */
   U32 lnkTstTx;               /* link test transmitted */
   U32 lnkTstAckRx;            /* link test ack received */
   U32 lnkTstAckTx;            /* link test ack transmitted */
   U32 txDrop;                 /* transmit messages dropped */
   U32 txCongDrop;             /* MSUs dropped due to link congestion */
   U32 sifOctTx;               /* number of SIF octets transmitted */
   U32 sifOctRx;               /* number of SIF octets received */
   U32 sioOctTx;               /* number of SIO octets transmitted */
   U32 sioOctRx;               /* number of SIO octets received */
   U32 msuTx;                  /* number of MSU transmitted */
   U32 msuRx;                  /* number of MSU received */
   U32 cong0;                  /* link congestion at threshold 0 */
   U32 cong1;                  /* link congestion at threshold 1 */
   U32 cong2;                  /* link congestion at threshold 2 */
   U32 cong3;                  /* link congestion at threshold 3 */
   U32 durLnkUnav;             /* duration of link unavailability */
   U32 durLnkCong;             /* duration of link congestion */
} Mtp3DLSAPStats;

typedef struct _Mtp3RoutStats  /* MTP Level 3 route statistics */
{
   U32 dpc;                    /* dpc */
   U32 routeTstTx;             /* route set test transmitted */
   U32 routeTstRx;             /* route set test received */
   U32 congTstRx;              /* route set congestion test received */
   U32 congTstTx;              /* route set congestion test transmitted */
   U32 txProhibTx;             /* transfer prohibited transmitted */
   U32 txProhibRx;             /* transfer prohibited received */
   U32 txRestrictRx;           /* transfer restricted received */
   U32 txAllowTx;              /* transfer allowed transmitted */
   U32 txAllowRx;              /* transfer allowed received */
   U32 rteUnavCnt;             /* route unavailable */
   U32 sifOctTx;               /* number of SIF octets transmitted */
   U32 sioOctTx;               /* number of SIO octets transmitted */
   U32 durRteUnav;             /* duration of route unavailability */
} Mtp3RoutStats;

typedef struct _Mtp3LnkSetStats  /* MTP Level 3 link set statistics */
{
   U32 strtLnkSetFail;         /* start of linkset failure */
   U32 stopLnkSetFail;         /* stop of linkset failure */
   U32 durLnkSetUnav;          /* duration of linkset unavailability */
} Mtp3LnkSetStats;



typedef struct _Mtp3SAPStatus   /* MTP Level 3 - SAP status */
{
    U8 sapState;                /* sap state */
    U8 flcSt;                   /* flow control state */
    U8 spare1;                  /* Alignment */
    U8 spare2;                  /* Alignment */
} Mtp3SAPStatus;

typedef struct _Mtp3DLSAPStatus     /* MTP Level 3 - signalling link status */
{
    U8 state;                   /* link state */
    U8 flcSt;                   /* flow control state */
    S16 tQSize;                 /* transmit queue size */
    S16 rTQSize;                /* retransmit queue size */
    Bool blkd;                  /* link is blocked */
    Bool locBlkd;               /* link is blocked locally */
    Bool rmtBlkd;               /* link is blocked remotely */
    Bool inhbt;                 /* link is inhibited */
    Bool locInhbt;              /* local inhibit indication */
    Bool rmtInhbt;              /* remote inhibit indication */
    Bool uninhbt;               /* uninhibit signalling indication */
    Bool cong;                  /* link is congested */
    Bool emerg;                 /* emergency indication */
    U8  spare1;                 /* End structures on 4-byte boundaries */
} Mtp3DLSAPStatus;

typedef struct _Mtp3LnkSetStatus      /* MTP Level 3 - link set status */
{
   Bool cfgFlg;                 /* configuration flag */
   U8 spare1;                   /* alignment */
   U16 nmbActLnks;              /* number of active links */
   U16 nmbCongLnks;             /* number of congested links */
   Bool cong;                   /* link set is congested */
   U8 state;                    /* link set state */
} Mtp3LnkSetStatus;

typedef struct _Mtp3RoutStatus        /* MTP Level 3 - route status */
{
   U32 dpc;                     /* dpc */
   U16 nmbActvLnkSets;          /* number of active link sets */
   U16 nmbCongLnkSets;          /* number of congested link sets */
   U8 state;                    /* route state */
   Bool cong;                   /* route set is congested */
   Bool rstFlg;                 /* SP restarting flag */
   U8  spare1;                  /* End structures on 4-byte boundaries */
} Mtp3RoutStatus;

typedef struct _Mtp3SPStatus          /* MTP Level 3 - signalling point status */
{
    Bool spRst;                 /* SP restarting flag */
    U8 haState;                 /* High availability state */
    U8 spare1;                  /* Alignment */
    U8 spare2;                  /* Alignment */
} Mtp3SPStatus;


/* management message */

typedef struct _Mtp3Mngmt
{
   DPRH     hdr;                  /* TXn000 DPR header */
   U16      start;                /* offset of 1st byte */
   U16      end;                  /* offset of last byte + 1 */
   union {
        Mtp3GenCfg      gencfg;     /* MTP 3 - general */
        Mtp3NSAPCfg     nsapcfg;    /* MTP 3 - transport SAP */
        Mtp3DLSapCfg    dlsapcfg;   /* MTP 3 - data link SAP */
        Mtp3LnkSetCfg   lnksetcfg;  /* MTP 3 - link set */
        Mtp3RoutCfg     routecfg;   /* MTP 3 - Routing table Entry */
   } req;
} Mtp3Mngmt;

#endif

