/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   init.c
**
**      Source code for an sccp demo program
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/


/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                    **
**                                                                      **
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
/* #include <sys/stream.h> */
#include <poll.h>
#include <string.h>
#endif                                  /* UNIX */


#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#endif                                  /* WIN32 */


#include "ss7type.h"
#include "sccpdef.h"
#include "sccpapi.h"
#include "sccpmapi.h"
#include "cfgscan.h"

#ifdef UNIX
#include "sol-dep.h"
#endif /* UNIX */

#ifdef UNIX
#define _getch getchar
#define Sleep(x) usleep(x * 1000)
#endif


DWORD InitializeSCCP (U8);
DWORD retrieveMessage(void);
DWORD TermSCCP       (void);
DWORD SendUData      (void);


U8    boardNum   = 1;
U8    sapid      = 0;
U8    ssn        = 254;
U32   loops      = 1;
U8    inloop     = 1;
U32   delay      = 5000;


CTAHD      FstCtaHd  = NULL_CTAHD;
CTAQUEUEHD FstQueue;


#ifdef UNIX
BOOL serverflag  = FALSE;
#else
BOOLEAN serverflag  = FALSE;
#endif


U8  msgdata[500];
SccpAddr cgAddr;
SccpAddr cdAddr;
U8      pcDispFmt = SS7_PCDISP_DFLT; /* point code display format, defaults to "8.8.8" (+hex) */
U8      pcLen = PCLEN_ANSI;          /* point code length, defaults to 24 bits */
U8      AddrType = SW_ANSI;  /* Addressing type: ANSI/ITU */
U8  runState;


void ErrorAndExit(CTAHD ctahd, DWORD errId, char *str);


void usage(char *pname, int opt)
{
    fprintf(stderr, "opt - %d\n"
                    "Usage: %s [options] Called Address [Calling Address]\n"
                    "\n"
                    "where the Called Address and Calling Address are specified as:\n"
                    "  <ssn>                   (requires default routing enabled)\n"
                    "  <x.y.z>:<ssn>\n"
                    "  <n>:<ssn>               (<n> is decimal or hex point code)\n"
                    "  GT<n>/<digits>          (<n> in range 1..4, digits '0'..'F')\n"
                    "  GT<n>/<digits>:<ssn>    (<n> in range 1..4, digits '0'..'F')\n"
                    "  NONE                    (i.e., the keyword 'NONE')\n"
                    "\n"
                    "options:\n"
                    "       -b number       specify board, default 1, max 8\n"
                    "       -p number       specify SAP ID, default 0, max 255\n"
                    "       -n number       specify subsytem number, default 254, max 255\n"
                    "       -i number       specify number of iterations, default 1, max 1,000,000,000\n"
                    "       -j number       specify delay between iterations(msec), default 5000, max 1,000,000\n"
                    "       -s              act as server, default act as client\n"
                    /*"       -t              use for tracing at some point\n"*/
                    "\n", opt, pname);
    exit(128);
}


/* scatScanAddr
 *
 * Scans an input address in one of the allowed forms:
 *
 *      <ssn>                   (requires default routing enabled)
 *      <x.y.z>:<ssn>
 *      <n>:<ssn>               (<n> is decimal or hex point code)
 *      GT<n>/<digits>          (<n> in range 1..4, digits '0'..'F')
 *      GT<n>/<digits>:<ssn>    (<n> in range 1..4, digits '0'..'F')
 *      NONE                    (i.e., the keyword "NONE")
 *
 * and translates it into an SCCP address structure.
 *
 * Parameters:
 *  pToken      ptr to null terminated input address string
 *  pAddr       ptr to SccpAddr structure to populate
 *
 * Return value:    0       Address translated successfully
 *                  -1      Address translation failed
 */
S16 scatScanAddr( char *pToken, SccpAddr *pAddr )
{
    U16         a1, a2, a3;
    U16         ssn = 0;
    U16         gtfmt;
    int         matchcnt;
    SCCP_STATUS status;
    U32         pc;
    U32         gltlen;
    char        gtdigits[MAX_GLT_SZ];
    int         scan_pc = 0;


    /* first, clear out address structure */
    memset( pAddr, 0, sizeof( SccpAddr ) );


    /* take the easy case first - no address */
    if( strcmp( pToken, "NONE" ) == 0 )
    {
        pAddr->presInd = NOT_PRESENT;
    }
    else if( (sscanf( pToken, "%hu.%hu.%hu:%hu", &a1, &a2, &a3, &ssn ) == 4) ||
             (sscanf( pToken, "%d:%hu", &scan_pc, &ssn ) == 2) )
    {
        /* convert point code to address based on address type (ANSI or ITU) */
        pc = (U32)scan_pc;
        if( CfgScanPc( pToken, &pc, pcDispFmt ) != 0 )
        {
            printf( "*Error: Address %s invalid for current address type\n", pToken );
            return( -1 );
        }
        else
        {
            pAddr->presInd = PRESENT;
            pAddr->pointCodeInd = PTCODE_INC;
            pAddr->pointCode = pc;
            pAddr->subsystemInd = SUBSYS_INC;
            pAddr->subsystem = (U8) ssn;
            pAddr->swType = AddrType;
            pAddr->routingInd = ROUTE_PC_SN;
            pAddr->natIntInd = ADDRIND_NAT;
        }
    }
    else if( sscanf( pToken, "%hu", &ssn ) == 1 )
    {
        pAddr->presInd = PRESENT;
        pAddr->pointCodeInd = PTCODE_NONE;
        pAddr->subsystemInd = SUBSYS_INC;
        pAddr->subsystem = (U8) ssn;
        pAddr->swType = AddrType;
        pAddr->routingInd = ROUTE_PC_SN;
        pAddr->natIntInd = ADDRIND_NAT;
    }
    else if( (matchcnt = sscanf( pToken, "GT%hu/%32[0123456789]:%hu", &gtfmt, gtdigits, &ssn )) == 2 ||
             (matchcnt == 3) )
    {
        printf( "Scanned Address: GT format %u, digits %s, ssn %u\n", gtfmt, gtdigits, ssn );  /* debug */


        /* set up address structure for passing to API */
        pAddr->presInd      = PRESENT;
        pAddr->pointCodeInd = PTCODE_NONE;
        if( matchcnt == 3 )
        {
            pAddr->subsystemInd = SUBSYS_INC;
            pAddr->subsystem = (U8) ssn;
        }
        else
        {
            pAddr->subsystemInd = SUBSYS_NONE;
        }
        pAddr->swType       = AddrType;
        pAddr->routingInd   = ROUTE_GLT;
        pAddr->glTitleInd   = (U8) gtfmt;
        pAddr->natIntInd    = ADDRIND_NAT;
        pAddr->glTransType  = 1;                /* default, hard coded for now */
        pAddr->numPlan      = NP_ISDN;          /* default, hard coded for now */
        pAddr->natAddrInd   = NATIND_SUBS;      /* default, hard coded for now */


        /* convert actual address digits to BCD - use the SCCP mgmt API for this */
        if( (status = SccpAsciiNumToBcd( gtdigits, pAddr->glTitle, MAX_GLT_SZ, &gltlen )) != SCCP_SUCCESS )
        {
            printf( "*Error (%d) converting global title (%s) to BCD\n", status, gtdigits );
            return( -1 );
        }
        pAddr->glTitleLen   = (U8) ((gltlen + 1) >> 1);    /* convert number of digits to byte length */
        if( (gltlen & 1) == 0 )
            pAddr->encoding = ENC_BCD_EVEN;
        else
            pAddr->encoding = ENC_BCD_ODD;


    }
    else
    {
        printf( "*Error: Invalid address format (%s)\n", pToken );
        return( -1 );
    }


    return( 0 );
}


int main( int argc, char *argv[] )
{
    DWORD        Ret;
    CTA_EVENT    Event;
    SccpUSapCfg  usapcfg;
    char        *p;
    int          shvar;
    short        optind, i;
    int          c;


    /* parse command line for optional parameters */
    for( optind = 1; optind < argc; optind++ )
    {
         p = argv[optind];
         if ( *p == '-' )
         {
              switch (*++p)
              {
                 case 's' :     /* server flag */
                     serverflag = TRUE;;
                     break;


                 case 'b' :    /* Set Board number */
                     if (*++p)         /* was there no space ? */
                         boardNum = (U8)atoi(p);
                     else
                         boardNum = (U8)atoi(argv[++optind]);


                     if((boardNum < 1) || (boardNum > 8) )
                          usage(argv[0], optind);


                     break;


                 case 'p' :    /* Set SAP ID */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);


                     if((shvar < 0) || (shvar > 255) )
                          usage(argv[0], optind);


                     sapid = (U8)shvar;
                     break;


                 case 'n' :    /* Set Subsytem Number */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);


                     if((shvar < 0) || (shvar > 255) )
                          usage(argv[0], optind);


                     ssn = (U8)shvar;
                     break;


                 case 'i' :    /* Set number of iterations */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);


                     if((shvar < 0) || (shvar > 1000000000) )
                          usage(argv[0], optind);


                     loops = shvar;
                     break;


                 case 'j' :    /* Set delay between iterations */
                     if (*++p)         /* was there no space ? */
                         delay = atoi(p);
                     else
                         delay = atoi(argv[++optind]);


                     if((delay < 0) || (delay > 1000000) )
                          usage(argv[0], optind);


                     break;



                 case 't' :     /* use for tracing at some point */
                     break;


                 default:
                     usage(argv[0], optind);
                     break;
              }       /* end switch */
         }
         else
         {
              /* end of optional parameters */
              break;          /* quit for loop without incrementing optind */
         }
    }         /* end for */



    /* initialize API */
    if ( (Ret = InitializeSCCP(boardNum)) != SCCP_SUCCESS )
    {
        fprintf(stderr, "Error %d initializing API\n", Ret);
        exit(130);
    }


    /* init mgmt api */
    if ( (Ret = SccpMgmtInit(boardNum, 0x25, 0)) != SCCP_SUCCESS )
    {
        fprintf(stderr, "Error %d calling SccpMgmtInit\n", Ret);
        exit(130);
    }
    /* get SAP type */
    if ( (Ret = SccpGetUSapCfg(boardNum, &usapcfg, sapid)) != SCCP_SUCCESS )
    {
        fprintf(stderr, "Error %d calling SccpGetUSapCfg\n", Ret);
        exit(130);
    }
    if (( usapcfg.swType == SCCP_SW_ITU88) || ( usapcfg.swType == SCCP_SW_ITU92))
    {
        AddrType = SW_ITU;    /* change to ITU addressing */
        pcDispFmt = SS7_PCDISP_INTL;
        pcLen = PCLEN_ITU;


    }
    else
    {
        AddrType = SW_ANSI;    /* change to ANSI addressing */
        pcDispFmt = SS7_PCDISP_DFLT;
        pcLen = PCLEN_ANSI;
    }



    if ( !serverflag )
    {
        if (optind >= argc)
        {
            /* no destination point code specified */
            printf("No Called Address specified\n");
            usage(argv[0], optind);
        }


        /* Read Called Address */
        if (scatScanAddr(argv[optind], &cdAddr) != 0)
        {
            printf("Badly formatted Called Address\n");
            usage(argv[0], optind);
        }


        ++optind;


        /* Read Calling Address */
        if (optind >= argc)
        {
            /* No Calling Address specified, use default */
            cgAddr.presInd = PRESENT;
            cgAddr.pointCodeInd = PTCODE_NONE;
            cgAddr.subsystemInd = SUBSYS_NONE;
            cgAddr.swType = AddrType;
            cgAddr.routingInd = ROUTE_PC_SN;
            cgAddr.natIntInd = ADDRIND_NAT;
        }
        else
        {
            if (scatScanAddr(argv[optind], &cgAddr) != 0)
            {
                printf("Badly formatted Calling Address\n");
                usage(argv[0], optind);
            }
        }



        /* load the test data, msgdata[] */
        for ( i = 0; i < 200; i++)
        {
            msgdata[i] = (U8)i;
        }



        while ( inloop )
        {
           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, delay );
           if ( Ret != SCCP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }
           switch ( Event.id )
           {
               case SCCPEVN_DATA:
                   /* an TCAP event has occured, call TCAPRetrieveMessage() to get the event */
                   retrieveMessage();
                   break;

               case SCCPEVN_CONGEST:
                   /* an SCCP congestion event has occured */
                   printf("Congestion event, level = %d\n",Event.value);
                   break;


               case CTAEVN_WAIT_TIMEOUT:
                   /* Delay passed, check for keyboard input */
                   /* make sure a key was hit */
                   if (_kbhit())
                   {
                       c = _getch();
                       if ( c == 'q' || c == 'Q' )
                       {
                           inloop = 0;
                           break;
                       }
                   }


                   /* we want to sent 'loops' number of requests */
                   if ( loops <= 0)
                   {
                       inloop = 0;
                       break;
                   }


                   if ( runState != SPRS_BACKUP )
                   {
                       SendUData();
                       loops--;
                   }


                   break;
           }
        }                          /* end while */


    }                              /* end if ( !serverflag ) */


    if ( serverflag )
    {
       /* if we are a server, wait for requests for 5 sec */
       while ( inloop )
       {
           if ( runState == SPRS_BACKUP )
           {
               printf("ERROR: Server State is BACKUP - no response\n");
               continue;
           }


           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
           if ( Ret != SCCP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }


           switch ( Event.id )
           {
               case SCCPEVN_DATA:
                   /* an SCCP event has occured, call SCCPRetrieveMessage() to get the event */
                   retrieveMessage();
                   break;

               case SCCPEVN_CONGEST:
                   /* an SCCP congestion event has occured */
                   printf("Congestion event, level = %d\n",Event.value);
                   break;

               case CTAEVN_WAIT_TIMEOUT:
                   /* 5 seconds passed, check for keyboard input */
                   /* make sure a key was hit */
                   if (_kbhit())
                   {
                      c = _getch();
                      if (c == 'q' || c == 'Q')
                      {
                         inloop = 0;
                         break;
                      }
                   }
                   break;
           }
       }
    }


    /* if we fall through to here, user has killed us -
     * release any active connections                    */
    TermSCCP();
}



#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;


   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;


   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }


   return (RetCode);
}
#endif
