/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   sccodemo.c
**
**      Source code for an sccp connection-oriented  demo program
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
/* #include <sys/stream.h> */
#include "sol-dep.h"
#endif                                  /* UNIX */

#ifdef WIN32
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <conio.h>
#endif                                  /* WIN32 */

#include "ss7type.h"
#include "sccpdef.h"
#include "sccpapi.h"
#include "cfgscan.h"

#ifdef UNIX
#define Sleep(x) usleep(x * 1000)
#endif

#define SUID        0x21    /* application reference number */
#define DPRCHAN     SUID    /* Dual Port RAM channel number */
#define BOARD_ZERO  0       /* our local board number       */

#define CONNIDX     1

#define SEND_CONNECT 0
#define SEND_DATA    1
#define SEND_RELEASE 2
#define WAIT_FOR_RESPONSE 3
extern U32 ConnFlag;

extern U8           boardNum;
extern U8           sapid;
extern U8           ssn;

extern CTAHD        FstCtaHd;
extern CTAQUEUEHD   FstQueue;
extern DWORD        delay;
extern U8           runState;

extern U8  msgdata[500];
extern SccpAddr cgAddr;
extern SccpAddr cdAddr;
extern U8      pcDispFmt; /* point code display format, defaults to "8.8.8" (+hex) */
extern U8      pcLen;          /* point code length, defaults to 24 bits */
extern U8      AddrType;  /* Addressing type: ANSI/ITU */

extern SccpConnId connId;

#ifdef UNIX
extern BOOL serverflag;
#else
extern BOOLEAN serverflag;
#endif


void ErrorAndExit(DWORD errId, char *str)
{
   printf( "Error : (%d) error returned in (%s) call.\n", errId, str );
   exit( 1 );
}

DWORD NMSSTDCALL ErrHdlr( CTAHD ctahd,
                          DWORD errorcode,
                          char *errtxt,
                          char *func )
{
    /* print out the error and exit */
    char text[40];
    ctaGetText(ctahd, errorcode, text, sizeof(text) );
    printf( "ErrorHandler: %s on ctahd=%#x, %s\n", errtxt, ctahd, func );
    return errorcode;
}

DWORD InitializeSCCP(U8 boardNum)
{
    CTA_EVENT        Event;
    DWORD            Ret;
    CTA_SERVICE_DESC sccpOpenSvcLst[]   = {{{"SCCP", "SCCPMGR"}, {0}, {0}, {0}}};
    CTA_SERVICE_NAME sccpServiceNames[] = {{ "SCCP", "SCCPMGR" }};
    CTA_INIT_PARMS   sccpInitparms      = { 0 };
    CTA_REV_INFO     info;

    ctaGetVersion(&info, sizeof(CTA_REV_INFO));
    if (info.compatlevel != CTA_COMPATLEVEL)
    {
        printf("CTA compatibility level does not match. re-compile.\n");
        exit(1);
    }

    /* bind this application as a user of the TCAP layer */
    sccpInitparms.size           = sizeof(CTA_INIT_PARMS);
    sccpInitparms.traceflags     = CTA_TRACE_ENABLE;
    sccpInitparms.parmflags      = CTA_PARM_MGMT_SHARED;
    sccpInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    /* ctaSetErrorHandler(ErrHdlr, NULL); */

    Ret = ctaInitialize( sccpServiceNames,
                         sizeof(sccpServiceNames) / sizeof(sccpServiceNames[0]),
                        &sccpInitparms);
    if (Ret != SCCP_SUCCESS)
    {
        /* ctdaemon is not running.
        *   Initialize without tracing.
        */
        sccpInitparms.traceflags = 0;
        sccpInitparms.parmflags  = 0;

        Ret = ctaInitialize( sccpServiceNames,
                             sizeof(sccpServiceNames) / sizeof(sccpServiceNames[0]),
                            &sccpInitparms);
        if (Ret != SUCCESS)
        {
            ErrorAndExit( Ret, "ctaInitialize" );
        }
    }

    Ret = ctaCreateQueue( NULL, 0, &FstQueue );
    if (Ret != SCCP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateQueue" );
    }

    /* Open FstCtaHd on FstQueue                   */
    Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
    if (Ret != SCCP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateContext" );
    }

    sccpOpenSvcLst[0].svcargs.args[0] = boardNum;           /* board number             */
    sccpOpenSvcLst[0].svcargs.args[1] = BOARD_ZERO;         /* srcInst                  */
    sccpOpenSvcLst[0].svcargs.args[2] = DPRCHAN + sapid;    /* srcEnt                   */
    sccpOpenSvcLst[0].svcargs.args[3] = 1;                  /* autobind                 */
    sccpOpenSvcLst[0].svcargs.args[5] = SUID;               /* suId                     */
    sccpOpenSvcLst[0].svcargs.args[4] = sapid;              /* spId                     */
    sccpOpenSvcLst[0].svcargs.args[6] = ssn;                /* ssn                      */
    sccpOpenSvcLst[0].svcargs.args[7] = 0;                  /* poolsize, use default    */

    Ret = ctaOpenServices( FstCtaHd, sccpOpenSvcLst, 1 );
    if (Ret != SCCP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete.*/
    Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
    if (Ret != SCCP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    if( Event.id == CTAEVN_OPEN_SERVICES_DONE )
    {
        if( Event.value != CTA_REASON_FINISHED )
        {
            printf("ERROR: Reason Value = %d ", Event.value);
            ErrorAndExit( Ret, "Reason value" );
        }
    }
    else
    {
        if( Event.id == CTAEVN_WAIT_TIMEOUT )
        {
            ErrorAndExit( Ret, "ctaWaitEvent: REASON = TIMEOUT " );
        }
        else
        {
            printf("ERROR: Open Services failed  id = %d ", Event.id);
            ErrorAndExit( Ret, "Open Services failed" );
        }
    }

    /* send message that subsystem is in service */
    SCCPStateRqst(FstCtaHd, sapid, ssn, SS_OOS);
    Sleep(100);
    return(SCCPStateRqst(FstCtaHd, sapid, ssn, SS_IS));
}

DWORD TermSCCP(void)
{
    CTA_EVENT  Event;
    DWORD      Ret;
    char*      sccpCloseSvcLst[] = {"SCCP"};

  /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, sccpCloseSvcLst, 1 );
   if ( Ret != SCCP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != SCCP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != SCCP_SUCCESS )
   {
        ErrorAndExit(  Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != SCCP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != SCCP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyQueue" );
   }

   return SCCP_SUCCESS;
}

DWORD SendConnectRequest(void)
{
    SccpConnRqst   *Msg;
    SccpProtoClass  qos;
    DWORD           retVal;

    printf("Sending Connection Request\n");

    if ((Msg = malloc(sizeof(SccpConnRqst))) == NULL)
        return(ENOMEM);

    memset(Msg, 0, sizeof(SccpConnRqst));

    qos.classInd = SCCP_CLASS2;  /* class 2 connection-oriented service */
    qos.msgHandling = MSG_RETURN;
    memcpy(&Msg->protoClass, &qos, sizeof(SccpProtoClass));

    memcpy(&Msg->calledPty, &cdAddr, sizeof(SccpAddr));
    memcpy(&Msg->callingPty, &cgAddr, sizeof(SccpAddr));

    connId.suId = SUID;        /* The same suId value defined during the OpenService call */
    connId.spId = sapid;       /* The same spId value defined during the OpenService call */
    connId.suConnId = CONNIDX; /* A unique id for the connection specified by the application */
    connId.spConnId  = 0;      /* The first received message contains the destination's connection ID,
                                *  which must be saved for all further messages */

    /* Data may be sent along with the Connection Request */
    Msg->data.presInd = PRESENT;
    Msg->data.dataLen = 10;
    memcpy(Msg->data.data,msgdata, 10);

    /* send transaction */
    if ((retVal = SCCPConnectRqst(FstCtaHd, &connId, Msg)) != SCCP_SUCCESS)
    {
        printf("Error on SCCPConnectRqst, err = %d\n", retVal);
    }

    /* release Msg and Parameter Buffer */
    free(Msg);

    return retVal;
}

DWORD SendConnectResponse(SccpConnId *pconnId, SccpConnRqst *Msg)
{
    DWORD           retVal;

    printf("Sending Connection Response\n");

    /* send transaction */
    if ((retVal = SCCPConnectResp(FstCtaHd, pconnId, Msg)) != SCCP_SUCCESS)
    {
        printf("Error on SCCPConnectResp, err = %d\n", retVal);
    }

    return retVal;
}

DWORD SendData(void)
{
    SccpDataRqst   *Msg;
    DWORD           retVal;

    printf("Sending Data over Connection\n");

    if ((Msg = malloc(sizeof(SccpDataRqst))) == NULL)
        return(ENOMEM);

    memset(Msg, 0, sizeof(SccpDataRqst));

    /* Data may be sent along with the Connection Request */
    Msg->data.presInd = PRESENT;
    Msg->data.dataLen = 100;
    memcpy(Msg->data.data,msgdata, 100);

    /* send transaction */
    if ((retVal = SCCPDataRqst(FstCtaHd, &connId, Msg)) != SCCP_SUCCESS)
    {
        printf("Error on SCCPDataRqst, err = %d\n", retVal);
    }

    /* release Msg and Parameter Buffer */
    free(Msg);

    return retVal;
}

DWORD SendReleaseRequest(void)
{
    SccpReleased    *Msg;
    DWORD           retVal;

    printf("Sending Release Request\n");

    if ((Msg = malloc(sizeof(SccpReleased))) == NULL)
        return(ENOMEM);

    memset(Msg, 0, sizeof(SccpReleased));


    /* Data may be sent along with the Connection Request */
    Msg->relCause = RELCSCCPUSER;
    Msg->relOrig  = ORIGENDUSER;
    Msg->rspPty.presInd = NOT_PRESENT;
    Msg->data.presInd   = NOT_PRESENT;

    /* send transaction */
    if ((retVal = SCCPReleaseRqst(FstCtaHd, &connId, Msg)) != SCCP_SUCCESS)
    {
        printf("Error on SCCPReleaseRqst, err = %d\n", retVal);
    }

    /* release Msg and Parameter Buffer */
    free(Msg);

    return retVal;
}


DWORD retrieveMessage(void)
{
    SccpAllMsgs   sccpMsg;
    SccpRcvInfoBlk infoBlk;
    DWORD         Ret;

    /* get SCCP message from API */
            if ((Ret = SCCPRetrieveMessage(FstCtaHd, &sccpMsg, &infoBlk, 0)) != SCCP_SUCCESS)
            {
               printf("SCCPRetrieveMessage failed, %d\n", Ret);
               return(SCCPERR_INTERNAL);
            }

            switch (infoBlk.indType)
            {
                case SCCPRUNSTATEIND:
                {
                    runState = infoBlk.evntType;
                    switch (runState)
                    {
                    case SPRS_BACKUP:
                         printf("RUN STATE Indication = BACKUP\n");
                         break;
                    case SPRS_PRIMARY:
                         printf("RUN STATE Indication = PRIMARY\n");
                         break;
                    case SPRS_STANDALONE:
                         printf("RUN STATE Indication = STANDALONE\n");
                         break;
                    default:
                         printf("ERROR: RUN STATE Indication = UNKNOWN\n");
                         break;
                    }
                    break;
                }

                case SCCPSTAIND:
                {
                    printf("Status Indication received, status = %d\n",
                            infoBlk.evntType);
                    break;
                }

                case SCCPCOORDIND:
                {
                SccpCoordRqst *coordRqst = &sccpMsg.coordRqst;

                printf( "SCCPCOORDIND received, SAP=%d [ssn=%d, smi = %d]\n",
                            infoBlk.suId, coordRqst->aSsn, coordRqst->smi );
                break;
                }

                case SCCPCOORDCFM:
                {
                SccpCoordRqst *coordRqst = &sccpMsg.coordRqst;

                printf( "SCCPCOORDCFM received, SAP=%d [ssn=%d, smi = %d]\n",
                            infoBlk.suId, coordRqst->aSsn, coordRqst->smi );
                break;
                }

                case SCCPPCSTIND:
                {
                SccpCoordRqst *coordRqst = &sccpMsg.coordRqst;

                switch( coordRqst->status )
                {
                    case SP_ACC:
                        printf( "*INFO: SP %s Now Accessable\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;

                    case SP_INACC:
                    case SP_INACC_NODROP:
                        printf( "*INFO: SP %s Now Inaccessable\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;

                    case SP_CONG1:
                        printf( "*INFO: SP %s Now at Congestion Level 1\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;

                    case SP_CONG2:
                        printf( "*INFO: SP %s Now at Congestion Level 2\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;

                    case SP_CONG3:
                        printf( "*INFO: SP %s Now at Congestion Level 3\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;


                    case SP_CONG_OFF:
                        printf( "*INFO: SP %s Congestion Ended\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;

                    default:
                        printf("*WARNING: Unknown PC State indication received, APC=%s, state=%d\n",
                                CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ), coordRqst->status );
                        break;
                }                                /* inner switch */
                break;
                }

                case SCCPSTATEIND:
                {
                SccpCoordRqst *coordRqst = &sccpMsg.coordRqst;

                switch( coordRqst->status )
                {
                    case SS_IS:
                        printf( "*INFO: SSN %d at PC %s Now In-Service\n",
                                coordRqst->aSsn, CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;
                    case SS_OOS:
                        printf( "*INFO: SSN %d at PC %s Now Out-of-Service\n",
                                coordRqst->aSsn, CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ) );
                        break;
                    default:
                        printf("*WARNING: Unknown Subsystem State indication received, SSN=%d, APC=%s, state=%d\n",
                                coordRqst->aSsn, CfgFmtPc( infoBlk.opc, pcLen, pcDispFmt ), coordRqst->status );
                        break;
                }                                /* inner switch */
                break;
                }

                case SCCPUDATIND:
                {
                    printf( "Unitdata Indication received, len = %d\n", sccpMsg.udataRqst.data.dataLen );
                    /*(void) validateSccpData( &sccpMsg.udataRqst.data );*/
                    break;
                }

                case SCCPCONNIND:
                {
                    printf( "Connection Indication received \n");
                    /* Copy the connection ID parameters */
                    memcpy(&connId, &infoBlk.connId, sizeof(SccpConnId));
                    /* Set the local connection ID */
                    connId.suConnId = CONNIDX;
                    /* Send SCCPConnectResp() */
                    SendConnectResponse(&connId, &sccpMsg.connRqst);

                    break;
                }

                case SCCPCONNCFM:
                {
                    printf( "Connection Confirmation received for connection %d\n",
                            infoBlk.connId.suConnId );
                    /* Save the detinations connection ID */
                    connId.spConnId = infoBlk.connId.spConnId;
                    ConnFlag = SEND_DATA;
                    break;
                }

                case SCCPRELIND:
                {
                    printf( "Release Indication received on connection %d, cause = %d\n",
                            infoBlk.connId.suConnId, sccpMsg.released.relCause );
                    break;
                }

                case SCCPDATIND:
                {
                    printf( "Data Indication received on connection %d, len = %d\n",
                            infoBlk.connId.suConnId, sccpMsg.dataRqst.data.dataLen );
                    break;
                }

                case SCCPEDATIND:
                {
                    printf( "Expedited Data Indication received on connection %d, len = %d\n",
                            infoBlk.connId.suConnId, sccpMsg.dataRqst.data.dataLen );
                     break;
                }

                case SCCPDACKIND:
                {
                    printf( "Data Acknowledgement Indication received on connection %d, len = %d\n",
                            infoBlk.connId.suConnId, sccpMsg.dataRqst.data.dataLen );
                    break;
                }

                case SCCPRESETIND:
                {
                    printf( "Reset Indication received on connection %d\n", infoBlk.connId.suConnId);
                    break;
                }

                case SCCPRESETCFM:
                {
                    printf( "Reset Confirmation received on connection %d\n", infoBlk.connId.suConnId);
                    break;
                }


                default:
                    printf("\nUnknown SCCP indication %d received\n", infoBlk.indType);
                    break;
            }
    return(SCCP_SUCCESS);
}
