/*
         Copyright (C) Dialogic Corporation 2012-2013. All Rights Reserved.

 -----  ---------  -----  ---------------------------------------------------
 Issue    Date      By                    Changes
 -----  ---------  -----  ---------------------------------------------------
        02-Jul-13   JLP   CN042TXA - add PSP options and option to turn off RC
 */

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 2008 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/*
 * m3uacfg.c
 *
 * Scans ascii text configuration file and downloads configuration to M3UA
 * task on specified board.
 */

/* header include files -- defines (.h) */
#include <sys/types.h>
/* #include <stropts.h> */
/* #include <poll.h> */
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

/*
#define IO
    #include "cpkuser.h"
    #include "dpruser.h"
#undef IO
*/

#include "ss7type.h"
#include "m3uamapi.h"
#include "cfgscan.h"
#include "m3uaver.h"

/* global variables */
short   bDebug = 0;                  /* prints each packet sent/received */

M3UAGenCfg gencfg;                  /* Make global so subsequent sections can use the info */

char configFile[256] = "M3UA.cfg";
static char *usage = "Usage: m3uacfg [ -b <board number> ] [-f <filename>]\n";
char buf[80];

/* macro to extract port type from port id (hi byte) */
#define TXPORTTYPE( portid )    ((portid >> 8) & 0xFF)

/* macro to extract port number from port id (low byte) */
#define TXPORTNUM( portid )     (portid & 0xff)

/* macro to extract port type from port id (hi byte) & convert to printable char */
#define TXASCPORTTYPE( portid ) ((((portid >> 8) & 0xFF) == PORT_TYPE_TDM) ? "T" : \
                                 (((portid >> 8) & 0xFF) == PORT_TYPE_REMOTE) ? "R" : "S")

/* Support functions */

/*----------------------------------------------
 * inet_aton
 *   Converts a dot notation IP string to a U32
 *   Returns converted address or 0 if failure.
 *---------------------------------------------*/
U32 inet_aton (char *ip)
{
	U8 ipArr[4];
	short i;
	char *pos;
	U32 ipL;

	i = 0;
	while(ip)
	{
		pos = strchr(ip,'.');
		if (!pos)
			break;

		*pos = '\0';

        ipArr[i] = (U8)atoi(ip);
		ip = pos+1;
		i++;
	}
	if( i != 3 )
		return(0); //error
    ipArr[i] =(U8) atoi(ip);
    ipL = ipArr[3] + (ipArr[2] << 8) + (ipArr[1] << 16) + (ipArr[0] << 24);
	return (ipL);
}

/*
* doM3uaGenCfg
*
* Configure general parameters for m3ua
*/
STATUS doM3uaGenCfg( U8 board )
{
   STATUS	status;
   U32		temp;

    memset(&gencfg,0,sizeof(M3UAGenCfg));
    status = M3uaInitGenCfg(&gencfg);
    if ( status != M3UA_SUCCESS )
    {
        /* Should never happen, init func always returns success */
        printf( "M3UA Gen Config: m3uaInitCfg error %d, configuration load failed\n", status );
        return( status );
    }

    /* Scan for remainder of optional parameters */
    if (getcfgval("NODE_TYPE", CFGT_STRING, (void *) buf, 0) > 0)
    {
        if (strcmp(buf, "ASP") == 0)
            gencfg.nodeType = M3UA_TYPE_ASP;
        else if (strcmp(buf, "SGP") == 0)
            gencfg.nodeType = M3UA_TYPE_SGP;
        else
        {
            printf( "M3UA Gen Config: Invalid NODE_TYPE [%s], defaulting to ASP \n", buf );
            gencfg.nodeType = M3UA_TYPE_ASP;
        }
    }

#ifdef NON_CONFIGURABLE
    /* Currently do not allow configuration of DPC_LRN_FLG.  All PC's statically configured */
    if (getcfgval("DPC_LRN_FLAG", CFGT_BOOL, (void *) &temp, 0) > 0)    /* disable/enable dpc learning */
        gencfg.dpcLrnFlag = (Bool) temp;
#endif

    if (getcfgval("MAX_NSAP", CFGT_INT, (void *) &temp, 0) > 0)         /* Max Number of NSAPs */
        gencfg.maxNmbNSap = (U16) temp;

#ifdef NON_CONFIGURABLE
    /* Currently do not allow configuration of MAX_SCT_SAP.  One SCT SAP will always suffice */
    if (getcfgval("MAX_SCT_SAP", CFGT_INT, (void *) &temp, 0) > 0)      /* MAX SCT SAPs  */
        gencfg.maxNmbSctSap = (U16) temp;
#endif

    if (getcfgval("MAX_NETWORK", CFGT_INT, (void *) &temp, 0) > 0)      /* Max number of networks */
        gencfg.maxNmbNwk = (U16) temp;

    if (getcfgval("MAX_ROUTE", CFGT_INT, (void *) &temp, 0) > 0)        /* Max number of route entries */
        gencfg.maxNmbRtEnt = (U16) temp;

    if (getcfgval("MAX_DPC", CFGT_INT, (void *) &temp, 0) > 0)          /* Max number of DPCs Supported */
        gencfg.maxNmbDpcEnt = (U16) temp;

    if (getcfgval("MAX_PS", CFGT_INT, (void *) &temp, 0) > 0)           /* Max number of peer servers  */
        gencfg.maxNmbPs = (U16) temp;

#if (defined(ITASP) && defined(OG_RTE_ON_LPS_STA))
    if (getcfgval("MAX_LPS", CFGT_INT, (void *) &temp, 0) > 0)          /* Max number of local peer servers */
        gencfg.maxNmbLps = (U16) temp;
#endif
    if (getcfgval("MAX_PSP", CFGT_INT, (void *) &temp, 0) > 0)          /* Max number of peer server process */
        gencfg.maxNmbPsp = (U8) temp;

    if (getcfgval("MAX_MSG", CFGT_INT, (void *) &temp, 0) > 0)          /* Max number of msgs in transit */
        gencfg.maxNmbMsg = (U16) temp;

    if (getcfgval("MAX_RNDRBN_LS", CFGT_INT, (void *) &temp, 0) > 0)    /* Max PS's using round-robin load sharing */
        gencfg.maxNmbRndRbnLs = (U16) temp;

    if (getcfgval("MAX_SLS_LS", CFGT_INT, (void *) &temp, 0) > 0)       /* Max PS's using SL- based load sharing */
        gencfg.maxNmbSlsLs = (U16) temp;

    if (getcfgval("MAX_SLS", CFGT_INT, (void *) &temp, 0) > 0)          /* Max total of SLS values used by all PS's */
        gencfg.maxNmbSls = (U32) temp;

#ifdef NON_CONFIGURABLE
    /* As an ASP we never accept DRKM.  But IPSP? */
    if (getcfgval("DRKM_SUPP", CFGT_BOOL, (void *) &temp, 0) > 0)       /* Dynamic Registration of Routing Key  */
        gencfg.drkmSupp = (Bool) temp;

    /* We will treat DRST as DAVA */
    if (getcfgval("DRST_SUPP", CFGT_BOOL, (void *) &temp, 0) > 0)       /* Restricted status of Route  */
        gencfg.drstSupp = (Bool) temp;
#endif

    if (getcfgval("QUEUE_SIZE", CFGT_INT, (void *) &temp, 0) > 0)       /* Outgoing congestion queue size per assoc */
        gencfg.qSize = (U32) temp;

    if (getcfgval("CONG_LEVEL1", CFGT_INT, (void *) &temp, 0) > 0)      /* Congestion level 1  */
        gencfg.congLevel1 = (U32) temp;

    if (getcfgval("CONG_LEVEL2", CFGT_INT, (void *) &temp, 0) > 0)      /* Congestion level 2  */
        gencfg.congLevel2 = (U32) temp;

    if (getcfgval("CONG_LEVEL3", CFGT_INT, (void *) &temp, 0) > 0)      /* Congestion level 3  */
        gencfg.congLevel3 = (U32) temp;

    if (getcfgval("TRACE_DATA", CFGT_BOOL, (void *) &temp, 0) > 0)      /* Data tracing on or off*/
    {
        if ((Bool) temp == FALSE)
            gencfg.traceMask = 0;
        else
            gencfg.traceMask = 0xFFFFFFFF;      /* Turn all tracing on */
    }

    if (getcfgval("DEBUG_LOG", CFGT_BOOL, (void *) &temp, 0) > 0)       /* Debug logging on or off*/
    {
        if ((Bool) temp == FALSE)
            gencfg.debugMask = 0;
        else
            gencfg.debugMask = 0xFFFFFFFF;      /* Turn all debugging on */
    }

    /* Scan for timer overrides */
    if (getcfgval("TMR_RESTART", CFGT_INT, (void *) &temp, 0) > 0)      /* Restart hold-off time */
        SET_TIMER( gencfg.tmr.tmrRestart, (U16)temp);

#ifdef NON_CONFIGURABLE
    /* SG only */
    if (getcfgval("TMR_MTP3_STA", CFGT_INT, (void *) &temp, 0) > 0)
        SET_TIMER( gencfg.tmr.tmrMtp3Sta, (U16)temp);
#endif

    if (getcfgval("TMR_AS_PEND", CFGT_INT, (void *) &temp, 0) > 0)      /* Time PS can remain in AS_PENDING state */
        SET_TIMER( gencfg.tmr.tmrAsPend, (U16)temp);

#ifdef NON_CONFIGURABLE
    /* We will not use M3UA heartbeats since SCTP has them */
    if (getcfgval("TMR_HEARTBEAT", CFGT_INT, (void *) &temp, 0) > 0)
        SET_TIMER( gencfg.tmr.tmrHeartbeat, (U16)temp);
#endif

    if (getcfgval("TMR_ASP_UP1", CFGT_INT, (void *) &temp, 0) > 0)      /* Initial time between ASPUP retries */
        SET_TIMER( gencfg.tmr.tmrAspUp1, (U16)temp);

    if (getcfgval("NMB_ASP_UP1", CFGT_INT, (void *) &temp, 0) > 0)      /* Number of initial attempts sending ASPUP */
        gencfg.tmr.nmbAspUp1 = (U8)temp;

    if (getcfgval("TMR_ASP_UP2", CFGT_INT, (void *) &temp, 0) > 0)      /* Steady-state time between ASPUP retries */
        SET_TIMER( gencfg.tmr.tmrAspUp2, (U16)temp);

    if (getcfgval("TMR_ASPDN", CFGT_INT, (void *) &temp, 0) > 0)        /* Time between ASPDN retries */
        SET_TIMER( gencfg.tmr.tmrAspDn, (U16)temp);

    if (getcfgval("TMR_ASPM", CFGT_INT, (void *) &temp, 0) > 0)         /* Time to wait after tx ASPM before failing */
        SET_TIMER( gencfg.tmr.tmrAspM, (U16)temp);

    if (getcfgval("TMR_DAUD", CFGT_INT, (void *) &temp, 0) > 0)         /* Time between DAUD msgs */
        SET_TIMER( gencfg.tmr.tmrDaud, (U16)temp);

    if (getcfgval("TMR_DRKM", CFGT_INT, (void *) &temp, 0) > 0)         /* Time between DRKM msgs */
        SET_TIMER( gencfg.tmr.tmrDrkm, (U16)temp);

    if (getcfgval("NMB_DRKM", CFGT_INT, (void *) &temp, 0) > 0)         /* Number of DRKM attempts before failing */
        gencfg.tmr.maxNmbRkTry = (U8)temp;

#ifdef NON_CONFIGURABLE
    /* SG only */
    if (getcfgval("TMR_DUNA_SETTLE", CFGT_INT, (void *) &temp, 0) > 0)
        SET_TIMER( gencfg.tmr.tmrDunaSettle, (U16)temp);
#endif

    if (getcfgval("TMR_SEQ_CNTRL", CFGT_INT, (void *) &temp, 0) > 0)     /* Delay when diverting traffic */
        SET_TIMER( gencfg.tmr.tmrSeqCntrl, (U16)temp);

    /* Generate the config request */
    status = M3uaSetGenCfg( board, &gencfg );
    switch( status )
    {
        case M3UA_SUCCESS:
            printf("M3UA GenConfig: Success\n");
            break;

        case M3UA_PARAM:
            printf( "M3UA Gen Config: Invalid parameter from board %d\n", board );
            break;

        case M3UA_TIMEOUT:
            printf( "M3UA Gen Config: No response from board %d\n", board );
            break;

        case M3UA_BOARD:
            printf( "M3UA Gen Config: Invalid board %d\n", board );
            break;

        case M3UA_HANDLE:
            printf( "M3UA Gen Config: Board %d not initialized\n", board );
            break;

        case M3UA_RESPONSE:
            printf( "M3UA Gen Config: Board %d returned unexpected response\n", board );
            break;

        default:
            printf( "M3UA Gen Config: Error %x sending config request to board %d\n", status, board );
            break;
    }

    return( status );
} /* doM3uaGenCfg */

/*
* m3uaNwkCfg
*
* M3UA Network Configuration
*/
void doM3uaNwkCfg( U8 board )
{
    M3UANwkCfg  nwkCfg;
    STATUS      status;
    S32         temp, net_id_line, next, last;
    U16         i;

    for( last=0, i=0; i<gencfg.maxNmbNwk; i++ )
    {
        if ((net_id_line = getcfgval("NETWORK_ID", CFGT_INT, &temp, last)) == 0)
            break;
        last = net_id_line;

        /* Initialize Network params block to default values */
        status = M3uaInitNwkCfg ( &nwkCfg, (U8)temp );
        if( status != M3UA_SUCCESS )
        {
            /* Should never happen  */
            printf( "M3UA Network Config:  Init() Network ID: %d failed\n", temp);
            continue;
        }

        /* Retrieve optional default overrides */

        /* May be multiple network appearances.  I don't yet understand why. */
        /* JP - Multiple NA's not implemented at this time */
        if (getcfgval("NETWORK_APPEAR", CFGT_INT, (void *) &temp, net_id_line) > 0 )
        {
           /* for (j = 0; j < M3UA_MAX_PSP; j++)
            {
                nwkCfg.nwkApp[j] = (U32)temp;
            }
	    */
            nwkCfg.nwkApp[0] = (U32)temp;

        }

        if (getcfgval("SSF", CFGT_STRING, (void *) buf, net_id_line) > 0 )
        {
            if ((strcmp(buf,"NAT")) == 0 )
                nwkCfg.ssf = M3UA_SSF_NAT;
            else if ((strcmp(buf,"INTL")) == 0 )
                nwkCfg.ssf = M3UA_SSF_INTL;
            else if ((strcmp(buf,"SPARE")) == 0 )
                nwkCfg.ssf = M3UA_SSF_SPARE;
            else if ((strcmp(buf,"RES")) == 0 )
                nwkCfg.ssf = M3UA_SSF_RES;
            else
                printf( "M3UA Network Config:  Invalid SUB_SERVICE type %s : set to default.\n", buf);
        }

        if (getcfgval("DPC_LENGTH", CFGT_INT, (void *) &temp, net_id_line) > 0)
        {
            if ( temp == 24 )
                nwkCfg.dpcLen = DPC24;
            else if ( temp == 16 )
                nwkCfg.dpcLen = DPC16;
            else if ( temp == 14 )
                nwkCfg.dpcLen = DPC14;
            else
                printf( "M3UA Network Config:  Invalid DPC_LEN type %d : set to default.\n", temp);
        }
        if (getcfgval("SLS_LENGTH", CFGT_INT, (void *) &temp, net_id_line) > 0)
            nwkCfg.slsLen = (U8)temp;

        if (getcfgval("SERVICE_USER_VAR", CFGT_STRING, (void *) buf, net_id_line) > 0 )
        {
            if ((strcmp(buf,"ANSI")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_ANS;
            else if ((strcmp(buf,"ITU")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_ITU;
            else if ((strcmp(buf,"CHINA")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_CHINA;
            else if ((strcmp(buf,"BICI")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_BICI;
            else if ((strcmp(buf,"TTC")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_TTC;
            else if ((strcmp(buf,"NTT")) == 0 )
                nwkCfg.suSwtch = M3UA_SW_NTT;
            else
            	printf( "M3UA Network Config:  Invalid SERVICE_USER_VAR type %s : set to default.\n",buf);
        }
        if (getcfgval("SERVICE_USER_VAR2", CFGT_STRING, (void *) buf, net_id_line) > 0 )
        {
            if ((strcmp(buf,"ANSI")) == 0 )
                nwkCfg.su2Swtch = M3UA_SW2_ANS;
            else if ((strcmp(buf,"ITU")) == 0 )
                nwkCfg.su2Swtch = M3UA_SW2_ITU;
            else if ((strcmp(buf,"ETSI")) == 0 )
                nwkCfg.su2Swtch = M3UA_SW2_ETS;
            else if ((strcmp(buf,"TTC")) == 0 )
                nwkCfg.su2Swtch = M3UA_SW2_TTC;
            else if ((strcmp(buf,"UNUSED")) == 0 )
                nwkCfg.su2Swtch = M3UA_SW2_UNUSED;
            else
                printf( "M3UA Network Config:  Invalid SERVICE_USER_VAR2 type %s : set to default.\n",buf);
        }
        /* Generate the config request */
        status = M3uaSetNwkCfg( board, &nwkCfg );
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA Network Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA Network Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA Network Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA Network Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA Network Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "M3UA Network Config: Error %d sending config request to board %d\n", status, board );
                break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }

} /* doM3uaNwkCfg */

/*
* doM3uaNSapCfg
*
* Configure M3UA NSAP
*/
void doM3uaNSapCfg( U8 board )
{
    M3UANSapCfg     nsapcfg;
    U16             status;
    short           i;
    S16             nsapno;
    S32             temp, last, next;

    for( last=0, i=0; i<gencfg.maxNmbNSap; i++ )
    {
        if ((next = getcfgval("NSAP_ID", CFGT_INT, &temp, last)) == 0)
            break;
        last = next;
        nsapno = (S16)temp;

        /* Initialize link params block to default values */
        status = M3uaInitNSapCfg( &nsapcfg, nsapno );
        if( status != M3UA_SUCCESS )
        {
            /* should never happen, we've already validated link type */
            printf( "M3UA NSap Config: Init NSap %d configuration failed\n", temp );
            continue;
        }

        /* Retrieve NSAP NetWork type */
        if ((getcfgval("NWK_ID", CFGT_INT, (void *) &temp, last)) > 0 )
        {
            nsapcfg.nwkId = (U8)temp;
        }

        if ((getcfgval("SERVICE_TYPE", CFGT_STRING, buf, last)) > 0 )
        {
            if ((strcmp(buf, "ISUP")) == 0)
                nsapcfg.suType = M3UA_SU_ISUP;
            else if ( (strcmp(buf, "SCCP")) == 0 )
                nsapcfg.suType = M3UA_SU_SCCP;
            else if ( (strcmp(buf, "TUP")) == 0 )
                nsapcfg.suType = M3UA_SU_TUP;
            else if ( (strcmp(buf, "BICC")) == 0 )
                nsapcfg.suType = M3UA_SU_BICC;
            else
                printf( "M3UA NSAP Config: NSAP %d Service type [%s] unknown - default used\n", nsapno, buf );
        }

        /* Generate the config request */
        status = M3uaSetNSapCfg( board, &nsapcfg );
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA NSAP Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA NSAP Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA NSAP Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA NSAP Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA NSAP Config: Board %d returned unexpected response\n", board );
                break;

            default:
                printf( "M3UA NSAP Config: Error %d sending config request to board %d\n", status, board );
            break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;

    }   /*end for loop */

}  /* doM3uaNSapCfg */

/*
*
* doM3uaSctSapCfg
*
* Configures M3UA SCT SAP Parameters
*
*/
void doM3uaSctSapCfg ( U8 board )
{
    M3UASctSapCfg   sctsapcfg;
    STATUS          status;
    S32             temp, last, next;
    U16             i;

    for( last = 0, i = 0; i < gencfg.maxNmbSctSap; i++ )
    {
        if ((next = getcfgval("SCT_SAP_ID", CFGT_INT, &temp, last)) == 0)
            break;
        last = next;

        /* initialize SCT SAP params block to default values */
        status = M3uaInitSctSapCfg ( &sctsapcfg, (S16)temp );
        if( status != M3UA_SUCCESS )
        {
            /* should never happen */
            printf( "M3UA SCT SAP Config:  Init() SAP ID : %d failed\n", temp);
            continue;
        }

        /* retrieve optional default overrides */

        if ((getcfgval("SRC_PORT", CFGT_INT, (void *) &temp, last)) > 0)
            sctsapcfg.srcPort = (U16) temp;

        if ((getcfgval("SP_ID", CFGT_INT, (void *) &temp, last)) > 0)
            sctsapcfg.spId = (S16)temp;

        status = M3uaSetSctSapCfg(board, &sctsapcfg);
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA SCT SAP Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA Network Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA Network Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA Network Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA Network Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "M3UA Network Config: Error %d sending config request to board %d\n", status, board );
                break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }

} /* doM3uaSctSapCfg */

/*
*
* doM3uaPspCfg
*
* Configures M3UA PSP Parameters
*
*/
void doM3uaPspCfg ( U8 board )
{
    M3UAPspCfg      pspcfg;
    STATUS          status;
    S32             temp, last, next;
    U32             ipAddr;
    U16             i;

    for( last = 0, i = 0; i < gencfg.maxNmbPsp; i++ )
    {
        if ((next = getcfgval("PSP_ID", CFGT_INT, &temp, last)) == 0)
            break;
        last = next;

        if ((getcfgval("PRIME_DEST_ADDR", CFGT_STRING, (void *) buf, last)) > 0)  /* Primary destination address */
        {
            ipAddr = inet_aton(buf);
        }
        else
        {
            printf("M3UA PSP Config: Mandatory PRIME_DEST_ADDR not specified \n");
            break;
        }

        /* initialize PSP params block to default values */
        status = M3uaInitPspCfg ( &pspcfg, (U16)temp,  (U32) ipAddr);
        if( status != M3UA_SUCCESS )
        {
            /* should never happen */
            printf( "M3UA PSP Config:  Init() PSP ID : %d failed !! \n", temp);
            continue;
        }

        /* retrieve optional default overrides */
        if ((getcfgval("PSP_TYPE", CFGT_STRING, (void *) buf, last)) > 0)
        {
            if ( (strcmp(buf, "ASP")) == 0 )
                pspcfg.pspType = M3UA_PSPTYPE_ASP;
            else if ( strcmp(buf, "SGP") == 0 )
                pspcfg.pspType = M3UA_PSPTYPE_SGP;
            else if ( strcmp(buf, "IPSP") == 0 )
                pspcfg.pspType = M3UA_PSPTYPE_IPSP;
            else
            {
                pspcfg.pspType = M3UA_PSPTYPE_IPSP;
                printf("M3UA PSP Config: Unknown PSP Type (%s) - Defaulted to IPSP \n", buf);
            }
        }

#ifdef NON_CONFIGURABLE
        /* SG only */
        if ((getcfgval("LOAD_SHARE_MODE", CFGT_STRING, (void *) buf, last)) > 0)
        {
            if ( (strcmp(buf, "ROUND_ROBIN")) == 0 )
                pspcfg.dfltLshareMode = M3UA_LOADSH_RNDROBIN;
            else if ( strcmp(buf, "SLS") == 0 )
                pspcfg.dfltLshareMode = M3UA_LOADSH_SLS;
            else
                printf("M3UA PSP Config: Unknown Load share mode - Defaulted \n");

        }
#endif
#ifdef SGVIEW
        if ((getcfgval("SG_ID", CFGT_INT, &temp, last)) > 0)
        {
            pspcfg.sgId = (U32)temp;
        }
#endif
        if ((getcfgval("IPSP_MODE", CFGT_STRING, (void *) buf, last)) > 0)
        {
            if ( (strcmp(buf, "SE")) == 0 )
                pspcfg.ipspMode = M3UA_IPSPMODE_SE;
            else if ( (strcmp(buf, "DE")) == 0 )
                pspcfg.ipspMode = M3UA_IPSPMODE_DE;
            else
            {
                printf("M3UA PSP Config: Unknown IPSP mode - Set to default \n");
            }
        }

        if ((getcfgval("DRKM_ALLOWED", CFGT_BOOL, (void *) &temp, 0)) > 0)       /* PSP allows dynamic registration key? */
            pspcfg.dynRegRkallwd = (Bool) temp;

        if ((getcfgval("USE_NWK_APP", CFGT_BOOL, (void *) &temp, 0)) > 0)        /* Whether to include nwk appearance in msgs */
            pspcfg.nwkAppIncl = (Bool) temp;

        if (getcfgval("ASP_ID_MAND", CFGT_STRING, (void *) buf, last) > 0)      /* Whether ASP ID mandatory for TX or RX */
        {
            if ( (strcmp(buf, "NONE")) == 0 )
                pspcfg.rxTxAspId = 0;
            else if ( (strcmp(buf, "RX")) == 0 )
                pspcfg.rxTxAspId = M3UA_RX_ASPID;
            else if ( (strcmp(buf, "TX")) == 0 )
                pspcfg.rxTxAspId = M3UA_TX_ASPID;
            else if ( (strcmp(buf, "BOTH")) == 0 )
                pspcfg.rxTxAspId = M3UA_RX_ASPID | M3UA_TX_ASPID;
            else
            {
                printf("M3UA PSP Config: Unknown ASP_ID_MAND - Set to default \n");
            }
        }

        if ((getcfgval("ASP_ID", CFGT_INT, &temp, last)) > 0)       /* Self ASP ID to use in ASPUP/ASPUP-ACK */
        {
            pspcfg.selfAspId = (U32)temp;
        }

        if ((getcfgval("NWK_ID", CFGT_INT, &temp, last)) > 0)       /* Default network context for incoming msgs */
        {
            pspcfg.nwkId = (U8)temp;
        }


#ifdef LITV10
        if ((getcfgval("RC_MAND", CFGT_BOOL, (void *) &temp, 0)) > 0)       /* Is routing context mandatory in ASPAC msgs */
            pspcfg.rcIsMand = (Bool) temp;
#endif
#ifdef NON_CONFIGURABLE
        if ((getcfgval("REREG_ALLOWED", CFGT_BOOL, (void *) &temp, 0)) > 0) /* Is DRKM re-registration allowed? */
            pspcfg.modRegRkallwd = (Bool) temp;
#endif

        if ((getcfgval("CFG_ALL_LPS", CFGT_BOOL, (void *) &temp, 0)) > 0)   /* Configure for all local peer servers? */
            pspcfg.cfgForAllLps = (Bool) temp;

        if ((getcfgval("DEST_PORT", CFGT_INT, (void *) &temp, last)) > 0)   /* Port number */
            pspcfg.assocCfg.dstPort = (U16)temp;

        if ((getcfgval("NMB_OUT_STREAMS", CFGT_INT, (void *) &temp, last)) > 0) /* Number of desired outbound streams */
            pspcfg.assocCfg.locOutStrms = (U16)temp;

        if ((getcfgval("CLIENT_SIDE", CFGT_BOOL, (void *) &temp, 0)) > 0)   /* If true, we initiate assocation when IPSP */
            pspcfg.assocCfg.clientSide = (Bool) temp;

        if ((getcfgval("DISABLED", CFGT_BOOL, (void *) &temp, 0)) > 0)   /* If true, we don't initiate assocation automatically */
            pspcfg.assocCfg.disabled = (Bool) temp;

        if ((getcfgval("TOS", CFGT_INT, &temp, last)) > 0)       /* Type-of-Service to use in outgoing messages */
        {
            pspcfg.assocCfg.tos = (U8)temp;
        }

        /* 
         * Options can be set directly and/or by specific parameters 
         * This allows for new options to be added quickly for use in the elf file 
         * without having to change the host software here to parse a new parameter
         */
        if ((getcfgval("OPTIONS", CFGT_INT, &temp, last)) > 0)   /* Options for PSP (including disable RC) */
        {
           pspcfg.options = (U32)temp;
        }
        
        /* specific options parameters */
        if ((getcfgval("RTE_CTX_DISABLE", CFGT_BOOL, (void *) &temp, 0)) > 0)   /* true/false we set/clear the appropriate options bit */
        {
          if((Bool)temp)
            pspcfg.options |= M3UA_PSP_OPT_RC_DISABLE;
          else
            pspcfg.options &= ~M3UA_PSP_OPT_RC_DISABLE;
        }




#ifdef NON_CONFIGURABLE
        /* We don't allow configuration of a dest address list.  We'll use what comes in the INIT/INIT-ACK for the assoc */
        numAddr = 0;

        if ((getcfgval("ASSOC_ADDR_LIST", CFGT_STRING, (void *) buf, last)) > 0 )
        {
           pspcfg.assocCfg.dstAddrLst.nmb = 1;
           pspcfg.assocCfg.dstAddrLst.nAddr[numAddr].type = CM_NETADDR_IPV4;
           pspcfg.assocCfg.dstAddrLst.nAddr[numAddr].u.ipv4NetAddr = inet_aton(buf);
        }
        else
            printf("M3UA PSP Config: Associated Address list is Empty\n");
#endif

        status = M3uaSetPspCfg(board, &pspcfg);
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA PSP Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA PSP Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA PSP Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA PSP Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA PSP Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "M3UA PSP Config: Error %d sending config request to board %d\n", status, board );
                break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }

} /* doM3uaPspCfg */

/*
*
* doM3uaPsCfg
*
* Configures M3UA PS Parameters
*
*/
void doM3uaPsCfg ( U8 board )
{
    M3UAPsCfg   pscfg;
    STATUS      status;
    S32         temp, last, next, nextPsp;
    U16         nmbPsp, i, j;

    for( last = 0, i = 0; i < gencfg.maxNmbPs; i++ )
    {
        nmbPsp = 0;

        if ((next = getcfgval("PS_ID", CFGT_INT, &temp, last)) == 0)        /* Peer server ID */
            break;
        last = next;
        /* initialize PS params block to default values */
        status = M3uaInitPsCfg ( &pscfg, (U32)temp );
        if( status != M3UA_SUCCESS )
        {
            /* should never happen */
            printf( "M3UA PS Config:  Init() PS ID : %d failed !! \n", temp);
            continue;
        }

        if ((getcfgval("NWK_ID", CFGT_INT, &temp, last)) > 0)           /* Network ID */
            pscfg.nwkId  = (U8)temp;

        if ((getcfgval("RTE_CTX", CFGT_INT, &temp, last)) > 0)          /* Route context passed in ASPAC messages */
            pscfg.routCtx  = (U32)temp;

        if (getcfgval("LOCAL", CFGT_BOOL, &temp, last) > 0)             /* Is this the local AS? */
            pscfg.lclFlag = (Bool) temp;

        if ((getcfgval("MODE", CFGT_STRING, buf, last)) > 0)            /* Active/Standby or load sharing */
        {
            if ( strcmp(buf, "ACTSTANDBY") == 0 )
                pscfg.mode = M3UA_MODE_ACTSTANDBY;
            else if ( strcmp(buf, "LOADSHARE") == 0 )
                pscfg.mode = M3UA_MODE_LOADSHARE;
            else if ( strcmp(buf, "BROADCAST") == 0 )
                pscfg.mode = M3UA_MODE_BROADCAST;
            else
                printf("M3UA PS Config: Unknown peer server mode (%s) - Defaulted\n", buf);
        }

        if ((getcfgval("LOAD_SHARE_TYPE", CFGT_STRING, buf, last)) > 0) /* Round robin or SLS (remote PS only) */
        {
            if ( strcmp(buf, "ROUND_ROBIN") == 0 )
                pscfg.loadShareMode = M3UA_LOADSH_RNDROBIN;
            else if ( strcmp(buf, "SLS") == 0 )
                pscfg.loadShareMode = M3UA_LOADSH_SLS;
            else
                printf("M3UA PS Config: Unknown load share type (%s) - Defaulted\n", buf);
        }

        for (j = 0; j < gencfg.maxNmbPsp; j++)
        {
            if ((nextPsp = (getcfgval("PSP", CFGT_INT, &temp, last))) > 0)          /* List of PSP's */
            {
                pscfg.psp[j] = (U16)temp;
                nmbPsp++;
                last = nextPsp;
            }
        }

        pscfg.nmbPsp = nmbPsp;                                          /* Set number of PSPs making up this PS */
        status = M3uaSetPsCfg(board, &pscfg);
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA Peer Server Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA PS Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA PS Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA PS Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA PS Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "M3UA PS Config: Error %d sending config request to board %d\n", status, board );
                break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }
} /* doM3uaPsCfg */

/*
* doM3uaRouteCfg
*
* Configures Routing Table
*/
void doM3uaRouteCfg( U8 board )
{
    M3UARteCfg      rtecfg;
    STATUS          status;
    S32             temp, last, next;
    U32             pc;
    U16             i;

    for( last = 0, i = 0; i < gencfg.maxNmbRtEnt; i++ )
    {
        if ((next = getcfgval("DPC", CFGT_PC, &pc, last)) == 0)
            break;
        last = next;

        M3uaInitRteCfg(&rtecfg, (U32)pc);
        /* override defaults */
        if ((getcfgval("RTE_TYPE", CFGT_STRING, buf, last)) > 0)
        {
             if ( strcmp(buf, "LOCAL") == 0 )
                rtecfg.rtType = M3UA_RTTYPE_LOCAL;
             else if ( strcmp(buf, "PS") == 0 )
                rtecfg.rtType = M3UA_RTTYPE_PS;
             else
                printf("M3UA Route Config: Unknown rtType %s - Defaulted\n", buf);
        }

        if ((getcfgval("NWK_ID", CFGT_INT, &temp, last)) > 0)
            rtecfg.nwkId  = (U8)temp;

        if ((getcfgval("RTE_PS_ID", CFGT_INT, &temp, last)) > 0)        /* Only used for route type PS */
            rtecfg.psId = temp;

        if ((getcfgval("NSAP_ID", CFGT_INT, &temp, last)) > 0)
        {
            rtecfg.nSapIdPres = TRUE;
            rtecfg.nSapId = (S16)temp;
        }
        /* Get routing filter overrides */
        if ((getcfgval("DPC_MASK", CFGT_PC, &pc, last)) > 0)
            rtecfg.rtFilter.dpcMask = pc;

        if ((getcfgval("OPC", CFGT_PC, &pc, last)) > 0)
            rtecfg.rtFilter.opc = pc;

        if ((getcfgval("OPC_MASK", CFGT_PC, &pc, last)) > 0)
            rtecfg.rtFilter.opcMask = pc;

        if ((getcfgval("SIO", CFGT_STRING, buf, last)) > 0 )
        {
            if ((strcmp(buf, "ISUP")) == 0)
                rtecfg.rtFilter.sio = M3UA_SU_ISUP;
            else if ( (strcmp(buf, "SCCP")) == 0 )
                rtecfg.rtFilter.sio = M3UA_SU_SCCP;
            else if ( (strcmp(buf, "TUP")) == 0 )
                rtecfg.rtFilter.sio = M3UA_SU_TUP;
            else if ( (strcmp(buf, "BICC")) == 0 )
                rtecfg.rtFilter.sio = M3UA_SU_BICC;
            else
            {
                /* Accept the above strings or a number between 0 and 0xF */
                if ((getcfgval("SIO", CFGT_INT, &temp, last)) > 0)
                {
                    rtecfg.rtFilter.sio = (U8)temp;
                }
                else
                {
                    printf( "M3UA Route Config: SIO type [%s] unknown - no SIO used\n", buf );
                    rtecfg.rtFilter.sio = 0;
                }
            }
        }

        if ((getcfgval("SIO_MASK", CFGT_INT, &temp, last)) > 0)
            rtecfg.rtFilter.sioMask = (U8)temp;

        status = M3uaSetRteCfg(board, &rtecfg);
        switch( status )
        {
            case M3UA_SUCCESS:
                printf("M3UA Route Config: Success\n");
                break;

            case M3UA_TIMEOUT:
                printf( "M3UA Route Config: No response from board %d\n", board );
                break;

            case M3UA_BOARD:
                printf( "M3UA Route Config: Invalid board %d\n", board );
                break;

            case M3UA_HANDLE:
                printf( "M3UA Route Config: Board %d not initialized\n", board );
                break;

            case M3UA_RESPONSE:
                printf( "M3UA Route Config: Board %d returned unexpected response.\n", board );
                break;

            default:
                printf( "M3UA Route Config: Error %d sending config request to board %d\n", status, board );
                break;
        }

        next=getcfgval("END",CFGT_INT,&temp,last);
        last = next;
    }   /* end for loop */
} /* doMtp3RouteCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    unsigned short      board = 1;                  /* default board is 1 */
    STATUS         status;
    int                 i;
    char                *p;

    printf("\n%s: sample M3UA configuration application version %s %s\n",
            argv[0], M3UACFG_VERSION, __DATE__);
    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++)
    {
		p = argv[i];
        if (*p == '-')
        {
            switch (*++p)
            {
            case 'f' :     /* configuration file name */
				i++;

                if (argv[i] == NULL)
                {
                  printf( usage );
                  exit ( 1 );
                }
                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
				break;

			case 'b' :    /* Set Board number */
				i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%hu", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
				break;

            case 'd' :
                bDebug = 1;              /* i/o packet dumps */
                CDebug = 1;             /* config scanner debugging */
                break;

			default:
                printf( usage );
                exit( 1 );
                break;
			}       /* end switch */
        }
        else
        {
            printf( usage );
            exit( 1 );
		}

	}       /* end for */

    /* Initialize the m3ua management library */
    status = M3uaMgmtInit( (U8)board, ENTSM, 0 );
    if( status != M3UA_SUCCESS )
    {
        printf( "Unable to open management library, error %d\n", status );
        exit(1);
    }

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if( initcfg( configFile ) != 0 )
    {
        /* initcfg() prints error message if it fails */
        exit( 1 );
    }

    /* do the m3ua configuration */
    status = doM3uaGenCfg( (U8)board );
    if ( status != M3UA_SUCCESS )
    {
        /* gen config failure is fatal; all others non-fatal */
        endcfg();
        (void) M3uaMgmtTerm( (U8)board );
        printf("General configuration failed (%d).  Terminate mgmt API and exit!\n", status);
        exit( 1 );
    }

    doM3uaNwkCfg( (U8)board );
    doM3uaSctSapCfg( (U8)board );
    doM3uaNSapCfg( (U8)board );
    doM3uaPspCfg( (U8)board );
    doM3uaPsCfg( (U8)board );
    doM3uaRouteCfg( (U8)board );

    /* allow libraries to clean up and terminate */
    endcfg();
    (void) M3uaMgmtTerm( (U8)board );               /* ignore return code */
    exit (0);           /* Exit code success */
}
