/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1996 - 2014 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/*
 * sctpcfg.c
 *
 * Scans ascii text configuration file and downloads configuration to SCTP
 * task on specified board.
 */

/* header include files -- defines (.h) */

#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "ss7type.h"
#include "sctpmapi.h"
#include "cfgscan.h"
#include "sigerr.h"
#include "sctpver.h"

/* global variables */
U8      myInst;
U8      myEntId;
short   bDebug = 0;                  /* prints each packet sent/received */

U16 gMaxSctSap = 1;                  /* Make global so subsequent sections can use the info
                                      * We do not let user configure this value always 1*/
U16 gMaxTsap   = 1;

char configFile[256] = "SCTP.cfg";
static char *usage = "Usage: sctpcfg [ -b <board number> ] [-f <filename>]\n";
char buf[80];

/*
* doSctpGenCfg
*
* Configures general parameters for SCTP
*/
SCTP_STATUS doSctpGenCfg( U8 board )
{
    SCTP_STATUS status;
    U32         temp;
    SCTPGenCfg  genCfg;

    memset(&genCfg, 0, sizeof(SCTPGenCfg));
    if ( SctpInitGenCfg(&genCfg) != SCTP_SUCCESS )
    {
        /* should never hit here*/
        return(SCTP_PARAM);
    }

    if ((getcfgval("MAX_ASSOC", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbAssoc = (U16) temp;

    if ((getcfgval("MAX_DEST_ADDRS", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbDstAddr = (U16) temp;

    if ((getcfgval("MAX_TX_QUEUE", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbTxChunks = (U32) temp;

    if ((getcfgval("MAX_RX_QUEUE", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbRxChunks = (U32) temp;

    if ((getcfgval("MAX_INSTREAM", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbInStrms = (U16) temp;

    if ((getcfgval("MAX_OUTSTREAM", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.maxNmbOutStrms = (U16) temp;

    if ((getcfgval("MTU_INITIAL", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.mtuInitial = (U16) temp;

    if ((getcfgval("MTU_MIN", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.mtuMinInitial = (U16) temp;

    if ((getcfgval("MTU_MAX", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.mtuMaxInitial = (U16) temp;

    if ((getcfgval("PERFORM_MTU", CFGT_BOOL, (void *) &temp, 0)) > 0)
        genCfg.performMtu = (Bool) temp;

    if ((getcfgval("USE_HOSTNAME", CFGT_BOOL, (void *) &temp, 0)) > 0)
        genCfg.useHstName = (Bool) temp;

    if ((getcfgval("HOSTNAME", CFGT_STRING, (void *) buf, 0)) > 0)
        strcpy((char *)genCfg.hostname,(char *) buf);

    if (getcfgval("TRACE_DATA", CFGT_BOOL, (void *) &temp, 0) > 0)      /* Data tracing on or off*/
    {
        if ((Bool) temp == FALSE)
            genCfg.traceMask = 0;
        else
            genCfg.traceMask = 0xFFFFFFFF;      /* Turn all tracing on */
    }

    if (getcfgval("DEBUG_LOG", CFGT_BOOL, (void *) &temp, 0) > 0)       /* Debug logging on or off*/
    {
        if ((Bool) temp == FALSE)
            genCfg.debugMask = 0;
        else
            genCfg.debugMask = 0xFFFFFFFF;      /* Turn all debugging on */
    }

    if ((getcfgval("MAX_INIT_RETRY", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.maxInitReTx = (U8) temp;

    if ((getcfgval("MAX_ASSOC_RETRY", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.maxAssocReTx = (U8) temp;

    if ((getcfgval("MAX_DEST_RETRY", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.maxPathReTx = (U8) temp;

    if ((getcfgval("ACCEPT_ALT", CFGT_BOOL, (void *) &temp, 0)) > 0)
        genCfg.reConfig.altAcceptFlg = (Bool) temp;

    if ((getcfgval("TMR_MD5_KEY", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.keyTm = (U16) temp;

    if ((getcfgval("RTT_ALPHA", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.alpha = (U16) temp;

    if ((getcfgval("RTT_BETA", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.reConfig.beta= (U16) temp;

    if ((getcfgval("RX_WINDOW_SIZE", CFGT_INT, (void *) &temp, 0)) > 0)
        genCfg.initARwnd = (U32) temp;

    status =  SctpSetGenCfg(board, &genCfg);
    switch( status )
    {
        case SCTP_SUCCESS:
            printf("SCTP Gen Config: Success\n");
            break;

        case SCTP_TIMEOUT:
            printf( "SCTP Gen Config: No response from board %d\n", board );
            break;

        case SCTP_BOARD:
            printf( "SCTP Gen Config: Invalid board %d\n", board );
            break;

        case SCTP_HANDLE:
            printf( "SCTP Gen Config: Board %d not initialized\n", board );
            break;

        case SCTP_RESPONSE:
            printf( "SCTP Gen Config: Board %d returned unexpected response.  \n", board);
            break;

        default:
            printf( "SCTP Gen Config: Board %d returned Failure. Unknown Error (0x%X) \n", board, status);
            break;
    }
    return(status);
} /* doSctpGenCfg */

/*
* doSctpSctSapCfg
*
* Configures General parameters for SCTP
*/
SCTP_STATUS doSctpSctSapCfg( U8 board )
{
    SCTPSctSapCfg   sctSapCfg;
    S32             temp, next, last;
    SCTP_STATUS     status = 0;
    U16             i;

    last = 0;
    for ( i = 0; i < gMaxSctSap; i++ )
    {
        if ((next = getcfgval("SCT_SAP_ID", CFGT_INT, (void *)&temp, last)) == 0)
            break;
        last = next;

        if ( SctpInitSctSapCfg(&sctSapCfg, (U16) temp) != SCTP_SUCCESS )
        {
            /* should never hit here*/
            return(SCTP_PARAM);
        }

        /* get values from config file. overwrite defaults */
        if ((getcfgval("TMR_ACK_DELAY", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.maxAckDelayTm = (U16) temp;

        if ((getcfgval("NMB_ACK_DELAY", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.maxAckDelayDg = (U16) temp;

        if ((getcfgval("TMR_INIT_RTO", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.rtoInitial = (U16) temp;

        if ((getcfgval("TMR_MIN_RTO", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.rtoMin = (U16) temp;

        if ((getcfgval("TMR_MAX_RTO", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.rtoMax = (U16) temp;
#ifdef NON_CONFIGURABLE
        if ((getcfgval("TMR_FREEZE", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.freezeTm = (U16) temp;
#endif
        if ((getcfgval("TMR_BUNDLE", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.bundleTm = (U16) temp;

        if ((getcfgval("TMR_COOKIE_LIFE", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.cookieLife = (U16) temp;

        if ((getcfgval("TMR_HB_INTERVAL", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.intervalTm = (U16) temp;
#ifdef LSB8
        if ((getcfgval("MAX_BURST", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.maxBurst = (U16) temp;

        if ((getcfgval("MAX_HB_BURST", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.maxHbBurst = (U16) temp;

        if ((getcfgval("TMR_SD_GUARD", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.t5SdownGrdTm = (U16) temp;
#endif
#ifdef NON_CONFIGURABLE
        if ((getcfgval("SCTP_HANDLE_INIT", CFGT_BOOL, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.handleInitFlg = (Bool) temp;
#endif
        if ((getcfgval("ABORT_ON_STREAM", CFGT_BOOL, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.negAbrtFlg = (Bool) temp;

        if ((getcfgval("ENABLE_HEARTBEAT", CFGT_BOOL, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.hBeatEnable = (Bool) temp;

        if ((getcfgval("FLOW_START_THRESH", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.flcUpThr = (U32) temp;

        if ((getcfgval("FLOW_STOP_THRESH", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.flcLowThr = (U32) temp;

#ifdef NON_CONFIGURABLE
#ifdef SB_ETSI
        if ((getcfgval("MAX_DATA_SIZE", CFGT_INT, (void *) &temp, last)) > 0)
            sctSapCfg.reConfig.maxDataSize = (U16) temp;
#endif
#endif
        status =  SctpSetSctSapCfg(board, &sctSapCfg);
        switch( status )
        {
            case SCTP_SUCCESS:
                printf( "SCTP SCT SAP Config: Success\n");
                break;

            case SCTP_TIMEOUT:
                printf( "SCTP SCT SAP Config: No response from board %d\n", board );
                break;

            case SCTP_BOARD:
                printf( "SCTP SCT SAP Config: Invalid board %d\n", board );
                break;

            case SCTP_HANDLE:
                printf( "SCTP SCT SAP Config: Board %d not initialized\n", board );
                break;

            case SCTP_RESPONSE:
                printf( "SCTP SCT SAP Config: Board %d returned unexpected response.  \n", board);
                break;

            default:
               printf( "SCTP SCT SAP Config: Board %d returned Failure. Unknown Error (%d) \n", board, status);
               break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }
    return(status);
} /* doSctpSctSapCfg */

/*
* Uitility function to convert str ip to long
* iptol(char *ip)
* Params: ip IP address in string format
*          ipL returned as long.
*/
U32 iptol(char *ip)
{
    int     ipArr[4];
    short   i;
    char   *pos;
    U32     ipL;

    i = 0;
    while(ip)
    {
        pos = strchr(ip,'.');
        if (!pos)
            break;

        *pos = '\0';

        ipArr[i] = atoi(ip);
        ip = pos+1;
        i++;
    }
    if( i != 3 )
        return(0); //error
    ipArr[i] = atoi(ip);
    ipL = ipArr[3] + (ipArr[2] << 8) + (ipArr[1] << 16) + (ipArr[0] << 24);
    return (ipL);
}/* iptol */

/*
* doSctpTSapCfg
*
* Configures General Config. for SCTP
*/
SCTP_STATUS doSctpTSapCfg( U8 board )
{
    SCTPTSapCfg    tSapCfg;
    SCTP_STATUS status = 0;
    S32         temp, next, last;
    U16         i, tsapId;

    last = 0;
    for(i = 0; i < gMaxTsap; i++)
    {
        if ((next = getcfgval("TSAP_ID", CFGT_INT, (void *)&temp, last)) == 0)
            break;
        last = next;
        tsapId = (U16) temp;

        if ( SCTP_SUCCESS != SctpInitTSapCfg(&tSapCfg, tsapId))
        {
            /* should never hit here*/
            return(SCTP_PARAM);
        }

        /* get values from config file. overwrite defaults */
        if (getcfgval("SAP_ID", CFGT_INT, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.spId = (S16) temp;

        if (getcfgval("MAX_BIND_RETRY", CFGT_INT, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.maxBndRetry = (U8) temp;

        if (getcfgval("TMR_BIND_CFM", CFGT_INT, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.tIntTmr = (U16) temp;

#ifdef NON_CONFIGURABLE
        if (getcfgval("USE_DNS", CFGT_BOOL, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.sbDnsCfg.useDnsLib = (Bool) temp;

        if (getcfgval("DNS_ADDR", CFGT_STRING, (void *) &buf, 0) > 0)
        {
            ipL = iptol((char *)buf);
            if ( ipL > 0 )
                tSapCfg.reConfig.sbDnsCfg.dnsAddr.u.ipv4TptAddr.address = ipL;
            else
                printf("Invalid IP for DNS_ADDR");
        }
        if (getcfgval("TMR_DNS_TIMEOUT", CFGT_INT, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.sbDnsCfg.dnsTmOut = (U16) temp;

        if (getcfgval("MAX_DNS_RETRY", CFGT_INT, (void *) &temp, 0) > 0)
            tSapCfg.reConfig.sbDnsCfg.maxRtxCnt = (U8) temp;
#endif

        status =  SctpSetTSapCfg(board, &tSapCfg);
        switch( status )
        {
            case SCTP_SUCCESS:
                printf("SCTP TSAP Config: Success\n");
                break;

            case SCTP_TIMEOUT:
                printf( "SCTP TSAP Config: No response from board %d\n", board );
                break;

            case SCTP_BOARD:
                printf( "SCTP TSAP Config: Invalid board %d\n", board );
                break;

            case SCTP_HANDLE:
                printf( "SCTP TSAP Config: Board %d not initialized\n", board );
                break;

            case SCTP_RESPONSE:
                printf( "SCTP TSAP Config: Board %d returned unexpected response.  \n", board);
                break;

            default:
               printf( "SCTP TSAP Config: Board %d returned Failure. Unknown Error\n", board);
               break;
        }
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    } /* for loop */
    return(status);
} /* doSctpTSapCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    SCTP_STATUS         status;
    U16                 board = 1;                  /* default board is 1 */
    S16                 i;
    char                *p;

    printf("\n%s: sample SCTP configuration application version %s %s\n",
            argv[0], SCTPCFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++)
    {
        p = argv[i];
        if (*p == '-')
        {
            switch (*++p)
            {
            case 'f' :     /* configuration file name */
                i++;
                if (argv[i] == NULL)
                {
                    printf( usage );
                    exit ( 1 );
                }

                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
                break;

            case 'b' :    /* Set Board number */
                i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%hu", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
                break;

            case 'd' :
                bDebug = 1;              /* i/o packet dumps */
                CDebug = 1;             /* config scanner debugging */
                break;

            default:
                printf( usage );
                exit( 1 );
                break;
            }       /* end switch */
        }
        else
        {
            printf( usage );
            exit( 1 );
        }

    }       /* end for */


    /* Initialize the  management library */
    status = SctpMgmtInit( (U8)board, ENTSM, 0 );
    if( status != SCTP_SUCCESS )
    {
        printf( "Unable to open management library, error %d\n", status );
        exit(1);
    }

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if( initcfg( configFile ) != 0 )
    {
        /* initcfg() prints error message if it fails */
        exit( 1 );
    }
    status = doSctpGenCfg((U8)board);
    if (status != SCTP_SUCCESS)
    {
        (void) SctpMgmtTerm( (U8)board );
        exit( 1 );
    }
    status = doSctpSctSapCfg((U8) board);
    status = doSctpTSapCfg((U8) board);

    /* allow libraries to clean up and terminate */
    endcfg();
    (void) SctpMgmtTerm( (U8)board );               /* ignore return code */

    exit (0);           /* Exit code success */
}
