/* SCTP management test application
 *
 * Sample SCTP management application; sends commands to SCTP task on Tx series
 * processor and waits for response (status and optional data, such as entity
 * status or statistics).
 *
 * To run:
 *
 * sctpmgr  [-b <board>]
 *
 *      <board>     Optional Board number to open (defaults to 1)
 *
 * User Commands:
 *    TODO
 *      BOARD   <board number>          ! attach to new board
 *      Q                               ! quit
 */


#include <stdio.h>
#include <ctype.h>
#ifdef UNIX
#define getch	getchar
#else
#include <conio.h>          /* For getch() call */
#endif
#include <string.h>
#include <stdlib.h>
#include "ss7type.h"
#include "osdep.h"
#include "sctpmapi.h"
#include "sigerr.h"
#include "cfgscan.h"                    /* for point code display and scanning utilities */
#include "sctpver.h"

 /* defines */
#define MAX_PARMS   10
#define CMD_HELP    0
#define CMD_STATS   1
#define CMD_STATUS  2
#define CMD_BOARD   3
#define CMD_TSAP    4
#define CMD_DEBUG   5
#define CMD_TRACE   6

#define SCTP_STS    10
#define SCTSAP_STS  11
#define TSAP_STS    12

#define SYS_STA     20
#define SAP_STA     21
#define GEN_STA     22
#define ASSOC_STA   23
#define DEST_TX_STA 24

#define CMD_QUIT    99
#define UNKNOWN     255


struct cmdTbl {
    char *cmd;
    U16   val;
};
struct cmdTbl cmdList[] = {
    {"?", CMD_HELP},
    {"STATS", CMD_STATS},
    {"STATUS",CMD_STATUS},
    {"BOARD", CMD_BOARD},
    {"TSAP",  CMD_TSAP},
    {"DEBUG", CMD_DEBUG},
    {"TRACE", CMD_TRACE},
    {"Q",  CMD_QUIT},
    {"UNKNOWN", UNKNOWN}
};
struct cmdTbl stsList[] = {
    {"SCTP", SCTP_STS},
    {"SCTSAP", SCTSAP_STS},
    {"TSAP", TSAP_STS},
    {"UNKNOWN", UNKNOWN}
};
struct cmdTbl staList[] = {
    {"SYS", SYS_STA},
    {"SAP", SAP_STA},
    {"SCTP", GEN_STA},
    {"ASSOC", ASSOC_STA},
    {"DESTTXADDR", DEST_TX_STA},
    {"UNKNOWN", UNKNOWN}
};

struct cmdTbl tsapCtrlList[] = {
    {"ENA", SCTP_CTRL_TSAP_ENA},
    {"DIS", SCTP_CTRL_TSAP_DIS},
    {"UNKNOWN", UNKNOWN}
};

struct cmdTbl debugCtrlList[] = {
    {"ENA", SCTP_CTRL_DEBUG_ON},
    {"DIS", SCTP_CTRL_DEBUG_OFF},
    {"UNKNOWN", UNKNOWN}
};

struct cmdTbl traceCtrlList[] = {
    {"ENA", SCTP_CTRL_TRACE_ON},
    {"DIS", SCTP_CTRL_TRACE_OFF},
    {"UNKNOWN", UNKNOWN}
};

struct cmdTbl err2Str[] = {
    {"Invalid Parameter", SCTP_PARAM},
	{"Invalid Board", SCTP_BOARD},
	{"Driver Error",SCTP_DRIVER},
	{"Uninitialized Board",SCTP_HANDLE},
	{"Request Timeout",SCTP_TIMEOUT},
	{"Invalid Response",SCTP_RESPONSE},
    {"Invalid SAP Id",12},
	{"Invalid Parameter value", 13},
	{"Unknown Error", UNKNOWN}
};

char *assocStateStr[] = {"CLOSED", "OPEN", "COOKIE_WAIT", "ESTABLISHED", "COOKIE_SEND", "SHUTDOWN_PEND","SHUTDOWN_SENT","SHUTDOWN_RCVD"};

short cp = 1;
Bool bDebug = FALSE;
Bool forever = TRUE;
void sctpCtrl( char *parms );
void statusReq( char *parms[] );
void statsReq( char *parms[] );
void tsapCtrl (char *parms[]);
void HelpCommand( char *parms[]);
void setCP( char *parms[] );
void quit();
char * getErrStr(S16 err);
void doGenStats(char *parms[]);
void doSctSapStats(char *parms[]);
void doTsapStats(char * parms[]);
void doSapStatus(char * parms[]);
void doGenStatus(char * parms[]);
void doAssocStatus(char * parms[]);
void doDestTransAddrStatus(char * parms[]);

/* u32ToIp
 *
 * This function converts a U32 into an IP dot notation string and returns it
 */
void u32ToIp (U32 addr, char* ipStr)
{
    sprintf (ipStr, "%d.%d.%d.%d", (addr & 0xFF000000) >> 24, (addr & 0x00FF0000) >> 16,
                                 (addr & 0x0000FF00) >> 8, (addr & 0x000000FF));
}

/* Display Usage help */
void usage( char *pName )
{
    printf("\n usage: %s [ -b <board>] \n", pName );
    printf("\n\t<board>\tboard number to attach to [default 1]\n");
    exit(1);
}
void quit()
{
    forever = FALSE;
}
char * getErrStr(S16 err)
{
	short i;
	for (i = 0; err2Str[i].val != UNKNOWN; i++)
	{
		if (err2Str[i].val == err)
			return(err2Str[i].cmd);
	}
	return("Unknown Error");
}
/*
*	Uitility function to convert str ip to long
*	iptol(char *ip)
* 	Params: ip IP address in string format
*			ipL returned as long.
*/
U32 iptol(char *ip)
{
	int ipArr[4];
	short i;
	char *pos;
	unsigned long ipL;

	i = 0;
	while(ip)
	{
		pos = strchr(ip,'.');
		if (!pos)
			break;

		*pos = '\0';

		ipArr[i] = atoi(ip);
		ip = pos+1;
		i++;
	}
	if( i != 3 )
		return(0); //error
	ipArr[i] = atoi(ip);
	ipL = ipArr[3] + (ipArr[2] * 256) + (ipArr[1] * 256 *256) + (ipArr[0] * 256 * 256 * 256);
	return (ipL);
}/* iptol */

void sctpCtrl( char *parms )             /* TODO */
{
    UNREFERENCED_PARAMETER(parms);
}

void statusReq( char *parms[] )            /* TODO */
{
    short i;
    Bool found = FALSE;
    if ( NULL == parms[0] )
    {
        printf("Invalid STATUS Command.\n");
        printf("Usage: STATUS SYS | SCTP | SAP | ASSOC \n");
        return;
    }
    /* first search Ent table for match */
    found = FALSE;
    for( i = 0; staList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( parms[0], staList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found ) {
        printf("Error: Unknown STATUS Command (%s)\n", parms[0]);
        return;
    }
    switch (staList[i].val)
    {
            case SAP_STA :
                doSapStatus((char **)&parms[1]);
            break;
            case GEN_STA :
                doGenStatus((char **)&parms[1]);
            break;
            case ASSOC_STA :
                doAssocStatus((char **)&parms[1]);
            break;
            case DEST_TX_STA :
                doDestTransAddrStatus((char **)&parms[1]);
            break;
            default:
                printf("Error: Unknown STATUS Command!\n");
    }

}
/*
 * Func: doSapStatus
 * Sends Sap Status Request to SCTP Layer
 */
void doSapStatus(char * parms[])
{
	S16 			sapId;
	SCTP_STATUS 	status;
	SCTPSapSta    	sta;
	SCTPSctSapCfg	cfg;
	SCTPTSapCfg		tSapCfg;
    U32             i;
	memset(&sta, 0, sizeof(SCTPSapSta));
	memset(&tSapCfg,0,sizeof(SCTPTSapCfg));
	memset(&cfg,0,sizeof(SCTPSctSapCfg));

	if ( NULL != parms[0])
	{
		sapId = (S16)(atoi(parms[0]));
	}
    else
    {
        printf("STATUS SAP Error: Invalid SAPID\n");
        return;
    }
	status = SctpSapStatus((U8)cp, &sta, sapId);
	if( SCTP_SUCCESS != status )
	{
		printf("Error: SAP Status Request returned Error: %s \n", getErrStr(status));
		return;
	}
	printf("Protocol Switch :%d\n", sta.swtch);
	printf("SAP Status      :%s\n", sta.hlSt == 0 ? "UBND" : "BND");
	memset(&cfg,0,sizeof(SCTPSctSapCfg));
	status = SctpGetSctSapCfg((U8)cp, &cfg, sapId);
	if ( SCTP_SUCCESS != status )
	{
		printf("Error: Getting SCT SAP Configuration failed: %s \n",getErrStr(status));
		return;
	}
	printf("=====================Current SCT SAP Configurations=====================\n");
    printf("swtch         = %-8u",cfg.swtch);
    printf("spId          = %-8u",cfg.spId);
    printf("selector      = %-8u\n",cfg.sel);

    printf("MemRegion     = %-8u",cfg.memId.region);
    printf("Mempool       = %-8u",cfg.memId.pool);
    printf("prior         = %-8u\n",cfg.prior);

	printf("route         = %-8u",cfg.route);
	/* reconfig Structure*/
    printf("maxAckDelayTm = %-8u",cfg.reConfig.maxAckDelayTm);
	printf("maxAckDelayDg = %-8u\n",cfg.reConfig.maxAckDelayDg);

	printf("rtoInitial    = %-8u",cfg.reConfig.rtoInitial);
	printf("rtoMin        = %-8u",cfg.reConfig.rtoMin);
	printf("rtoMax        = %-8u\n",cfg.reConfig.rtoMax);

	printf("freezeTm      = %-8u",cfg.reConfig.freezeTm);
	printf("bundleTm      = %-8u",cfg.reConfig.bundleTm);
	printf("cookieLife    = %-8u",cfg.reConfig.cookieLife);
	printf("intervalTm    = %-8u\n",cfg.reConfig.intervalTm);
#ifdef LSB8
	printf("maxBurst      = %-8u",cfg.reConfig.maxBurst);
	printf("maxHbBurst    = %-8u",cfg.reConfig.maxHbBurst);
	printf("t5SdownGrdTm  = %-8u\n",cfg.reConfig.t5SdownGrdTm);
#endif
	printf("handleInitFlg = %-8s",(cfg.reConfig.handleInitFlg == TRUE ? "TRUE" : "FALSE"));
	printf("negAbrtFlg    = %-8s",(cfg.reConfig.negAbrtFlg == TRUE ? "TRUE" : "FALSE"));
	printf("hBeatEnable   = %-8s\n",(cfg.reConfig.hBeatEnable == TRUE ? "TRUE" : "FALSE"));

	printf("flcUpThr      = %-8u",cfg.reConfig.flcUpThr);
	printf("flcLowThr     = %-8u\n",cfg.reConfig.flcLowThr);
#ifdef TDS_ROLL_UPGRADE_SUPPORT
	printf("remIntfValid  = %-8u",cfg.reConfig.remIntfValid);
	printf("remIntfVer    = %-8u",cfg.reConfig.remIntfVer);
#endif
#ifdef SB_ETSI
	printf("maxDataSize   = %-8u",cfg.reConfig.maxDataSize);
#endif
#ifdef SB_CHECKSUM_DUAL
	printf("checksumType  = %-8u",cfg.reConfig.checksumType);
#endif
	memset(&tSapCfg,0,sizeof(SCTPTSapCfg));
	status = SctpGetTSapCfg((U8)cp, &tSapCfg, sapId);
	if ( SCTP_SUCCESS != status )
	{
		printf("Error: Getting TSAP Configuration failed: %s \n",getErrStr(status));
		return;
	}
	printf("======================Current TSAP Configurations======================\n");
	printf("swtch         = %-8u",tSapCfg.swtch);
    printf("suId          = %-8u",tSapCfg.suId);
    printf("sel           = %-8u\n",tSapCfg.sel);

    printf("ent           = %-8s",tSapCfg.ent == 117 ? "ENTHI" : "UNKN");
	printf("inst          = %-8u",tSapCfg.inst);
	printf("procId        = %-8u\n",tSapCfg.procId);

	printf("MemPool       = %-8u",tSapCfg.memId.pool);
    printf("prior         = %-8u",tSapCfg.prior);
    printf("route         = %-8u\n",tSapCfg.route);

	for(i = 0; i < tSapCfg.srcNAddrLst.nmb; i ++)
	{
		printf("SrcAddrType   = %-8s",(tSapCfg.srcNAddrLst.nAddr[i].type == 4 ? "IPv4" : "IPv6"));
		printf("srcAddress    = %-8x",tSapCfg.srcNAddrLst.nAddr[i].u.ipv4NetAddr);
	}
    printf("srcNAdrNmb    = %-8u\n",tSapCfg.srcNAddrLst.nmb);
	/* reconfig Structure*/
    printf("spId          = %-8u",tSapCfg.reConfig.spId);
	printf("maxBndRetry   = %-8u",tSapCfg.reConfig.maxBndRetry);
	printf("BndCfmTmr     = %-8u\n",tSapCfg.reConfig.tIntTmr);

#ifdef TDS_ROLL_UPGRADE_SUPPORT
	printf("remIntfValid  = %d\n",tSapCfg.reConfig.remIntfValid);
	printf("remIntfVer    = %d\n",tSapCfg.reConfig.remIntfVer);
#endif
	if ( TRUE == tSapCfg.reConfig.sbDnsCfg.useDnsLib )
	{
		printf("dnsAddrType   = %-8u",tSapCfg.reConfig.sbDnsCfg.dnsAddr.type);
		printf("DnsPort       = %-8u",tSapCfg.reConfig.sbDnsCfg.dnsAddr.u.ipv4TptAddr.port);
		printf("sbDnsAddr     = %-8X\n",tSapCfg.reConfig.sbDnsCfg.dnsAddr.u.ipv4TptAddr.address);
		printf("DnsUseDnsLib  = %-8s",(tSapCfg.reConfig.sbDnsCfg.useDnsLib == TRUE ? "TRUE" : "FALSE"));
		printf("DnsTmOut      = %-8u",tSapCfg.reConfig.sbDnsCfg.dnsTmOut);
		printf("DnsRtxCnt     = %-8u\n",tSapCfg.reConfig.sbDnsCfg.maxRtxCnt);
	}
}
/*
 * Func: doGenStatus
 *      : sends General (SCTP) Status request to Layer and print resuls
 * Parameter:   parms user passed arguements  (may be we don't need this)
 */
void doGenStatus(char * parms[])
{
    SCTPGenSta      sta;
	SCTPGenCfg		cfg;
    SCTP_STATUS     status;

    UNREFERENCED_PARAMETER(parms);

    status = SctpGenStatus((U8)cp, &sta);
    if ( SCTP_SUCCESS != status )
    {
       printf("Error: SCTP (gen) Status Request returned Error: %s \n", getErrStr(status));
       return;
    }
    printf("Associations 	  = %d\n",sta.nmbAssoc);
    printf("Local Addr in Use = %d\n", sta.nmbLocalAddr);
    printf("Peer Addr in Use  = %d\n", sta.nmbPeerAddr);
	/* get Current Configuration */
	memset(&cfg,0,sizeof(SCTPGenCfg));
	status = SctpGetGenCfg( (U8)cp, &cfg);
	if ( SCTP_SUCCESS != status )
	{
		printf("Error: SCTP Get Config Failed.! %s\n", getErrStr(status));
        return;
	}
	printf("======================Current SCTP Configurations======================\n");
	printf("serviceType    = %-8s",(cfg.serviceType == 0 ? "UDP" : "TCP"));
	printf("maxNmbSctSaps  = %-8u",cfg.maxNmbSctSaps);
	printf("maxNmbTSaps    = %-8u\n",cfg.maxNmbTSaps);

	printf("maxNmbEndp     = %-8u",cfg.maxNmbEndp);
	printf("maxNmbAssoc    = %-8u",cfg.maxNmbAssoc);
	printf("maxNmbDstAddr  = %-8u\n",cfg.maxNmbDstAddr);

	printf("maxNmbSrcAddr  = %-8u",cfg.maxNmbSrcAddr);
	printf("maxNmbTxChunks = %-8u",cfg.maxNmbTxChunks);
	printf("maxNmbRxChunks = %-8u\n",cfg.maxNmbRxChunks);

	printf("maxNmbInStrms  = %-8u",cfg.maxNmbInStrms);
	printf("maxNmbOutStrms = %-8u",cfg.maxNmbOutStrms);
	printf("initARwnd      = %-8u\n",cfg.initARwnd);

	printf("mtuInitial     = %-8u",cfg.mtuInitial);
	printf("mtuMinInitial  = %-8u",cfg.mtuMinInitial);
	printf("mtuMaxInitial  = %-8u\n",cfg.mtuMaxInitial);

	printf("performMtu     = %-8s",(cfg.performMtu == TRUE ? "TRUE" : "FALSE"));
	printf("UseHstName     = %-8s",(cfg.useHstName == TRUE ? "TRUE" : "FALSE"));
	printf("Hostname       = %-8s\n",cfg.hostname);

	printf("timeRes        = %-8u",cfg.timeRes);
	printf("maxInitReTx    = %-8u",cfg.reConfig.maxInitReTx);
	printf("maxAssocReTx   = %-8u\n",cfg.reConfig.maxAssocReTx);

	printf("maxPathReTx    = %-8u",cfg.reConfig.maxPathReTx);
	printf("altAcceptFlg   = %-8u",cfg.reConfig.altAcceptFlg);
	printf("keyTm          = %-8u\n",cfg.reConfig.keyTm);

	printf("alpha          = %-8u",cfg.reConfig.alpha);
	printf("beta           = %-8u\n",cfg.reConfig.beta);
	printf("========================================================================\n");

}
/*
 * Func: doAssocStatus
 *      : This function sends Association Status request and prints out results.
 * Parameters:  parms - User passed arguements.
 */

void doAssocStatus(char * parms[])
{
    SCTPAssocSta  sta;
    SCTP_STATUS   status;
    U32           assocId;                   /* association ID */
    U32           i;
    char          ipStr[16];

    if ( NULL != parms[0] )
    {
        assocId = (U32)(atol(parms[0]));
    }
    else
    {
        printf("Error: Invalid Association Id. Try ? for help!\n");
        return;
    }
    status = SctpAssocStatus((U8)cp, &sta, assocId);
    if ( SCTP_SUCCESS != status)
    {
        printf("Error: Association Status Request returned %s\n", getErrStr(status));
        return;
    }
    printf("\tAssociation Status for %lu\n", sta.assocId);
    printf("\tState:    %s\n",assocStateStr[sta.assocState]);
    printf("\tDestination Address List\n");
    for ( i = 0; i < sta.dstNAddrLst.nmb; i++)
    {
        /* assuming IPv4 Address*/
        u32ToIp(sta.dstNAddrLst.nAddr[i].u.ipv4NetAddr, &ipStr[0]);
        printf("\t(%d) %s\n", i+1, ipStr);
    }
    printf("\tSource Address List\n");
    for ( i = 0; i < sta.srcNAddrLst.nmb; i++)
    {
        /* assuming IPv4 Address*/
        u32ToIp(sta.srcNAddrLst.nAddr[i].u.ipv4NetAddr, &ipStr[0]);
        printf("\t(%d) %s\n", i+1, ipStr);
    }
    u32ToIp(sta.priNAddr.u.ipv4NetAddr, &ipStr[0]);
    printf("\tPrimary Network Address: %s\n", ipStr);
    printf("\tSource Port:             %d\n", sta.srcPort);
    printf("\tDestination Port:        %d\n", sta.dstPort);
}
 /*
  * Func: doDestTransAddrStatus
  *     : This function sends Destination Transport address Sts
  * Parameters: parms used passed arguments i.e dest addr, port association id
  */
void doDestTransAddrStatus(char * parms[])
{
    SCTPDtaSta      sta;
    SCTP_STATUS     status;
    U32             destAddr;
    char            *ips;
	S16				sapId;
    U16             destPort;
    U32             assocId;
    U8              i;
    for (i = 0; i < 4 ; i++)
    {
        if ( NULL == parms[i] )
        {
            printf("Error: Invalid command\n");
            return;
        }
    }
	/* Format: status destTxaddr <sapid> <destIp> <destport> <assocId> */
	sapId			=	(S16)(atoi(parms[0]));
    ips = malloc(strlen(parms[1]));  /* save a copy to print in . format later */
    strcpy(ips,parms[1]);
    destAddr        = iptol(parms[1]);
    destPort        = (U16)(atoi(parms[2]));
    assocId         = (U32)(atol(parms[3]));
    sta.dstNAddr.type = SCTP_IPV4ADDR_TYPE;
    sta.dstNAddr.u.ipv4NetAddr = destAddr;
    //sta.dstNAddr.u.ipv4NetAddr.port    = destPort;
    sta.assocId     =  assocId;
    status = SctpDestTransAddrStatus((U8)cp, &sta, sapId);
    if ( SCTP_SUCCESS != status)
    {
        printf("Error: Association Status Request Failed: %s \n",getErrStr(status));
        return;
    }
    printf("\tDestination Adrress Status for %s \n",ips);
    printf("\tState:            %d\n", sta.state);
    printf("\tMTU  :            %d\n", sta.mtu);
    printf("\tReTx Timeoutes:   %d\n", sta.rto);
}
/*
 * Handle Statistics Requesst
 * Parameters: parms user passed arguements.
 */
void statsReq( char *parms[] )
{
    short i;
    Bool found = FALSE;

    if ( NULL == parms[0] )
    {
        printf("Invalid STATS Command. Try ? for help\n");
        return;
    }
    /* first search Ent table for match */
    found = FALSE;
    for( i = 0; stsList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( parms[0], stsList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found ) {
        printf("Error: Unknown STATS Command (%s)\n", parms[0]);
    }
    switch (stsList[i].val)
    {
            case SCTP_STS :
                doGenStats((char **)&parms[1]);
            break;
            case SCTSAP_STS :
                doSctSapStats((char **)&parms[1]);
            break;
            case TSAP_STS :
                doTsapStats((char **)&parms[1]);
            break;
            default:
                printf("Error: Unknown STATS Command!\n");
    }
}
/*
 *  doTsapStats
 *            : get T SAP Statistics
 * Parameters: pointer to user passed arguments *parms
 */
void doTsapStats(char * parms[])
{
    SCTPTSapSts     sts;
    SCTP_STATUS     status;
    S16             sapId;
    Bool            isReset = FALSE;

    memset(&sts,0,sizeof(SCTPTSapSts));
    if ( NULL != parms[0] )
    {
        sapId = (S16)(atoi(parms[0]));
        if ( (NULL != parms[1]) && (0 == (strcmp("RESET", parms[1]))) )
        {
            isReset = TRUE;
        }
    }
    else
    {
        printf("Error: Invalid TSAP Id \n");
        return;
    }
    status = SctpTSapStatistics((U8)cp, &sts, sapId, isReset);
    if ( SCTP_SUCCESS != status )
    {
       printf("Error: SCTSAP Statistics returned Error: %s \n", getErrStr(status));
       return;
    }
   printf("=================TSAP %d Statistics Since %02d-%02d-%02d %02d:%02d:%02d===============\n",
        sapId,sts.dt.month, sts.dt.day, sts.dt.year+1900, sts.dt.hour, sts.dt.min, sts.dt.sec);
    printf("\tCounter     %-12s   %-12s   %-12s\n","Tx", "Rx", "Re-Tx");
    printf("=====================================================================\n");
    printf("\tINIT        %-12u   %-12u   %-12u\n", sts.sbChunkSts.noInitTx,sts.sbChunkSts.noInitRx, sts.sbChunkSts.noInitReTx);
    printf("\tINIT ACK    %-12u   %-12u   %-12s\n", sts.sbChunkSts.noIAckTx,sts.sbChunkSts.noIAckRx, "n/a");
    printf("\tSHUTDOWN    %-12u   %-12u   %-12u\n", sts.sbChunkSts.noShDwnTx,sts.sbChunkSts.noShDwnRx, sts.sbChunkSts.noShDwnReTx);
    printf("\tDOWN ACK    %-12u   %-12u   %-12u\n", sts.sbChunkSts.noShDwnAckTx,sts.sbChunkSts.noShDwnAckRx, sts.sbChunkSts.noShDwnAckReTx);
    printf("\tCOOKIES     %-12u   %-12u   %-12u\n", sts.sbChunkSts.noCookieTx,sts.sbChunkSts.noCookieRx, sts.sbChunkSts.noCookieReTx);
    printf("\tCK ACKS     %-12u   %-12u   %-12s\n", sts.sbChunkSts.noCkAckTx,sts.sbChunkSts.noCkAckRx, "n/a");
    printf("\tDATA        %-12u   %-12u   %-12u\n", sts.sbChunkSts.noDataTx,sts.sbChunkSts.noDataRx, sts.sbChunkSts.noDataReTx);
    printf("\tDAT ACKS    %-12u   %-12u   %-12s\n", sts.sbChunkSts.noDAckTx,sts.sbChunkSts.noDAckRx, "n/a");
    printf("\tDWN COMPL   %-12u   %-12u   %-12s\n", sts.sbChunkSts.noShDwnCmpltTx,sts.sbChunkSts.noShDwnCmpltRx, "n/a");
#ifdef LSB2
    printf("\tHBEAT       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noHBeatTx,sts.sbChunkSts.noHBeatRx, "n/a");
    printf("\tHB ACK      %-12u   %-12u   %-12s\n", sts.sbChunkSts.noHBAckTx,sts.sbChunkSts.noHBAckRx, "n/a");
#endif
#ifdef LSB3
    printf("\tABORT       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noAbortTx,sts.sbChunkSts.noAbortRx, "n/a");
#endif
#ifdef LSB6
    printf("\tERROR       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noErrorTx,sts.sbChunkSts.noErrorRx, "n/a");
#endif
    printf("\tBYTES       %-12u   %-12u   %-12s\n", sts.sbByteSts.bytesTx,sts.sbByteSts.bytesRx, "n/a");
    printf("\tBIND RETRY  %-12s   %-12s   %-12u\n","n/a", "n/a", sts.nmbBndRetry);
}

/*
 *  doSctSapStats
 *            : get SCT SAP Statistics
 * Parameters: pointer to user passed arguments *parms
 */
void doSctSapStats(char *parms[])
{
    SCTPSctSapSts   sts;
    SCTP_STATUS     status;
    S16             sapId;
    Bool            isReset = FALSE;

    if ( NULL != parms[0])
    {
        sapId = (S16)(atoi(parms[0]));
        if ( NULL != parms[1] && (0 == (strcmp("RESET", parms[1]))) )
        {
            isReset = TRUE;
        }
    }
    else
    {
	   printf("Error: Invalid SCT SAP Id\n");
	   return;
    }

    status = SctpSctSapStatistics((U8)cp, &sts, sapId, isReset);
    if ( SCTP_SUCCESS != status )
    {
       printf("Error: SCTSAP Statistics returned Error: %s \n", getErrStr(status));
       return;
    }
   printf("=================SCTSAP %d Statistics Since %02d-%02d-%02d %02d:%02d:%02d===============\n",
        sapId,sts.dt.month, sts.dt.day, sts.dt.year+1900, sts.dt.hour, sts.dt.min, sts.dt.sec);
   printf("\tTotal Bytes Received:\t%d\n", sts.sbByteSts.bytesRx);
   printf("\tTotal Bytes Transmit:\t%d\n", sts.sbByteSts.bytesTx);
}

/*
 *  doGenStats
 *            : get General SCTP Statistics
 * Parameters: pointer to user passed arguments *parms
 */
void doGenStats(char *parms[])
{
    SCTPGenSts      sts;
    SCTP_STATUS     status;
    Bool            isReset = FALSE;
    memset(&sts,0,sizeof(SCTPGenSts));
    if ( (NULL != parms[0]) && (0 == (strcmp("RESET", parms[0]))) )
    {
        isReset = TRUE;
    }
    status = SctpGenStatistics((U8)cp, &sts, isReset);

    if ( SCTP_SUCCESS != status )
    {
       printf("Error: General (SCTP) Statistics returned Error: %s \n", getErrStr(status));
       return;
    }
    printf("============SCTP Statistics Since %02d-%02d-%02d %02d:%02d:%02d=============\n",
            sts.dt.month, sts.dt.day, sts.dt.year+1900, sts.dt.hour, sts.dt.min, sts.dt.sec);
    printf("Statistics for Rx & Tx Chunks\n");
    printf("\tCounter     %-12s   %-12s   %-12s\n","Tx", "Rx", "Re-Tx");
    printf("=============================================================\n");
    printf("\tINIT        %-12u   %-12u   %-12u\n", sts.sbChunkSts.noInitTx,sts.sbChunkSts.noInitRx, sts.sbChunkSts.noInitReTx);
    printf("\tINIT ACK    %-12u   %-12u   %-12s\n", sts.sbChunkSts.noIAckTx,sts.sbChunkSts.noIAckRx, "n/a");
    printf("\tSHUTDOWN    %-12u   %-12u   %-12u\n", sts.sbChunkSts.noShDwnTx,sts.sbChunkSts.noShDwnRx, sts.sbChunkSts.noShDwnReTx);
    printf("\tDOWN ACK    %-12u   %-12u   %-12u\n", sts.sbChunkSts.noShDwnAckTx,sts.sbChunkSts.noShDwnAckRx, sts.sbChunkSts.noShDwnAckReTx);
    printf("\tCOOKIES     %-12u   %-12u   %-12u\n", sts.sbChunkSts.noCookieTx,sts.sbChunkSts.noCookieRx, sts.sbChunkSts.noCookieReTx);
    printf("\tCK ACKS     %-12u   %-12u   %-12s\n", sts.sbChunkSts.noCkAckTx,sts.sbChunkSts.noCkAckRx, "n/a");
    printf("\tDATA        %-12u   %-12u   %-12u\n", sts.sbChunkSts.noDataTx,sts.sbChunkSts.noDataRx, sts.sbChunkSts.noDataReTx);
    printf("\tDAT ACKS    %-12u   %-12u   %-12s\n", sts.sbChunkSts.noDAckTx,sts.sbChunkSts.noDAckRx, "n/a");
    printf("\tDWN COMPL   %-12u   %-12u   %-12s\n", sts.sbChunkSts.noShDwnCmpltTx,sts.sbChunkSts.noShDwnCmpltRx, "n/a");
#ifdef LSB2
    printf("\tHBEAT       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noHBeatTx,sts.sbChunkSts.noHBeatRx, "n/a");
    printf("\tHB ACK      %-12u   %-12u   %-12s\n", sts.sbChunkSts.noHBAckTx,sts.sbChunkSts.noHBAckRx, "n/a");
#endif
#ifdef LSB3
    printf("\tABORT       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noAbortTx,sts.sbChunkSts.noAbortRx, "n/a");
#endif
#ifdef LSB6
    printf("\tERROR       %-12u   %-12u   %-12s\n", sts.sbChunkSts.noErrorTx,sts.sbChunkSts.noErrorRx, "n/a");
#endif
    printf("\tBYTES       %-12u   %-12u   %-12s\n", sts.sbByteSts.bytesTx,sts.sbByteSts.bytesRx, "n/a");
    printf("\tDNS QRY     %-12u   %-12u   %-12u\n", sts.sbDnsSts.noQueryTx,sts.sbDnsSts.noQueryRspRx, sts.sbDnsSts.noQueryReTx);
}

void tsapCtrl( char *parms[] )
{
    short i;
    Bool found = FALSE;
    SCTP_STATUS status;

    if ( NULL == parms[0] )
    {
        printf("Invalid command arguments\n");
        return;
    }

    /* first search table for match */
    for( i = 0; tsapCtrlList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( parms[0], tsapCtrlList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found )
    {
        printf("Error: Unknown TSAP Command (%s)\n", parms[0]);
        return;
    }

    status = SctpMgmtCtrl( (U8) cp, 0, (U8)tsapCtrlList[i].val );     /* Always sap 0 */
    if( status != SCTP_SUCCESS )
	{
        printf( "Board %d TSAP %d Control Request %d failed: status = 0x%2x\n",
                cp, 0, tsapCtrlList[i].val, status );
    }
}

void debugCtrl( char *parms[] )
{
    short i;
    Bool found = FALSE;
    SCTP_STATUS status;
    if ( NULL == parms[0] )
    {
        printf("Invalid command arguments\n");
        return;
    }
    /* first search table for match */
    for( i = 0; debugCtrlList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( parms[0], debugCtrlList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found )
    {
        printf("Error: Unknown DEBUG Command (%s)\n", parms[0]);
        return;
    }
    status = SctpMgmtCtrl( (U8) cp, 0, (U8)debugCtrlList[i].val );     /* Entity unused currently */
    if( status != SCTP_SUCCESS )
	{
        printf( "Board %d DEBUG %d Control Request %d failed: status = 0x%2x\n",
                cp, 0, debugCtrlList[i].val, status );
    }
}

/* Enable / Disable Trace */
void traceCtrl( char *parms[] )
{
    short i;
    Bool found = FALSE;
    SCTP_STATUS status;
    if ( NULL == parms[0] )
    {
        printf("Invalid command arguments\n");
        return;
    }
    /* first search table for match */
    for( i = 0; traceCtrlList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( parms[0], traceCtrlList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found )
    {
        printf("Error: Unknown TRACE Command (%s)\n", parms[0]);
        return;
    }
    status = SctpMgmtCtrl( (U8) cp, 0, (U8)traceCtrlList[i].val );     /* Entity unused currently */
    if( status != SCTP_SUCCESS )
	{
        printf( "Board %d DEBUG %d Control Request %d failed: status = 0x%2x\n",
                cp, 0, traceCtrlList[i].val, status );
    }
}

/*---------------------------------
 * Change Current board to new board
 * ---------------------------------*/
void setCP( char *parms[] )
{
    short         newCp;
    if ( NULL == parms[0] )
    {
        printf("Invalid board number \n");
        return;
    }
    newCp = (short) atoi(parms[0]);
    if ( newCp > 8 || newCp < 1)
    {
        printf( "Board %d out of Range. Must be in range 1..8\n", newCp);
        return;
    }
    if ( SCTP_SUCCESS != (SctpMgmtInit( (U8) newCp, ENTSB, 0 )) )   /* Use SCTP entity ID for mgr since cfg & task
                                                                     * are using MTP2's entId */
    {
        printf( "Board %d Initialisation failed!\n", newCp);
        return;
    }
    else
    {
        SctpMgmtTerm( (U8)cp );
        cp = newCp;
    }
}
/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
    char    *p = buf;
    char    *endp;
    short   cnt;

    /* initialize token pointers to all null */
    for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
        tokens[cnt] = 0;

    /* begin scanning for tokens */
    cnt = 0;
    while( *p != 0 && cnt <= MAX_PARMS )
    {
        /* skip leading space or tabs */
        while( *p == ' ' || *p == '\t' )
            p++;

        /* beginning of token (or end of line) */
        if( *p != 0 )
        {
            /* scan for end of token, converting to upper case as we go */
            endp = p;
            while( *endp != ' ' && *endp != '\t' && *endp != 0 )
            {
                if( islower( *endp ) )
                {
                    *endp = 'A' + (*endp - 'a');    /* convert to upper */
                }
                endp++;
            }

            /* now have a begin & end of token */
            tokens[cnt++] = p;
            if( *endp == 0 )
            {
                p = endp;   /* end of input - terminate scan */
            }
            else
            {
                *endp = 0;      /* null terminate token */
                p = &endp[1];   /* start next scan at after null */
            }
        }   /* end if *p != 0 */
    }   /* end while *p != 0 .... */

    return( cnt );
}
/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
    S16                 i;
    Bool                found;

    UNREFERENCED_PARAMETER(cnt);

    /* first search command table for command match */
    found = FALSE;
    if ( NULL == tokens[0] )
    {
        printf("Error: Unknown Command\n");
        return;
    }
    for( i = 0; cmdList[i].val != UNKNOWN; i++ )
    {
        if( strcmp( tokens[0], cmdList[i].cmd ) == 0 )
        {
            found = TRUE;
            break;      /* command matched */
        }
    }   /* end for */
    if ( !found ) {
        printf("Error: Unknown Command (%s)\n", tokens[0]);
    }
    else
    {
        switch (cmdList[i].val) {
            case CMD_HELP :
                 HelpCommand((char **)&tokens[1]);
            break;
            case CMD_STATS :
                statsReq ((char **)&tokens[1]);
            break;
            case CMD_STATUS :
                statusReq((char **)&tokens[1]);
            break;
            case CMD_TSAP :
                tsapCtrl((char **)&tokens[1]);
            break;
            case CMD_DEBUG :
                debugCtrl((char **)&tokens[1]);
            break;
            case CMD_TRACE :
                traceCtrl((char **)&tokens[1]);
            break;
            case CMD_BOARD :
                setCP((char **)&tokens[1]);
            break;
            case CMD_QUIT :
                quit();
            break;
            default:
                printf("Unknown Commcand");
        } /* switch */
    }
}
void HelpCommand(char *parms[])
{
    UNREFERENCED_PARAMETER(parms);

    printf("Statistics Commands\n");
    printf("-------------------\n");
    printf("STATS SCTP [RESET] - Display/Reset SCTP Stats\n");
	printf("STATS SCTSAP <SAPID> [RESET] - Display/Reset SCTSap Stats\n");
	printf("STATS TSAP <SAPID> [RESET] - Display/Reset TSAP Stats\n\n");
   /* printf("STATUS SYS - Display System Id information\n"); */
    printf("Status Commands\n");
    printf("---------------\n");
    printf("STATUS SAP <SAPID> - Display Sap status for <sapid>\n");
	printf("STATUS SCTP - Display General (SCTP) status\n");
	printf("STATUS ASSOC <ASSOCID> - Display Sap Association status\n\n");
    printf("Control Commands\n");
    printf("----------------\n");
    /* printf("SCTSAP ENA/DIS - Enable/disable upper SCTSAP\n"); */
    printf("TSAP   ENA/DIS - Enable/disable lower TSAP\n");
    printf("DEBUG  ENA/DIS - Enable/disable debug logging\n");
    printf("TRACE  ENA/DIS - Enable/disable data tracing\n");
/*	printf("STATUS DESTTXADDR <SAPID> <DEST IP> <DEST PORT> ASSOCID -Display Dest.Addr Status\n"); */
    printf("BOARD  <number> - Sets Current board\n");
	printf("Q      Quit sctpmgr\n");
	printf("?      Help\n");

}
/* Main routine */
int main( int argc, char *argv[] )
{

    S16     cnt=0;
    S16     i;
	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	char	*p;
    SCTP_STATUS status;
    printf("\n\n%s: sample SCTP management application version %s %s\n\n",
            argv[0], SCTPMGR_VERSION, __DATE__);
	/* parse command line for interrupt & cp # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
			case 'b' :    /* Set Board number */
				i++;
				if((argv[i] == NULL) || (sscanf(argv[i], "%hi", &cp) != 1) || (cp < 1) || (cp > 8) )
				{
                    usage( argv[0] );
				}
				break;

            case 'd' :      /* Set debug mode on */
                bDebug = TRUE;
                break;

			default:
                usage( argv[0] );
				break;
			}	/* end switch */
		}
		else
		{
            usage( argv[0] );
		}

	}	/* end for */
    status = SctpMgmtInit( (U8) cp, ENTSB, 0 );
    if( status != SCTP_SUCCESS )
	{
        printf( "MgmtInit failed [%d]\n", status );
		exit(1);
	}
    //status = SctpGetGenCfg( (U8) cp, &genCfg );
    if (status != SCTP_SUCCESS)
    {
       exit(1);
    }
       /* command processing loop */
    printf( "sctpmgr[%-1u]>", cp );
    while( (forever == TRUE) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        cnt = tokenize( buffer, tokens );
        if( cnt > 0 )
            processCmd( tokens, cnt );
            printf( "sctpmgr[%-1u]>", cp );
    }   /* end while */

    status = SctpMgmtTerm( (U8) cp );
return(0);
}   /* end main() */
