/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   find800.c  ITU edition
**
**      Source code for an 800 number client/server that uses the
**      TCAP API. Contains the send and receive logic.
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
/* #include <sys/stream.h> */
#endif	                                /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#endif	                                /* __OS2__ */

#ifdef WIN32
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#endif	                                /* WIN32 */

#include "ss7type.h"
#include "tcapdef.h"
#include "sccpdef.h"
#include "osdep.h"

#ifdef UNIX
#define Sleep(x) usleep(x * 1000)
#endif

#define SUID        0x21    /* application reference number */
#define DPRCHAN     SUID    /* Dual Port RAM channel number */
#define BOARD_ZERO  0       /* our local board number       */
#define TRANS_ID    0       /* arbitrary transaction ID     */
#define COMP_800    0x01    /* component OP Code            */

extern U8           boardNum;
extern U32          pointCode;
extern U8           sapid;
extern U8           ssn;
extern U8           destssn;
extern U8           AddrType;

extern CTAHD        FstCtaHd;
extern CTAQUEUEHD   FstQueue;
extern U8           phnum[5];
extern DWORD        delay;
extern U8           runState;

#ifdef UNIX
extern BOOL dumpflag;
extern BOOL serverflag;
#else
extern BOOLEAN dumpflag;
extern BOOLEAN serverflag;
#endif


U8            invoke_id = 0;
U32           trans_id  = TRANS_ID;
U32           send_num  = 0;
U32           rspd_num  = 0;
unsigned char phrealnum[5];
char          str800num[11];
char          strrealnum[11];
unsigned char pbuf[512];
char          notfound[] = "not found";
unsigned char uinfo[] = { 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,0x01, 0x02, 0x03, 0x04, 0x00 };

unsigned char query800[] =
            {
              0xF2, 0x0B,                             /* Set Parameter */
              0x84, 0x09, 0xF2, 0x0B, 0xF4, 0x0A,     /* Digit parameter */
              0x00, 0x00, 0x00, 0x00, 0x00,           /* BCD encoded phone number */
              0x00                                    /* NULL terminator, just for the heck of it */
            };


void ErrorAndExit(DWORD errId, char *str)
{
   printf( "Error : (%d) error returned in (%s) call.\n", errId, str );
   exit( 1 );
}

/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
     ByteCnt    # Bytes in Buf
OUT: <file>      gets multiple lines
*/
static void dumpMsg(char          *Buf,
                    unsigned short ByteCnt)
{
   unsigned short Li, LMax, ToDo, Bi, Ai;
   char           Ascii[80];
   unsigned char  B;

    /* Dump to std out in hex followed by ascii */
   for( ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax )
   {
         LMax = (ToDo > 16) ? 16 : ToDo;             /* max bytes this line */
         printf("%04X| ", Bi);
         Ai = 0;

         for( Li = 0; Li < LMax; Li++ )              /* each byte this line */
         {
                B = *Buf++;                          /* next byte to show */
                printf("%02X ", B);                  /* as hex */
                if (!isprint(B))                     /* printable ascii? */
                       B = '.';                      /* no, use this */
                Ascii[Ai++] = B;                     /* into ascii portiob */
         }      /* for Li */

         for( Li = LMax; Li < 16; Li++ )             /* pad short ln if necessary */
                printf("   ");                       /* so asci lines up */

         Ascii[Ai] = 0;                              /* terminate the ascii */
         printf("    ");
         printf("%s\n", Ascii);                      /* show the ascii */
         Bi += LMax;
   }    /* for ToDo */
}       /* dumpMsg */

DWORD NMSSTDCALL ErrHdlr( CTAHD ctahd,
                          DWORD errorcode,
                          char *errtxt,
                          char *func )
{
    /* print out the error and exit */
    char text[40];
    ctaGetText(ctahd, errorcode, text, sizeof(text) );
    printf( "ErrorHandler: %s on ctahd=%#x, %s\n", errtxt, ctahd, func );
    return errorcode;
}

DWORD InitializeTCAP(U8 boardNum)
{
    CTA_EVENT        Event;
    DWORD            Ret;
    CTA_SERVICE_DESC tcapOpenSvcLst[]   = {{{"TCAP", "TCAPMGR"}, {0}, {0}, {0}}};
    CTA_SERVICE_NAME tcapServiceNames[] = {{ "TCAP", "TCAPMGR" }};
    CTA_INIT_PARMS   tcapInitparms      = { 0 };
    CTA_REV_INFO     info;

    ctaGetVersion(&info, sizeof(CTA_REV_INFO));
    if (info.compatlevel != CTA_COMPATLEVEL)
    {
        printf("CTA compatibility level does not match. re-compile.\n");
        exit(1);
    }

    /* bind this application as a user of the TCAP layer */
    tcapInitparms.size           = sizeof(CTA_INIT_PARMS);
    tcapInitparms.traceflags     = CTA_TRACE_ENABLE;
    tcapInitparms.parmflags      = CTA_PARM_MGMT_SHARED;
    tcapInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    /* ctaSetErrorHandler(ErrHdlr, NULL); */

    Ret = ctaInitialize( tcapServiceNames,
                         sizeof(tcapServiceNames) / sizeof(tcapServiceNames[0]),
                        &tcapInitparms);
    if (Ret != TCAP_SUCCESS)
    {
        /* ctdaemon is not running.
        *   Initialize without tracing.
        */
        tcapInitparms.traceflags = 0;
        tcapInitparms.parmflags  = 0;

        Ret = ctaInitialize( tcapServiceNames,
                             sizeof(tcapServiceNames) / sizeof(tcapServiceNames[0]),
                            &tcapInitparms);
        if (Ret != SUCCESS)
        {
            ErrorAndExit( Ret, "ctaInitialize" );
        }
    }

    Ret = ctaCreateQueue( NULL, 0, &FstQueue );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateQueue" );
    }

    /* Open FstCtaHd on FstQueue                   */
    Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateContext" );
    }

    tcapOpenSvcLst[0].svcargs.args[0] = boardNum;           /* board number             */
    tcapOpenSvcLst[0].svcargs.args[1] = DPRCHAN + sapid;    /* srcEnt                   */
    tcapOpenSvcLst[0].svcargs.args[2] = BOARD_ZERO;         /* srcInst                  */
    tcapOpenSvcLst[0].svcargs.args[3] = SUID;               /* suId                     */
    tcapOpenSvcLst[0].svcargs.args[4] = sapid;              /* spId                     */
    tcapOpenSvcLst[0].svcargs.args[5] = ssn;                /* ssn                      */

    Ret = ctaOpenServices( FstCtaHd, tcapOpenSvcLst, 1 );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete.*/
    Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    if( Event.id == CTAEVN_OPEN_SERVICES_DONE )
    {
        if( Event.value != CTA_REASON_FINISHED )
        {
            printf("ERROR: Reason Value = %d ", Event.value);
            ErrorAndExit( Ret, "Reason value" );
        }
    }
    else
    {
        if( Event.id == CTAEVN_WAIT_TIMEOUT )
        {
            ErrorAndExit( Ret, "ctaWaitEvent: REASON = TIMEOUT " );
        }
        else
        {
            printf("ERROR: Open Services failed  id = %d ", Event.id);
            ErrorAndExit( Ret, "Open Services failed" );
        }
    }

    /* send message that subsystem is in service */
    TCAPStateReq(FstCtaHd, sapid, ssn, SS_OOS);
    Sleep(100);
    return(TCAPStateReq(FstCtaHd, sapid, ssn, SS_IS));
}

DWORD TermTCAP(void)
{
    CTA_EVENT  Event;
    DWORD      Ret;
    char*      tcapCloseSvcLst[] = {"TCAP"};

  printf("TermTCAP: %d were sent\n          %d were responded\n          trans_id = %d\n", send_num, rspd_num, trans_id);
  /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, tcapCloseSvcLst, 1 );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyQueue" );
   }

   return TCAP_SUCCESS;
}

DWORD Send800(void)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;
    TcapDlgSect     dlg;

    printf("Send800, pc = %d.%d.%d:%d ",
           ((pointCode >> 16) & 0xff)  ,
           ((pointCode >>  8) & 0xff)  ,
           ((pointCode >>  0) & 0xff)  , destssn);

    if ( trans_id > 0xfffffff0)
    {
        trans_id = 0;
        send_num = 0;
        rspd_num = 0;
    }

    send_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_BEGIN;
    tInfo.suDlgId        = ++trans_id;  /* arbitrary value */
    tInfo.spDlgId        = 0;         /* must be zero on first request for a transaction */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType )
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = destssn;
    tInfo.cdAddr.pointCode    = pointCode;
    tInfo.cdAddr.glTransType  = 0; /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /*Sample  transaction dialog  */
    dlg.dlgType = TCAP_DLGP_REQ;
    dlg.resPres = NOT_PRESENT;
    dlg.apConName.length = 8;
    dlg.apConName.octStr[0] = 0x30;
    dlg.apConName.octStr[1] = 0x31;
    dlg.apConName.octStr[2] = 0x32;
    dlg.apConName.octStr[3] = 0x33;
    dlg.apConName.octStr[4] = 0x34;
    dlg.apConName.octStr[5] = 0x35;
    dlg.apConName.octStr[6] = 0x36;
    dlg.apConName.octStr[7] = 0x00;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    /* increment invoke_id */
    if ( invoke_id == 0xFF )
        invoke_id = 0x00;
    else
        invoke_id++;

    printf("trans_id = %d invoke_id = %d\n", trans_id, invoke_id);

    Comp.compType                              = TCAP_INVOKE;
    Comp.uProt.ituComp.invokeId.present        = PRESENT;
    Comp.uProt.ituComp.invokeId.compId         = invoke_id;
    Comp.uProt.ituComp.linkedId.present        = NOT_PRESENT;
    Comp.uProt.ituComp.linkedId.compId         = 0;
    Comp.uProt.ituComp.invokeTimer             = 0;         /* use default */
    Comp.uProt.ituComp.opClass                 = 1;         /* both success and failure are reported */
    Comp.uProt.ituComp.uComp.opcode.opCodeType = TCAP_LOCAL;
    /* opCode set to anything */
    Comp.uProt.ituComp.uComp.opcode.opCode.length    = 1;
    Comp.uProt.ituComp.uComp.opcode.opCode.octStr[0] = 0x01;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phnum, 5);

    /* copy 13 bytes of 800 query into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
	{
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    /* release Msg and Parameter Buffer */
    free(Msg);
    free(pParam);

    return retVal;
}

DWORD Respond800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;
    TcapDlgSect     dlg;

    printf( "Respond800, pc = %d.%d.%d:%d, invoke_id = %d\n",
           ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
             srctInfo->cgAddr.subsystem,
             srccomp->uProt.ituComp.invokeId.compId);

    rspd_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_END;
    tInfo.suDlgId        = ++trans_id;        /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem; /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                          /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;


    /* Sample transaction dialog */
    dlg.dlgType = TCAP_DLGP_RSP;
    dlg.resPres = PRESENT;
    dlg.result = TCAP_DLG_REJ_PERM;
    dlg.resSrc = TCAP_DLG_SP_TAG;
    dlg.resDiag = TCAP_DLG_RSD_NCDLG;
    dlg.apConName.length = 8;
    dlg.apConName.octStr[0] = 0x30;
    dlg.apConName.octStr[1] = 0x31;
    dlg.apConName.octStr[2] = 0x32;
    dlg.apConName.octStr[3] = 0x33;
    dlg.apConName.octStr[4] = 0x34;
    dlg.apConName.octStr[5] = 0x35;
    dlg.apConName.octStr[6] = 0x36;
    dlg.apConName.octStr[7] = 0x00;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    Comp.compType                                    = TCAP_RET_RES_L;
    Comp.uProt.ituComp.invokeId.present              = PRESENT;
    /* use invoke ID from recevied packet */
    Comp.uProt.ituComp.invokeId.compId               = srccomp->uProt.ituComp.invokeId.compId;
    Comp.uProt.ituComp.linkedId.present              = NOT_PRESENT;
    Comp.uProt.ituComp.linkedId.compId               = 0;
    Comp.uProt.ituComp.invokeTimer                   = 0;          /* use default */
    Comp.uProt.ituComp.opClass                       = 1;          /* both success and failure are reported */
    Comp.uProt.ituComp.uComp.opcode.opCodeType       = TCAP_LOCAL;
    /* opCode set to COMP_800 */
    Comp.uProt.ituComp.uComp.opcode.opCode.length    = 1;
    Comp.uProt.ituComp.uComp.opcode.opCode.octStr[0] = COMP_800;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phonenum, 5);

    /* copy 13 bytes of 800 response into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
	{
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);
    free(pParam);

    return retVal;
}


DWORD Failed800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    printf( "Failed800, pc = %d.%d.%d:%d, invoke_id = %d\n",
           ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
             srctInfo->cgAddr.subsystem,
             srccomp->uProt.ituComp.invokeId.compId);

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_END;
    tInfo.suDlgId        = ++trans_id;        /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem;  /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                           /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0;           /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    Comp.compType                                    = TCAP_RET_ERR;
    Comp.uProt.ituComp.invokeId.present              = PRESENT;
    /* use invoke ID from recevied packet */
    Comp.uProt.ituComp.invokeId.compId               = srccomp->uProt.ituComp.invokeId.compId;
    Comp.uProt.ituComp.linkedId.present              = NOT_PRESENT;
    Comp.uProt.ituComp.linkedId.compId               = 0;
    Comp.uProt.ituComp.invokeTimer                   = 0;           /* use default */
    Comp.uProt.ituComp.opClass                       = 1;           /* both success and failure are reported */
    Comp.uProt.ituComp.uComp.opcode.opCodeType       = TCAP_LOCAL;
    /* opCode set to COMP_800 */
    Comp.uProt.ituComp.uComp.opcode.opCode.length    = 1;
    Comp.uProt.ituComp.uComp.opcode.opCode.octStr[0] = COMP_800;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phonenum, 5);

    /* copy 13 bytes of 800 response into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
	{
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);
    free(pParam);

    return retVal;
}

DWORD End800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum)
{
    U8             *Msg;
    TcapTransInfo   tInfo;
    DWORD           retVal;

    UNREFERENCED_PARAMETER(phonenum);

    printf( "End800, pc = %d.%d.%d:%d, invoke_id = %d\n",
           ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
             srctInfo->cgAddr.subsystem,
             srccomp->uProt.ituComp.invokeId.compId);

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    /* modify to TCAP_ABORT or TCAP_END or TCAP_CONTINUE */
    tInfo.msgType        = TCAP_END;

    /* if ABORT, set AbortCause */

    /* modify su to copy input if this is a CONTINUE */
    tInfo.suDlgId        = ++trans_id;         /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId;  /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    /* modify if this is a prearanged END */
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem; /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                          /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0;            /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);

    return retVal;
}

DWORD retrieveMessage(void)
{
    U8            Msg[800];
    TcapRecvInfo  infoBlk;
    TcapComp      comp = {0};
    DWORD         Ret;
    U8            index;
    U16           plen;
    unsigned char i, ch;
    TcapDlgSect  *pdlg;

#ifdef UNIX
    FILE         *hFile;
    char          line[40];
    char         *xptr, *yptr;
#endif

    /* get TCAP message from API */
            if ((Ret = TCAPRetrieveMessage(FstCtaHd, (U8 *)Msg, &infoBlk)) != TCAP_SUCCESS)
            {
               if( Ret != TCAP_NOMSG )  /* NOMSG is valid - could be congestion indication */
                    printf("TCAPRetrieveMessage failed, %d\n", Ret);
               return(TCAP_NOMSG);
            }

            switch (infoBlk.eventType)
            {
                case TCAP_EVENT_RUN_STATE:
                {
                    runState = infoBlk.event.runstate.status;
                    switch (runState)
                    {
                    case TCAP_BACKUP:
                         printf("RUN STATE Indication = BACKUP\n");
                         break;
                    case TCAP_PRIMARY:
                         printf("RUN STATE Indication = PRIMARY\n");
                         break;
                    case TCAP_STANDALONE:
                         printf("RUN STATE Indication = STANDALONE\n");
                         break;
                    default:
                         printf("ERROR: RUN STATE Indication = UNKNOWN\n");
                         break;
                    }
                    break;
                }

                case TCAP_EVENT_STA_IND:
                {
                    printf("Status Indication received, status = %d, suId = %d\n",
                            infoBlk.event.status.status,
                            infoBlk.event.status.suId);
                    break;
                }

                case TCAP_EVENT_COORD_IND:
                    printf("Coordinated Status change, SSN = %d, smi = %d\n",
                            infoBlk.event.coord.aSsn,
                            infoBlk.event.coord.smi);
                    break;

                case TCAP_EVENT_COORD_CFM:
                {
                    printf("Coordinated Status change confirm, SSN = %d, smi = %d\n",
                            infoBlk.event.coord.aSsn,
                            infoBlk.event.coord.smi);
                    break;
                }

                case TCAP_EVENT_SSN_STATE:
                {
                    printf(  "Subsystem status change, pc = %d.%d.%d:%d, smi = %d, status = %d\n",
                            ((infoBlk.event.ssnstate.aPc >> 16) & 0xff),
                            ((infoBlk.event.ssnstate.aPc >>  8) & 0xff),
                            ((infoBlk.event.ssnstate.aPc >>  0) & 0xff),
                              infoBlk.event.ssnstate.aSsn,
                              infoBlk.event.ssnstate.smi,infoBlk.event.ssnstate.status);
                    break;
                }

                case TCAP_EVENT_PC_STATE:
                {
                    printf(  "PointCode status change, pc = %d.%d.%d, status = %d\n",
                            ((infoBlk.event.pcstate.aPc >> 16) & 0xff),
                            ((infoBlk.event.pcstate.aPc >>  8) & 0xff),
                            ((infoBlk.event.pcstate.aPc >>  0) & 0xff),
                              infoBlk.event.pcstate.status);
                    break;
                }

                case TCAP_EVENT_NOT_IND:
                {
                    printf("Notify Event received - cause = %d\n", infoBlk.event.notif.retcause);
                    printf("TransInfo, event = %d abort cause = %d\nsuId = %d, spId = %d, cdpc = %d.%d.%d:%d, cgpc = %d.%d.%d:%d\n",
                              infoBlk.event.notif.transInfo.msgType, infoBlk.event.notif.transInfo.abortCause,
                              infoBlk.event.notif.transInfo.suDlgId, infoBlk.event.notif.transInfo.spDlgId,
                            ((infoBlk.event.notif.transInfo.cdAddr.pointCode >> 16) & 0xff),
                            ((infoBlk.event.notif.transInfo.cdAddr.pointCode >>  8) & 0xff),
                            ((infoBlk.event.notif.transInfo.cdAddr.pointCode >>  0) & 0xff),
                              infoBlk.event.notif.transInfo.cdAddr.subsystem,
                            ((infoBlk.event.notif.transInfo.cgAddr.pointCode >> 16) & 0xff),
                            ((infoBlk.event.notif.transInfo.cgAddr.pointCode >>  8) & 0xff),
                            ((infoBlk.event.notif.transInfo.cgAddr.pointCode >>  0) & 0xff),
                              infoBlk.event.notif.transInfo.cgAddr.subsystem);
                    break;
                }

                case TCAP_EVENT_DAT_IND:
                {
                    if ( runState == TCAP_BACKUP )
                    {
                         printf("Data Indication received but ignored. Reason I AM BACKUP\n");
                         break;
                    }

                    if ( dumpflag )
                    {
                    printf("Data Indication received - from pc = %d.%d.%d\n",
                            ((infoBlk.event.data.opc >> 16) & 0xff),
                            ((infoBlk.event.data.opc >>  8) & 0xff),
                            ((infoBlk.event.data.opc >>  0) & 0xff));
                    printf("TransInfo, event = %d abort cause = %d\nsuId = %d, spId = %d, cdpc = %d.%d.%d:%d, cgpc = %d.%d.%d:%d\n",
                            infoBlk.event.data.transInfo.msgType, infoBlk.event.data.transInfo.abortCause,
                            infoBlk.event.data.transInfo.suDlgId, infoBlk.event.data.transInfo.spDlgId,
                            ((infoBlk.event.data.transInfo.cdAddr.pointCode >> 16) & 0xff),
                            ((infoBlk.event.data.transInfo.cdAddr.pointCode >>  8) & 0xff),
                            ((infoBlk.event.data.transInfo.cdAddr.pointCode >>  0) & 0xff),
                            infoBlk.event.data.transInfo.cdAddr.subsystem,
                            ((infoBlk.event.data.transInfo.cgAddr.pointCode >> 16) & 0xff),
                            ((infoBlk.event.data.transInfo.cgAddr.pointCode >>  8) & 0xff),
                            ((infoBlk.event.data.transInfo.cgAddr.pointCode >>  0) & 0xff),
                            infoBlk.event.data.transInfo.cgAddr.subsystem);
                    }

                    /* dump dialog part and user info part */
                    if ( dumpflag )
                    {
                       if ( infoBlk.event.data.dlgPart.dlgType != TCAP_DLGP_NONE)
                       {
                          printf("\nDialog Part:\n");
                          dumpMsg((char *)&(infoBlk.event.data.dlgPart), sizeof(TcapDlgSect));
                       }

                       if ( infoBlk.event.data.userInfoLen)
                       {
                          printf("\nUser Info Part:\n");
                          dumpMsg((char *)infoBlk.event.data.pUserInfo, infoBlk.event.data.userInfoLen);
                       }
                    }

                    pdlg = &(infoBlk.event.data.dlgPart);
                    if ( pdlg->dlgType != TCAP_DLGP_NONE)
                    {
                         printf("dlgtype = %d, respres = %d, result = %d, ressrc = %d, resdiag = %d, abrtsrc = %d\n",
                                 pdlg->dlgType,pdlg->resPres,pdlg->result,pdlg->resSrc,pdlg->resDiag,pdlg->abrtSrc);
                         printf("apconname len = %d, apcon = %s\n",pdlg->apConName.length,pdlg->apConName.octStr);
                    }

                    for ( index=0; index < infoBlk.event.data.numComps; index++)
                    {
                       plen = 512;
                       if ((Ret = TCAPGetComp((U8 *)&Msg, index, &comp, &plen, pbuf)) != TCAP_SUCCESS)
                       {
                          printf("TCAPGetComp failed, %d", Ret);
                          return(TCAP_NOMSG);
                       }

                       /* dump component */
                       if ( dumpflag)
                          dumpMsg((char *)&comp, sizeof(TcapComp));

                       /* dump parameter buffer */
                       if ( dumpflag)
                          dumpMsg((char *)pbuf,plen);
                    }

                    /* if we are a server, and this is an INVOKE request, respond */
                    if (serverflag && (comp.compType == TCAP_INVOKE) )
                    {
                       send_num++;
                       /* if component opcode is for our 800 number request, check the param buffer for the number */
                       if ( comp.uProt.ituComp.uComp.opcode.opCode.octStr[0] == COMP_800)
                       {
                          /* decode the BCD encoded phone number, put in str800num */
                          for ( i = 0; i < 5; i++)
                          {
                             /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
                             str800num[i*2] = ((pbuf[(8+i)] & 0xF0) >> 4) + 0x30;
                             str800num[(i*2 +1)] = ((pbuf[(8+i)]) & 0x0F) + 0x30;
                          }
                          str800num[10] = 0x00;     /* null terminate for getprofilestring to work */
                          printf("server received %s",str800num);
                          /* find the corresponding real number, put in strrealnum[] */
#ifdef UNIX
                          strcpy(strrealnum,notfound);
                          hFile = fopen("./numbers.800","r");
                          if ( hFile != NULL)
                          {
                             /* get each line in file, in succession */
                             while ( fgets(line,40,hFile) != NULL)
                             {
                                while ( (strlen(line) > 0) && (line[strlen(line) - 1] < ' ') )
                                    line[strlen(line) - 1] = '\0';  /* remove any trailing non-printable characters */
                                yptr = strchr(line,'=');
                                if ( yptr == NULL)
                                   continue;
                                *yptr = 0x00;
                                yptr++;
                                xptr = line;
                                /* check if 800 number matches first entry in line */
                                if (!( strcmp(xptr,str800num)))
                                    strcpy(strrealnum,yptr);
                             }
                          }
                          fclose(hFile);
#else
                          GetPrivateProfileString("800 Numbers", str800num, notfound, strrealnum, 11, ".\\numbers.800");
#endif
                          if ( !(strcmp(strrealnum,notfound)))
                          {
                             printf("  server didn't find a match\n");
                             Failed800(&(infoBlk.event.data.transInfo), &comp, phrealnum);
                          }
                          else
                          {
                             printf("  server found the matching number %s\n",strrealnum);

                             /* BCD encode the phone number */
                             for ( i = 0; i < 5; i++)
                             {
                                /* 2 numbers in a byte  */
                                ch = strrealnum[(i*2)] - 0x30;
                                phrealnum[i] = (ch << 4) & 0xF0;
                                ch = strrealnum[((i*2)+1)] - 0x30;
                                phrealnum[i] += ch;
                             }
                             /* make sure to swap addresses and send back response */
                             Respond800(&(infoBlk.event.data.transInfo), &comp, phrealnum);
                          }
                       }
                    }

                     /*if we are a client, and this is a RET_RES request, print the number */
                    if ((!serverflag) && (comp.compType == TCAP_RET_RES_L) )
                    {
                       rspd_num++;
                       /* if component opcode is for our 800 number request, check the param buffer for the number */
                       if ( comp.uProt.ituComp.uComp.opcode.opCode.octStr[0] == COMP_800)
                       {
                          /* decode the BCD encoded phone number, put in strrealnum */
                          for ( i = 0; i < 5; i++)
                          {
                             /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
                             strrealnum[i*2] = ((pbuf[(8+i)] & 0xF0) >> 4) + 0x30;
                             strrealnum[(i*2 +1)] = ((pbuf[(8+i)]) & 0x0F) + 0x30;
                          }

                          strrealnum[10] = 0x00;     /* null terminate for printf to work */
                          printf("client got a returned actual number of %s\n",strrealnum);
                       }
                    }

                     /*if we are a client, and this is a RET_ERR request, print the number */
                    if ((!serverflag) && (comp.compType == TCAP_RET_ERR) )
                    {
                       printf("client got a RET_ERR, 800 number not found\n");
                    }

                    /* Receive a Reject */
                    if (comp.compType == TCAP_REJECT)
                    {
                        /*
                         *  if the Reject was in a Begin message, we MUST send a Continue or End with no
                         * components. The TCAP stack will attach the Reject component and send to
                         * the original sender.
                         */
                        if ( (infoBlk.event.data.transInfo.msgType == TCAP_BEGIN))
                        {
                           printf("got a Begin/Reject, send End\n");
                           End800(&(infoBlk.event.data.transInfo), &comp, phrealnum);
                        }
                        /*
                         *  If the Reject was in a Continue message, the TCAP stack has already sent
                         *  the rejected component back to the Sender. At this point, we can try to
                         *  continue the trasnaction or End it. For this sample, we End it.
                         */
                        else if ( (infoBlk.event.data.transInfo.msgType == TCAP_CONTINUE))
                        {
                           printf("got a Continue/Reject, send End\n");
                           End800(&(infoBlk.event.data.transInfo), &comp, phrealnum);
                        }
                        /*
                         *  The only other message that can carry components is the End.
                         *  If the Reject was in a End message, the TCAP stack has already closed the
                         *  transaction, so the application doesn't need to do anything.
                         */
                        else
                        {
                           printf("got a End/Reject, do nothing\n");
                        }
                    }

                    /* if we get a Reject in a Begin Messsage, send an End */
                    if (comp.compType == TCAP_CANCEL)
                    {
                        /*
                         * An invoke timeout occured. The invoke is closed, but the trasnaction is
                         * still open. We can try another invoke, or simple End the trasnaction.
                         * For this sample, we End the transaction.
                         */
                        printf("got an Invoke Timeout, send End\n");
                        End800(&(infoBlk.event.data.transInfo), &comp, phrealnum);
                    }
                    break;
                }

                default:
                    printf("\nUnknown TCAP indication %d received\n", infoBlk.eventType);
                    break;
            }
    return(TCAP_SUCCESS);
}
