/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   init.c   ITU edition
**
**      Source code for an 800 number client/server that uses the
**      TCAP API. Contains the main() and parameter parsing.
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <string.h>
/* #include <sys/stream.h> */
#include "sol-dep.h"
#endif                                  /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#endif                                  /* __OS2__ */

#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#endif                                  /* WIN32 */

#include "ss7type.h"
#include "tcapdef.h"

#define TCAP_BACKUP     0x30
#define TCAP_PRIMARY    0x31
#define TCAP_STANDALONE 0x32

#ifdef UNIX
#define _getch getchar
#define Sleep(x) usleep(x * 1000)
#endif

DWORD InitializeTCAP (U8);
DWORD retrieveMessage(void);
DWORD TermTCAP       (void);
DWORD Send800        (void);
DWORD End800         (TcapTransInfo*, TcapComp*, unsigned char*);

U8    boardNum   = 1;
U32   pointCode;
U8    sapid      = 0;
U8    ssn        = 254;
U8    destssn    = 254;
U32   loops      = 1;
U8    inloop     = 1;
U32   delay      = 5000;

CTAHD      FstCtaHd  = NULL_CTAHD;
CTAQUEUEHD FstQueue;

#ifdef UNIX
BOOL dumpflag   = FALSE;
BOOL serverflag = FALSE;
#else
BOOLEAN dumpflag   = FALSE;
BOOLEAN serverflag = FALSE;
#endif

S8  strnum[32];
U8  phnum[5];
U8  AddrType = 0;           /* default to ITU addressing */
U8  runState;

void ErrorAndExit(CTAHD ctahd, DWORD errId, char *str);

void usage(char *pname, int opt)
{
    fprintf(stderr, "opt - %d\n"
                    "Usage: %s [options] pointcode:ssn phonenum\n"
                    "\n"
                    "where:\n"
                    "       pointcode is the destination in the form n.n.n\n"
                    "       ssn is the destination subsystem number(0 to 255)\n"
                    "\n"
                    "       phonenum is the phone number to be decoded(i.e. 8004321234)\n"
                    "\n"
                    "options:\n"
                    "       -b number       specify board, default 1, max 8\n"
                    "       -p number       specify SAP ID, default 0, max 255\n"
                    "       -n number       specify subsytem number, default 254, max 255\n"
                    "       -i number       specify number of iterations, default 1, max 1,000,000,000\n"
                    "       -j number       specify delay between iterations(msec), default 5000, max 1,000,000\n"
                    "       -s              act as server, default act as client\n"
                    "       -d              detailed dump of each packet sent/rcvd\n"
                    "       -a              use ANSI addressing(default: ITU)\n"
                    "\n", opt, pname);
    exit(128);
}

int main( int argc, char *argv[] )
{
    DWORD        Ret;
    CTA_EVENT    Event;
    char        *p;
    int          shvar;
    short        optind, i;
    int          lssn;
    int          hbyte, mbyte, lbyte;
    U8           ch;
    int          c;
    U8           congLvl;
    TCAPAPISTATS tcApiStats;
    char         text[40];

    /* parse command line for optional parameters */
    for( optind = 1; optind < argc; optind++ )
    {
        p = argv[optind];
        if ( *p == '-' )
        {
           switch (*++p)
           {
              case 's' :     /* server flag */
                  serverflag = TRUE;;
                  break;

              case 'd' :     /* dump flag */
                  dumpflag = TRUE;
                  break;

              case 'b' :    /* Set Board number */
                  if (*++p)         /* was there no space ? */
                      boardNum = (U8)atoi(p);
                  else
                      boardNum = (U8)atoi(argv[++optind]);

                  if((boardNum < 1) || (boardNum > 8) )
                       usage(argv[0], optind);

                  break;

              case 'p' :    /* Set SAP ID */
                  if (*++p)         /* was there no space ? */
                      shvar = atoi(p);
                  else
                      shvar = atoi(argv[++optind]);

                  if((shvar < 0) || (shvar > 255) )
                       usage(argv[0], optind);

                  sapid = (U8)shvar;
                  break;

              case 'n' :    /* Set Subsytem Number */
                  if (*++p)         /* was there no space ? */
                      shvar = atoi(p);
                  else
                      shvar = atoi(argv[++optind]);

                  if((shvar < 0) || (shvar > 255) )
                       usage(argv[0], optind);

                  ssn = (U8)shvar;
                  break;

              case 'i' :    /* Set number of iterations */
                  if (*++p)         /* was there no space ? */
                      shvar = atoi(p);
                  else
                      shvar = atoi(argv[++optind]);

                  if((shvar < 0) || (shvar > 1000000000) )
                       usage(argv[0], optind);

                  loops = shvar;
                  break;

              case 'j' :    /* Set delay between iterations */
                  if (*++p)         /* was there no space ? */
                      delay = atoi(p);
                  else
                      delay = atoi(argv[++optind]);

                  if((delay < 0) || (delay > 1000000) )
                       usage(argv[0], optind);

                  break;

              case 'a' :     /* specify ANSI addressing */
                  AddrType = 1;
                  break;

              default:
                  usage(argv[0], optind);
                  break;
           }       /* end switch */
        }
        else
        {
           /* end of optional parameters */
           break;          /* quit for loop without incrementing optind */
        }
    }         /* end for */

    /* initialize API */
    if ( (Ret = InitializeTCAP(boardNum)) != TCAP_SUCCESS )
    {
        fprintf(stderr, "Error %d initializing API\n", Ret);
        exit(130);
    }

    if ( !serverflag )
    {
        if (optind >= argc)
        {
            /* no destination point code specified */
            usage(argv[0], optind);
        }

        if (sscanf(argv[optind], "%u.%u.%u:%u", &hbyte, &mbyte, &lbyte, &lssn) != 4)
        {
            usage(argv[0], optind);
        }

        if ((hbyte > 255) || (mbyte > 255) || (lbyte > 255))
        {
            usage(argv[0], optind);
        }

        pointCode = (hbyte << 16) + (mbyte << 8) + lbyte;
        destssn = (U8)lssn;
        ++optind;

        if (optind >= argc)
        {
           /* no phone number specified */
           usage(argv[0], optind);
        }

        /* read phone number */
        if (sscanf(argv[optind], "%s", strnum) != 1)
        {
            usage(argv[0], optind);
        }

        if ( strlen(strnum) < 10)
        {
            usage(argv[0], optind);
        }

        /* BCD encode the phone number */
        for ( i = 0; i < 5; i++)
        {
            /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
            ch = strnum[(i*2)] - 0x30;
            phnum[i] = (ch << 4) & 0xF0;
            ch = strnum[((i*2)+1)] - 0x30;
            phnum[i] += ch;
        }

        while ( inloop )
        {
           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, delay );
           if ( Ret != TCAP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }
           switch ( Event.id )
           {
               case TCAPEVN_DATA:
                   /* an TCAP event has occured, call TCAPRetrieveMessage() to get the event */
                   retrieveMessage();
                   break;

               case TCAPEVN_CONGEST:
                   /* TCAP layer or API is congested; retrieve some statistics and display them */
                   congLvl = (U8) Event.value;
                   if( (Ret = TCAPGetApiStats( Event.ctahd, &tcApiStats, 0 )) == TCAP_SUCCESS )
                   {
                        printf( "TCAP Congestion level %d, API queue length/max = %ld/%ld\n",
                                congLvl, tcApiStats.qCount, tcApiStats.qPeak );
                   }
                   else
                   {
                        strcpy( text, "" );
                        ctaGetText( Event.ctahd, Ret, text, sizeof( text ) );
                        printf( "TCAP Congestion level %d, API stats not available [error: 0x%lx, %s]\n",
                                congLvl, Ret, text );
                   }
                   break;

               case CTAEVN_WAIT_TIMEOUT:
                   /* Delay passed, check for keyboard input */
                   /* make sure a key was hit */
                   if (_kbhit())
                   {
                       c = _getch();
                       if ( c == 'q' || c == 'Q' )
                       {
                          inloop = 0;
                          break;
                       }
                   }

                   /* we want to sent 'loops' number of requests */
                   if ( loops <= 0)
                   {
                       inloop = 0;
                       break;
                   }

                   if ( runState != TCAP_BACKUP )
                   {
                       Send800();
                       loops--;
                   }

                   break;
           }
        }                          /* end while */
    }                              /* end if ( !serverflag ) */

    if ( serverflag )
    {
       /* if we are a server, wait for requests for 5 sec */
       while ( inloop )
       {
           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
           if ( Ret != TCAP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }

           switch ( Event.id )
           {
               case TCAPEVN_DATA:
                   /* an TCAP event has occured, call TCAPRetrieveMessage() to get the event */
                   retrieveMessage();
                   break;

               case TCAPEVN_CONGEST:
                   /* TCAP layer or API is congested; retrieve some statistics and display them */
                   congLvl = (U8) Event.value;
                   if( (Ret = TCAPGetApiStats( Event.ctahd, &tcApiStats, 0 )) == TCAP_SUCCESS )
                   {
                        printf( "TCAP Congestion level %d, API queue length/max = %ld/%ld\n",
                                congLvl, tcApiStats.qCount, tcApiStats.qPeak );
                   }
                   else
                   {
                        strcpy( text, "" );
                        ctaGetText( Event.ctahd, Ret, text, sizeof( text ) );
                        printf( "TCAP Congestion level %d, API stats not available [error: 0x%lx, %s]\n",
                                congLvl, Ret, text );
                   }
                   break;

               case CTAEVN_WAIT_TIMEOUT:
                   /* 5 seconds passed, check for keyboard input */
                   /* make sure a key was hit */
                   if (_kbhit())
                   {
                      c = _getch();
                      if (c == 'q' || c == 'Q')
                      {
                         inloop = 0;
                         break;
                      }
                   }
                   break;
           }
       }
    }

    /* if we fall through to here, user has killed us -
     * release any active connections                    */
    TermTCAP();
}

#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;

   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;

   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }

   return (RetCode);
}
#endif
