/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   find800.c   ANSI edition
**
**      Source code for an 800 number client/server that uses the
**      TCAP API. Contains the send and receive logic.
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
/* #include <sys/stream.h> */
#endif                                  /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#endif                                  /* __OS2__ */

#ifdef WIN32
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <conio.h>
#endif                                  /* WIN32 */

#include <time.h>
#include "ss7type.h"
#include "tcapdef.h"
#include "sccpdef.h"
#include "tcap800.h"
#include "osdep.h"

#define SUID        0x21    /* application reference number */
#define DPRCHAN     SUID    /* Dual Port RAM channel number */
#define BOARD_ZERO  0       /* our local board number       */
#define TRANS_ID    0       /* arbitrary transaction ID     */
#define COMP_800    0x01    /* component OP Code            */
#define NUMRetRes   3       /* 1/(random return results)    */
#define NUMTrans    100000  /* keeping transaction number   */

/* Macro to get a random integer within a specified range */
#define getrandom( min, max ) (( rand() % (int)((( max ) + 1 ) - ( min ))) + ( min ))

extern U8           boardNum;
extern U32          pointCode;
extern U8           sapid;
extern U8           ssn;
extern U8           destssn;
extern U8           AddrType;

extern CTAHD        FstCtaHd;
extern CTAQUEUEHD   FstQueue;
extern U8           phnum[5];
extern U8           runState;
extern U8           readFlag;
extern FILE        *name;

#ifdef UNIX
extern BOOL dumpflag;
extern BOOL serverflag;
extern BOOL SendPending;
extern BOOL noresponseflag;
#else
extern BOOLEAN dumpflag;
extern BOOLEAN serverflag;
extern BOOLEAN SendPending;
extern BOOLEAN noresponseflag;
#endif

U8            invoke_id = 0;
U8            pend_id;
U8            transKeep[NUMTrans];
U32           trans_id  = TRANS_ID;
U32           send_num  = 0;
U32           rspd_num  = 0;
unsigned char phrealnum[5];
char          str800num[11];
char          strrealnum[11];
unsigned char pbuf[512];
char          notfound[] = "not found";
TcapRecvInfo  infoBlkPvs;
TcapComp      compPvs;

U32      connPerSec = 0;      /* start with clear per-second statistics */
time_t   lastPerSec = 0;      /* last time per-second stats output */

void check_per_second( S8 *who, S8 *what );

unsigned char query800[] =
             {
              0xF2, 0x0B,                             /* Set Parameter */
              0x84, 0x09, 0xF2, 0x0B, 0xF4, 0x0A,     /* Digit parameter */
              0x00, 0x00, 0x00, 0x00, 0x00,           /* BCD encoded phone number */
              0x00                                    /* NULL terminator, just for the heck of it */
             };


void ErrorAndExit(DWORD errId, char *str)
{
   printf( "Error : (%d) error returned in (%s) call.\n", errId, str );
   exit( 1 );
}

/*
  func to dump data in hex+ascii format

  IN:  Buf        data to show
       ByteCnt    # Bytes in Buf
  OUT: <file>      gets multiple lines
*/
static void dumpMsg(char          *Buf,
                    unsigned short ByteCnt)
{
   unsigned short Li, LMax, ToDo, Bi, Ai;
   char           Ascii[80];
   unsigned char  B;

   /* Dump to std out in hex followed by ascii */
   for( ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax )
   {
         LMax = (ToDo > 16) ? 16 : ToDo;             /* max bytes this line */
         printf("%04X| ", Bi);
         Ai = 0;

         for( Li = 0; Li < LMax; Li++ )              /* each byte this line */
         {
                B = *Buf++;                          /* next byte to show */
                printf("%02X ", B);                  /* as hex */
                if (!isprint(B))                     /* printable ascii? */
                       B = '.';                      /* no, use this */
                Ascii[Ai++] = B;                     /* into ascii portiob */
         }      /* for Li */

         for( Li = LMax; Li < 16; Li++ )             /* pad short ln if necessary */
                printf("   ");                       /* so asci lines up */

         Ascii[Ai] = 0;                              /* terminate the ascii */
         printf("    ");
         printf("%s\n", Ascii);                      /* show the ascii */
         Bi += LMax;
   }    /* for ToDo */
}       /* dumpMsg */

DWORD NMSSTDCALL ErrHdlr( CTAHD ctahd,
                          DWORD errorcode,
                          char *errtxt,
                          char *func )
{
    /* print out the error and exit */
    char text[40];
    ctaGetText(ctahd, errorcode, text, sizeof(text) );
    printf( "ErrorHandler: %s on ctahd=%#x, %s\n", errtxt, ctahd, func );
    return errorcode;
}

DWORD InitializeTCAP(U8 boardNum)
{
    CTA_EVENT        Event;
    DWORD            Ret;
    CTA_SERVICE_DESC tcapOpenSvcLst[]   = {{{"TCAP", "TCAPMGR"}, {0}, {0}, {0}}};
    CTA_SERVICE_NAME tcapServiceNames[] = {{ "TCAP", "TCAPMGR" }};
    CTA_INIT_PARMS   tcapInitparms      = { 0 };
    CTA_REV_INFO     info;

    ctaGetVersion(&info, sizeof(CTA_REV_INFO));
    if (info.compatlevel != CTA_COMPATLEVEL)
    {
        printf("CTA compatibility level does not match. re-compile.\n");
        exit(1);
    }

    /* bind this application as a user of the TCAP layer */
    tcapInitparms.size           = sizeof(CTA_INIT_PARMS);
    tcapInitparms.traceflags     = CTA_TRACE_ENABLE;
    tcapInitparms.parmflags      = CTA_PARM_MGMT_SHARED;
    tcapInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    /* ctaSetErrorHandler(ErrHdlr, NULL); */

    Ret = ctaInitialize( tcapServiceNames,
                         sizeof(tcapServiceNames) / sizeof(tcapServiceNames[0]),
                        &tcapInitparms);
    if (Ret != TCAP_SUCCESS)
    {
        /* ctdaemon is not running.
        *   Initialize without tracing.
        */
        tcapInitparms.traceflags = 0;
        tcapInitparms.parmflags  = 0;

        Ret = ctaInitialize( tcapServiceNames,
                             sizeof(tcapServiceNames) / sizeof(tcapServiceNames[0]),
                            &tcapInitparms);
        if (Ret != SUCCESS)
        {
            ErrorAndExit( Ret, "ctaInitialize" );
        }
    }

    Ret = ctaCreateQueue( NULL, 0, &FstQueue );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateQueue" );
    }

    /* Open FstCtaHd on FstQueue                   */
    Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateContext" );
    }

    tcapOpenSvcLst[0].svcargs.args[0] = boardNum;           /* board number             */
    tcapOpenSvcLst[0].svcargs.args[1] = DPRCHAN + sapid;    /* srcEnt                   */
    tcapOpenSvcLst[0].svcargs.args[2] = BOARD_ZERO;         /* srcInst                  */
    tcapOpenSvcLst[0].svcargs.args[3] = SUID;               /* suId                     */
    tcapOpenSvcLst[0].svcargs.args[4] = sapid;              /* spId                     */
    tcapOpenSvcLst[0].svcargs.args[5] = ssn;                /* ssn                      */

    Ret = ctaOpenServices( FstCtaHd, tcapOpenSvcLst, 1 );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete.*/
    Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
    if (Ret != TCAP_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    if( Event.id == CTAEVN_OPEN_SERVICES_DONE )
    {
        if( Event.value != CTA_REASON_FINISHED )
        {
            printf("ERROR: Reason Value = %d ", Event.value);
            ErrorAndExit( Ret, "Reason value" );
        }
    }
    else
    {
        if( Event.id == CTAEVN_WAIT_TIMEOUT )
        {
            ErrorAndExit( Ret, "ctaWaitEvent: REASON = TIMEOUT " );
        }
        else
        {
            printf("ERROR: Open Services failed  id = %d ", Event.id);
            ErrorAndExit( Ret, "Open Services failed" );
        }
    }

    /* send message that subsystem is in service */
    TCAPStateReq(FstCtaHd, sapid, ssn, SS_OOS);
    return(TCAPStateReq(FstCtaHd, sapid, ssn, SS_IS));
}

DWORD TermTCAP(void)
{
    CTA_EVENT  Event;
    DWORD      Ret;
    char*      tcapCloseSvcLst[] = {"TCAP"};

  printf("TermTCAP: %d were sent\n          %d were responded\n", send_num, rspd_num);

  /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, tcapCloseSvcLst, 1 );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != TCAP_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyQueue" );
   }

#ifdef WIN32
   _fcloseall();
#endif

  for( Ret = 1; Ret < NUMTrans; Ret++ )
        if( transKeep[Ret] > 1 && Ret <= trans_id && !serverflag )
            printf("Trans = %d has status %d\n", Ret, transKeep[Ret]);

   return TCAP_SUCCESS;
}

DWORD Send800(U8 last)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    if( dumpflag )
    {
        printf("Send800, pc = %d.%d.%d:%d  trans = %d ",
            ((pointCode >> 16) & 0xff)  ,
            ((pointCode >>  8) & 0xff)  ,
            ((pointCode >>  0) & 0xff)  ,
                destssn,
                trans_id);
    }

    if ( send_num > 0x7ffffff0)
    {
        trans_id = TRANS_ID;
        send_num = 0;
        rspd_num = 0;
    }

    while( transKeep[(trans_id + 1) % NUMTrans] > 1 )
    {
        trans_id++;
    }

    send_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_QRY_PRM;
    tInfo.suDlgId        = ++trans_id;  /* arbitrary value */
    tInfo.spDlgId        = 0;         /* must be zero on first request for a transaction */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType )
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = destssn;
    tInfo.cdAddr.pointCode    = pointCode;
    tInfo.cdAddr.glTransType  = 0; /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    /* increment invoke_id */
    if ( invoke_id == 0xFF )
       invoke_id = 0x00;
    else
       invoke_id++;

    if( dumpflag )
    {
        printf("invoke_id = %d\n",invoke_id);
    }

    if ( last )
       Comp.compType                          = TCAP_INVOKE_L;
    else
       Comp.compType                          = TCAP_INVOKE_NL;

    Comp.uProt.ansiComp.invokeId.present      = PRESENT;
    Comp.uProt.ansiComp.invokeId.compId       = invoke_id;
    Comp.uProt.ansiComp.corrId.present        = NOT_PRESENT;
    Comp.uProt.ansiComp.corrId.compId         = 0;
    Comp.uProt.ansiComp.invokeTimer           = 10;        /* use default */
    Comp.uProt.ansiComp.paramFlg              = TCAP_SET;
    Comp.uProt.ansiComp.uComp.opcode.opCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.opcode.opFamily = TCAP_ANSI_NOF_PR_INS;
    Comp.uProt.ansiComp.uComp.opcode.opSpec   = TCAP_ANSI_OS_START;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);


    /* load BCD encoded phone number */
    memcpy(&query800[8], phnum, 5);

    /* copy 13 bytes of 800 query into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    transKeep[trans_id % NUMTrans] = 2;
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }
    else
    {
       SendPending = TRUE;
       pend_id = invoke_id;
       connPerSec++;                 /* 1 more Connection Request transmitted */
       if ( (connPerSec & 0xFFFFFFE0) == connPerSec )  /* only consider every 32 connects */
            check_per_second( "tcapdemo", "Requests" );    /* check whether time to output period stats */

    }

    /* release Msg and Parameter Buffer */
    free(Msg);
    free(pParam);

    return retVal;
}

DWORD SendCon800(U32 spId, U8 last)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    if( dumpflag )
    {
        printf("SendCon800, pc = %d.%d.%d:%d ",
            ((pointCode >> 16) & 0xff)  ,
            ((pointCode >>  8) & 0xff)  ,
            ((pointCode >>  0) & 0xff)  , destssn);
    }
    send_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    if ( last )
       tInfo.msgType     = TCAP_CNV_PRM;
    else
       tInfo.msgType     = TCAP_CNV_NO_PRM;

    tInfo.suDlgId        = trans_id;  /* arbitrary value */
    tInfo.spDlgId        = spId;      /* must be zero on first request for a transaction */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType )
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = destssn;
    tInfo.cdAddr.pointCode    = pointCode;
    tInfo.cdAddr.glTransType  = 0; /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    /* increment invoke_id */
    if ( invoke_id == 0xFF )
       invoke_id = 0x00;
    else
       invoke_id++;

    if( dumpflag )
    {
        printf("invoke_id = %d\n",invoke_id);
    }

    if ( last )
       Comp.compType                          = TCAP_INVOKE_L;
    else
       Comp.compType                          = TCAP_INVOKE_NL;

    Comp.uProt.ansiComp.invokeId.present      = PRESENT;
    Comp.uProt.ansiComp.invokeId.compId       = invoke_id;
    Comp.uProt.ansiComp.corrId.present        = NOT_PRESENT;
    Comp.uProt.ansiComp.corrId.compId         = 0;
    Comp.uProt.ansiComp.invokeTimer           = 10;        /* use default */
    Comp.uProt.ansiComp.paramFlg              = TCAP_SET;
    Comp.uProt.ansiComp.uComp.opcode.opCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.opcode.opFamily = TCAP_ANSI_NOF_PR_INS;
    Comp.uProt.ansiComp.uComp.opcode.opSpec   = TCAP_ANSI_OS_START;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);


    /* load BCD encoded phone number */
    memcpy(&query800[8], phnum, 5);

    /* copy 13 bytes of 800 query into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    transKeep[trans_id % NUMTrans]++;
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }
    else
    {
       SendPending = TRUE;
       pend_id = invoke_id;

       connPerSec++;                 /* 1 more Connection Request transmitted */
       if ( (connPerSec & 0xFFFFFFE0) == connPerSec )  /* only consider every 32 connects */
            check_per_second( "tcapdemo", "Requests" );    /* check whether time to output period stats */

    }

    /* release Msg and Parameter Buffer */
    free(Msg);
    free(pParam);

    return retVal;
}

DWORD SendConRR(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    if( dumpflag )
    {
        printf( "SendConRR, pc = %d.%d.%d:%d, invoke_id = %d\n suDlgId = %d, spDlgId = %d\n",
            ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
                srctInfo->cgAddr.subsystem,
                srccomp->uProt.ansiComp.invokeId.compId + 1,
                srctInfo->suDlgId,
                srctInfo->spDlgId );
    }
   rspd_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_CNV_NO_PRM;
    tInfo.suDlgId        = srctInfo->suDlgId; /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* first request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem; /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                          /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    Comp.compType                        = TCAP_INVOKE_NL;
    Comp.uProt.ansiComp.invokeId.present = PRESENT;
    Comp.uProt.ansiComp.invokeId.compId  = srccomp->uProt.ansiComp.invokeId.compId + 1;
    Comp.uProt.ansiComp.corrId.present   = NOT_PRESENT;
    Comp.uProt.ansiComp.corrId.compId    = 0;
    Comp.uProt.ansiComp.invokeTimer      = 10;        /* use default */
    Comp.uProt.ansiComp.paramFlg         = TCAP_SET;

    Comp.uProt.ansiComp.uComp.opcode.opCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.opcode.opFamily = TCAP_ANSI_NOF_PR_INS;
    Comp.uProt.ansiComp.uComp.opcode.opSpec   = TCAP_ANSI_OS_START;

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phonenum, 5);

    /* copy 13 bytes of 800 response into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    transKeep[trans_id % NUMTrans]--;
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);
    free(pParam);

    return retVal;
}

DWORD Respond800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum, U8 last)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    if( dumpflag )
    {
        printf( "Respond800, pc = %d.%d.%d:%d, invoke_id = %d\n suDlgId = %d, spDlgId = %d\n",
            ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
                srctInfo->cgAddr.subsystem,
                srccomp->uProt.ansiComp.invokeId.compId,
                srctInfo->suDlgId,
                srctInfo->spDlgId );
    }
   rspd_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_RESPONSE;
    tInfo.suDlgId        = srctInfo->suDlgId; /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem; /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                          /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    if ( last )
       Comp.compType                     = TCAP_RET_RES_L;
    else
       Comp.compType                     = TCAP_RET_RES_NL;

    Comp.uProt.ansiComp.invokeId.present = NOT_PRESENT;
    /* use invoke ID from recevied packet */
    Comp.uProt.ansiComp.invokeId.compId  = 0;
    Comp.uProt.ansiComp.corrId.present   = PRESENT;
    Comp.uProt.ansiComp.corrId.compId    = srccomp->uProt.ansiComp.invokeId.compId;
    Comp.uProt.ansiComp.invokeTimer      = 0;        /* use default */
    Comp.uProt.ansiComp.paramFlg         = TCAP_SET;
    /* not used for replies */
    /*
    Comp.uProt.ansiComp.uComp.opcode.opCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.opcode.opFamily = TCAP_ANSI_1;
    Comp.uProt.ansiComp.uComp.opcode.opSpec   = COMP_800;
    */

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phonenum, 5);

    /* copy 13 bytes of 800 response into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    transKeep[trans_id % NUMTrans]--;
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);
    free(pParam);

    return retVal;
}

DWORD RespondCon800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum, U8 last)
{
    U8             *Msg, *pParam;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;
    U16             len;

    if( dumpflag )
    {
        printf( "RespondCon800, pc = %d.%d.%d:%d, invoke_id = %d\n suDlgId = %d, spDlgId = %d\n",
            ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
                srctInfo->cgAddr.subsystem,
                srccomp->uProt.ansiComp.invokeId.compId,
                srctInfo->suDlgId,
                srctInfo->spDlgId );
    }
   rspd_num++;

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    if ( last )
       tInfo.msgType     = TCAP_CNV_PRM;
    else
       tInfo.msgType     = TCAP_CNV_NO_PRM;

    tInfo.suDlgId        = srctInfo->suDlgId; /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem; /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                          /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    if ( last )
       Comp.compType                     = TCAP_RET_RES_L;
    else
       Comp.compType                     = TCAP_RET_RES_NL;

    Comp.uProt.ansiComp.invokeId.present = NOT_PRESENT;
    /* use invoke ID from recevied packet */
    Comp.uProt.ansiComp.invokeId.compId  = 0;
    Comp.uProt.ansiComp.corrId.present   = PRESENT;
    Comp.uProt.ansiComp.corrId.compId    = srccomp->uProt.ansiComp.invokeId.compId;
    Comp.uProt.ansiComp.invokeTimer      = 0;        /* use default */
    Comp.uProt.ansiComp.paramFlg         = TCAP_SET;
    /* not used for replies */
    /*
    Comp.uProt.ansiComp.uComp.opcode.opCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.opcode.opFamily = TCAP_ANSI_1;
    Comp.uProt.ansiComp.uComp.opcode.opSpec   = COMP_800;
    */

    /* allocate parameter buffer */
    if ((pParam = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load BCD encoded phone number */
    memcpy(&query800[8], phonenum, 5);

    /* copy 13 bytes of 800 response into parameter buffer */
    len = 13;
    memcpy(pParam, query800, len);

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, len, pParam)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    transKeep[trans_id % NUMTrans]--;
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);
    free(pParam);

    return retVal;
}

DWORD Failed800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *phonenum)
{
    U8             *Msg;
    TcapTransInfo   tInfo;
    TcapComp        Comp;
    DWORD           retVal;

    UNREFERENCED_PARAMETER(phonenum);

    printf( "Failed800, pc = %d.%d.%d:%d, invoke_id = %d\n",
           ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
           ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
             srctInfo->cgAddr.subsystem,
             srccomp->uProt.ansiComp.invokeId.compId);

    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    tInfo.msgType        = TCAP_RESPONSE;
    tInfo.suDlgId        = ++trans_id;        /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId; /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem;  /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0;                           /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0;           /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on InitTrans\n");
    }

    Comp.compType                               = TCAP_RET_ERR;
    Comp.uProt.ansiComp.invokeId.present        = NOT_PRESENT;
    /* use invoke ID from recevied packet */
    Comp.uProt.ansiComp.invokeId.compId         = 0;
    Comp.uProt.ansiComp.corrId.present          = PRESENT;
    Comp.uProt.ansiComp.corrId.compId           = srccomp->uProt.ansiComp.invokeId.compId;
    Comp.uProt.ansiComp.invokeTimer             = 0;        /* use default */
    Comp.uProt.ansiComp.paramFlg                = TCAP_SET;
    Comp.uProt.ansiComp.uComp.errcode.errCodeId = TCAP_NATIONAL;
    Comp.uProt.ansiComp.uComp.errcode.errCode   = TCAP_ANSI_ERR_DAT_UA;

    /* add component to transaction */
    if ((retVal = TCAPAddComp(Msg, &Comp, 0, NULL)) != TCAP_SUCCESS)
    {
        printf("Error on AddComp, err = %d\n", retVal);
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
    {
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);

    return retVal;
}

DWORD End800(void)
{
    U8             *Msg;
    TcapTransInfo   tInfo;
    DWORD           retVal;

    if( dumpflag )
    {
        printf( "End800, pc = %d.%d.%d:%d\n",
            ((pointCode >> 16) & 0xff),
            ((pointCode >>  8) & 0xff),
            ((pointCode >>  0) & 0xff), destssn);
    }
    if ((Msg = (U8 *)malloc(2048)) == NULL)
         return(ENOMEM);

    /* load transaction info */
    /* modify to TCAP_ABORT or TCAP_END or TCAP_CONTINUE */
    tInfo.msgType        = TCAP_RESPONSE;

    /* if ABORT, set AbortCause */

    /* modify su to copy input if this is a CONTINUE */
    tInfo.suDlgId        = trans_id;         /* arbitrary value */
    tInfo.spDlgId        = 0;
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    /* modify if this is a prearanged END */
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = destssn;      /* calling address is now called address */
    tInfo.cdAddr.pointCode    = pointCode;
    tInfo.cdAddr.glTransType  = 0;            /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0;            /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);

    return retVal;
}

DWORD Cnv800(TcapTransInfo *srctInfo, TcapComp *srccomp, unsigned char *fphonenum)
{
    U8             *Msg;
    TcapTransInfo   tInfo;
    DWORD           retVal;

    UNREFERENCED_PARAMETER(fphonenum);

    if( dumpflag )
    {
        printf( "Conv800, pc = %d.%d.%d:%d, invoke_id = %d\n",
            ((srctInfo->cgAddr.pointCode >> 16) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  8) & 0xff),
            ((srctInfo->cgAddr.pointCode >>  0) & 0xff),
                srctInfo->cgAddr.subsystem,srccomp->uProt.ansiComp.invokeId.compId);
    }
    if ((Msg = (U8 *)malloc(2048)) == NULL)
        return(ENOMEM);

    /* load transaction info */
    /* modify to TCAP_ABORT or TCAP_END or TCAP_CONTINUE */
    tInfo.msgType        = TCAP_CNV_PRM;

    /* if ABORT, set AbortCause */

    /* modify su to copy input if this is a CONTINUE */
    tInfo.suDlgId        = ++trans_id;           /* arbitrary value */
    tInfo.spDlgId        = srctInfo->spDlgId;  /* use Dlg ID from request */
    tInfo.qos.priority   = 0;
    tInfo.qos.retOpt     = MSG_RETURN;
    tInfo.qos.seqDlvy    = TCAP_QOS_NORMAL;
    /* modify if this is a prearanged END */
    tInfo.preArgEnd      = 0;
    tInfo.chkpt          = TCAP_CHKPT_DEFAULT;
    tInfo.inactvTimer    = 0;
    tInfo.cdAddr.presInd = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType)
        tInfo.cdAddr.swType   = SW_ANSI;
    else
        tInfo.cdAddr.swType   = SW_ITU;

    tInfo.cdAddr.subsystemInd = SUBSYS_INC;
    tInfo.cdAddr.pointCodeInd = PTCODE_INC;
    tInfo.cdAddr.glTitleInd   = GLT_NONE;
    tInfo.cdAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cdAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cdAddr.subsystem    = srctInfo->cgAddr.subsystem;      /* calling address is now called address */
    tInfo.cdAddr.pointCode    = srctInfo->cgAddr.pointCode;
    tInfo.cdAddr.glTransType  = 0; /* not used */
    tInfo.cdAddr.encoding     = ENC_UNKNOWN;
    tInfo.cdAddr.numPlan      = NP_UNK;
    tInfo.cdAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cdAddr.glTitleLen   = 0;
    /* not using glTitle */
    tInfo.cgAddr.presInd      = PRESENT;

    /* If the high byte is zero, assume this is an ITU address */
    if ( AddrType )
        tInfo.cgAddr.swType   = SW_ANSI;
    else
        tInfo.cgAddr.swType   = SW_ITU;

    tInfo.cgAddr.subsystemInd = SUBSYS_NONE;
    tInfo.cgAddr.pointCodeInd = PTCODE_NONE;
    tInfo.cgAddr.glTitleInd   = GLT_NONE;
    tInfo.cgAddr.routingInd   = ROUTE_PC_SN;
    tInfo.cgAddr.natIntInd    = ADDRIND_NAT;
    tInfo.cgAddr.subsystem    = 0;
    tInfo.cgAddr.pointCode    = 0;
    tInfo.cgAddr.glTransType  = 0; /* not used */
    tInfo.cgAddr.encoding     = ENC_UNKNOWN;
    tInfo.cgAddr.numPlan      = NP_UNK;
    tInfo.cgAddr.natAddrInd   = NATIND_SUBS;
    tInfo.cgAddr.glTitleLen   = 0;

    /* initialize the transaction */
    if ((retVal = TCAPInitTrans(Msg, &tInfo, NULL, 0, NULL)) != TCAP_SUCCESS)
	{
        printf("Error on InitTrans\n");
    }

    /* send transaction */
    if ((retVal = TCAPTransRqst(FstCtaHd, sapid, Msg)) != TCAP_SUCCESS)
	{
        printf("Error on TransRqst, err = %d\n", retVal);
    }

    free(Msg);

    return retVal;
}

DWORD retrieveMessage(TcapRecvInfo* infoBlk)
{
    U8            Msg[800];
    TcapComp      comp = {0};
    DWORD         Ret;
    U8            index;
    U16           plen;
    unsigned char i, ch;
    Trans         evt;

#ifdef UNIX
    FILE         *hFile;
    char          line[40];
    char         *xptr, *yptr;
#endif

    /* get TCAP message from API */
            if ((Ret = TCAPRetrieveMessage(FstCtaHd, (U8 *)Msg, infoBlk)) != TCAP_SUCCESS)
            {
               if( Ret != TCAP_NOMSG )  /* NOMSG is valid - could be congestion indication */
                  printf("TCAPRetrieveMessage failed, %d\n", Ret);
               return(TCAP_NOMSG);
            }

            switch (infoBlk->eventType)
            {
                case TCAP_EVENT_RUN_STATE:
                {
                    if ( runState                       == TCAP_PRIMARY &&
                         infoBlk->event.runstate.status == TCAP_BACKUP )
                    {
                        printf("RUN STATE: Send CheckPoint\n");
                        evt.position  = ftell(name);
                        evt.transId   = trans_id;
                        evt.invokeId  = invoke_id;
                        evt.readFlag  = readFlag;
                        CheckPoint(&evt);
                    }

                    runState = infoBlk->event.runstate.status;

                    if ( runState == TCAP_BACKUP )
                        readFlag = 0;

                    switch (runState)
                    {
                    case TCAP_BACKUP:
                         printf("RUN STATE Indication = BACKUP\n");
                         break;
                    case TCAP_PRIMARY:
                         printf("RUN STATE Indication = PRIMARY\n");
                         break;
                    case TCAP_STANDALONE:
                         printf("RUN STATE Indication = STANDALONE\n");
                         break;
                    default:
                         printf("ERROR: RUN STATE Indication = UNKNOWN\n");
                         break;
                    }
                    break;
                }

                case TCAP_EVENT_STA_IND:
                {
                    if( dumpflag )
                    {
                        printf("Status Indication received, status = %d, suId = %d\n",
                                infoBlk->event.status.status,
                                infoBlk->event.status.suId);
                    }
                    break;
                }

                case TCAP_EVENT_COORD_IND:
                    printf("Coordinated Status change, SSN = %d, smi = %d\n",
                            infoBlk->event.coord.aSsn,
                            infoBlk->event.coord.smi);
                    break;

                case TCAP_EVENT_COORD_CFM:
                {
                    printf("Coordinated Status change confirm, SSN = %d, smi = %d\n",
                            infoBlk->event.coord.aSsn,
                            infoBlk->event.coord.smi);
                    break;
                }

                case TCAP_EVENT_SSN_STATE:
                {
                    printf(  "Subsystem status change, pc = %d.%d.%d:%d, smi = %d, status = %d\n",
                            ((infoBlk->event.ssnstate.aPc >> 16) & 0xff),
                            ((infoBlk->event.ssnstate.aPc >>  8) & 0xff),
                            ((infoBlk->event.ssnstate.aPc >>  0) & 0xff),
                              infoBlk->event.ssnstate.aSsn,
                              infoBlk->event.ssnstate.smi,
                              infoBlk->event.ssnstate.status);
                    break;
                }

                case TCAP_EVENT_PC_STATE:
                {
                    printf(  "PointCode status change, pc = %d.%d.%d, status = %d\n",
                            ((infoBlk->event.pcstate.aPc >> 16) & 0xff),
                            ((infoBlk->event.pcstate.aPc >>  8) & 0xff),
                            ((infoBlk->event.pcstate.aPc >>  0) & 0xff),
                              infoBlk->event.pcstate.status);
                    break;
                }

                case TCAP_EVENT_NOT_IND:
                {
                    printf("Notify Event received - cause = %d\n", infoBlk->event.notif.retcause);
                    printf("TransInfo, event = %d abort cause = %d\nsuId = %d, spId = %d, cdpc = %d.%d.%d:%d, cgpc = %d.%d.%d:%d\n",
                              infoBlk->event.notif.transInfo.msgType, infoBlk->event.notif.transInfo.abortCause,
                              infoBlk->event.notif.transInfo.suDlgId, infoBlk->event.notif.transInfo.spDlgId,
                            ((infoBlk->event.notif.transInfo.cdAddr.pointCode >> 16) & 0xff),
                            ((infoBlk->event.notif.transInfo.cdAddr.pointCode >>  8) & 0xff),
                            ((infoBlk->event.notif.transInfo.cdAddr.pointCode >>  0) & 0xff),
                              infoBlk->event.notif.transInfo.cdAddr.subsystem,
                            ((infoBlk->event.notif.transInfo.cgAddr.pointCode >> 16) & 0xff),
                            ((infoBlk->event.notif.transInfo.cgAddr.pointCode >>  8) & 0xff),
                            ((infoBlk->event.notif.transInfo.cgAddr.pointCode >>  0) & 0xff),
                              infoBlk->event.notif.transInfo.cgAddr.subsystem);
                    break;
                }

                case TCAP_EVENT_DAT_IND:
                {
                    if ( runState == TCAP_BACKUP )
                    {
                         printf("Data Indication received but ignored. Reason I AM BACKUP\n");
                         break;
                    }

                    if( serverflag )
                    {
                        connPerSec++;                 /* 1 more Connection indication received */
                        if ( (connPerSec & 0xFFFFFFE0) == connPerSec )      /* only consider every 32 connects */
                            check_per_second( "tcapdemo", "Indications" );  /* check whether time to output period stats */
                        if( noresponseflag )  /* don't respond to the message to get open transactions */
                                break;
                    }

                    if ( dumpflag )
                    {
                       printf("Data Indication received - from pc = %d.%d.%d\n",
                            ((infoBlk->event.data.opc >> 16) & 0xff),
                            ((infoBlk->event.data.opc >>  8) & 0xff),
                            ((infoBlk->event.data.opc >>  0) & 0xff));
                       printf("TransInfo, event = %d abort cause = %d\nsuId = %d, spId = %d, cdpc = %d.%d.%d:%d, cgpc = %d.%d.%d:%d\n",
                            infoBlk->event.data.transInfo.msgType, infoBlk->event.data.transInfo.abortCause,
                            infoBlk->event.data.transInfo.suDlgId, infoBlk->event.data.transInfo.spDlgId,
                            ((infoBlk->event.data.transInfo.cdAddr.pointCode >> 16) & 0xff),
                            ((infoBlk->event.data.transInfo.cdAddr.pointCode >>  8) & 0xff),
                            ((infoBlk->event.data.transInfo.cdAddr.pointCode >>  0) & 0xff),
                            infoBlk->event.data.transInfo.cdAddr.subsystem,
                            ((infoBlk->event.data.transInfo.cgAddr.pointCode >> 16) & 0xff),
                            ((infoBlk->event.data.transInfo.cgAddr.pointCode >>  8) & 0xff),
                            ((infoBlk->event.data.transInfo.cgAddr.pointCode >>  0) & 0xff),
                            infoBlk->event.data.transInfo.cgAddr.subsystem);

                       /* dump dialog part and user info part */
                       if ( infoBlk->event.data.dlgPart.dlgType != TCAP_DLGP_NONE)
                       {
                          printf("\nDialog Part:\n");
                          dumpMsg((char *)&(infoBlk->event.data.dlgPart), sizeof(TcapDlgSect));
                       }
                       if ( infoBlk->event.data.userInfoLen)
                       {
                          printf("\nUser Info Part:\n");
                          dumpMsg((char *)infoBlk->event.data.pUserInfo, infoBlk->event.data.userInfoLen);
                       }
                    }

                    for ( index=0; index < infoBlk->event.data.numComps; index++)
                    {
                       plen = 512;
                       if ((Ret = TCAPGetComp((U8 *)&Msg, index, &comp, &plen, pbuf)) != TCAP_SUCCESS)
                       {
                          printf("TCAPGetComp failed, %d", Ret);
                          return(TCAP_NOMSG);
                       }

                       /*
                        * Check if the component matches the pending invoke ID. If so, the
                        * invoke is no longer pending.
                        */
                       if (pend_id == comp.uProt.ansiComp.corrId.compId)
                          SendPending = FALSE;

                       /* dump component */
                       if ( dumpflag)
                          dumpMsg((char *)&comp, sizeof(TcapComp));

                       /* dump parameter buffer */
                       if ( dumpflag)
                          dumpMsg((char *)pbuf,plen);
                    }

                    /* if we are a server, and this is an INVOKE request, respond */
                    if ( serverflag )
                    {
                       send_num++;

                       /* decode the BCD encoded phone number, put in str800num */
                       for ( i = 0; i < 5; i++)
                       {
                          /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
                          str800num[i*2] = ((pbuf[(8+i)] & 0xF0) >> 4) + 0x30;
                          str800num[(i*2 +1)] = ((pbuf[(8+i)]) & 0x0F) + 0x30;
                       }

                       str800num[10] = 0x00;     /* null terminate for getprofilestring to work */
                       if( dumpflag )
                       {
                        printf("server received %s",str800num);
                       }
                       /* find the corresponding real number, put in strrealnum[] */
#ifdef UNIX
                       strcpy(strrealnum,notfound);
                       hFile = fopen("./numbers.800","r");
                       if ( hFile != NULL)
                       {
                          /* get each line in file, in succession */
                          while ( fgets(line,40,hFile) != NULL)
                          {
                             while ( (strlen(line) > 0) && (line[strlen(line) - 1] < ' ') )
                                line[strlen(line) - 1] = '\0';  /* remove any trailing non-printable characters */
                             yptr = strchr(line,'=');
                             if ( yptr == NULL)
                                continue;
                             *yptr = 0x00;
                             yptr++;
                             xptr = line;
                             /* check if 800 number matches first entry in line */
                             if (!( strcmp(xptr,str800num)))
                                 strcpy(strrealnum,yptr);
                          }
                       }
                       fclose(hFile);
#else
                       GetPrivateProfileString("800 Numbers", str800num, notfound, strrealnum, 11, ".\\numbers.800");
#endif
                       if ( !(strcmp(strrealnum, notfound)) && comp.compType != TCAP_RET_RES_NL )
                       {
                          printf("  server didn't find a match\n");
                          Failed800(&(infoBlk->event.data.transInfo), &comp, phrealnum);
                       }
                       else
                       {
                          if( dumpflag )
                          {
                            printf("  server found the matching number %s\n",strrealnum);
                          }

                          /* BCD encode the phone number */
                          for ( i = 0; i < 5; i++)
                          {
                             /* 2 numbers in a byte  */
                             ch = strrealnum[(i*2)] - 0x30;
                             phrealnum[i] = (ch << 4) & 0xF0;
                             ch = strrealnum[((i*2)+1)] - 0x30;
                             phrealnum[i] += ch;
                          }

                          if ( (infoBlk->event.data.transInfo.msgType == TCAP_QRY_PRM))
                          {
                             if ( trans_id != TRANS_ID )
                             {
                                printf("End of trans, send Response\n");
                                End800();
                                trans_id = TRANS_ID;
                             }
                             else
                             {
                                trans_id = infoBlk->event.data.transInfo.spDlgId;
                             }
                          }

                          /* make sure to swap addresses and send back response */
                          switch ( comp.compType )
                          {
                          case TCAP_INVOKE_L:
                             if( readFlag == 0 )
                             {
                                readFlag = 1;
                                printf("Server responds for previous Conversation 0\n");
                                RespondCon800(&(infoBlkPvs.event.data.transInfo), &compPvs, phrealnum, 0);
                             }
                             trans_id = TRANS_ID;
                             Respond800(&(infoBlk->event.data.transInfo), &comp, phrealnum, 1);
                             break;

                          case TCAP_INVOKE_NL:
                             if ( !getrandom(0, NUMRetRes) &&
                                  (infoBlk->event.data.transInfo.msgType != TCAP_QRY_PRM) )
                             {
                                 printf("Server is sending Conversation\n");
                                 memcpy(&infoBlkPvs, infoBlk, sizeof(TcapRecvInfo));
                                 memcpy(&compPvs,   &comp,    sizeof(TcapComp));
                                 readFlag = 0;

                                 SendConRR(&(infoBlk->event.data.transInfo), &comp, phrealnum);
                             }
                             else
                             {
                                  if( readFlag == 0)
                                 {
                                    readFlag = 1;
                                    printf("Server responds for previous Conversation 1\n");
                                    RespondCon800(&(infoBlkPvs.event.data.transInfo), &compPvs, phrealnum, 0);
                                 }
                                 RespondCon800(&(infoBlk->event.data.transInfo), &comp, phrealnum, 0);
                             }
                             break;

                          case TCAP_RET_RES_NL:
                             readFlag = 1;
                             printf("Server responds for previous Conversation\n");
                             RespondCon800(&(infoBlkPvs.event.data.transInfo), &compPvs, phrealnum, 0);
                             break;

                          }
                       }
                    }                   /* if ( serverflag ) */

                    /* if we are a client, and this is a RET_RES request, print the number */
                    if ( !serverflag )
                    {
                       /* decode the BCD encoded phone number, put in strrealnum */
                       for ( i = 0; i < 5; i++)
                       {
                          /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
                          strrealnum[i*2] = ((pbuf[(8+i)] & 0xF0) >> 4) + 0x30;
                          strrealnum[(i*2 +1)] = ((pbuf[(8+i)]) & 0x0F) + 0x30;
                       }

                       switch ( comp.compType )
                       {
                       case TCAP_RET_RES_L:
                       case TCAP_RET_RES_NL:
                          readFlag = 1;
                          rspd_num++;
                          transKeep[trans_id % NUMTrans]--;

                          strrealnum[10] = 0x00;     /* null terminate for printf to work */
                          if( dumpflag )
                          {
                            printf("client got a returned actual number of %s\n",strrealnum);
                          }
                          break;

                       case TCAP_INVOKE_NL:
                          readFlag = 0;
                          rspd_num--;
                          transKeep[trans_id % NUMTrans]++;
                          invoke_id = comp.uProt.ansiComp.invokeId.compId;

                          if( dumpflag )
                          {
                            printf("client respond a 800-number\n");
                          }
                          RespondCon800(&(infoBlk->event.data.transInfo), &comp, phnum, 0);
                          break;
                       }
                    }

                    /* if we are a client, and this is a RET_ERR request, print the number */
                    if ((!serverflag) && (comp.compType == TCAP_RET_ERR) )
                    {
                       printf("client got a RET_ERR, 800 number not found\n");
                    }

                    /* Receive a Reject */
                    if (comp.compType == TCAP_REJECT)
                    {
                        /*
                         *  if the Reject was in a Query with Permission message, we MUST send a
                         *  Conversation or Response with no components. The TCAP stack will
                         *  attach the Reject component and send to the original sender.
                         */
                        if ( (infoBlk->event.data.transInfo.msgType == TCAP_QRY_PRM))
                        {
                           printf("got a QueryPrm/Reject, send Response\n");
                           End800();
                        }
                        /*
                         *  If the Reject was in a Query with No Permission message, we MUST
                         *  send a Conversation with no components. We aren't allowed to end
                         *  the trasnaction.
                         */
                        else if ( (infoBlk->event.data.transInfo.msgType == TCAP_QRY_NO_PRM))
                        {
                           printf("got a QueryNoPrm/Reject, send Conversation\n");
                           Cnv800(&(infoBlk->event.data.transInfo), &comp, phrealnum);
                        }
                        /*
                         *  If the Reject was in a Conversation w/Perm message, the TCAP stack has already sent
                         *  the rejected component back to the Sender. At this point, we can try to
                         *  continue the trasnaction or end it. For this sample, we end it.
                         */
                        else if ( (infoBlk->event.data.transInfo.msgType == TCAP_CNV_PRM))
                        {
                           printf("got a CnvPrm/Reject, send Response\n");
                           End800();
                        }
                        /*
                         *  If the Reject was in a Conversation w/o Permission message, the TCAP
                         *  stack has already sent the rejected component back to the Sender.
                         *  At this point, we can only try to continue the transaction, since we
                         *  aren't allowed to end the transaction. So, do nothing.
                         */
                        else if ( (infoBlk->event.data.transInfo.msgType == TCAP_CNV_NO_PRM))
                        {
                           printf("got a CnvNoPrm/Reject, send nothing\n");
                        }
                        /*
                         *  The only other message that can carry components is the Response.
                         *  If the Reject was in a Response message, the TCAP stack has already closed the
                         *  transaction, so the application doesn't need to do anything.
                         */
                        else
                        {
                           printf("got a Response/Reject, do nothing\n");
                        }
                    }
                    break;
                }

                default:
                    printf("\nUnknown TCAP indication %d received\n", infoBlk->eventType);
                    break;
            }
    return infoBlk->event.data.transInfo.msgType;
}

/****************************************************************************
 * check_per_second - check whether time to output periodic stats
 *
 * This is called every 256th ( <RX NOTIF> +
 *                              <SEND SUCCESS (counted prior to TX NOTIF)>
 *
 * With an even traffic load, this a variance of +/- 128 pkts per sec in
 * each direction.
 *
 * NOTE:
 *   Greater accuracy could be achieved by changing the mask used to determine
 *   how often to call check_per_second [at the cost of host CPU].
 ****************************************************************************/
void check_per_second( S8 *who, S8 *what )
{
    time_t          now;

    now = time(NULL);
    if (now >= lastPerSec + 1)
    {
        printf( "%s: %05d Connect %s/sec\n", who, connPerSec, what );

        lastPerSec = now;
        connPerSec = 0;
    }
}

