/*****************************************************************************
******************************************************************************
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
**
**   init.c    ANSI edition
**
**      Source code for an 800 number client/server that uses the
**      TCAP API. Contains the main() and parameter parsing.
**
*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------
******************************************************************************
*****************************************************************************/

/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1998 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/


/*
Fordingbridge change history:

    22-Nov-12   MH      - CN007TXA - 'Smoke test failure - unsigned types'
                          Remove unnecessary range checks.
*/


#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <string.h>
/* #include <sys/stream.h> */
#include "sol-dep.h"
#endif                                  /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#endif                                  /* __OS2__ */

#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <process.h>
#endif                                  /* WIN32 */

#include "osdep.h"
#include "ss7type.h"
#include "tcapdef.h"
#include "tcap800.h"

#ifdef UNIX
#define _getch getchar
#define Sleep(x) usleep(x * 1000)
#endif

DWORD InitializeTCAP (U8);
DWORD retrieveMessage(TcapRecvInfo*);
DWORD TermTCAP       (void);
DWORD Send800        (U8);
DWORD SendCon800     (U32, U8);
DWORD End800         (void);
DWORD SendConRR      (TcapTransInfo*, TcapComp*, unsigned char*);
DWORD Cnv800         (TcapTransInfo*, TcapComp*, unsigned char*);
DWORD Respond800     (TcapTransInfo*, TcapComp*, unsigned char*, U8);
DWORD RespondCon800  (TcapTransInfo*, TcapComp*, unsigned char*, U8);
DWORD Failed800      (TcapTransInfo*, TcapComp*, unsigned char*);
void  ErrorAndExit   (CTAHD , DWORD , char*);

U8    boardNum   = 1;
U16   locPort    = 4096;
U16   remPort    = 4096;
U32   remAddr;
U32   pointCode;
U32   position   = 0;
U8    sapid      = 0;
U8    ssn        = 254;
U8    destssn    = 254;
int   loops      = -1;
U8    inloop     = 1;
DWORD delay      = 0;
FILE* name;

CTAHD      FstCtaHd  = NULL_CTAHD;
CTAQUEUEHD FstQueue;

#ifdef UNIX
BOOL SendPending = FALSE;
BOOL dumpflag    = FALSE;
BOOL serverflag  = FALSE;
BOOL noresponseflag = FALSE;
char  flName[]   = "./commands.800";
#else
BOOLEAN SendPending = FALSE;
BOOLEAN dumpflag    = FALSE;
BOOLEAN serverflag  = FALSE;
BOOLEAN noresponseflag = FALSE;
char  flName[]   = ".\\commands.800";
#endif

S8  strnum[32];
U8  phnum[5];
U8  AddrType = 1;                       /* use ANSI addressing by default */
U8  runState;
U8  readFlag = 1;

void usage(char *pname, int opt)
{
    fprintf(stderr, "opt - %d\n"
                    "Usage: %s [options] pointcode:ssn phonenum\n"
                    "\n"
                    "where:\n"
                    "       pointcode is the destination in the form n.n.n\n"
                    "       ssn is the destination subsystem number(0 to 255)\n"
                    "\n"
                    "       phonenum is the phone number to be decoded(i.e. 8004321234)\n"
                    "\n"
                    "options:\n"
                    "       -b number       specify board, default 1, max 8\n"
                    "       -p number       specify SAP ID, default 0, max 255\n"
                    "       -n number       specify subsytem number, default 254, max 255\n"
                    "       -i number       specify number of iterations, default 1, max 32000\n"
                    "       -j number       specify delay between iterations(msec), default no, max 65536\n"
                    "       -s              act as server, default act as client\n"
                    "       -d              detailed dump of each packet sent/rcvd\n"
                    "       -t              use ITU addressing(default: ANSI addressing)\n"
                    "       -dp port        destination port\n"
                    "       -da address     destination address\n"
                    "       -dp name        destination name\n"
                    "       -lp port        local port\n"
                    "       -r              server doesn't send responses. HAS to be set on both server and client sides\n"

                    "\n", opt, pname);
    exit(128);
}

int main( int argc, char *argv[] )
{
    DWORD           Ret;
    CTA_EVENT       Event;
    TcapRecvInfo    infoBlk = {0};
    char           *p;
    int             shvar;
    short           optind, i;
    short           ctatimeout=5000;
    U8              hbyte, mbyte, lbyte, lssn;
    U8              ch;
    int             c;
    static char     in[133];
    THostent        *hostent;        /* ptr to struct returned by gethostbyname() */
    TThrId          udpThreadId;
    U8              congLvl;
    TCAPAPISTATS    tcApiStats;
    char            text[40];

    /* Initialize remote address to loopback */
    remAddr = htonl(INADDR_LOOPBACK);

    /* parse command line for optional parameters */
    for( optind = 1; optind < argc; optind++ )
    {
         p = argv[optind];
         if ( *p == '-' )
         {
              switch (*++p)
              {
                 case 's' :     /* server flag */
                     serverflag = TRUE;
                     break;

                 case 'd':
                     ++p;
                     if (*p == 'p')
                     {
                         if (*++p)         /* was there no space ? */
                             remPort = htons((U16)atoi(p));
                         else
                             remPort = htons((U16)atoi(argv[++optind]));
                     }
                     else if (*p == 'a')
                     {
                         if (*++p)         /* was there no space ? */
                             ;               /* str is OK now */
                         else
                             p = argv[++optind];

                         remAddr = inet_addr(p);
                     }
                     else if (*p == 'n')
                     {
                         if (*++p)         /* was there no space ? */
                             ;               /* str is OK now */
                         else
                             p = argv[++optind];
                         if (osGetHostByName(p, hostent) != RET_SUCCESS)
                         {
                             fprintf(stderr,  "UDPThread terminating,  invalid mate IP address/host name [%s]\n", p);
                         }

                         /* got host info for far end, use 1st IP address */
                         memcpy((char *)&(remAddr), hostent->h_addr, hostent->h_length);
                     }
                     else
                         dumpflag = TRUE;  /* dump flag */

                     break;

                 case 'l':
                     ++p;
                     if (*p == 'p')
                     {
                         if (*++p)         /* was there no space ? */
                             locPort = htons((U16)atoi(p));
                         else
                             locPort = htons((U16)atoi(argv[++optind]));
                     }
                     break;

                 case 'b' :    /* Set Board number */
                    if (*++p)         /* was there no space ? */
                        boardNum = (U8)atoi(p);
                    else
                        boardNum = (U8)atoi(argv[++optind]);

                    if((boardNum < 1) || (boardNum > 8) )
                         usage(argv[0], optind);

                     break;

                 case 'p' :    /* Set SAP ID */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);

                     if((shvar < 0) || (shvar > 255) )
                          usage(argv[0], optind);

                     sapid = (U8)shvar;
                     break;

                 case 'n' :    /* Set Subsytem Number */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);

                     if((shvar < 0) || (shvar > 255) )
                          usage(argv[0], optind);

                     ssn = (U8)shvar;
                     break;

                 case 'i' :    /* Set number of iterations */
                     if (*++p)         /* was there no space ? */
                         shvar = atoi(p);
                     else
                         shvar = atoi(argv[++optind]);

                     if((shvar < 0) || (shvar > 32000) )
                          usage(argv[0], optind);

                     loops = shvar;
                     break;

                 case 'j' :    /* Set delay between iterations */
                     if (*++p)         /* was there no space ? */
                         delay = atoi(p);
                     else
                         delay = atoi(argv[++optind]);

                     if((delay < 0) || (delay > 65536) )
                          usage(argv[0], optind);

                     break;

                 case 't' :     /* specify ITU addressing */
                     AddrType = 0;
                     break;

                case 'r' :              /* server doesn't respond to the messages
                                         * so that open transactions are formed  */
                    noresponseflag = TRUE;
                    ctatimeout = 500;   /* less timeout because the client doesn't need to wait for the responses for the
                                         * server, and it can send out requests quickly */
                    break;

                 default:
                     usage(argv[0], optind);
                     break;
              }       /* end switch */
         }
         else
         {
              /* end of optional parameters */
              break;          /* quit for loop without incrementing optind */
         }
    }         /* end for */

    if ((THREAD_CREATE(0, (TPtrThrFunc) UDPThread, argv, 0, udpThreadId)) != RET_SUCCESS)
    {
        printf( "ERROR: Unable to create UDP thread, error %ld", osGetLastError() );
        return(-1);
    }

    if ((name = fopen(flName, "r")) == NULL)
    {
        printf("ERROR: wrong file name\n");
        return(-1);
    }

    /* initialize API */
    if ( (Ret = InitializeTCAP(boardNum)) != TCAP_SUCCESS )
    {
        fprintf(stderr, "Error %d initializing API\n", Ret);
        exit(130);
    }

    if ( !serverflag )
    {
        if (optind >= argc)
        {
            /* no destination point code specified */
            usage(argv[0], optind);
        }

        if (sscanf(argv[optind], "%u.%u.%u:%u", &hbyte, &mbyte, &lbyte, &lssn) != 4)
        {
            usage(argv[0], optind);
        }

        pointCode = (hbyte << 16) + (mbyte << 8) + lbyte;
        destssn = lssn;
        ++optind;

        if (optind >= argc)
        {
            /* no phone number specified */
            usage(argv[0], optind);
        }

        /* read phone number */
        if (sscanf(argv[optind], "%s", strnum) != 1)
        {
            usage(argv[0], optind);
        }

        if ( strlen(strnum) < 10)
        {
            usage(argv[0], optind);
        }

        /* BCD encode the phone number */
        for ( i = 0; i < 5; i++)
        {
            /* 2 numbers in a byte , phnum[5] is dest, strnum[] is source */
            ch = strnum[(i*2)] - 0x30;
            phnum[i] = (ch << 4) & 0xF0;
            ch = strnum[((i*2)+1)] - 0x30;
            phnum[i] += ch;
        }

        while ( inloop )
        {
           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, ctatimeout );
           if ( Ret != TCAP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }
           switch ( Event.id )
           {
           case TCAPEVN_DATA:
               /* an TCAP event has occured, call TCAPRetrieveMessage() to get the event */
               Ret = retrieveMessage(&infoBlk);
               break;

           case TCAPEVN_CONGEST:
                /* TCAP layer or API is congested; retrieve some statistics and display them */
                congLvl = (U8) Event.value;
                if( (Ret = TCAPGetApiStats( Event.ctahd, &tcApiStats, 0 )) == TCAP_SUCCESS )
                {
                    printf( "TCAP Congestion level %d, API queue length/max = %ld/%ld\n",
                            congLvl, tcApiStats.qCount, tcApiStats.qPeak );
                }
                else
                {
                    strcpy( text, "" );
                    ctaGetText( Event.ctahd, Ret, text, sizeof( text ) );
                    printf( "TCAP Congestion level %d, API stats not available [error: 0x%lx, %s]\n",
                            congLvl, Ret, text );
                }
                break;

           case CHKPNT_EVT:
               readFlag = 1;
               printf("EVENT: CHKPNT_EVT received\n");
               if ( position )
               {
                   position = 0;
                   Send800(0);
               }
               continue;

           case CTAEVN_WAIT_TIMEOUT:
               /* 5 seconds passed, check for keyboard input */
               /* make sure a key was hit */
               if (_kbhit())
               {
                   c = _getch();
                   if ( c == 'q' || c == 'Q' )
                   {
                       inloop = 0;
                       break;
                   }
               }
               if ( runState == TCAP_BACKUP )
               {
                    Sleep( delay );     /* delay */
                    /* printf("Data do NOT send. Reason I AM BACKUP\n");  */
               }
               if( !noresponseflag)
                continue;
           }

           /* check for keyboard input make sure a key was hit */
           if (_kbhit())
           {
               c = _getch();
               if (c == 'q' || c == 'Q')
               {
                  inloop = 0;
                  break;
               }
           }

           /* we are a client, start the outgoing request */
//           if (  (runState != TCAP_BACKUP) &&
//                  readFlag )
           if (runState != TCAP_BACKUP)
           {
                if( (!noresponseflag) && (!readFlag) )
                {
                    /* don't do anything here */
                }
                else
                {
                    Sleep( delay );              /* delay */
                    if ( (fgets(in, 132, (FILE*)name) == NULL)
                    ||  (Ret                         == TCAP_ANSI_PABORT) )
                    {
                        rewind(name);
                        fgets(in, 132, (FILE*)name);
                        while ( (strlen(in) > 0) && (in[strlen(in) - 1] < ' ') )
                            in[strlen(in) - 1] = '\0';      /* remove any trailing non-printable characters */
                    }

                    switch ( in[0] )
                    {
                    case 'Q':
                    case 'q':
                        if ( in[3] == 'L' || in[3] == 'l' )
                            Send800(1);

                        if ( in[3] == 'N' || in[3] == 'n' )
                            Send800(0);

                        break;

                    case 'C':
                    case 'c':
                        if ( in[3] == 'L' || in[3] == 'l' )
                            SendCon800(infoBlk.event.data.transInfo.spDlgId, 1);

                        if ( in[3] == 'N' || in[3] == 'n' )
                            SendCon800(infoBlk.event.data.transInfo.spDlgId, 0);

                        break;

                    default:
                        printf("ERROR: command\n");
                        continue;
                    }                   /* end switch ( in[0] ) */
                }
           }                        /* end if ( (runState != TCAP_BACKUP) && readFlag ) */

           /* we want to sent 'loops' number of requests */
           if ( loops > 0)
           {
               if ( !(--loops) )
               {
                   inloop = 0;
               }
           }
        }                               /* end while( inloop ) */

        if ( SendPending )
        {    /* end transaction if no response rcvd */
            End800();
        }
    }                                   /* end if ( !serverflag ) */

    if ( serverflag )
    {
       /* if we are a server, wait for requests for 5 sec */
       while ( inloop )
       {
           if ( runState == TCAP_BACKUP )
           {
               printf("ERROR: Server State is BACKUP - no response\n");
               Sleep( delay );     /* delay */
               continue;
           }

           /* cta wait event */
           Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
           if ( Ret != TCAP_SUCCESS )
           {
               ErrorAndExit( FstCtaHd, Ret, "ctaWaitEvent" );
           }

           switch ( Event.id )
           {
               case TCAPEVN_DATA:
                   /* an TCAP event has occured, call TCAPRetrieveMessage() to get the event */
                   Sleep( delay );     /* delay */
                   retrieveMessage(&infoBlk);
                   break;

               case TCAPEVN_CONGEST:
                   /* TCAP layer or API is congested; retrieve some statistics and display them */
                   congLvl = (U8) Event.value;
                   if( (Ret = TCAPGetApiStats( Event.ctahd, &tcApiStats, 0 )) == TCAP_SUCCESS )
                   {
                        printf( "TCAP Congestion level %d, API queue length/max = %ld/%ld\n",
                                congLvl, tcApiStats.qCount, tcApiStats.qPeak );
                   }
                   else
                   {
                        strcpy( text, "" );
                        ctaGetText( Event.ctahd, Ret, text, sizeof( text ) );
                        printf( "TCAP Congestion level %d, API stats not available [error: 0x%lx, %s]\n",
                                congLvl, Ret, text );
                   }
                   break;

              case CTAEVN_WAIT_TIMEOUT:
                   /* 5 seconds passed, check for keyboard input */
                   /* make sure a key was hit */
                   if (_kbhit())
                   {
                      c = _getch();
                      if (c == 'q' || c == 'Q')
                      {
                         inloop = 0;
                         break;
                      }
                   }
                   break;
           }                            /* end switch ( Event.id ) */
       }                                /* end while( inloop )     */
    }                                   /* end if ( serverflag )   */

    /* if we fall through to here, user has killed us -
     * release any active connections                    */
    TermTCAP();

    return( 0 );
}

#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;

   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;

   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }

   return (RetCode);
}
#endif
