/*
 * tupldtst.c - sample call origination application
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
#include <conio.h>
#endif

#include "nmstypes.h"
#include "ctadef.h"
#include "txcpi.h"
#include "tuptype.h"
#include "tupmsgs.h"
#include "tupdef.h"
#include "osdep.h"

#ifdef UNIX
#include "sol-dep.h"
#endif /* UNIX */

#define MAX_CIRCS  1000

extern CPI_WAIT_TYPE stdInput;

#ifdef UNIX
#include <unistd.h>
#define _getch getchar
#define Sleep(x) usleep(x * 1000)
#endif

/* function prototypes from subs.c */

DWORD sendResetReq(CTAHD ctahd, U8 board, TupSuId suId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendContCheckRsp(CTAHD ctahd, U8 board, TupSuId suId, TupInstId suInstId, TupInstId spInstId,
                       CirIdx circuit, U8 contInd);
DWORD sendIAM1(CTAHD ctahd, U8 board, TupSuId suId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit,
              char *called, char *calling);
DWORD sendInfoResp(CTAHD ctahd, U8 board, TupSuId suId, TupInstId suInstId, TupInstId spInstId,
                   CirIdx circuit, char *called, char *calling);
DWORD sendReleaseReq(CTAHD ctahd, U8 board, TupSuId suId, TupInstId suInstId,
                     TupInstId spInstId, CirIdx circuit);
DWORD sendReleaseRsp(CTAHD ctahd, U8 board, TupRelEvnt *evnt, TupRcvInfoBlk *infoBlk);
DWORD sendBlockReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendUnblockReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendContinuityCheckReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendContinuityReport(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendContinuityFailure(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendACM(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendAnswer(CTAHD ctahd, U8 board, TupAllSdus *evnt, TupRcvInfoBlk *infoBlk);
DWORD NMSAPI     TUPRetrieveMessage(CTAHD ctahd, TupAllSdus *event, TupRcvInfoBlk *infoBlk);
DWORD NMSAPI     TUPConnectResp(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId,
                                CirIdx circuit, TupConRspEvnt *conRspEvnt);
DWORD NMSAPI     TUPConnectStatusReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId,
                                     CirIdx circuit, TupCnStEvnt *conStEvnt, U8 eventType);
DWORD NMSAPI     TUPReleaseReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId,
                               CirIdx circuit, TupRelEvnt *relEvnt);
DWORD NMSAPI     TUPReleaseResp(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId,
                                CirIdx circuit, TupRelEvnt *relEvnt);
DWORD NMSAPI     TUPConnectReq(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId,
                               Bool cirSelFlg, CirIdx circuit, TupConEvnt *conEvnt);
DWORD NMSAPI     TUPGetApiStats( CTAHD ctahd, TUPAPISTATS *pstats, U8 bReset );

/* globals */

U8       boardNum = 1;
CirIdx    circuit = 1;
TupInstId suInstId;
TupInstId spInstId;
TupSuId     suId;
char   called[11];
char   calling[11];      /* default = no calling pty info in IAM */
TupAllSdus   rcvEvent;
TupRcvInfoBlk  rcvInfo;
U32  circdiff;
U32  MaxCircuits;
U32  spInstIds[3500];
U16 poolsize = 200;
TUPAPISTATS apiStats;
long  delay_time = 0;     /* Introducing delay to process the incoming messages for Queue congestion*/

/* Define Instance and Entity IDs required of any application that
 * utilized the TUP API.
 */
#define INST_ID    0       /* Always 0 */
#define OUR_ENT    0x21

/* This application always uses tup sap 0 */
#define SAP_ID      0


/* internal function prototypes */
void handleStatInd( CTAHD ctahd, TupAllSdus *event, TupRcvInfoBlk *infoBlk );
void handleConStaInd( CTAHD ctahd, TupCnStEvnt *event, TupRcvInfoBlk *infoBlk );

/* handleTupMessage
 *
 * This function is called to handle all incoming TUP messages.
 * It just calls the proper message handling function based on the
 * type of message received.
 */
DWORD handleTupMessage(CTAHD ctahd)
{
    DWORD status;

    if( delay_time != 0 )
        Sleep( delay_time );
    status = TUPRetrieveMessage( ctahd, &rcvEvent, &rcvInfo );
    if ( status == TUP_NOMSG ) {
        return(TUP_SUCCESS);
    }
    if (status !=  TUP_SUCCESS)
	{
		fprintf(stderr, "TUPRetrieveMessage() failed, returned %d\n", status);
		return(status);
	}
	/* determine indication/confirmation type received */
    switch( rcvInfo.indType ) {

      case EVTTUPCONCFM:              /* connect confirmation */
        /*printf("tupldtst: Connect Confirmation received for circuit %d\n", (int) rcvInfo.circuit );*/
        /*APM*/
        TUPConnectResp(ctahd, boardNum, SAP_ID, 0, spInstIds[rcvInfo.circuit-circdiff],
                      rcvInfo.circuit-circdiff, (TupConRspEvnt*)&rcvEvent);
		break;

      case EVTTUPCONIND:              /* connect indication (incoming call) */
        printf("tupldtst: Connect Indication for circuit %d spInstId %d\n", rcvInfo.circuit, rcvInfo.spInstId );
        spInstIds[rcvInfo.circuit] = rcvInfo.spInstId;

        TUPConnectReq(ctahd, boardNum, SAP_ID, 0, 0,(Bool)((rcvInfo.circuit+circdiff) ? 1 : 0),
                      rcvInfo.circuit+circdiff, (TupConEvnt *)&rcvEvent );
        break;

      case EVTTUPCNSTIND:             /* connect status indication */
        printf("tupldtst: Connect Status Indication received for circuit %d, event %d spInstId %d\n", (int) rcvInfo.circuit,
               rcvInfo.evntType, rcvInfo.spInstId );
        if ( rcvInfo.evntType == TUPADDRCMPLT) {
           spInstIds[rcvInfo.circuit] = rcvInfo.spInstId;

           TUPConnectStatusReq( ctahd, boardNum, SAP_ID, 0, spInstIds[rcvInfo.circuit-circdiff],
                                rcvInfo.circuit-circdiff, (TupCnStEvnt*)&rcvEvent, rcvInfo.evntType );
        }
        break;

      case EVTTUPRELIND:              /* Release indication */
        printf("tupldtst: Release Indication for circuit %d\n", (int) rcvInfo.circuit );
        TUPReleaseResp( ctahd, boardNum, SAP_ID, 0, spInstIds[rcvInfo.circuit],
                        rcvInfo.circuit, (TupRelEvnt*)&rcvEvent );
        break;

      case EVTTUPRELCFM:              /* Release confirmation (release complt) */
        printf("tupldtst: Circuit %d released\n", (int) rcvInfo.circuit );
        break;

      case EVTTUPMGTSTAIND:               /* status indication */
      case EVTTUPSTAIND:                  /* status indication */
        handleStatInd( ctahd, &rcvEvent, &rcvInfo );
		break;

      case EVTTUPBNDCFM :                            /* Bind confirmation */
        printf("Bind confirm received\n");
        break;

      default:
		/* unrecognized event */
        printf("tupldtst: Unrecognized indication [%d] for circuit %d\n",
				(int) rcvInfo.indType, (int) rcvInfo.circuit );
		break;

	}   /* end switch */

    return(TUP_SUCCESS);

}   /* end handleTupMessage */

/* handleStatInd
*
* This function handles a status indication from the far exchange
* or TUP layer. A ERRORIND indicates that the call has failed, so
* the application terminates. Other indications are just logged (a real
* application would need to respond to some of these indications, such
* as circuit/group blocking/unblocking requests).
*/
void handleStatInd( CTAHD ctahd, TupAllSdus *event, TupRcvInfoBlk *infoBlk )
{

	switch (infoBlk->evntType)
	{

       case TUPSWITCHCONG:
       case TUPCIRCUITCONG:
       case TUPNATIONALCONG:
       case TUPADDRESSINCOMP:
       case TUPCALLFAILURE:
       case TUPSUBBUSY:
       case TUPUNALLOCATEDNUM:
       case TUPSENDSPECIAL:
       case TUPOUTOFSERVICE:
       case TUPACCESSBARRED:
       case TUPNODIGITAL:
       case TUPEXTENDEDINFO:
       case TUPSUBLOCALBUSY:
       case TUPSUBTOLLBUSY:
          printf("Unsuccessful Call Message Rcvd (%d) circuit %d, sending release\n",infoBlk->evntType,infoBlk->circuit);
          sendReleaseReq(ctahd, boardNum, SAP_ID, 0, spInstIds[infoBlk->circuit], infoBlk->circuit);
          break;

       case TUPCLEARBKW:
          printf("Clear Backwards received for circuit %d\n",infoBlk->circuit);
          sendReleaseReq(ctahd, boardNum, SAP_ID, 0, spInstIds[infoBlk->circuit], infoBlk->circuit);
          break;

       case TUPDUALSEIZURE:
          printf("Call failure due to dual seizure circuit %d\n",infoBlk->circuit);
          printf("Please retry the call on another circuit\n");
          break;

       case TUPERRORIND:   /* ERROR indication from TUP layer */
          printf("tupldtst: ERROR Status, circ=%d, suId=%d, spId=%d, indicator=%d\n",
                 infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId,
                 (int) event->errInfo.errorCode );
          break;

       case TUPCONTCHK:           /* continuity check requested */
          printf("tupldtst: Unexpected Continuity Check request received for circuit=%d\n", infoBlk->circuit);
          break;

       case TUPCONTREP:
          printf("tupldtst: Unexpected Continuity Response Rcvd, circ=%d, suId=%d, spId=%d\n",
                 infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId );
          break;

       case TUPCIRBLKREQ:
          printf("tupldtst: Circuit Block Request, circuit %d", infoBlk->circuit);
          break;

       case TUPCIRUNBLKREQ:
          printf("tupldtst: Circuit Unblock Request, circuit %d\n", infoBlk->circuit);
          break;

       case TUPCRMGRPBLKREQ:
          printf("tupldtst: Circuit Group Block, starting at circuit %d\n", infoBlk->circuit);
          break;

       case TUPCRMGRPUNBLKREQ:
          printf("tupldtst: Circuit Group Unblock, starting at circuit %d\n", infoBlk->circuit);
          break;

       case TUPCIRRESREQ:
          printf("tupldtst: Circuit Reset circuit %d.\n", infoBlk->circuit);
          break;

       case TUPCIRGRPRESREQ:
          printf("tupldtst: Circuit Group Reset, starting at circuit %d.\n", infoBlk->circuit);
          break;

       case TUPMTP3RESUME:
          printf("tupldtst: MTP3 Resume Received: PointCode %x.\n", event->m.staEvnt.pointCode.pointCode);
          break;

       case TUPMTP3PAUSE:
          printf("tupldtst: MTP3 Pause Received: PointCode %x.\n", event->m.staEvnt.pointCode.pointCode);
          break;

       case TUPMTP3STANDALONE :
        printf("tuldtst: MTP3 Standalone Received: PointCode %x.\n", event->m.staEvnt.pointCode.pointCode);
        break;

       case TUPMTP3PRIMARY :
        printf("tupldtst: MTP3 Primary Received: PointCode %x.\n", event->m.staEvnt.pointCode.pointCode);
        break;

       case TUPMTP3BACKUP :
        printf("tupldtst: MTP3 Backup Received: PointCode %x.\n", event->m.staEvnt.pointCode.pointCode);
        break;

       case TUPMTP3CONGESTION :
        printf("tupldtst: MTP3 Congestion Received: PointCode %x., Congestion level: %d\n",
                event->m.staEvnt.pointCode.pointCode, event->m.staEvnt.mtpCongestionLvl);
        break;

       case TUPMTP3STOPCONGESTION :
        printf("tupldtst: MTP3 stop Congestion Received: PointCode %x., Congestion level: %d\n",
                event->m.staEvnt.pointCode.pointCode, event->m.staEvnt.mtpCongestionLvl);
        break;


       default:    /* all other status indications */
          printf("tupldtst: Unexpected Status received, circ=%d, suId=%d, spId=%d, type=%d\n",
                 infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId,
                 (int) infoBlk->evntType );
          break;

   }    /* switch */

} /* end of handleStatInd */

/* handleConStaInd
*
* This function handles a connection status indication event; if this
* is a info request for calling party number, an INFormation message is
* generated containing calling party number (only).
*
*/
void handleConStaInd( CTAHD ctahd, TupCnStEvnt *event, TupRcvInfoBlk *infoBlk )
{
	int     circuit;

    UNREFERENCED_PARAMETER(event);

	circuit = (int) infoBlk->circuit;

	switch (infoBlk->evntType) {

  case TUPADDRCMPLT:
		printf( "Addr Complete received for circuit %d\n", infoBlk->circuit );
		/* Service provider ID (spId) MUST be saved for use in all subsequent
		* messages associated with this connection
		*/
		circuit  = infoBlk->circuit;
		suInstId = infoBlk->suInstId;
		spInstId = infoBlk->spInstId;
		suId     = infoBlk->suId;
		break;

  case TUPFRWDTRSFR:                         /* forward transfer */
		printf( "Forward Transfer indication received for circuit %d\n", infoBlk->circuit );
		break;

  case TUPINFORMATION:                       /* information indication */
		printf( "Information indication received for circuit %d\n",
				(int) infoBlk->circuit );
		break;

  case TUPINFORMATREQ:                       /* information rqst */
		printf( "Information Rqst received for circuit %d\n", infoBlk->circuit );
        sendInfoResp(ctahd, boardNum, SAP_ID, 0, 0, circuit, called, calling);
		break;

  case TUPSUBSADDR:                          /* subsequent address */
		printf( "Subsequent Address received for circuit %d\n", infoBlk->circuit );
		break;

	default:
		/* unrecognized connect status indication */
		printf( "Unrecognized connect status ind., circ=%d, suId=%d, spId=%d\n",
				(int) infoBlk->circuit, (int) infoBlk->suInstId, (int) infoBlk->spInstId );
		break;

	}   /* of switch */

} /* end of handleCnStInd */


static void ErrorAndExit( DWORD ErrId, char *Fcn )
{

   printf( "Error : (%d) error returned in (%s) call.\n", ErrId, Fcn );
   exit( 1 );
}


DWORD NMSSTDCALL ErrHdlr( CTAHD ctahd,
			  DWORD errorcode,
			  char *errtxt,
			  char *func )
{
    /* print out the error and exit */
    char text[40];
    ctaGetText(ctahd, errorcode, text, sizeof(text) );
    if ( errorcode != TUP_NOMSG ) {
        printf( "ErrorHandler: %s on ctahd=%#x, %s\n", errtxt, ctahd, func );
        return errorcode;
    }
    return (TUP_SUCCESS);
}



/* main()
 *
 * This is the main routine for the TUP incoming test application.
 * It initializes the application's local circuit state database,
 * initializes the TUP API (by binding to TUP), and then just loops
 * waiting for messages from TUP and dispatching the appropriate
 * routine to handle the message.
 */

int main( int argc, char *argv[] )
{
   /* OS_HSYNC fdSync; */
   /* short   status;   */
	short   optind = 1;
	char   *str;

    CTA_SERVICE_DESC tupOpenSvcLst[] = {{{"TUP", "TUPMGR"}, {0}, {0}, {0}}};
    char *tupCloseSvcLst[] = { "TUP" };

    CTAQUEUEHD          FstQueue = 0;
    CTAHD               FstCtaHd = NULL_CTAHD;
    CTA_EVENT           Event;
    DWORD               Ret;
    CTA_INIT_PARMS      tupInitparms =                 { 0 };
    CTA_SERVICE_NAME    tupServiceNames[] =            { { "TUP", "TUPMGR" } };
    U8                  inloop = 1;
    int                 c;
    DWORD               status;
    CTA_REV_INFO        info;
    /*TupConEvnt          tupConEvnt; */ /* Connect Event */

    ctaGetVersion(&info, sizeof(CTA_REV_INFO));
    if (info.compatlevel != CTA_COMPATLEVEL)
    {
       printf("CTA compatibility level does not match. re-compile.\n");
       exit(1);
    }

    /* check for required and optional parameters */
    if( argc < 2 )
	{
        printf( "Usage: %s [-b <board>] <Maximum Number of circuits> \n", argv[0] );
		exit(1);
    }

    str = argv[optind];
    if (str)
	{
      if (*str == '-')
      {
		optind++;
		switch (*++str)
		{
		case 'b':   /* which channel should we speak through */
			if (*++str)         /* was there a space ? */
            boardNum = (U8)atoi(str);
			else
            boardNum = (U8)atoi(argv[optind++]);
			break;
        case 's' :
            if (*++str)         /* was there a space ? */
               delay_time = atol(str);
			else
               delay_time = atol(argv[optind++]);
            if( delay_time == 0 )
               printf( "Warning: Invalid delay time [%s] specified, using 0\n", delay_time);
            break;

		default:
			printf( "Usage: %s [-b <board>] \n", argv[0] );
			exit(1);
		}
      }
	}

    /*Get Maximum number of circuits defined from argv to determine
     * circdiff value used to make outbound calls
     */
    MaxCircuits = atoi(argv[optind++]);
    circdiff = MaxCircuits+5;

    /*ctaSetErrorHandler(ErrHdlr, NULL); */


    /* bind this application as a user of the TUP layer */
    tupInitparms.size = sizeof(CTA_INIT_PARMS);
    tupInitparms.traceflags = CTA_TRACE_ENABLE;
    tupInitparms.parmflags  = CTA_PARM_MGMT_SHARED;
    tupInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    Ret = ctaInitialize( tupServiceNames,
                         sizeof(tupServiceNames) / sizeof(tupServiceNames[0]),
                         &tupInitparms);
    if (Ret != SUCCESS)
    {
       /*-----------------3/7/00 10:58AM-------------------
        * ctdaemon is not running. Initialize
        * without tracing
        * --------------------------------------------------*/
       tupInitparms.traceflags = 0;
       tupInitparms.parmflags  = 0;
       Ret = ctaInitialize( tupServiceNames,
                            sizeof(tupServiceNames) / sizeof(tupServiceNames[0]),
                            &tupInitparms);
       if ( Ret != SUCCESS)
       {
           ErrorAndExit( Ret, "ctaInitialize" );
       }
    }

    Ret = ctaCreateQueue( NULL, 0, &FstQueue );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaCreateQueue" );
    }

    /* Open FstCtaHd on FstQueue                   */
    Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaCreateContext" );
    }

    tupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
    tupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
    tupOpenSvcLst[0].svcargs.args[2] = OUR_ENT;      /* dprChan */
    tupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
    tupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
    tupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */
    tupOpenSvcLst[0].svcargs.args[6] = poolsize;     /* pool size */

    Ret = ctaOpenServices( FstCtaHd, tupOpenSvcLst, 1 );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete. */
    Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
    if ( Ret != SUCCESS )
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    if (( Event.id != CTAEVN_OPEN_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
    {
            ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    else
       printf("Bind Confirmation Received and TUP Service Opened\n");

    while ( inloop)
    {
        /* cta wait event */
        Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
        if ( Ret != SUCCESS )
        {
                ErrorAndExit( Ret, "ctaWaitEvent" );
        }

        switch ( Event.id )
        {
            case TUPEVN_DATA:
               /* an TUP event has occured, call TUPRetrieveMessage() to get the event */
               handleTupMessage(FstCtaHd);
               break;

            case TUPEVN_CONGEST:
                printf("Congestion event received: Congestion level: [%d]\n", Event.value);
                Ret = TUPGetApiStats( FstCtaHd, &apiStats, 0 );
                if ( Ret != SUCCESS) {
                   printf("TUPGetApiStats() fialed: error= %d\n", Ret);
                   break;
                }
                printf("Events Received:%d\n", apiStats.rxSuccess);
                printf("Successful transmit requests:%d\n", apiStats.txSuccess);
                printf("Outstanding transmit requests:%d\n", apiStats.txPending);
                break;

            case CTAEVN_WAIT_TIMEOUT:
            /* 5 seconds passed, check for keyboard input */
                /* make sure a key was hit */
                if (_kbhit())
                {
                    c = _getch();
                    if (c == 'q')
                       inloop = 0;
                    else if (c == 'r')
                    {
                       status = sendReleaseReq(FstCtaHd, boardNum, suId, suInstId, spInstId, circuit);
                       if (status != TUP_SUCCESS)
                          printf("tupldtst: sendReleaseReq() failed status = %d\n", status);
                       else
                          printf("tupldtst: Release sent\n");
                    }
                    else if (c == 'i')
                       sendIAM1(FstCtaHd, boardNum, SAP_ID, 0, 0, circuit, called, calling);
                    else if (c == 'k')
                       sendBlockReq(FstCtaHd, boardNum, SAP_ID, 0, 0, circuit);
                    else if (c == 'u')
                       sendUnblockReq(FstCtaHd, boardNum, SAP_ID, 0, 0, circuit);
                    else if(c == 't')
                       sendReleaseRsp(FstCtaHd, boardNum, &rcvEvent.m.relEvnt, &rcvInfo);
                    else if(c == 'f')
                       sendContinuityFailure(FstCtaHd, boardNum, SAP_ID, 0, 0, circuit);
                    else if(c == 'c')
                       sendACM(FstCtaHd, boardNum, rcvInfo.suId, rcvInfo.suInstId, rcvInfo.spInstId, rcvInfo.circuit);
                    else if(c == 'a')
                       sendAnswer(FstCtaHd, boardNum, &rcvEvent, &rcvInfo);
                }
                break;

            default:
                printf( "CTAHD(0x%08X) : Unhandled event id (0x%x) during "
                        "wait.\nExiting...\n", Event.ctahd,
                        Event.id );
        }

    }           /* end while */

    /*
    *   Close service and destroy context.
    */
    Ret = ctaCloseServices( FstCtaHd, tupCloseSvcLst, 1 );
    if ( Ret != SUCCESS )
    {
           ErrorAndExit( Ret, "ctaCloseServices" );
    }

    /* Wait for service close to complete. */
    Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
    if ( Ret != SUCCESS )
    {
           ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
    {
           ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    Ret = ctaDestroyContext( FstCtaHd );

    if ( Ret != SUCCESS )
    {
           ErrorAndExit( Ret, "ctaDestroyContext" );
    }

    /* Wait for context close to complete. */
    Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
    if ( Ret != SUCCESS )
    {
           ErrorAndExit( Ret, "ctaWaitEvent" );
    }
    if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
    {
           ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    Ret = ctaDestroyQueue( FstQueue );
    if ( Ret != SUCCESS )
    {
           ErrorAndExit( Ret, "ctaDestroyQueue" );
    }

} /* end of main */
