/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1996 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/*
 * TUPCfg.c
 *
 * Scans ascii text configuration file and downloads configuration to
 * TUP task on specified board.
 */

/* header include files -- defines (.h) */

#ifdef UNIX
#include <sys/types.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#endif                                  /* UNIX */

#ifdef WIN32
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* WIN32 */

#ifdef __OS2__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* __OS2__ */

/*#include "txcpi.h" */

#define TUP_CFG
#include "cfgscan.h"
#include "tupmgmt.h"
#include "tupver.h"
#include "tupdef.h"

extern int CDebug;
/* OS-specific definitions */
#define TRUE    1
#define FALSE   0

/* UNIX */

/* forward references */
U32 scanConPCs(U32 *pCpcList, U16 *numconpc, U32 tlast);

short GenCfg(void);
short SapCfg(void);
short NSapCfg(void);
short CircCfg(void);

void showHex(unsigned char *Buf, unsigned short ByteCnt);

/* global variables */
short           debug = 0;      /* prints each packet sent/received */
S32             temp;           /* temp used by scanner macros */
char            buf[80];
U8  boardNum = 1;                  /* default board is 1 */
U16 MaxSaps;
U16 MaxNetSaps;
U16 MaxRoutes;
U32 MaxCircuits;
U16 MaxCircGrp;
U16 MaxDpcs;
U16 MaxOpcs;
U32 opc;

/* global buffers used for netmgr requests & responses */

/* limits from general configuration section */

char configFile[80] = "TUP.cfg";
static char *usage = "Usage: TUPcfg [ -b <board number> ] [-f <filename>]\n";


/* Handy macro to get & range check a long, print a warning if its out
 * of range, and copy the value (possibly cast to a smaller type)
 * to "dest" if its in range; assumes "dest" is previously set to
 * default value. "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_INT(name, min, max, dest, cast)         \
{                                                           \
    if (getcfgval(name, CFGT_INT, &temp, 0) > 0) {          \
    if (temp < min || temp > max) {                   \
        fprintf(stderr,                                    \
            "Warning: %s [%ld] out of range, setting to default [%d]\n",\
            name, temp, dest);                         \
        } else {                                            \
        dest = (cast) temp;                             \
        }                                                   \
    }                                                       \
}

#define GET_CHECK_INT_FROM(name, min, max, dest, cast, from, next)    \
{                                                                       \
    if ((next = getcfgval(name, CFGT_INT, &temp, from)) > 0) {          \
    if (temp < min || temp > max) {                               \
    fprintf(stderr,                                                \
        "Warning: %s [%ld] out of range, setting to default [%d]\n",\
        name, temp, dest);                                     \
    } else {                                                        \
        dest = (cast) temp;                                         \
    }                                                               \
    }                                                                   \
}

/* Handy macro to get an address
 */
#define GET_ADDRESS(name, dest)                                       \
{                                                                               \
    char temp[CFGT_MAXSTR + 1];                                                                                     \
                                                                        \
    if (getcfgval(name, CFGT_STRING, temp, 0) > 0)                                  \
    {                                                                   \
        (dest).length = strlen(temp);                                                                   \
        strncpy((dest).string, temp, (dest).length);                    \
    }                                                                   \
}

#define GET_ADDRESS_FROM(name, dest, from, next)                      \
{                                                                       \
    char temp[CFGT_MAXSTR + 1];                                         \
                                                                        \
    if ((next = getcfgval(name, CFGT_STRING, temp, from)) > 0)          \
    {                                                                   \
        (dest).length = strlen(temp);                                   \
        strncpy((dest).string, temp, (dest).length);                    \
    }                                                                   \
    else                                                                \
        (dest).length = 0;                                              \
}

/* Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER(name, timer)                            \
{                                                           \
  if (getcfgval(name, CFGT_INT, &temp, 0) > 0)              \
    timer.value = (U16) temp;                               \
  timer.enable = (timer.value != 0) ? 1 : 0;                \
}

#define GET_TIMER_FROM(name, timer, from, next)               \
{                                                               \
  if ((next = getcfgval(name, CFGT_INT, &temp, from)) > 0)      \
    timer.value = (U16) temp;                                   \
  timer.enable = (timer.value != 0) ? 1 : 0;                    \
}

/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
     ByteCnt    #Bytes in Buf
OUT: <file>      gets multiple lines
*/
void showHex(unsigned char *Buf, unsigned short ByteCnt)
{
        unsigned short Li, LMax, ToDo, Bi, Ai;
        char Ascii[80];
        unsigned char B;

/* Dump to std out in hex followed by ascii */

for (ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax)
    {LMax = (ToDo > 16) ? 16 : ToDo;            /* max bytes this line */
     printf("%04X| ", Bi);
     Ai = 0;
     for (Li = 0; Li < LMax; Li++)              /* each byte this line */
         {B = *Buf++;                           /* next byte to show */
          printf("%02X ", B);                           /* as hex */
          if (!isprint(B))                      /* printable ascii? */
             B = '.';                           /* no, use this */
          Ascii[Ai++] = B;                      /* into ascii portiob */
         }      /* for Li */
     for (Li = LMax; Li < 16; Li++)             /* pad short ln if necessary */
         printf("   ");                                 /* so asci lines up */
     Ascii[Ai] = 0;                             /* terminate the ascii */
     printf("    ");
     printf("%s\n", Ascii);                     /* show the ascii */
     Bi += LMax;
    }       /* for ToDo */

}       /* ShowHex */

/*
* GenCfg
*
* Configure general parameters for TUP
*/
short GenCfg(void)
{
    TUPGenParms cfg;
    U16        status;

    /* Initialize configuration block to defaults */

    TUPInitGenCfg(&cfg);

    /* Scan for overrides from config file */
    GET_CHECK_INT("MAX_SAPS",     1, 255,   cfg.maxSaps,     U16);
    GET_CHECK_INT("MAX_NSAPS",    1, 255,   cfg.maxNetSaps,  U16);
    GET_CHECK_INT("MAX_CIRCUITS", 0, 65535, cfg.maxCircuits, U32);
    GET_CHECK_INT("TXMON_TIMEOUT",0, 65535, cfg.txmonTimeout, U32);
    GET_CHECK_INT("MAX_GROUPS",   0, 65535, cfg.maxCircGrp,  U16);
    GET_CHECK_INT("MAX_DPCS",     1, 255,   cfg.maxDpcs,  U16);
    GET_CHECK_INT("MAX_ROUTES",   0, 65535, cfg.maxRoutes,   U16);
    GET_CHECK_INT("ALARM_LEVEL",  0, 255,   cfg.alarmLevel,  U16);

    cfg.maxOpcs = cfg.maxDpcs;          /* If maxOpcs and maxDpcs are not the same, then there is a
                                         * possibility of breaking logic of MTP pause/resume
                                         * timer routines on the board  */

    if (getcfgval("TRACE_EVENT", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= TRACE_EVENT;
        else
            cfg.traceFlags &= ~TRACE_EVENT;
    }

    if (getcfgval("TRACE_DATA", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= TRACE_DATA;
        else
            cfg.traceFlags &= ~TRACE_DATA;
    }

    if (getcfgval("TRACE_WARNING", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= ENUM_WARNING;
        else
            cfg.traceFlags &= ~ENUM_WARNING;
    }

    if (getcfgval("TRACE_ERROR", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= ENCODE_ERROR;
        else
            cfg.traceFlags &= ~ENCODE_ERROR;
    }
    if (getcfgval("TRACE_ELEMENT", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= INIT_ELMT_DBG;
        else
            cfg.traceFlags &= ~INIT_ELMT_DBG;
    }

    if (getcfgval("TIMER_TRACE", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= TMR_TRACE;
        else
            cfg.traceFlags &= ~TMR_TRACE;
    }

    if (getcfgval("CHECKPOINT_TYPE", CFGT_BOOL, &temp, 0) > 0)
    {
         if (temp)
                cfg.chkptType = TUP_CHKPT_AUTO;
         else
                cfg.chkptType = TUP_CHKPT_NONE;
    }

    GET_TIMER("MTPPAUSE_TIMER",   cfg.MtpPauseTmr);

    /* Save some globals for use by other config sections */
    MaxSaps     = cfg.maxSaps;
    MaxNetSaps  = cfg.maxNetSaps;
    MaxRoutes   = cfg.maxRoutes;
    MaxCircuits = cfg.maxCircuits;
    MaxCircGrp  = cfg.maxCircGrp;
    MaxDpcs = cfg.maxDpcs;
    MaxOpcs = cfg.maxOpcs;

#ifdef CFGTEST
    printf("General Configuration\n");
    showHex((unsigned char *)&cfg, sizeof(cfg));
#else
    /* Call the Config Request */
    status = TUPSetGenCfg(boardNum, &cfg);
    if (status == CPI_TIMEOUT)
    {
        printf("TUP Gen Config: No response from board\n");
        return(status);
    }
    else if (status != TUP_SUCCESS)
    {
        printf("TUP Gen Config: Error %d\n", status);
        return(status);
    }

#endif                               /* ! CFGTEST */

    return(0);

} /* TUPGenCfg */

/*
* TUPSapCfg
*
* Configure TUP upper saps
*/
short SapCfg(void)
{
    TUPUSapCfg  cfg;
    short       status;
    S32         last, next;
    U16         sapno;

    for (last = 0; ;)
    {
        memset(&cfg, 0, sizeof(cfg));

        next = getcfgval("USER_SAP", CFGT_INT, &temp, last);
        sapno = (U16)temp;
        if ( next <= 0)
            break;          /* no more user SAPs */

        last = next;        /* save starting line in config file */

        if (sapno > MaxSaps)
        {
            fprintf(stderr, "User SAP number %d exceeds maximum [%d]\n",
                    sapno, MaxSaps);
            continue;
        }

        /* Initialize user SAP with default values */
        TUPInitUSapCfg(&cfg);

        /* Fetch overides from config file */
        if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, 0) > 0)
        {
            if (strcmp(buf, "ITU-T") == 0)
                cfg.switchType = TUP_SW_ITU;
            else if (strcmp(buf, "CHINA") == 0)
                cfg.switchType = TUP_SW_CHN;
            else
            {
                fprintf(stderr, "Warning: SAP %d unknown switch type [%s]"
                    "ignored, defaulting to ITU-T\n", sapno, buf);
            }
        }

        /* Get Queue congestion onset values */
        if ( getcfgval("QCONGONSET1", CFGT_INT, &temp, 0) > 0)
                cfg.QCongOnsetLvl1 = (S16)temp;

        if ( getcfgval("QCONGABATE1", CFGT_INT, &temp, 0) > 0)
                cfg.QCongAbateLvl1 = (S16)temp;

        if ( getcfgval("QCONGONSET2", CFGT_INT, &temp, 0) > 0)
                cfg.QCongOnsetLvl2 = (S16)temp;

        if ( getcfgval("QCONGABATE2", CFGT_INT, &temp, 0) > 0)
                cfg.QCongAbateLvl2 = (S16)temp;

        if ( getcfgval("QCONGONSET3", CFGT_INT, &temp, 0) > 0)
                cfg.QCongOnsetLvl3 = (S16)temp;

        if ( getcfgval("QCONGABATE3", CFGT_INT, &temp, 0) > 0)
                cfg.QCongAbateLvl3 = (S16)temp;

#ifdef CFGTEST
    printf("User SAP Configuration\n");
    showHex((unsigned char *)&cfg, sizeof(cfg));
#else

        /* Call the Config Request */
        status = TUPSetUSapCfg(boardNum, &cfg, sapno);
        if (status == CPI_TIMEOUT)
        {
            printf("TUP USAP Config: No response from board\n");
            return(status);
        }
        else if (status != TUP_SUCCESS)
        {
            printf("TUP USAP Config: Error %d\n", status);
            return(status);
        }

#endif                               /* ! CFGTEST */

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }   /* end for loop */

    return(0);

} /* TUPSapCfg */


/*
* TUPNSapCfg
*
* Configure lower saps with mtp3
*/
short NSapCfg(void)
{
    TUPNSapCfg      cfg;
    short           status;
    S32             last, next;
    U16             nsapno, mtpSap;

    last = 0;

    if ((next = getcfgval("NSAP", CFGT_INT, &temp, last)) <= 0)
        return(-1);                          /* no more NSAPs */

    last = next;                        /* save starting line in config file */
    nsapno = (U16) temp;
    if (nsapno > MaxNetSaps)
    {
        fprintf(stderr, "NSAP number %d exceeds maximum [%d]\n",
                nsapno, MaxNetSaps);
        return(0);
    }

    /* Fetch non-default-able values from config */
    if (getcfgval("OPC", CFGT_PC, &temp, last) <= 0)
    {
        printf("TUP Gen Config: OPC not specified\n");
        return(-1);
    }
    opc = (U32) temp;

    if (getcfgval("MTPSAP", CFGT_INT, &temp, last) <= 0)
    {
        printf("TUP Gen Config: MTPSAP not specified\n");
        return(-1);
    }
    mtpSap = (U16)temp;

    /* Initialize nsap with specified values */
    TUPInitNSapCfg(&cfg, nsapno, mtpSap, opc);

    /* Get default-able values from config and default them if necessary */
    if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0)
    {
        if (strcmp(buf, "CHINA") == 0)
            cfg.switchType = TUP_SW_CHN;
        else if (strcmp(buf, "ITU-T") == 0)
            cfg.switchType = TUP_SW_ITU;
        else
            fprintf(stderr, "Warning: NSAP %d unknown switch type [%s]"
                     "ignored, defaulting to ITU-T\n", nsapno, buf);
    }

    if (getcfgval("SSF", CFGT_STRING, buf, last) > 0)
    {
        if (strcmp(buf, "SSF_NAT") == 0)
            cfg.ssf = SSF_NAT;
        else if (strcmp(buf, "SSF_INTL") == 0)
            cfg.ssf = SSF_INTL;
        else if (strcmp(buf, "SSF_SPARE") == 0)
            cfg.ssf = SSF_SPARE;
        else if (strcmp(buf, "SSF_RES") == 0)
            cfg.ssf = SSF_RES;
        else
            fprintf(stderr, "Warning: NSAP %d unknown SSF [%s]"
                     "ignored, defaulting to SSF_NAT\n", nsapno, buf);
    }

#ifdef CFGTEST
    printf("Network SAP Configuration\n");
    showHex((unsigned char *)&cfg, sizeof(cfg));
#else

    /* Call the Config Request */
    status = TUPSetNSapCfg(boardNum, &cfg);
    if (status == CPI_TIMEOUT)
    {
        printf("TUP NSAP Config: No response from board\n");
        return(status);
    }
    else if (status != TUP_SUCCESS)
    {
        printf("TUP NSAP Config: Error %d\n", status);
        return(status);
    }

#endif                               /* ! CFGTEST */

    /* Skip to end of section */
    next = getcfgval("END", CFGT_INT, &temp, last);
    last = next;

    return(0);
}  /* TUPNSapCfg */

/*
* TUPCircCfg
*
* Configure Address Table
*/
short CircCfg(void)
{
    TUPCircCfg  cfg;
    short       status;
    S32         last, next;
    U32         dpc, altOpc, circuit;



    /* initialize circuit defaults */

    for(last = 0; ;)
    {
       /* Get non-default-able values from config */
       if ((next = getcfgval("CIRCUIT", CFGT_INT, &temp, last)) <= 0)
           break;                          /* no more circuits */

       circuit = (U32)temp;
       last = next;                        /* save starting line in config file */

       if ((next = getcfgval("DPC", CFGT_PC, &temp, last)) <= 0)
       {
           printf("Error: CIRCUIT %ld missing DPC\n", circuit);
           break;
       }
       dpc = (U32)temp;

       if ((next = getcfgval("ALT_OPC", CFGT_PC, &temp, last)) > 0)
           altOpc = (U32)temp;
       else
           altOpc = opc;

       /* Initialize circuit cfg structure */
       TUPInitCircCfg(&cfg, circuit, altOpc, dpc);

       /* Get default-able values from config and override if present */
       if ((next = getcfgval("CIC", CFGT_INT, &temp, last)) > 0)
           cfg.cic = (U16)temp;

       if ((next = getcfgval("NUM_CIRCUITS", CFGT_INT, &temp, last)) > 0)
           cfg.numCircs = (U16)temp;

       if ((next = getcfgval("GROUP_ID", CFGT_INT, &temp, last)) > 0)
           cfg.groupId  = (U16)temp;

       if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0)
       {
           if (strcmp(buf, "ITU-T") == 0)
               cfg.switchType = TUP_SW_ITU;
           else if (strcmp(buf, "CHINA") == 0)
               cfg.switchType = TUP_SW_CHN;
           else
           {
               fprintf(stderr, "Warning: CIRC %d unknown switch type [%s]"
                       "ignored, defaulting to ITU-T\n", circuit, buf);
           }
       }

       if ((next = getcfgval("NSAP_ID", CFGT_INT, &temp, last)) > 0)
           cfg.nsapId  = (U16)temp;

       GET_TIMER("T1_TIMER",   cfg.t1);
       GET_TIMER("T2_TIMER",   cfg.t2);
       GET_TIMER("T3_TIMER",   cfg.t3);
       GET_TIMER("T4_TIMER",   cfg.t4);
       GET_TIMER("T5_TIMER",   cfg.t5);
       GET_TIMER("T6_TIMER",   cfg.t6);
       GET_TIMER("T7_TIMER",   cfg.t7);
       GET_TIMER("T8_TIMER",   cfg.t8);
       GET_TIMER("T9_TIMER",   cfg.t9);
       GET_TIMER("T10_TIMER",  cfg.t10);
       GET_TIMER("T11_TIMER",  cfg.t11);
       GET_TIMER("T12_TIMER",  cfg.t12);
       GET_TIMER("T13_TIMER",  cfg.t13);
       GET_TIMER("T14_TIMER",  cfg.t14);
       GET_TIMER("T15_TIMER",  cfg.t15);
       GET_TIMER("T16_TIMER",  cfg.t16);
       GET_TIMER("T17_TIMER",  cfg.t17);
       GET_TIMER("T18_TIMER",  cfg.t18);
       GET_TIMER("T19_TIMER",  cfg.t19);
       GET_TIMER("T25_TIMER",  cfg.t25);


#ifdef CFGTEST
       printf("Circuit Configuration\n");
       showHex((unsigned char *)&cfg, sizeof(cfg));
#else

       /* Call the Config Request */
       status = TUPSetCircCfg(boardNum, &cfg, 0);  /* Hardcoded spId */
       if (status == CPI_TIMEOUT)
       {
           printf("TUP Circuit Config: No response from board\n");
           return(status);
       }
       else if (status != TUP_SUCCESS)
       {
           printf("TUP Circuit Config: Error %d\n", status);
           return(status);
       }

#endif                               /* ! CFGTEST */
       /* skip up to end of section */
       next = getcfgval("END", CFGT_INT, &temp, last);
       last = next;
    }   /* end for loop */
    return(0);

} /* TUPCircCfg */

/*
*
*  TUPcfg
*
*  Configures the TUP process by calling the individual configuration section
*  scanner routines in the right order.
*/
void TUPcfg(void)
{
    short   status;

    status = GenCfg();               /* General Configuration */
    if (status != 0)
        return;                         /* fatal */

    status = SapCfg();               /* User SAPs */
    if (status != 0)
        return;                         /* fatal */

    status = NSapCfg();       /* Network Saps */
    if (status != 0)
        return;                         /* fatal */

    CircCfg();                       /* Circuit Configuration (non-fatal)*/

} /* end of TUPcfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main(int argc, char *argv[])
{

    int             i;
    char           *p;
    S16             status;
    unsigned short  tmp;
    CDebug = 0;

    printf("\n%s: sample TUP configuration application version %s %s\n",
            argv[0], TUPCFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++) {
        p = argv[i];
        if (*p == '-') {
            switch (*++p) {
               case 'f' :     /* configuration file name */
                  i++;
                  strncpy(configFile, argv[i], sizeof(configFile) - 1);
                  configFile[sizeof(configFile) - 1] = '\0';  /* ensure null term */
                  break;

               case 'b' :    /* Set Board number */
                  i++;
                  if ((sscanf(argv[i], "%hu", &tmp) != 1) || (tmp < 1) || (tmp > 8))
                  {
                     printf(usage);
                     exit(1);
                  }
                  else
                     boardNum = (U8)tmp;
                  break;

               case 'd' :
                  debug = 1;              /* i/o packet dumps */
                  CDebug = 1;             /* config scanner debugging */
                  break;

               default:
                  printf(usage);
                  exit(1);
                  break;
            }       /* end switch */
        } else {
            printf(usage);
            exit(1);
        }

    }       /* end for */

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if (initcfg(configFile) != 0)
    {
       exit(1);
    }
    if ((status = TUPInitMgmtAPI(boardNum, TUP_ENT)) < 0)
    {
       fprintf(stderr, "Error %d initializing API\n", status);
       exit(1);
    }

    /* do the TUP configuration */
    TUPcfg( );

    TUPTermMgmtAPI(boardNum);

    endcfg();
}
