/* incoming.c:
/*
/* This is the incoming circuit thread
*/

#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "ctadef.h"
#include "ctaerr.h"


#include "osdep.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"
#include "tupapi.h"

#include "tupdemo.h"

static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForACMevt(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForANSWERevt(Circuit *circuit, Event *evt, DWORD evtId);
static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForCLFmsg(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForRLGevt(Circuit *circuit, Event *evt, DWORD evtId);

DWORD sendACM(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);
DWORD sendAnswer(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit);

StateDesc incStates[INC_MAXSTATES] =
{
    { "INCST_IDLE",			CallIdle		},
    { "INCST_WT4ACMEVT",	WaitForACMevt	},
    { "INCST_WT4ANSWEREVT",	WaitForANSWERevt},
    { "INCST_CONNECTED",	CallConnected	},
    { "INCST_WT4CLFMSG",	WaitForCLFmsg	},
	{ "INCST_WT4RLGEVT",	WaitForRLGevt	}
};

extern CTAHD		tupHndl;
extern short		traceFlags;
extern S16			switchType;
extern U8			boardNum;
extern Circuit	   *circuits;
extern char	       *eventNames[];
extern TupInstId    spInstId;

extern CirIdx    numIn;
extern CirIdx    numOut;
extern CirIdx    firstIn;
extern CirIdx    lastIn;
extern CirIdx    firstOut;
extern CirIdx    lastOut;


extern Bool     multiThread;
extern Bool     noOutbound;

/* external function from udpthrd.c */
void CheckPoint(Circuit *circuit);
/* external function from startup.c */
CirIdx CircuitSelect(CirIdx inCirId, U8 testCall);
/* external function from tupthrd.c */
DWORD EventCircuit(CirIdx srcId, CirIdx dstId, DWORD evtId, Event *event);
/* external function from sendmsg.c */
DWORD sendRSC(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendCLF(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendBLO(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendUBL(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendCFL(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendGRS(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);
DWORD sendMGB(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);
DWORD sendMGU(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);
DWORD sendCBK(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendRLG(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId, TupRelEvnt *event);

void IncomingThread(void *arglist)
{
    MULTI_TYPE     *args = arglist;
    Circuit        *circuit = &circuits[args->as_u32];
    CTAQUEUEHD      queue;
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    CirIdx          i;

	/* Initialize the circuit firstIn */
    circuit->cirId          = args->as_u32;
    circuit->mateId			= 0;
    circuit->state			= INCST_IDLE;
	circuit->callState		= NOT_APPL;
    circuit->circuitState	= IDLE_STATE;

    if ((ret = ctaCreateQueue(NULL, 0, &queue)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d IncomongThread() terminating, ctaCreateQueue() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(queue, 0, NULL, &circuit->ctaHndl)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d IncomingThread() terminating, ctaCreateContext() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

	/* Initialize all INCOMING circuits to IDLE_STATE */
	if (multiThread != TRUE)
    {
        for (i = firstIn ; i <= lastIn; i++)
        {
            circuits[i].mateId			= 0;
            circuits[i].ctaHndl			= circuit->ctaHndl;
            circuits[i].cirId			= i;
			circuits[i].state			= INCST_IDLE;
			circuits[i].callState		= NOT_APPL;
            circuits[i].circuitState    = IDLE_STATE;
        }
    }
    for (;;)
    {
        if ((ret = ctaWaitEvent(queue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "Circuit %d IncomingThread() terminating, ctaWaitEvent() failed: %d\n", circuit->cirId, ret);
            THREAD_EXIT();
        }
        circuit = &circuits[ctaEvent.value];
        if (traceFlags & EVT_TRACE)
            fprintf(stderr, "IncomingThread: %4d %s %s \n", circuit->cirId, eventNames[(ctaEvent.id & 0xff)],
                    incStates[circuit->state].stateName);

		(*(incStates[circuit->state].state))(circuit, (Event *)ctaEvent.buffer, ctaEvent.id);
    }
}


static void ThreadError(Circuit *circuit, char *call, DWORD ret)
{
    fprintf(stderr, "Circuit %d IncomingThread() terminating, %s failed: : %d\n", circuit->cirId, call, ret);
     THREAD_EXIT();
}


static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {

		case IAM_MSG:
            /*Select a circuit to initiate an outgoing call. If it is a test call, then processes
             * only incoming call. Doesn't make an outgoing call
             */
            ret = CircuitSelect(circuit->cirId, (U8)(evt->sdu.m.conEvnt.cgPtyCat.callPartyCat == TEST_CALL));
            if (ret == 0)
            {
                /* No outgoing circuit available */
				sendCFL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);
				circuit->mateId			= 0;
				circuit->state			= INCST_WT4CLFMSG;
                circuit->callState		= WAIT_CNI_CLF;		/* This should be in transient state */
				circuit->circuitState	= BUSY_IN;
                CheckPoint(circuit);

                if ( evt != NULL )
                   free(evt);
                break;
            }

            if ( noOutbound )
            {
                sendACM(tupHndl, boardNum, 0, 0, 0, circuit->cirId);
                sendAnswer(tupHndl, boardNum, 0, 0, 0, circuit->cirId);
                circuit->callState = WAIT_CNI_CLF;
                circuit->state     = INCST_CONNECTED;
            }
		    else
			{
                /*Makes an outgoing call on outgoing circuit, if incoming IAM contains Calling Party
                 * Category = ordinary subscriber call
                 */
				if ((ret = EventCircuit(circuit->cirId, circuit->mateId, IAM_EVT, evt)) != SUCCESS)
				   ThreadError(circuit, "EventCircuit()\n", ret);
				circuit->state			= INCST_WT4ACMEVT;
				circuit->callState		= WAIT_CNI_ACM;
				circuit->circuitState	= BUSY_IN;
				CheckPoint(circuit);
			}

            break;

        case RSC_MSG:

            printf("Incoming Thread: RSC received from circuit %d\n", circuit->cirId);

			circuit->callState		= NOT_APPL;
            if (circuit->circuitState == IDLE_BOTH_BLK)
            {
				circuit->circuitState = IDLE_LOC_BLK;
            }
			else
            {
				circuit->circuitState = IDLE_STATE;
            }

            CheckPoint(circuit);

            if (evt != NULL)
            {
               free(evt);
            }
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == IDLE_LOC_BLK)
				circuit->circuitState = IDLE_BOTH_BLK;
			else
				circuit->circuitState = IDLE_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == IDLE_REM_BLK)
				circuit->circuitState = IDLE_BOTH_BLK;
			else
				circuit->circuitState = IDLE_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *)&circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case CCR_MSG:

            printf("Incoming Thread: CCR received from circuit %d\n.", circuit->cirId);
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n, resetting the circuit!", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallIdle state , resetting the circuit!\n",
                    circuit->cirId, evtId);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
	}
}


static void WaitForACMevt(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {
        case ACM_EVT:

            if(TUPConnectStatusReq(tupHndl, boardNum, 0, 0, 0, circuit->cirId, (TupCnStEvnt *)&evt->sdu.m.conStEvnt,
                                   TUPADDRCMPLT) != TUP_SUCCESS)
				printf("TUPConnectStatusReq() failed in sending ACM!");
            else
                printf("Incoming Thread: ACM sent for circuit %d\n", circuit->cirId);

			circuit->state		= INCST_WT4ANSWEREVT;
			circuit->callState	= WAIT_CNI_ANSWER;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case CLF_MSG:

			circuit->state		= INCST_WT4RLGEVT;
            circuit->callState	= WAIT_CNI_LOCAL_RLG;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case CBK_EVT:

            sendCBK(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state		= INCST_WT4CLFMSG;
			circuit->callState	= WAIT_CNI_CLF;

            if ( evt != NULL )
               free(evt);
            break;

        case CFL_EVT:

            sendCFL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state		= INCST_WT4CLFMSG;
			circuit->callState	= WAIT_CNI_CLF;

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_MSG:
            printf("Incoming Thread: RSC received from circuit %d\n", circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;


            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == BUSY_IN_LOC_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_LOC_BLK;
			else
				circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_REM_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_REM_BLK;
			else
			circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);
            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n, resetting the circuit!", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);
            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForACMevt state , resetting\n",
                    circuit->cirId, evtId);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
	}
}


static void WaitForANSWERevt(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {
        case ANC_EVT:

			if(TUPConnectResp(tupHndl, boardNum, 0, 0, 0, circuit->cirId, (TupConRspEvnt *)&evt->sdu.m.conRspEvnt)
				!= TUP_SUCCESS)
				printf("TUPConnectStatusReq() failed in sending ANC!");
            else
                printf("Incoming Thread: ANC sent for circuit %d\n", circuit->cirId);

			circuit->state		= INCST_CONNECTED;
			circuit->callState	= WAIT_CNI_CLF;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case CLF_MSG:
            printf("Incoming Thread: CLF received from circuit %d\n.", circuit->cirId);

			circuit->state		= INCST_WT4RLGEVT;
            circuit->callState	= WAIT_CNI_LOCAL_RLG;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case CBK_EVT:

            sendCBK(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state		= INCST_WT4CLFMSG;
			circuit->callState	= WAIT_CNI_CLF;

            if ( evt != NULL )
               free(evt);
            break;

        case CFL_EVT:

            sendCFL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state		= INCST_WT4CLFMSG;
			circuit->callState	= WAIT_CNI_CLF;

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_MSG:

            printf("Incoming Thread: RSC received from circuit %d\n", circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == BUSY_IN_LOC_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_LOC_BLK;
			else
				circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_REM_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_REM_BLK;
			else
			circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n, resetting the circuit!", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state          = INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForANSWERevt state , resetting\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
	}
}


static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {
        case CLF_MSG:
            sendRLG(tupHndl, boardNum, 0, 0, 0, circuit->cirId, &evt->sdu.m.relEvnt);

            circuit->state      = INCST_IDLE;
            circuit->callState  = NOT_APPL;
            switch (circuit->circuitState)
			{
                case BUSY_IN:
					circuit->circuitState = IDLE_STATE;
					break;

                case BUSY_IN_LOC_BLK:
					circuit->circuitState = IDLE_LOC_BLK;
					break;

                case BUSY_IN_REM_BLK:
					circuit->circuitState = IDLE_REM_BLK;
					break;

                case BUSY_IN_BOTH_BLK:
					circuit->circuitState = IDLE_BOTH_BLK;
					break;
            }

			CheckPoint(circuit);
            if ( evt != NULL)
                free(evt);
			break;

        case CBK_EVT:

            sendCBK(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state		= INCST_WT4CLFMSG;
			circuit->callState	= WAIT_CNI_CLF;

            if ( evt != NULL)
                free(evt);
            break;

        case RSC_MSG:
            printf("Incoming Thread: RSC received from circuit %d\n", circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);
            if ( evt != NULL)
                free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == BUSY_IN_LOC_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_LOC_BLK;
			else
				circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_REM_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_REM_BLK;
			else
			circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n, resetting the circuit!", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallConnected state , resetting\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
    }
}


static void WaitForCLFmsg(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {
        case CLF_MSG:

			circuit->state		= INCST_WT4RLGEVT;
            circuit->callState	= WAIT_CNI_LOCAL_RLG;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case RSC_MSG:
            printf("Incoming Thread: RSC received from circuit %d\n.", circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == BUSY_IN_LOC_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_LOC_BLK;
			else
				circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_REM_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_REM_BLK;
			else
			circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForCLFmsg state , resetting\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
	}
}


static void WaitForRLGevt(Circuit *circuit, Event *evt, DWORD evtId)
{
    DWORD           ret;
	CirIdx          i;

    switch (evtId)
    {
        case RLG_EVT:

			sendRLG(tupHndl, boardNum, 0, 0, 0, circuit->cirId, &evt->sdu.m.relEvnt);

			switch (circuit->circuitState)
			{
				case BUSY_IN:
					circuit->circuitState = IDLE_STATE;
					break;

				case BUSY_IN_LOC_BLK:
					circuit->circuitState = IDLE_LOC_BLK;
					break;

				case BUSY_IN_REM_BLK:
					circuit->circuitState = IDLE_REM_BLK;
					break;

				case BUSY_IN_BOTH_BLK:
					circuit->circuitState = IDLE_BOTH_BLK;
					break;
			}
			circuit->state		= INCST_IDLE;
            circuit->callState	= NOT_APPL;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

        case RSC_MSG:

            printf("Incoming Thread: RSC received from circuit %d\n", circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == BUSY_IN_LOC_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_LOC_BLK;
			else
				circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_REM_BLK)
				circuit->circuitState = BUSY_IN_BOTH_BLK;
			else
				circuit->circuitState = BUSY_IN_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_IN_BOTH_BLK)
				circuit->circuitState = BUSY_IN_REM_BLK;
			else
			circuit->circuitState = BUSY_IN;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
				printf("Incoming Thread: GRS received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= INCST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGB received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()\n", ret);
			else
                printf("Incoming Thread: MGU received from circuit %d\n.", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_IN_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_IN;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:

            /*printf("Circuit %2d got a checkpointing message:\n", circuit->cirId);
			printf("Circuit ID	= %2d\t", evt->chkPntMsg.cirId);
			printf("Mate ID		= %2d\n", evt->chkPntMsg.mateId);
			printf("callState	= %x\t", evt->chkPntMsg.callState);
			printf("circuitState= %x\n", evt->chkPntMsg.circuitState);
            printf("state       = %2d\n", evt->chkPntMsg.state);*/

			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForRLGevt state , resetting\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= INCST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
			circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
	}
}
