/* outgoing.c
/*
/* Outgoing circuit thread
*/


#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "ctadef.h"
#include "ctaerr.h"

#include "osdep.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"
#include "tupapi.h"

#include "tupdemo.h"

static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForACMmsg(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForANSWERmsg(Circuit *circuit, Event *evt, DWORD evtId);
static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForRLGmsg(Circuit *circuit, Event *evt, DWORD evtId);

StateDesc outStates[OUT_MAXSTATES] =
{
    { "OUTST_IDLE",			CallIdle		},
    { "OUTST_WT4ACMMSG",	WaitForACMmsg	},
    { "OUTST_WT4ANSWERMSG",	WaitForANSWERmsg},
    { "OUTST_CONNECTED",	CallConnected	},
    { "OUTST_WT4RLGMSG",	WaitForRLGmsg	}
};

extern CTAHD		tupHndl;
extern short		traceFlags;
extern Circuit		*circuits;
extern char		    *eventNames[];
extern U8			 boardNum;


extern CirIdx   numIn;
extern CirIdx   numOut;
extern CirIdx   firstIn;
extern CirIdx   lastIn;
extern CirIdx   firstOut;
extern CirIdx   lastOut;
extern Bool     multiThread;

/* external function from udpthrd.c */
void CheckPoint(Circuit *circuit);
/* external function from tupthrd.c */
DWORD EventCircuit(CirIdx srcId, CirIdx dstId, DWORD evtId, Event *event);
/* external function from sendmsg.c */
DWORD sendRSC(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendCLF(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendBLO(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendUBL(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);
DWORD sendGRS(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);
DWORD sendMGB(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);
DWORD sendMGU(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range);

void OutgoingThread(void *arglist)
{
    MULTI_TYPE     *args = arglist;
    Circuit        *circuit = &circuits[args->as_u32];
    CTAQUEUEHD      queue;
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    CirIdx          i;

	/* Initialize the circuit firstOut */
    circuit->cirId              = args->as_u32;
    circuit->mateId				= 0;
    circuit->callState			= NOT_APPL;
    circuit->circuitState		= IDLE_STATE;
	circuit->state				= OUTST_IDLE;

    if ((ret = ctaCreateQueue(NULL, 0, &queue)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d OutgoingThread() terminating, ctaCreateQueue() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(queue, 0, NULL, &circuit->ctaHndl)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d OutgoingThread() terminating, ctaCreateContext() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if (multiThread != TRUE)
    {
        for (i = firstOut ; i <= lastOut ; i++)
        {
            circuits[i].mateId		= 0;
            circuits[i].ctaHndl		= circuit->ctaHndl;
            circuits[i].cirId		= i;
			circuits[i].callState		= NOT_APPL;
			circuits[i].circuitState	= IDLE_STATE;
			circuits[i].state			= OUTST_IDLE;

		}
    }

    for (;;)
    {
        if ((ret = ctaWaitEvent(queue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "Circuit %d OutgoingThread() terminating, ctaWaitEvent() failed: %d\n", circuit->cirId, ret);
            THREAD_EXIT();
        }
		circuit = &circuits[ctaEvent.value];

        if (traceFlags & EVT_TRACE)
            fprintf(stderr, "OutgoingThread: % 4d %s %s \n", circuit->cirId, eventNames[(ctaEvent.id & 0xff)],
                    outStates[circuit->state].stateName);

			(*(outStates[circuit->state].state))(circuit, (Event *)ctaEvent.buffer, ctaEvent.id);
	}
}


static void ThreadError(Circuit *circuit, char *call, DWORD ret)
{
    fprintf(stderr, "Circuit %d OutgoingThread() terminating, %s failed: : %d\n", circuit->cirId, call, ret);
    THREAD_EXIT();
}


static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
	CirIdx          i;

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case IAM_EVT:

			/* Upon receiving XXX_EVT from mated circuit, send XXX message through TUP API calls*/
            if (TUPConnectReq(tupHndl, boardNum, 0, 0, 0, (Bool)((circuit) ? 1 : 0), circuit->cirId,
                              &evt->sdu.m.conEvnt) != TUP_SUCCESS)
				printf("TUPConnectReq() failed in sending IAM!\n");
            else
                printf("Outgoing Thread: IAM sent for circuit %d\n", circuit->cirId);

			/* Change outgoing state if necessary*/
			circuit->state = OUTST_WT4ACMMSG;

			/* Change callState for this call if necessary */
			circuit->callState		= WAIT_CNO_ACM;

			/* Change circuitState for this circuit if necessary*/
			circuit->circuitState	= BUSY_OUT;

			/* Checkpointing and release evt */
			CheckPoint(circuit);

            if ( evt != NULL )
                free(evt);
			break;

        case RSC_MSG:
            printf("Outgoing Thread: RSC received from circuit %d\n.)", circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if(circuit->circuitState == IDLE_LOC_BLK)
				circuit->circuitState = IDLE_BOTH_BLK;
			else
				circuit->circuitState = IDLE_REM_BLK;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_MSG:

			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == IDLE_REM_BLK)
				circuit->circuitState = IDLE_BOTH_BLK;
			else
				circuit->circuitState = IDLE_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: GRS received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGB received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGU received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
     		break;

		case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
			circuit->state			= evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

            circuit->mateId			= 0;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

            fprintf(stderr, "Error indication received for circuit %d, resetting the circuit!", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallIdle state , resetting the circuit!\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
    }
}


static void WaitForACMmsg(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
	CirIdx          i;

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case SAM_EVT:
        {
            if(TUPConnectStatusReq(tupHndl, boardNum, 0, 0, 0, circuit->cirId, (TupCnStEvnt *)&evt->sdu.m.conStEvnt, TUPSUBSADDR) != TUP_SUCCESS)
                printf("TUPConnectStatusReq() failed in sending SAM!");
            else
                printf("Outgoing Thread: SAM sent for circuit %d\n", circuit->cirId);

            if ( evt != NULL )
               free(evt);

            break;
        }
        case SAO_EVT:
        {
            if(TUPConnectStatusReq(tupHndl, boardNum, 0, 0, 0, circuit->cirId, (TupCnStEvnt *)&evt->sdu.m.conStEvnt, TUPSUBADDRONE) != TUP_SUCCESS)
                printf("TUPConnectStatusReq() failed in sending SAO!");
            else
                printf("Outgoing Thread: SAO sent for circuit %d\n", circuit->cirId);

            if ( evt != NULL )
               free(evt);

            break;
        }
        case ACM_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, ACM_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			circuit->state		= OUTST_WT4ANSWERMSG;
 			circuit->callState	= WAIT_CNO_ANSWER;

			CheckPoint(circuit);
            break;

        case CBK_MSG:

			circuit->callState  = WAIT_CNO_RELEASE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case CFL_MSG:

            /*Call failure occured, To free the circuit for next call, send RESET*/
            sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state          = OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case CLF_EVT:

			sendCLF(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state		= OUTST_WT4RLGMSG;
			circuit->callState	= WAIT_CNO_RLG;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_MSG:
			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);
            break;

        case BLO_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, BLO_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_LOC_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_REM_BLK;

			CheckPoint(circuit);
            break;

        case UBL_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, UBL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			else
				circuit->circuitState = BUSY_OUT;

			CheckPoint(circuit);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_REM_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_REM_BLK;
			else
				circuit->circuitState = BUSY_OUT;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: GRS received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGB received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
            break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGU received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case CHKPNT_EVT:

			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
            circuit->state          = evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

			fprintf(stderr, "Error indication received for circuit, resetting the circuit! %d\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForACMmsg state , resetting the circuit!\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);
            if ( evt != NULL)
                free(evt);
            break;
    }
}


static void WaitForANSWERmsg(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
	CirIdx          i;

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {

		case ANC_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, ANC_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			circuit->state			= OUTST_CONNECTED;
			circuit->callState		= WAIT_CNO_RELEASE;

			CheckPoint(circuit);
            break;

        case CBK_MSG:
			circuit->callState  = WAIT_CNO_RELEASE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case CFL_MSG:

            /*Call failure occured, To free the circuit for next call, send RESET*/
            sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state          = OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case CLF_EVT:
			sendCLF(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, CLF_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			circuit->state		= OUTST_WT4RLGMSG;
			circuit->callState	= WAIT_CNO_RLG;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_MSG:
			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case BLO_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, BLO_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_LOC_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_REM_BLK;

			CheckPoint(circuit);
            break;

        case UBL_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, UBL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			else
				circuit->circuitState = BUSY_OUT;

			CheckPoint(circuit);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);
			if(circuit->circuitState == BUSY_OUT_REM_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_REM_BLK;
			else
				circuit->circuitState = BUSY_OUT;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: GRS received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGB received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGU received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
            circuit->state          = evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;
			free(evt);
			break;

        case ERR_EVT:

			fprintf(stderr, "Error indication received for circuit %d, resetting the circuit!\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForANSWERmsg state , resetting the circuit!\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
    }
}


static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
	CirIdx          i;

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {

		case CBK_MSG:
            sendCLF(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

            circuit->state      = OUTST_WT4RLGMSG;
			circuit->callState	= WAIT_CNO_RLG;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);

	        break;


        case CLF_EVT:

			sendCLF(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state		= OUTST_WT4RLGMSG;
			circuit->callState	= WAIT_CNO_RLG;

			CheckPoint(circuit);

            if (evt != NULL)
               free(evt);
            break;

        case RSC_MSG:
			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
              free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);
            break;

        case BLO_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, BLO_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_LOC_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_REM_BLK;

			CheckPoint(circuit);
            break;

        case UBL_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, UBL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			else
				circuit->circuitState = BUSY_OUT;

			CheckPoint(circuit);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_REM_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_LOC_BLK;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_REM_BLK;
			else
				circuit->circuitState = BUSY_OUT;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: GRS received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGB received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGU received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case CHKPNT_EVT:
			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
            circuit->state          = evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;
            if ( evt != NULL )
              free(evt);
			break;

        case ERR_EVT:

			fprintf(stderr, "Error indication received for circuit %d, resetting the circuit!\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

		default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallConnected state , resetting the circuit!\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
    }
}


static void WaitForRLGmsg(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
	CirIdx          i;

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case RLG_MSG:
			switch (circuit->circuitState)
			{
				case BUSY_OUT:
					circuit->circuitState = IDLE_STATE;
					break;

				case BUSY_OUT_LOC_BLK:
					circuit->circuitState = IDLE_LOC_BLK;
					break;

				case BUSY_OUT_REM_BLK:
					circuit->circuitState = IDLE_REM_BLK;
					break;

				case BUSY_OUT_BOTH_BLK:
					circuit->circuitState = IDLE_BOTH_BLK;
					break;
			}
			circuit->state		= OUTST_IDLE;
			circuit->callState	= NOT_APPL;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case RSC_MSG:
			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_LOC_BLK;
			else
				circuit->circuitState = IDLE_STATE;

			CheckPoint(circuit);
            if ( evt != NULL )
               free(evt);
            break;

        case RSC_EVT:

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			if(circuit->circuitState == IDLE_BOTH_BLK)
				circuit->circuitState = IDLE_REM_BLK;
			else
				circuit->circuitState = IDLE_STATE;

            CheckPoint(circuit);
            break;

		case BLO_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, BLO_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_LOC_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_REM_BLK;

			CheckPoint(circuit);
            break;

        case UBL_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, UBL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			else
				circuit->circuitState = BUSY_OUT;

			CheckPoint(circuit);
            break;

        case BLO_EVT:

			sendBLO(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_REM_BLK)
				circuit->circuitState = BUSY_OUT_BOTH_BLK;
			else
				circuit->circuitState = BUSY_OUT_LOC_BLK;
			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

        case UBL_EVT:

			sendUBL(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			if(circuit->circuitState == BUSY_OUT_BOTH_BLK)
				circuit->circuitState = BUSY_OUT_REM_BLK;
			else
				circuit->circuitState = BUSY_OUT;

            CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
			break;

		case GRS_MSG:

			if ((ret = EventCircuit(circuit->cirId, circuit->mateId, GRS_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: GRS received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

		case GRS_EVT:

			sendGRS(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				circuits[(circuit->cirId + i)].callState	= NOT_APPL;
				circuits[(circuit->cirId + i)].state		= OUTST_IDLE;
				if(circuits[(circuit->cirId + i)].circuitState	== IDLE_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = IDLE_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = IDLE_STATE;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGB_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGB_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGB received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_LOC_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGB_EVT:

			sendMGB(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_REM_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_BOTH_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

        case MGU_MSG:

            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, MGU_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
			else
                printf("Outgoing Thread: MGU received from circuit %d\n.)", circuit->cirId);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_LOC_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}
			break;

        case MGU_EVT:

			sendMGU(tupHndl, boardNum, 0, 0, 0, circuit->cirId, evt->sdu.m.staEvnt.range.range);

			for (i = 0; i < evt->sdu.m.staEvnt.range.range; i++)
			{
				if(circuits[(circuit->cirId + i)].circuitState == BUSY_OUT_BOTH_BLK)
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT_REM_BLK;
				else
					circuits[(circuit->cirId + i)].circuitState = BUSY_OUT;

				CheckPoint((Circuit *) &circuits[(circuit->cirId + i)]);
			}

            if ( evt != NULL )
               free(evt);
			break;

		case CHKPNT_EVT:

            /*printf("Circuit %2d got a checkpointing message (WaitForRLGmsg State):\n", circuit->cirId);
			printf("Circuit ID	= %2d\t", evt->chkPntMsg.cirId);
			printf("Mate ID		= %2d\n", evt->chkPntMsg.mateId);
			printf("callState	= %x\t", evt->chkPntMsg.callState);
			printf("circuitState= %x\n", evt->chkPntMsg.circuitState);
            printf("state       = %2d\n", evt->chkPntMsg.state);*/

			circuit->mateId			= evt->chkPntMsg.mateId;
	        circuit->callState		= evt->chkPntMsg.callState;
			circuit->circuitState	= evt->chkPntMsg.circuitState;
            circuit->state          = evt->chkPntMsg.state;

            if ( evt != NULL )
               free(evt);
            break;

        case IDLE_EVT:

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

            if ( evt != NULL )
               free(evt);
			break;

        case ERR_EVT:

			fprintf(stderr, "Error indication received for circuit %d\n", evt->info.circuit);

			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;

		default:

            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForRLGmsg state , resetting\n",
                    circuit->cirId, evtId);

            /*Invalid event received, Reset this circuit and make it available for next call*/
			sendRSC(tupHndl, boardNum, 0, 0, 0, circuit->cirId);

			circuit->state			= OUTST_IDLE;
			circuit->callState		= NOT_APPL;
			circuit->circuitState	= IDLE_STATE;

			CheckPoint(circuit);

            if ( evt != NULL )
               free(evt);
            break;
    }
}



