/* sendmsg.c: Function calls to send TUP messages by using TUP APIs */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "nmstypes.h"
#include "ctadef.h"

#include "tuptype.h"
#include "tupmsgs.h"
#include "tupdef.h"
#include "tupapi.h"

#include "tupdemo.h"

extern Bool quietOutput;

DWORD sendRSC(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupStaEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, cirId, TUPCIRRESREQ, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPStatusReq() failed in sending RSC, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("RSC sent for circuit %d\n", cirId);
        }
	}
    return (status);
}


DWORD sendCLF(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupRelEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPReleaseReq(ctahd, board, spId, suInstId, spInstId, cirId, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPReleaseReq() failed in sending CLF, status = %d\n", status);
	}
    /*else
	{
        printf("CLF sent for circuit %d\n", cirId);
    } */
    return (status);
}


DWORD sendRLG(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId, TupRelEvnt *event)
{
    DWORD            status;

	/* send */
    status = TUPReleaseResp(ctahd, board, spId, suInstId, spInstId, cirId, event);

	if( status != TUP_SUCCESS )
	{
		printf("TUPReleaseResp() failed in sending RLG, status = %d\n", status);
	}
    /*else
	{
        printf("RLG sent for circuit %d\n", cirId);
    } */
    return (status);
}


DWORD sendBLO(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupStaEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, cirId, TUPCIRBLKREQ, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPStatusReq() failed in sending BLO, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("BLO sent for circuit %d\n", cirId);
        }
	}
    return (status);
}


DWORD sendUBL(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupStaEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, cirId, TUPCIRUNBLKREQ, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPStatusReq() failed in sending UBL, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("UBL sent for circuit %d\n", cirId);
        }
	}
    return (status);
}


DWORD sendCFL(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupStaEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, cirId, TUPCALLFAILURE, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPStatusReq() failed in sending CFL, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("CFL sent for circuit %d\n", cirId);
        }
	}
    return (status);
}


DWORD sendGRS(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range)
{
	TupStaEvnt  evnt;
    DWORD       status;
    U8		    i;

	memset(&evnt, 0, sizeof(evnt));

    evnt.range.eh.pres	= PRESENT;
    evnt.range.range	= range;

    for (i = 0; i< range; i++)
		evnt.range.statusMap[i] = 1;

    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, circuit, TUPCIRGRPRESREQ, &evnt);
	if(status != TUP_SUCCESS)
	{
        printf("TUPStatusReq() failed in sending GRS, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("GRS sent for circuit %d\n", circuit);
        }
	}
    return (status);
}

DWORD sendMGB(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range)
{

	TupStaEvnt  evnt;
    DWORD       status;
    U8			i;
	memset(&evnt, 0, sizeof(evnt));

	evnt.range.eh.pres  = PRESENT;
	evnt.range.range	= range;

    for (i = 0; i< range; i++)
		evnt.range.statusMap[i] = 1;

    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, circuit, TUPCRMGRPBLKREQ, &evnt);

	if (status != TUP_SUCCESS)
	{
		printf("TUPStatusReq() failed in sending SGB, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("SGB sent for circuit %d\n", circuit);
        }
	}
    return (status);
}


DWORD sendMGU(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit, U8 range)
{

	TupStaEvnt  evnt;
    DWORD       status;
    U8			i;
	memset(&evnt, 0, sizeof(evnt));

	evnt.range.eh.pres  = PRESENT;
	evnt.range.range	= range;

    for (i = 0; i< range; i++)
		evnt.range.statusMap[i] = 1;

    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, circuit, TUPCRMGRPUNBLKREQ, &evnt);

	if (status != TUP_SUCCESS)
	{
		printf("TUPStatusReq() failed in sending SGU, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("SGU sent for circuit %d\n", circuit);
        }
	}
    return (status);
}

DWORD sendCBK(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId)
{
    DWORD            status;
	TupStaEvnt       evnt;

	/* initialize message  */
	memset(&evnt, 0, sizeof(evnt));

	/* send */
    status = TUPStatusReq(ctahd, board, spId, suInstId, spInstId, cirId, TUPCLEARBKW, &evnt);

	if( status != TUP_SUCCESS )
	{
		printf("TUPStatusReq() failed in sending CBK, status = %d\n", status);
	}
	else
	{
        if( !quietOutput)
        {
            printf("CBK sent for circuit %d\n", cirId);
        }
	}
    return (status);
}

DWORD sendACM(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit)
{
	TupCnStEvnt      addrcmp;
    DWORD            status;

	/* initialize ACM message by copying parameters from original
	 * connect indication (IAM) received
	 */
	memset( &addrcmp, 0, sizeof( addrcmp ) );

	/* send the ACM */
    status = TUPConnectStatusReq( ctahd, board, spId, suInstId, spInstId, circuit, &addrcmp, TUPADDRCMPLT );
	if( status != TUP_SUCCESS )
	{
		printf( "TUPConnectStatusReq() failed status = %d\n", status );
	}
    else
	{
        if( !quietOutput)
        {
            printf( "Address Complete sent for circuit %d\n", circuit );
        }
    }
    return(status);
}       /* end of sendACM */

DWORD sendAnswer(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, CirIdx circuit)
{
    DWORD           status;
	TupConRspEvnt   evnt;


	evnt.answerType.eh.pres = 1;
	evnt.answerType.answerType = ANU_CHARGE;

    status = TUPConnectResp(ctahd, board, spId, suInstId,
                            spInstId, circuit, &evnt);
	if( status != TUP_SUCCESS )
	{
		printf( "TUPConnectResp() failed status = %d\n", status );
	}
    else
	{
        if( !quietOutput)
        {
            printf( "Answer sent for circuit %d\n", circuit );
        }
    }
    return(status);
} /*End of sendAnswer*/



