/* startup.c: Main function of tupdemo */

#ifdef WIN32
#include <process.h>
/*#include <winsock.h>*/
#endif

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "nmstypes.h"
#include "ctadef.h"
#include "osdep.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"
#include "tupmgmt.h"

#include "tupdemo.h"

#ifdef UNIX
#define Sleep(x) usleep(1000 * x)
#endif

CTA_SERVICE_NAME  demoServiceNames[] = {{"TUP", "TUPMGR"}, { "HMI", "HMIMGR" } };

S16				switchType = TUP_SW_ITU;
U8              boardNum    = 1;
U16             locPort     = 4096;
U16             remPort     = 4096;
U32             remAddr;

Bool            multiThread = FALSE;
Bool            noOutbound = FALSE;
Bool            quietOutput = FALSE;

short           traceFlags  = 0;
CirIdx          numIn       = 12;
CirIdx          numOut      = 12;
CirIdx          firstIn;
CirIdx          lastIn;
CirIdx          firstOut;
CirIdx          lastOut;
Circuit         *circuits;

Event           *rcvEvt;

TMutex          criticalSection;

void TUPThread(void *dummy);			/* tupthrd.c */
void HMIThread (void *dummy);			/* hmithrd.c */
void UDPThread (void *dummy);			/* udpthrd.c */
void UserThread (void *dummy);			/* userthrd.c */
void TestThread (void *dummy);			/* testthrd.c */
void OutgoingThread(void *arglist);		/* outgoing.c */
void IncomingThread(void *arglist);		/* incoming.c */

int main (int argc, char *argv[])
{
    short           optind = 1;
    char           *str;
    struct hostent *hostent;        /* ptr to struct returned by gethostbyname() */
    DWORD           ret;
    char            buf[80];
    CTA_INIT_PARMS  demoInitparms = {0};
    U8              inHigh = FALSE;
    CirIdx          i;
    TThrId			threadId;
	TUPGenParms		cfg;
    MULTI_TYPE      arg;

    UNREFERENCED_PARAMETER(argc);

    arg.as_ptr = NULL;


    /* Initialize remote address to our loopback address */
    remAddr = htonl(INADDR_LOOPBACK);

	/* initialize cfg  */
	memset(&cfg, 0, sizeof(cfg));

    str = argv[optind];
    while (str)
    {
        if (*str == '-')
        {
            optind++;
            switch (*++str)
            {
                case 'b':   /* which board should we speak to ? */
                    if (*++str)         /* was there no space ? */
                        boardNum = (U8)atoi(str);
                    else
                        boardNum = (U8)atoi(argv[optind++]);
                    break;

                case 'd':
                    ++str;
                    if (*str == 'p')
                    {
                        if (*++str)         /* was there no space ? */
                            remPort = htons((U16)atoi(str));
                        else
                            remPort = htons((U16)atoi(argv[optind++]));
                    }
                    else if (*str == 'a')
                    {
                        if (*++str)         /* was there no space ? */
                            ;               /* str is OK now */
                        else
                            str = argv[optind++];
                        remAddr = inet_addr(str);
                    }
                    else if (*str == 'n')
                    {
                        if (*++str)         /* was there no space ? */
                            ;               /* str is OK now */
                        else
                            str = argv[optind++];
                        if ((hostent = gethostbyname(str)) == NULL)
                        {
                            fprintf(stderr,  "UDPThread terminating,  invalid mate IP address/host name [%s]\n", str);
                            THREAD_EXIT();
                        }

                        /* got host info for far end, use 1st IP address */
                        memcpy((char *)&(remAddr), hostent->h_addr, hostent->h_length);
                    }
                    break;

                case 'i':
                    inHigh = TRUE;
                    break;

                case 'l':
                    ++str;
                    if (*str == 'p')
                    {
                        if (*++str)         /* was there no space ? */
                            locPort = htons((U16)atoi(str));
                        else
                            locPort = htons((U16)atoi(argv[optind++]));
                    }
                    break;

                case 'm':
                    multiThread = TRUE;
                    break;

                case 'n':   /* which board should we speak to ? */
                    ++str;
                    if (*str == '\0')
						numIn = numOut = atoi(argv[optind++]);
                    else if (*str == 'i')
                    {
                        ++str;
                        if (*str == '\0')
                            numIn = atoi(argv[optind++]);
                        else
                            numIn = atoi(str);
                    }
                    else if (*str == 'o')
                    {
                        ++str;
                        if (*str == '\0')
                            numOut = atoi(argv[optind++]);
                        else
                            numOut = atoi(str);
                    }
                    else
                        numIn = numOut = atoi(str);

					/* Getting the Max. no of circuit allowed */
				    ret = TUPInitMgmtAPI(boardNum, ENT_ID);
					if( ret != TUP_SUCCESS )
					{
						printf( "TUPInitMgmtAPI failed [0x%x]\n", ret );
						exit(1);
					}

					if(TUPGetGenCfg(boardNum, &cfg) != TUP_SUCCESS)
						printf("Error in getting TUP general configurations.\n");

				    ret = TUPTermMgmtAPI(boardNum);
					if( ret != TUP_SUCCESS )
					{
						printf( "TUPTermMgmtAPI failed [0x%x]\n", ret );
						exit(1);
					}

					if( cfg.maxCircuits < (numIn + numOut) )
					{
						printf("No. of circuits exceeds the no. allowed! Exiting....\n");
						exit(1);
					}
					break;

                case 's':   /* which board should we speak to ? */
                    if (*++str == '\0')         /* was there no space ? */
                        str = argv[optind++];

                    if ((strcmp(str, "ITU") == 0) ||
                        (strcmp(str, "ITU") == 0))
                        switchType = TUP_SW_ITU;

                    else if ((strcmp(str, "CHN") == 0) ||
                             (strcmp(str, "CHN") == 0))
                        switchType = TUP_SW_CHN;
                    break;

                case 't':   /* which board should we speak to ? */
                    ++str;
                    if (*str == 'e')
                        traceFlags |= EVT_TRACE;
                    if (*str == 'c')
                        traceFlags |= CHKPNT_TRACE;
                    if (*str == 'a')
                        traceFlags = 0xffff;
                    break;

                case 'o':
                    noOutbound = TRUE;
                    break;

                case 'q':   /* quiet output */
                    quietOutput = TRUE;
                    break;

                default:
                    break;
            }
        }
        else
            break;

        str = argv[optind];
    }

    if (inHigh == TRUE) /* incoming circuits use higher circuit IDs */
    {
        firstIn  = numOut + 1;
        lastIn   = numOut + numIn;
        firstOut = 1;
        lastOut  = numOut;
    }
    else
    {
        firstIn  = 1;
        lastIn   = numIn;
        firstOut = numIn + 1;
        lastOut  = numIn + numOut;
    }

    if ((circuits = (Circuit *)calloc(((numIn + numOut) + 1), sizeof(Circuit))) == NULL)
    {
        fprintf(stderr, "Could not allocate circuit control blocks - Exiting\n");
        exit(1);
    }

    /* Initialize CT Access health management (HMI) service */
    demoInitparms.size = sizeof(CTA_INIT_PARMS);
    demoInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    MUTEX_CREATE(criticalSection);

    if ((ret = ctaInitialize(demoServiceNames, 2, &demoInitparms)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        fprintf(stderr, "HMIThread terminating, error initializing CT Access: %s\n", buf);
        THREAD_EXIT();
    }

    THREAD_CREATE(0, (TPtrThrFunc) UDPThread, argv, 0, threadId);
    THREAD_CREATE(0, (TPtrThrFunc) TUPThread, argv, 0, threadId);
    THREAD_CREATE(0, (TPtrThrFunc) UserThread, argv, 0, threadId);
    THREAD_CREATE(0, (TPtrThrFunc) TestThread, &circuits[0], 0, threadId);  /* cirId = 0 -> TestThread */
    THREAD_CREATE(0, (TPtrThrFunc) HMIThread, argv, 0, threadId);

    if (multiThread)
    {
        for (i = firstIn; i <= lastIn; i++)
        {
            arg.as_u32 = i;
            circuits[i].cirId = i;
            THREAD_CREATE(0, (TPtrThrFunc) IncomingThread, &arg, 0, threadId);
        }
        for (i = firstOut; i <= lastOut; i++)
        {
            arg.as_u32 = i;
            circuits[i].cirId = i;
            THREAD_CREATE(0, (TPtrThrFunc) OutgoingThread, &arg, 0, threadId);
        }
    }
    else
    {
        arg.as_u32 = firstIn;
        THREAD_CREATE(0, (TPtrThrFunc) IncomingThread, &arg, 0, threadId);
        arg.as_u32 = firstOut;
        THREAD_CREATE(0, (TPtrThrFunc) OutgoingThread, &arg, 0, threadId);
    }
    for (;;)
#ifdef UNIX
        sleep(1000);
#else
        Sleep(1000);
#endif
}


CirIdx CircuitSelect(CirIdx inCirId, U8 testCall)
{
    static char      init    = 0;
    static CirIdx    nextOut;
    CirIdx           cirIdx;

    if (!init)
    {
        init = 1;
        nextOut = firstOut;
    }

    /* If incoming IAM specific it's a TEST_CALL */
	if (testCall == TRUE)
	{
        /*circuits[inCirId].mateId = 0; */  /* if test call, send to test thread */
		return(1);
	}

    /* Choose available circuit to be mate circuit */
    for (cirIdx = nextOut; cirIdx <= lastOut; cirIdx++)
    {
       if ((circuits[cirIdx].callState == NOT_APPL) && (circuits[cirIdx].circuitState == IDLE_STATE))
       {
            if (cirIdx == lastOut)
                nextOut = firstOut;
            else
                nextOut = (cirIdx + 1);
            circuits[inCirId].mateId = cirIdx;
            circuits[cirIdx].mateId  = inCirId;

			MUTEX_UNLOCK(criticalSection);
            return(cirIdx);
        }
    }

    MUTEX_UNLOCK(criticalSection);
    return((CirIdx)0);
}
