#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"
#include "tupapi.h"
#include "tupmgmt.h"

#include "tupdemo.h"

extern CTAHD    tupHndl;
extern short    traceFlags;
extern S16      switchType;
extern Circuit *circuits;

extern char *promptStr;

/* external function from udpthrd.c */
void CheckPoint(Circuit *circuit);
/* external function from startup.c */
CirIdx CircuitSelect(CirIdx inCirId, U8 testCall);

void TestThread(void *arglist)
{
    Circuit        *circuit = (Circuit *)arglist;
    CTAQUEUEHD      queue;
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    Event          *evt;

    if ((ret = ctaCreateQueue(NULL, 0, &queue)) != SUCCESS)
    {
        fprintf(stderr, "TestThread() terminating, ctaCreateQueue() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(queue, 0, NULL, &circuit->ctaHndl)) != SUCCESS)
    {
        fprintf(stderr, "TestThread() terminating, ctaCreateContext() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }


    for (;;)
    {
        if ((ret = ctaWaitEvent(queue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "TestThread() terminating, ctaWaitEvent() failed: %d\n", circuit->cirId, ret);
            THREAD_EXIT();
        }
        evt = (Event *)ctaEvent.buffer;

        switch (ctaEvent.id)
        {
            case IAM_EVT:
                printf("TestThread: IAM_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case ACM_EVT:
                printf("TestThread: ACM_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case ANC_EVT:
                printf("TestThread: ANC_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case CLF_EVT:
                printf("TestThread: CLF_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case CLF_MSG:
                printf("TestThread: CLF_MSG received from circuit %d\n", evt->info.circuit);
                break;

            case CBK_EVT:
                printf("TestThread: CBK_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case CFL_EVT:
                printf("TestThread: CFL_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case RLG_EVT:
                printf("TestThread: RLG_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case RSC_EVT:
                printf("TestThread: RSC_EVT received from circuit %d\n", ctaEvent.value);
				break;

            case BLO_EVT:
                printf("TestThread: BLO_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case UBL_EVT:
                printf("TestThread: UBL_EVT received from circuit %d\n", evt->info.circuit);
                free(evt);
                break;

            case GRS_EVT:
                printf("TestThread: GRS_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case MGB_EVT:
                printf("TestThread: MGB_EVT received from circuit %d\n", evt->info.circuit);
                break;

            case MGU_EVT:
                printf("TestThread: MGU_EVT received from circuit %d\n", evt->info.circuit);
                break;

            default:
                printf("TestThread: unhandled event 0x%x from circuit %d\n", ctaEvent.id, evt->info.circuit);
        }
        if ( evt != NULL )
           free(evt);

        printf(promptStr);
    }
}

