 /*
 * tupthrd.c: Receive TUP events and send messages to circuit threads
 *
 */

#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"

#include "dpruser.h"
#include "txcpi.h"

#include "tupdef.h"
#include "tuptype.h"
#include "tupmsgs.h"
#include "tupapi.h"

#include "tupdemo.h"

/* globals */
U32 lastTrans = 0;
U32 lastIncCnt   = 0;
U32 lastOutCnt   = 0;
U16 poolsize = 200;

extern S16      switchType;
extern U8       boardNum;
extern char    *promptStr;
extern Circuit	*circuits;
extern CirIdx    numIn;
extern CirIdx    numOut;
extern CirIdx    firstIn;
extern CirIdx    lastIn;
extern CirIdx    firstOut;
extern CirIdx    lastOut;
extern TupRcvInfoBlk	rcvInfoBlk;
extern short    traceFlags;

/* external function from sendmsg.c */
DWORD sendRSC(CTAHD ctahd, U8 board, TupSpId spId, TupInstId suInstId, TupInstId spInstId, U32 cirId);

CTAHD               tupHndl         = NULL_CTAHD;
CTAQUEUEHD          tupQueue        = 0;
CTA_SERVICE_DESC    tupOpenSvcLst[] = {{{"TUP", "TUPMGR"}, {0}, {0}, {0}}};
char               *tupCloseSvcLst[] = { "TUP" };

int                 standAlone = 0;

static void DeadState(CTA_EVENT *ctaEvent);
static void StartedState(CTA_EVENT *ctaEvent);
static void BackupState(CTA_EVENT *ctaEvent);
static void PrimaryState(CTA_EVENT *ctaEvent);
static void StandAloneState(CTA_EVENT *ctaEvent);

void (*tupState)(CTA_EVENT *ctaEvent) = DeadState;		/* initial state = DeadState */

static void HandleTUPEvent();
static void OpenTUP(CTA_EVENT *ctaEvent);
static void CloseTUP(CTA_EVENT *ctaEvent);

/* external function from udpthrd.c */
void CheckPoint(Circuit *circuit);
void AppReady();

DWORD EventCircuit(CirIdx srcId, CirIdx dstId, DWORD evtId, Event *event)
{
    CTA_EVENT   ctaEvent = {0};

    ctaEvent.id     = evtId;
    ctaEvent.ctahd  = circuits[dstId].ctaHndl;
    ctaEvent.buffer = event;
    ctaEvent.size   = (event == NULL) ? 0 : sizeof(Event);  /* if event is null, set length to 0 */
    ctaEvent.value  = (dstId) ? dstId : srcId;              /* if dstId is test thread, send our own cirId */

    return(ctaQueueEvent(&ctaEvent));
}


void TUPThread(void *dummy)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;

    UNREFERENCED_PARAMETER(dummy);

    if ((ret = ctaCreateQueue(NULL, 0, &tupQueue)) != SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, ctaCreatQueue() failed: %d\n", ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(tupQueue, 0, NULL, &tupHndl)) != SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, ctaCreatContext() failed: %d\n", ret);
        THREAD_EXIT();
    }

    OpenTUP(&ctaEvent);
    tupState = StandAloneState;
    promptStr = "Stand Alone> ";
    printf("%s\n", promptStr);

    for (;;)
    {
        if ((ret = ctaWaitEvent(tupQueue, &ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "tupThread() terminating, ctaWaitEvent() failed\n");
            THREAD_EXIT();
        }

        /*if (tupState == DeadState)
                printf("tupstate = DeadState\n");
		else if (tupState == StartedState)
                printf("tupstate = StartedState\n");
		else if (tupState == BackupState)
            printf("tupstate = BackupState\n");
		else if (tupState == PrimaryState)
                printf("tupstate = PrimaryState\n");
		else if (tupState == StandAloneState)
                printf("tupstate = StandAloneState\n");*/

        (*tupState)(&ctaEvent);
    }
}


static void DeadState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "TUPThread(): 0x%x %d DeadState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case TUPEVN_DATA:
            fprintf(stderr, "TUPThread(): Unexpected TUP event received in Dead state\n");
            break;

        case STARTED_EVT:
            tupState = StartedState;
			printf("-> StartedState\n");
            break;

        case BACKUP_EVT:
            OpenTUP(ctaEvent);
            AppReady();
            tupState = BackupState;
			printf("-> BackupState\n");
            break;

        case PRIMARY_EVT:
            OpenTUP(ctaEvent);
            AppReady();
            tupState = PrimaryState;
            printf("-> PrimaryState\n");
            break;

        case STANDALONE_EVT:
            OpenTUP(ctaEvent);
            tupState = StandAloneState;
			printf("-> StandAloneState\n");
            break;

        case DEAD_EVT:
            tupState = DeadState;
			printf("-> DeadState\n");
            return;

        case STOP_EVT:
            tupState = DeadState;
            return;

        case APPREADY_EVT:
			printf("-> DeadState\n");
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );
            return;
    } /* switch CT Access event */

}

static void StartedState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "TUPThread(): 0x%x %d StartedState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case TUPEVN_DATA:
            fprintf(stderr, "TUPThread(): Unexpected TUP event received in Started state\n");
            break;

        case DEAD_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
			printf("-> DeadState\n");
            break;

        case STARTED_EVT:
            tupState = StartedState;
			printf("-> StartedState\n");
            break;

        case BACKUP_EVT:
            OpenTUP(ctaEvent);
            AppReady();
            tupState = BackupState;
			printf("-> BackupState\n");
            break;

        case PRIMARY_EVT:
            OpenTUP(ctaEvent);
            AppReady();
            tupState = PrimaryState;
            printf("-> PrimaryState\n");
            break;

        case STANDALONE_EVT:
            OpenTUP(ctaEvent);
            standAlone = TRUE;
            tupState = StandAloneState;
			printf("-> StandAloneState\n");
            break;

        case APPREADY_EVT:
			printf("-> StartedState\n");
            break;

        case STOP_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
			printf("-> DeadState\n");
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void BackupState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "TUPThread(): 0x%x %d BackupState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case TUPEVN_DATA:
            HandleTUPEvent();
 			printf("-> BackupState\n");
            break;

        case DEAD_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
			printf("-> DeadState\n");
            break;

        case STARTED_EVT:
            tupState = StartedState;
			printf("-> StartedState\n");
            break;

        case BACKUP_EVT:
            tupState = BackupState;
			printf("-> BackupState\n");
            break;

        case PRIMARY_EVT:
            tupState = PrimaryState;
            printf("-> PrimaryState\n");
            break;

        case STANDALONE_EVT:
            tupState = StandAloneState;
			printf("-> StandAloneState\n");
            break;

        case APPREADY_EVT:
            AppReady();
 			printf("-> BackupState\n");
            break;

        case STOP_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
			printf("-> DeadState\n");
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void PrimaryState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "TUPThread(): 0x%x %d PrimaryState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case TUPEVN_DATA:
            HandleTUPEvent();
            break;

        case DEAD_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
 			printf("-> DeadState\n");
            break;

        case STARTED_EVT:
            tupState = StartedState;
 			printf("-> StartedState\n");
            break;

        case BACKUP_EVT:
            tupState = BackupState;
 			printf("-> BackupState\n");
            break;

        case PRIMARY_EVT:
            tupState = PrimaryState;
            printf("-> PrimaryState\n");
            break;

        case STANDALONE_EVT:
            tupState = StandAloneState;
 			printf("-> StandAloneState\n");
            break;

        case APPREADY_EVT:
	        {
			    CirIdx       cirIdx;

				for (cirIdx = 1; cirIdx <= (numIn + numOut); cirIdx++)
					CheckPoint((Circuit *) &circuits[cirIdx]);
		    }
			break;

        case STOP_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
 			printf("-> DeadState\n");
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void StandAloneState(CTA_EVENT *ctaEvent)
{
    if (traceFlags & EVT_TRACE)
        fprintf(stderr, "TUPThread(): 0x%x %d StandAloneState\n", ctaEvent->id, ctaEvent->value);

    switch (ctaEvent->id)
    {
        case TUPEVN_DATA:
            HandleTUPEvent();
            break;

        case DEAD_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
            break;

        case STARTED_EVT:
            tupState = StartedState;
            break;

        case BACKUP_EVT:
            tupState = BackupState;
            break;

        case PRIMARY_EVT:
            tupState = PrimaryState;
            break;

        case STANDALONE_EVT:
            tupState = StandAloneState;
            break;

        case STOP_EVT:
            CloseTUP(ctaEvent);
            tupState = DeadState;
            break;

        default:
            fprintf(stderr,  "CTAHD(0x%08X) : Unhandled event id (0x%x) during wait.\n", ctaEvent->ctahd, ctaEvent->id );

    } /* switch CT Access event */
}

static void HandleTUPEvent()
{
    Event          *evt;
    DWORD           ret;
    CirIdx           i;

    /*printf("Handling TUP Events!\n"); */

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        fprintf(stderr, "tupThread() terminating, could not allocate required resources\n");
        THREAD_EXIT();
    }

	if ((ret = TUPRetrieveMessage(tupHndl, &evt->sdu, &evt->info)) != TUP_SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, TUPRetrieveMessage() failed: %d\n", ret);
        THREAD_EXIT();
    }

	switch( evt->info.indType )
    {

		case EVTTUPMGTSTAIND:			/* Management Status indication */
			free(evt);					/* not handled in this example */
			break;

        case EVTTUPCONCFM:              /* Connect confirm: ANN/ANC/ANU*/
			switch (evt->sdu.m.conRspEvnt.answerType.answerType)
			{
				case ANU_NO_CHARGE:
				    EventCircuit(0, evt->info.circuit, ANN_MSG, evt);
					break;

				case ANU_CHARGE:
				    EventCircuit(0, evt->info.circuit, ANC_MSG, evt);
					break;

				case ANU_UNQUALIFIED:
				    EventCircuit(0, evt->info.circuit, ANU_MSG, evt);
					break;
			} break;

        case EVTTUPCONIND:              /* Connect indication: IAI/IAM */
			if ((evt->sdu.m.conEvnt.cgPtyNum.eh.pres == PRESENT) || (evt->sdu.m.conEvnt.origCdNum.eh.pres == PRESENT) ||
				(evt->sdu.m.conEvnt.clUsrGrp.eh.pres == PRESENT))
			{
				EventCircuit(0, evt->info.circuit, IAI_MSG, evt);
			}
			else
			{

				EventCircuit(0, evt->info.circuit, IAM_MSG, evt);
			}
            break;

        case EVTTUPRELIND:              /* Connection release indication */
            EventCircuit(0, evt->info.circuit, CLF_MSG, evt);
            break;

        case EVTTUPRELCFM:              /* Connection release confirmation */
            EventCircuit(0, evt->info.circuit, RLG_MSG, evt);
            break;

        case EVTTUPCNSTIND:             /* Connection progress status indications */
            switch (evt->info.evntType)
            {
				case TUPADDRCMPLT:						/* address complete (ACM)*/
                    EventCircuit(0, evt->info.circuit, ACM_MSG, evt);
                    break;

                case TUPSUBSADDR:			            /* subsequent address (SAM)*/
                    EventCircuit(0, evt->info.circuit, SAM_MSG, evt);
                    break;

                case TUPSUBADDRONE:                     /* one subsequest addr (SAO)*/
                    EventCircuit(0, evt->info.circuit, SAO_MSG, evt);
                    break;

                case TUPREANSWER:                       /* reanswer signal (RAN) */
                    EventCircuit(0, evt->info.circuit, RAN_MSG, evt);
                    break;

                default:
                    /* unrecognized connect status indication */
					/* unhandled messages -> Release circuit */
                    EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
                    free(evt);
                   	fprintf(stderr, "Invalid or unhandled EVTTUPCNSTIND indications! Reset circuit\n.");
					break;
            } break; /* switch connect status indication */

        case EVTTUPSTAIND:              /* Status indications */

			switch (evt->info.evntType)
            {
                case TUPERRORIND:         /* error indication */
                    printf("TUP error = %d\n", evt->sdu.errInfo.errorCode);
                    EventCircuit(0, evt->info.circuit, ERR_EVT, evt);
                    break;

                case TUPCONTCHK:					/* continuity check (CCR)*/
                    EventCircuit(0, evt->info.circuit, CCR_MSG, evt);
                    break;

                case TUPCONTREP:					/* continuity report (COT)*/
                    EventCircuit(0, evt->info.circuit, COT_MSG, evt);
                    break;

                case TUPCONFAIL:					/* continuity failure (CCF)*/
                    EventCircuit(0, evt->info.circuit, CCF_MSG, evt);
                    break;

                case TUPCIRBLKREQ:					/* circuit block request (BLO)*/
                    printf("Received BLO_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, BLO_MSG, evt);
                    break;

                case TUPCIRUNBLKREQ:				/* circuit unblock request (UBL)*/
                    printf("Received UBL_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, UBL_MSG, evt);
                    break;

                case TUPCIRRESREQ:					/* circuit reset request (RSC)*/
                    printf("Received RSC_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, RSC_MSG, evt);
                    break;

                case TUPCIRGRPRESREQ:				/* circuit group reset request (GRS)*/
                    printf("Received GRS_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, GRS_MSG, evt);
                    break;

                case TUPCRMGRPBLKREQ:				/* maint circ grp block request (MGB)*/
                    printf("Received MGB_MSG from circuit %3d\n", evt->info.circuit);
                    break;

                case TUPCRMGRPUNBLKREQ:				/* maint circ grp unblock request (MGU)*/
                    printf("Received MGU_MSG from circuit %3d\n", evt->info.circuit);
                    break;

                case TUPCRHGRPBLKREQ:				/* h/w circ grp block request (HGB)*/
                    printf("Received HGB_MSG from circuit %3d\n", evt->info.circuit);
                    break;

                case TUPCRHGRPUNBLKREQ:				/* h/w circ grp unblock request (HGU)*/
                    printf("Received HGU_MSG from circuit %3d\n", evt->info.circuit);
                    break;

				case TUPCRSGRPBLKREQ:				/* s/w circ grp block request (SGB)*/
                    printf("Received SGB_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, SGB_MSG, evt);
                    break;

				case TUPCRSGRPUNBLKREQ:				/* s/w circ grp unblock request (SGU)*/
                    printf("Received SGU_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, SGU_MSG, evt);
                    break;

				/* Below are release cause inducators */

                case TUPSWITCHCONG:
                    printf("Received SEC_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, SEC_MSG, evt);
                    break;

                case TUPCIRCUITCONG:
                    printf("Received CGC_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, CGC_MSG, evt);
                    break;

                case TUPNATIONALCONG:
                    printf("Received NNC_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, NNC_MSG, evt);
                    break;

                case TUPADDRESSINCOMP:
                    printf("Received ADI_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, ADI_MSG, evt);
                    break;

                case TUPCALLFAILURE :
                    printf("Received CFL_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, CFL_MSG, evt);
                    break;

                case TUPSUBBUSY:
                    printf("Received SSB_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, SSB_MSG, evt);
                    break;

                case TUPUNALLOCATEDNUM:
                    printf("Received UNN_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, UNN_MSG, evt);
                    break;

                case TUPOUTOFSERVICE :
                    printf("Received LOS_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, LOS_MSG, evt);
                    break;

                case TUPSENDSPECIAL:
                    printf("Received SST_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, SST_MSG, evt);
                    break;

                case TUPACCESSBARRED :
                    printf("Received ACB_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, ACB_MSG, evt);
                    break;

                case TUPNODIGITAL:
                    printf("Received DPN_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, DPN_MSG, evt);
                    break;

                case TUPCLEARBKW:
                    printf("Received CBK_MSG from circuit %3d\n", evt->info.circuit);
                    EventCircuit(0, evt->info.circuit, CBK_MSG, evt);
                    break;

                case TUPMTP3PAUSE:
                    printf("Received TUPMTP3PAUSE from task!\n");
                    free(evt);
                    break;

                case TUPMTP3RESUME:
                    printf("Received TUPMTP3RESUME from task!\n");
                    free(evt);
                    break;

                case TUPMTP3BACKUP:
					printf("Received TUPMTP3BACKUP from task!\n");
                    for (i = 1; i <= (numIn + numOut); i++)
                    {
                        /* Restore all circuit states from Primary */
						CheckPoint((Circuit *) &circuits[i]);

                        /* Set all circuits with transient states to IDLE */
                        if ( (circuits[i].callState != WAIT_CNI_CLF) && (circuits[i].callState != WAIT_CNO_RELEASE) &&
                             (circuits[i].callState != NOT_APPL))
							EventCircuit(0, i, IDLE_EVT, NULL);
                    }
                    free(evt);
                    break;

                case TUPMTP3PRIMARY:
					for (i = 1; i <= (numIn + numOut); i++)
                    {
                 		/* RESET all circuits with transient states */
                        if ( (circuits[i].callState != WAIT_CNI_CLF) && (circuits[i].callState != WAIT_CNO_RELEASE) &&
                             (circuits[i].callState != NOT_APPL))
						{

							if (circuits[i].circuitState == IDLE_STATE)
								printf("Circuit %3d = IDLE_STATE\n", i);
							printf("Resetting transient circuit = %3d\n", circuits[i].cirId);

							sendRSC(tupHndl, boardNum, 0, 0, 0, circuits[i].cirId);

							circuits[i].callState		= NOT_APPL;
							circuits[i].circuitState	= IDLE_STATE;
							if (i >= firstIn && i <= lastIn)
								circuits[i].state = INCST_IDLE;
							else
								circuits[i].state = OUTST_IDLE;
						}
					}
                    free(evt);
                    break;

                case TUPMTP3STANDALONE:
					free(evt);
                    break;

                default: /* all other status indications */
                    EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
                    fprintf(stderr, "Invalid or unhandled EVTTUPSTAIND indications! Reset circuit.\n", ret);
					free(evt);
                    break;
            } break;   /* switch status indication */

	        default: /* unrecognized event */
            EventCircuit(0, evt->info.circuit, RSC_EVT, NULL);
			fprintf(stderr, "Invalid or unhandled TUP indications! Reset circuit.\n", ret);
			free(evt);
            break;
    }
}


static void OpenTUP(CTA_EVENT *ctaEvent)
{
    DWORD           ret;

    tupOpenSvcLst[0].svcargs.args[0] = boardNum;     /* board number */
    tupOpenSvcLst[0].svcargs.args[1] = INST_ID;      /* srcInst */
    tupOpenSvcLst[0].svcargs.args[2] = ENT_ID;       /* dprChan */
    tupOpenSvcLst[0].svcargs.args[3] = 1;            /* AutoBind? (yes=1, no= 0) */
    tupOpenSvcLst[0].svcargs.args[4] = SAP_ID;       /* spId */
    tupOpenSvcLst[0].svcargs.args[5] = SAP_ID;       /* suId */
    tupOpenSvcLst[0].svcargs.args[6] = poolsize;     /* poolsize */


    if ((ret = ctaOpenServices(tupHndl, tupOpenSvcLst, 1)) != SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, ctaOpenServices() failed: %d\n", ret);
        THREAD_EXIT();
    }

    /* Wait for service open to complete. */
    if ((ret = ctaWaitEvent(tupQueue, ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, ctaWaitEvent() failed: %d\n", ret);
        THREAD_EXIT();
    }
    if ((ctaEvent->id != CTAEVN_OPEN_SERVICES_DONE) || (ctaEvent->value != CTA_REASON_FINISHED))
    {
        fprintf(stderr, "tupThread() terminating, ctaOpenServices() failed: id - 0x%x value - %d\n", ctaEvent->id, ctaEvent->value);
        THREAD_EXIT();
    }
	else
		printf("Bind Confirmation Received and TUP Service Opened!\n");
}


static void CloseTUP(CTA_EVENT *ctaEvent)
{
    DWORD           ret;

    if ((ret = ctaCloseServices(tupHndl, tupCloseSvcLst, 1)) != SUCCESS)
    {
        fprintf(stderr, "tupThread() terminating, ctaCloseServices() failed: %d\n", ret);
        THREAD_EXIT();
    }

    /* Wait for service open to complete. */
    for (;;)
    {
        if ((ret = ctaWaitEvent(tupQueue, ctaEvent, (unsigned)CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "tupThread() terminating, ctaWaitEvent() failed: %d\n", ret);
            THREAD_EXIT();
        }

        if (ctaEvent->id != CTAEVN_CLOSE_SERVICES_DONE)
            continue;
        else if (ctaEvent->value != CTA_REASON_FINISHED)
        {
            fprintf(stderr, "tupThread() terminating, ctaCloseServices() failed\n");
            THREAD_EXIT();
        }
        else
            return;
    }
}


char *eventNames[] =
{
	"IAM_MSG		", /* 0x00 */
	"IAI_MSG		", /* 0x01 */
	"ANN_MSG		", /* 0x02 */
	"ANC_MSG		", /* 0x03 */
	"ANU_MSG		", /* 0x04 */
	"CLF_MSG		", /* 0x05 */
	"RLG_MSG		", /* 0x06 */
	"ACM_MSG		", /* 0x07 */
	"SAM_MSG		", /* 0x08 */
	"SAO_MSG 		", /* 0x09 */
	"RAN_MSG		", /* 0x0a */
	"CCR_MSG		", /* 0x0b */
	"COT_MSG		", /* 0x0c */
	"CCF_MSG		", /* 0x0d */
	"BLO_MSG		", /* 0x0e */
	"UBL_MSG		", /* 0x0f */
	"RSC_MSG		", /* 0x10 */
	"GRS_MSG 		", /* 0x11 */
	"MGB_MSG		", /* 0x12 */
	"MGU_MSG		", /* 0x13 */
	"HGB_MSG		", /* 0x14 */
	"HGU_MSG		", /* 0x15 */
	"SEC_MSG		", /* 0x16 */
	"CGC_MSG 		", /* 0x17 */
	"NNC_MSG 		", /* 0x18 */
	"ADI_MSG 		", /* 0x19 */
	"CFL_MSG		", /* 0x1a */
	"SSB_MSG		", /* 0x1b */
	"UNN_MSG 		", /* 0x1c */
	"LOS_MSG 		", /* 0x1d */
	"SST_MSG		", /* 0x1e */
	"ACB_MSG 		", /* 0x1f */
	"DPN_MSG		", /* 0x20 */
	"CBK_MSG		", /* 0x21 */
	"SGB_MSG		", /* 0x22 */
	"SGU_MSG		", /* 0x23 */
	"UNDEFINED		", /* 0x24 */
	"UNDEFINED		", /* 0x25 */
	"UNDEFINED		", /* 0x26 */
	"UNDEFINED		", /* 0x27 */
	"UNDEFINED		", /* 0x28 */
	"UNDEFINED		", /* 0x29 */
	"UNDEFINED		", /* 0x2a */
	"UNDEFINED		", /* 0x2b */
	"UNDEFINED		", /* 0x2c */
	"UNDEFINED		", /* 0x2d */
	"UNDEFINED		", /* 0x2e */
	"UNDEFINED		", /* 0x2f */
	"IAM_EVT		", /* 0x30 */
	"IAI_EVT		", /* 0x31 */
	"ANN_EVT		", /* 0x32 */
	"ANC_EVT		", /* 0x33 */
	"ANU_EVT		", /* 0x34 */
	"CLF_EVT		", /* 0x35 */
	"RLG_EVT		", /* 0x36 */
	"ACM_EVT		", /* 0x37 */
	"SAM_EVT		", /* 0x38 */
	"SAO_EVT 		", /* 0x39 */
	"RAN_EVT		", /* 0x3a */
	"CCR_EVT		", /* 0x3b */
	"COT_EVT		", /* 0x3c */
	"CCF_EVT		", /* 0x3d */
	"BLO_EVT		", /* 0x3e */
	"UBL_EVT		", /* 0x3f */
	"RSC_EVT		", /* 0x40 */
	"GRS_EVT 		", /* 0x41 */
	"MGB_EVT		", /* 0x42 */
	"MGU_EVT		", /* 0x43 */
	"HGB_EVT		", /* 0x44 */
	"HGU_EVT		", /* 0x45 */
	"SEC_EVT		", /* 0x46 */
	"CGC_EVT 		", /* 0x47 */
	"NNC_EVT 		", /* 0x48 */
	"ADI_EVT 		", /* 0x49 */
	"CFL_EVT		", /* 0x4a */
	"SSB_EVT		", /* 0x4b */
	"UNN_EVT 		", /* 0x4c */
	"LOS_EVT 		", /* 0x4d */
	"SST_EVT		", /* 0x4e */
	"ACB_EVT 		", /* 0x4f */
	"DPN_EVT		", /* 0x50 */
	"CBK_EVT		", /* 0x51 */
	"SGB_EVT		", /* 0x52 */
	"SGU_EVT		", /* 0x53 */
	"UNDEFINED		", /* 0x54 */
	"UNDEFINED		", /* 0x55 */
	"UNDEFINED		", /* 0x56 */
	"UNDEFINED		", /* 0x57 */
	"UNDEFINED		", /* 0x58 */
	"UNDEFINED		", /* 0x59 */
	"UNDEFINED		", /* 0x5a */
	"UNDEFINED		", /* 0x5b */
	"UNDEFINED		", /* 0x5c */
	"UNDEFINED		", /* 0x5d */
	"UNDEFINED		", /* 0x5e */
	"UNDEFINED		", /* 0x5f */
	"ERR_EVT		", /* 0x60 */
	"DEAD_EVT		", /* 0x61 */
	"STARTED_EVT	", /* 0x62 */
	"BACKUP_EVT		", /* 0x63 */
	"PRIMARY_EVT	", /* 0x64 */
	"STANDALONE_EVT	", /* 0x65 */
	"CHKPNT_EVT		", /* 0x66 */
    "IDLE_EVT       ", /* 0x67 */
	"APPREADY_EVT	", /* 0x68 */
	"STOP_EVT 		", /* 0x69 */
};

