/* udpthrd.c
 *
 * This module provides the communication interface to the mate node.
 *
 */

#ifdef WIN32
#include <windows.h>
#include <winsock.h>
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>

#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"

#include "osdep.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"

#include "tupdemo.h"

/* Constants and defaults */
#define REQUIRED_WINSOCK_VERSION    0x0101  /* requires at least winsock version 1.1 */

/* Globals */
static TSockAddrIn           remSock;    /* IP address/port for mate application */
static TSockAddrIn           locSock;    /* this application's socket */
static TSocket               Socket;     /* socket to receive on */

extern U16      locPort;
extern U16      remPort;
extern U32      remAddr;
extern Circuit *circuits;
extern short    traceFlags;
extern CTAHD    tupHndl;

/* External functions from tupthrd.c */
DWORD EventCircuit(CirIdx srcId, CirIdx dstId, DWORD evtId, Event *event);

void TraceCheckPoint(ChkPntMsg *chkPntMsg, int dirFlag)
{
    char       *callState;
    char       *circuitState;
    char       *dir;

    switch (chkPntMsg->callState)
    {
        case NOT_APPL:
			callState = "NOT_APPL";
            break;

        case WAIT_CNI_CLF:
            callState = "WAIT_CNI_CLF		";
            break;

		case WAIT_CNI_INFO:
            callState = "WAIT_CNI_INFO		";
            break;

		case WAIT_CNI_ANSWER:
            callState = "WAIT_CNI_ANSWER	";
            break;

		case WAIT_CNI_ACM:
            callState = "WAIT_CNI_ACM		";
            break;

		case WAIT_CNI_LOCAL_RLG:
            callState = "WAIT_CNI_LOCAL_RLG	";
            break;

		case WAIT_CNI_COT:
            callState = "WAIT_CNI_COT		";
            break;

		case WAIT_CNI_CCR:
            callState = "WAIT_CNI_CCR		";
            break;

		case WAIT_CNI_CONT_CLF:
            callState = "WAIT_CNI_CONT_CLF	";
            break;

		case CNI_DEAD:
            callState = "CNI_DEAD			";
            break;

        case WAIT_CNO_RLG:
            callState = "WAIT_CNO_RLG		";
            break;

		case WAIT_CNO_RELEASE:
            callState = "WAIT_CNO_RELEASE	";
            break;

		case WAIT_CNO_INFO:
            callState = "WAIT_CNO_INFO		";
            break;

		case WAIT_CNO_ANSWER:
            callState = "WAIT_CNO_ANSWER	";
            break;

		case WAIT_CNO_ACM:
            callState = "WAIT_CNO_ACM		";
            break;

		case WAIT_CNO_COT:
            callState = "WAIT_CNO_COT		";
            break;

		case WAIT_CNO_CONT_CLF:
            callState = "WAIT_CNO_CONT_CLF	";
            break;

		case WAIT_CNO_CCR:
            callState = "WAIT_CNO_CCR		";
            break;

		case CNO_DEAD:
            callState = "CNO_DEAD			";
            break;

		default:
            callState = "UNKNOWN!			";
            break;
    }
    switch (chkPntMsg->circuitState)
    {
		case IDLE_STATE:
            circuitState = "IDLE_STATE			";
            break;

        case IDLE_LOC_BLK:
            circuitState = "IDLE_LOC_BLK		";
            break;

        case IDLE_REM_BLK:
            circuitState = "IDLE_REM_BLK		";
            break;

        case IDLE_BOTH_BLK:
            circuitState = "IDLE_BOTH_BLK		";
            break;

        case BUSY_IN:
            circuitState = "BUSY_IN				";
            break;

        case BUSY_IN_LOC_BLK:
            circuitState = "BUSY_IN_LOC_BLK		";
            break;

        case BUSY_IN_REM_BLK:
            circuitState = "BUSY_IN_REM_BLK		";
            break;

		case BUSY_IN_BOTH_BLK:
            circuitState = "BUSY_IN_BOTH_BLK	";
            break;

        case BUSY_OUT:
            circuitState = "BUSY_OUT			";
            break;

        case BUSY_OUT_LOC_BLK:
            circuitState = "BUSY_OUT_LOC_BLK	";
            break;

        case BUSY_OUT_REM_BLK:
            circuitState = "BUSY_OUT_REM_BLK	";
            break;

		case BUSY_OUT_BOTH_BLK:
            circuitState = "BUSY_OUT_BOTH_BLK	";
            break;

		case OOS_L:
            circuitState = "OOS_L				";
            break;

        case OOS_L_LOC_BLK:
            circuitState = "OOS_L_LOC_BLK		";
            break;

        case OOS_L_REM_BLK:
            circuitState = "OOS_L_REM_BLK		";
            break;

        case OOS_L_BOTH_BLK:
            circuitState = "OOS_L_BOTH_BLK		";
            break;

		case OOS_R:
            circuitState = "OOS_R				";
            break;

        case OOS_R_LOC_BLK:
            circuitState = "OOS_R_LOC_BLK		";
            break;

        case OOS_R_REM_BLK:
            circuitState = "OOS_R_REM_BLK		";
            break;

        case OOS_R_BOTH_BLK:
            circuitState = "OOS_R_BOTH_BLK		";
            break;

		case OOS_B:
            circuitState = "OOS_B				";
            break;

        case OOS_B_LOC_BLK:
            circuitState = "OOS_B_LOC_BLK		";
            break;

        case OOS_B_REM_BLK:
            circuitState = "OOS_B_REM_BLK		";
            break;

        case OOS_B_BOTH_BLK:
            circuitState = "OOS_B_BOTH_BLK		";
            break;

		default:
            circuitState = "UNKNOWN!			";
            break;
    }
    switch (dirFlag)
    {
        case DIR_TX:
            dir = "TX";
            break;

        case DIR_RX:
            dir = "RX";
            break;

        default:
            dir = "??";
            break;
    }

    fprintf(stderr, "CheckPoint %s: %d %d \n\t\t callstate: %s ; circuitstate: %s %s\n", dir, chkPntMsg->cirId, chkPntMsg->mateId, callState, circuitState);
}


/*
 * UDPThread
 *
 * This is the entry point to the receiver thread; its job is to loop continuously
 * doing blocking reads on the socket and posting any received packets to the SCTL
 * module in the form of "data indication" events, until a socket error is received
 * (indicating that the socket has been shut down and its time to stop).
 *
 * Parameters:
 *
 *  pParam      pointer to socket handle to use
 *
 * Returns:     Always 0
 *
 */
void UDPThread( void *dummy )
{

#ifdef WIN32
    WSADATA         wsadata;        /* sockets version data */
    int             wsaret;         /* sockets return code */

    UNREFERENCED_PARAMETER(dummy);

    wsaret = WSAStartup(REQUIRED_WINSOCK_VERSION, &wsadata);
    if (wsaret != 0)
    {
        fprintf(stderr,  "UDPThread terminating, unable to initialize sockets API, error %d\n ", wsaret );
        THREAD_EXIT();
    }
    else if ((LOBYTE(wsadata.wVersion) != 1) || HIBYTE(wsadata.wVersion) != 1)
    {
        /* incompatible SOCKETS API version */
        fprintf(stderr, "UDPThread terminating, unsupported sockets API version (%d.%d)\n",
                 HIBYTE(wsadata.wVersion), LOBYTE(wsadata.wVersion));
        THREAD_EXIT();
    }
#endif

    /* next create a UDP datagram socket */
    if((SOCKET_OPEN( AF_INET, SOCK_DGRAM, 0, Socket )) == RET_FAILED )
    {
        /* fatal error */
        fprintf(stderr, "UDPThread terminating, unable to create socket, error %d", osGetLastError());
        THREAD_EXIT();
    }
    locSock.sin_port = locPort;
    locSock.sin_family = AF_INET;
    locSock.sin_addr.s_addr = htonl(INADDR_ANY);

    remSock.sin_port = remPort;
    remSock.sin_family = AF_INET;
    remSock.sin_addr.s_addr = remAddr;

    /* Bind the socket to our configured port */
    if(SOCKET_BIND(Socket, (TSockAddr*)&locSock, sizeof(locSock))
       != RET_SUCCESS)
    {
        fprintf(stderr, "UDPThread terminating, unable to bind socket, error %d", osGetLastError());
        THREAD_EXIT();
    }

    for (;;)
    {
        CTA_EVENT           ctaEvent;
        TSockAddrIn         fromAddr;       /* IP address/port for mate application */
        Event              *evt;
        int                 fromLen;        /* length of socket address structure */
        int                 sret;           /* sockets return value */
        int                 recvLen;        /* length of received message */

        if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
        {
            fprintf(stderr, "UDPThread terminating, memory allocation failure\n");
            THREAD_EXIT();
        }

        fromLen = sizeof(struct sockaddr_in);

		if ((SOCKET_RECVFROM(Socket, (char*) &evt->chkPntMsg, sizeof(ChkPntMsg), 0,
                             (TSockAddr*) &fromAddr, &fromLen, recvLen)) != RET_SUCCESS)
		{
            sret = osGetLastError();
            if( sret == ERR_MSGSIZE )
            {
                /* message too big, probably from some other application due to port
                 * number conflict - display warning so users can resolve conflict
                 */
                fprintf(stderr,  "UDPThread: Message from %s too long [len=%d], discarded", inet_ntoa(fromAddr.sin_addr), recvLen );
            }
#ifdef UNIX
            else if (sret == ECONNREFUSED)
            {
                /* ignore this error, which occurs before other TUPdemo starts */
            }
#else
            else  if ((sret != WSAECONNREFUSED) &&
                      (sret != WSAECONNRESET) )
            {
                fprintf(stderr, "UDPThread terminating, socket receive failed %ld\n", osGetLastError());
                THREAD_EXIT();
            }
#endif
        }
        else if  (evt->chkPntMsg.msgId == CHKPNT_MSG)
        {
            if (traceFlags & CHKPNT_TRACE)
                TraceCheckPoint(&evt->chkPntMsg, DIR_RX);

            EventCircuit(0, evt->chkPntMsg.cirId, CHKPNT_EVT, evt);
        }
        else if  (evt->chkPntMsg.msgId == APPRDY_MSG)
        {
            memset(&ctaEvent, 0, sizeof(CTA_EVENT));

            ctaEvent.id     = APPREADY_EVT;
            ctaEvent.size   = 0;
            ctaEvent.buffer = NULL;
            ctaEvent.ctahd  = tupHndl;
            ctaQueueEvent(&ctaEvent);
            free(evt);
        }
        else
            fprintf(stderr, "UDPThread: Unknown message type 0x%x received\n", evt->chkPntMsg.msgId);
    }
}


void CheckPoint(Circuit *circuit)
{

    ChkPntMsg   chkPntMsg;         /* buffer for outgoing message */

    /* construct message */

    chkPntMsg.msgId			= CHKPNT_MSG;
    chkPntMsg.cirId			= circuit->cirId;
    chkPntMsg.mateId	    = circuit->mateId;
    chkPntMsg.callState		= circuit->callState;
    chkPntMsg.circuitState  = circuit->circuitState;
    chkPntMsg.state			= circuit->state;

    if (traceFlags & CHKPNT_TRACE)
      TraceCheckPoint(&chkPntMsg, DIR_TX);

    /*printf("Circuit %2d with following info. will be checkpointed:\n", circuit->cirId);
	printf("Circuit ID	= %2d\t", chkPntMsg.cirId);
	printf("Mate ID		= %2d\n", chkPntMsg.mateId);
	printf("callState	= %x\t", chkPntMsg.callState);
	printf("circuitState= %x\n", chkPntMsg.circuitState);
    printf("state       = %2d\n", chkPntMsg.state);*/

	if (SOCKET_SENDTO(Socket, (char *) &chkPntMsg, sizeof(ChkPntMsg), 0,
      (TSockAddr*) &remSock, sizeof(TSockAddrIn)) != RET_SUCCESS)
	fprintf(stderr,  "UDP: Unable to send checkpoint message, error %ld\n", osGetLastError());

}


void AppReady()
{

    ChkPntMsg   chkPntMsg;         /* buffer for outgoing message */

    /* construct message */

    chkPntMsg.msgId     = APPRDY_MSG;

    if (SOCKET_SENDTO(Socket, (char *) &chkPntMsg, sizeof(ChkPntMsg), 0,
                      (TSockAddr*) &remSock, sizeof(TSockAddrIn)) != RET_SUCCESS)
	    fprintf(stderr,  "UDP: Unable to send ready message, error %ld\n", osGetLastError());
}
