/*
 * Userthrd.c
 *
 * User Thread to accept user input
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef UNIX
#include <ctype.h>
#endif                                  /* UNIX */

#include "osdep.h"
#include "nmstypes.h"
#include "ctadef.h"

#include "tuptype.h"
#include "tupdef.h"
#include "tupmsgs.h"
#include "tupmgmt.h"

#include "tupapi.h"

#include "tupdemo.h"

#define MAX_ARGS    16
#define SEPCHARS    " \n"

/* global */
extern U32 lastTrans;
extern U32 lastIncCnt;
extern U32 lastOutCnt;

char           *promptStr = "Unknown> ";
extern U8       boardNum;
extern U16      switchType;
extern CirIdx   numIn;
extern CirIdx   numOut;
extern CirIdx   firstIn;
extern CirIdx   lastIn;
extern CirIdx   firstOut;
extern CirIdx   lastOut;
extern Circuit *circuits;
extern CTAHD    tupHndl;

/* External functions from tupthrd.c */
DWORD EventCircuit(CirIdx srcId, CirIdx dstId, DWORD evtId, Event *event);


void cmdHELP(char *args[])
{
    if (args[0] == NULL)
    {
        printf("QUIT        exit application\n");
        printf("IAM         send an initial address message\n");
        printf("ACM         send an address complete message\n");
        printf("ANC         send an answer message (Charge) \n");
        printf("CLF         send a clear forward\n");
        printf("RLG         send a release complete message\n");
        printf("RSC         send a circuit reset message\n");
        printf("BLO         send a blocking message\n");
        printf("UBL         send an unblocking message\n");
        printf("GRS         send a circuit group reset message\n");
        printf("MGB         send a maintainence circuit group blocking message\n");
        printf("MGU         send a maintainence circuit group unblocking message\n");
        printf("CBK         send a clear-back signal\n");
	    printf("CFL         send a call failure signal\n");
	    printf("CIRCUIT		check the status of a circuit\n");
    }
    else
    {
        if (strcmp(args[0], "Q") == 0)
            exit(0);
        if (strcmp(args[0], "QUIT") == 0)
            exit(0);
        if (strcmp(args[0], "IAM") == 0)
            printf("Syntax: IAM <circuid ID> <called>\n");
	    if (strcmp(args[0], "ACM") == 0)
            printf("Syntax: ACM <circuid ID>\n");
        if (strcmp(args[0], "ANM") == 0)
            printf("Syntax: ANM <circuid ID>\n");
        if (strcmp(args[0], "CLF") == 0)
            printf("Syntax: CLF <circuid ID>\n");
        if (strcmp(args[0], "RLG") == 0)
            printf("Syntax: RLG <circuid ID>\n");
        if (strcmp(args[0], "RSC") == 0)
            printf("Syntax: RSC <circuid ID>\n");
        if (strcmp(args[0], "BLO") == 0)
            printf("Syntax: BLO <circuid ID>\n");
        if (strcmp(args[0], "UBL") == 0)
            printf("Syntax: UBL <circuid ID>\n");
        if (strcmp(args[0], "GRS") == 0)
            printf("Syntax: GRS <circuid ID> <range>\n");
        if (strcmp(args[0], "MGB") == 0)
            printf("Syntax: MGB <circuid ID> <range>\n");
        if (strcmp(args[0], "MGU") == 0)
            printf("Syntax: MGU <circuid ID> <range>\n");
        if (strcmp(args[0], "CBK") == 0)
	        printf("Syntax: CBK <circuid ID>\n");
        if (strcmp(args[0], "CFL") == 0)
            printf("Syntax: CFL <circuid ID>\n");
	    if (strcmp(args[0], "CIRCUIT") == 0)
            printf("Syntax: CIRCUIT <circuid ID>\n");
    }
}


void cmdIAM(char *args[])
{

    CirIdx      cirId;
    Event      *evt;
    DWORD       ret;
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (args[1] == NULL)
    {
        printf("UserThread: Missing called number\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) < firstOut) || (cirId > lastOut))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    /* mandatory calling party category */
    evt->sdu.m.conEvnt.cgPtyCat.eh.pres      = PRESENT;
    evt->sdu.m.conEvnt.cgPtyCat.callPartyCat = ORDINARY_TOLL_CALLER; /* ordinary subscriber */

    /* mandatory message indicator */
    evt->sdu.m.conEvnt.msgInd.eh.pres  = PRESENT;
    evt->sdu.m.conEvnt.msgInd.msgInd   = IAM_MI_LOCAL_SUBSCRIBER;

    /* called party number */
    evt->sdu.m.conEvnt.addrSigs.eh.pres = PRESENT;
    TUPASCIItoADDR(args[1], &evt->sdu.m.conEvnt.addrSigs.addrSigs, (short)strlen(args[1]) );

    if ((ret = EventCircuit(0, cirId, IAM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdSAM(char *args[])
{

    CirIdx      cirId;
    Event      *evt;
    DWORD       ret;
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (args[1] == NULL)
    {
        printf("UserThread: Missing called number\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) < firstOut) || (cirId > lastOut))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    /* called party number */
    evt->sdu.m.conStEvnt.addrSigs.eh.pres = PRESENT;
    TUPASCIItoADDR(args[1], &evt->sdu.m.conStEvnt.addrSigs.addrSigs, (short)strlen(args[1]) );

    if ((ret = EventCircuit(0, cirId, SAM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdSAO(char *args[])
{

    CirIdx      cirId;
    Event      *evt;
    DWORD       ret;
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (args[1] == NULL)
    {
        printf("UserThread: Missing address signal(s)\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) < firstOut) || (cirId > lastOut))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    /* called party number */
    evt->sdu.m.conStEvnt.addrSigs.eh.pres = PRESENT;
    TUPASCIItoADDR(args[1], &evt->sdu.m.conStEvnt.addrSigs.addrSigs, 1 );

    if ((ret = EventCircuit(0, cirId, SAO_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdACM(char *args[])
{

    CirIdx      cirId;
    Event      *evt;
    DWORD       ret;
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("User Thread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    /* mandatory address complete inducator */
    evt->sdu.m.conStEvnt.msgInd.eh.pres   = PRESENT;
    evt->sdu.m.conStEvnt.msgInd.indicator = ACM_COMPLETE;

    if ((ret = EventCircuit(0, cirId, ACM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdANC(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    /* mandatory address complete inducator */
    evt->sdu.m.conRspEvnt.answerType.eh.pres    = PRESENT;
    evt->sdu.m.conRspEvnt.answerType.answerType = ANU_CHARGE;

    if ((ret = EventCircuit(0, cirId, ANC_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdCLF(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, CLF_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdRLG(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, RLG_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdRSC(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, RSC_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdBLO(char *args[])
{
    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, BLO_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdUBL(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, UBL_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdGRS(char *args[])
{

    DWORD       ret;
    DWORD       temp;
    U8          range;
    CirIdx      cirId;
    Event      *evt;
    char        buf[80];
    int         i;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if ((temp = (DWORD) atoi(args[1])) > 256)
    {
        printf("UserThread: Range exceed limit! (max. is 256) \n");
        return;
    }
    else
    {
        range = (U8) temp;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    evt->sdu.m.staEvnt.range.eh.pres    = PRESENT;
    evt->sdu.m.staEvnt.range.range      = range;
    for (i = 0; i < range; i++)
        evt->sdu.m.staEvnt.range.statusMap[i] = 1;

    if ((ret = EventCircuit(0, cirId, GRS_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdMGB(char *args[])
{
    DWORD       temp;
    DWORD       ret;
    U8          range;
    CirIdx      cirId;
    Event      *evt;
    char        buf[80];
    int         i;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if ((temp = (DWORD) atoi(args[1])) > 256)
    {
        printf("UserThread: Range exceed limit! (max. is 256) \n");
        return;
    }
    else
    {
        range = (U8) temp;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    evt->sdu.m.staEvnt.range.eh.pres    = PRESENT;
    evt->sdu.m.staEvnt.range.range      = range;
    for (i = 0; i < range; i++)
        evt->sdu.m.staEvnt.range.statusMap[i] = 1;

    if ((ret = EventCircuit(0, cirId, MGB_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdMGU(char *args[])
{
    DWORD       temp;
    DWORD       ret;
    U8          range;
    CirIdx      cirId;
    Event      *evt;
    char        buf[80];
    int         i;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if ((temp = (DWORD) atoi(args[1])) > 256)
    {
        printf("UserThread: Range exceed limit! (max. is 256) \n");
        return;
    }
    else
    {
        range = (U8) temp;
    }


    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    evt->sdu.m.staEvnt.range.eh.pres    = PRESENT;
    evt->sdu.m.staEvnt.range.range	= range;
    for (i = 0; i < range; i++)
        evt->sdu.m.staEvnt.range.statusMap[i] = 1;

    if ((ret = EventCircuit(0, cirId, MGU_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdCBK(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > ( + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, CBK_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdCFL(char *args[])
{

    DWORD       ret;
    CirIdx      cirId;
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > ( + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }
    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    /* clear out event buffer to zeros (0 => NOT_PRESENT) */
    memset( evt, 0, sizeof(Event) );

    if ((ret = EventCircuit(0, cirId, CFL_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}


void cmdCIRCUIT(char *args[])
{
    char       *callState;
    char       *circuitState;
    CirIdx      cirId;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn	+ numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    switch (circuits[cirId].callState)
    {
        case NOT_APPL:
            callState = "NOT_APPL";
            break;
        case WAIT_CNI_CLF:
            callState = "WAIT_CNI_CLF";
            break;
        case WAIT_CNI_INFO:
            callState = "WAIT_CNI_INFO";
            break;
        case WAIT_CNI_ANSWER:
            callState = "WAIT_CNI_ANSWER";
            break;
        case WAIT_CNI_ACM:
            callState = "WAIT_CNI_ACM";
            break;
        case WAIT_CNI_LOCAL_RLG:
            callState = "WAIT_CNI_LOCAL_RLG";
            break;
        case WAIT_CNI_COT:
            callState = "WAIT_CNI_COT";
            break;
        case WAIT_CNI_CCR:
            callState = "WAIT_CNI_CCR";
            break;
        case WAIT_CNI_CONT_CLF:
            callState = "WAIT_CNI_CONT_CLF";
            break;
        case CNI_DEAD:
            callState = "CNI_DEAD";
            break;
        case WAIT_CNO_RLG:
            callState = "WAIT_CNO_RLG";
            break;
        case WAIT_CNO_RELEASE:
            callState = "WAIT_CNO_RELEASE";
            break;
        case WAIT_CNO_INFO:
            callState = "WAIT_CNO_INFO";
            break;
        case WAIT_CNO_ANSWER:
            callState = "WAIT_CNO_ANSWER";
            break;
        case WAIT_CNO_ACM:
            callState = "WAIT_CNO_ACM";
            break;
        case WAIT_CNO_COT:
            callState = "WAIT_CNI_COT";
            break;
        case WAIT_CNO_CONT_CLF:
            callState = "WAIT_CNO_CONT_CLF";
            break;
        case WAIT_CNO_CCR:
            callState = "WAIT_CNO_CCR";
            break;
        case CNO_DEAD:
            callState = "CNO_DEAD";
            break;
        default:
            callState = "Unknown";
    }

    switch (circuits[cirId].circuitState)
    {
        case IDLE_STATE:
            circuitState = "IDLE_STATE";
            break;
        case IDLE_LOC_BLK:
            circuitState = "IDLE_LOC_BLK";
            break;
        case IDLE_REM_BLK:
            circuitState = "IDLE_REM_BLK";
            break;
        case IDLE_BOTH_BLK:
            circuitState = "IDLE_BOTH_BLK";
            break;
        case BUSY_IN:
            circuitState = "BUSY_IN";
            break;
        case BUSY_IN_LOC_BLK:
            circuitState = "BUSY_IN_LOC_BLK";
            break;
        case BUSY_IN_REM_BLK:
            circuitState = "BUSY_IN_REM_BLK";
            break;
        case BUSY_IN_BOTH_BLK:
            circuitState = "BUSY_IN_BOTH_BLK";
            break;
        case BUSY_OUT:
            circuitState = "BUSY_OUT";
            break;
        case BUSY_OUT_LOC_BLK:
            circuitState = "BUSY_OUT_LOC_BLK";
            break;
        case BUSY_OUT_REM_BLK:
            circuitState = "BUSY_OUT_REM_BLK";
            break;
        case BUSY_OUT_BOTH_BLK:
            circuitState = "BUSY_OUT_BOTH_BLK";
            break;
        case OOS_L:
            circuitState = "OOS_L";
            break;
        case OOS_L_LOC_BLK:
            circuitState = "OOS_L_LOC_BLK";
            break;
        case OOS_L_REM_BLK:
            circuitState = "OOS_L_REM_BLK";
            break;
        case OOS_L_BOTH_BLK:
            circuitState = "OOS_L_BOTH_BLK";
            break;
        case OOS_R:
            circuitState = "OOS_R";
            break;
        case OOS_R_LOC_BLK:
            circuitState = "OOS_R_LOC_BLK";
            break;
        case OOS_R_REM_BLK:
            circuitState = "OOS_R_REM_BLK";
            break;
        case OOS_R_BOTH_BLK:
            circuitState = "OOS_R_BOTH_BLK";
            break;
        case OOS_B:
            circuitState = "OOS_B";
            break;
        case OOS_B_LOC_BLK:
            circuitState = "OOS_B_LOC_BLK";
            break;
        case OOS_B_REM_BLK:
            circuitState = "OOS_B_REM_BLK";
            break;
        case OOS_B_BOTH_BLK:
            circuitState = "OOS_B_BOTH_BLK";
            break;
        default:
            circuitState = "Unknown";
    }
    printf("circuit %d:	callState = %s\n\t	circuitState = %s \n", cirId, callState, circuitState);
}


void strtoupper(char *s)
{
    while (*s != '\0')
        *s++ = (char)toupper(*s);
}


void UserThread(void *dummy)
{
    static char in[133];
    static char *inPtr = in;
    char        *cmd;
    char        *args[MAX_ARGS + 1];
    int         i;

    UNREFERENCED_PARAMETER(dummy);

    for (;;)
    {
        printf(promptStr);
        fgets(in, 132, stdin);
        while ( (strlen(in) > 0) && (in[strlen(in) - 1] < ' ') )
            in[strlen(in) - 1] = '\0';      /* remove any trailing non-printable characters */

        strtoupper(in);
        if ((cmd = strtok(in, SEPCHARS)) == NULL)
            continue;

        memset(args, 0, ((sizeof(char *) * MAX_ARGS) + 1));

        for (i = 0; i <= MAX_ARGS; i++)
        {
            if ((args[i] = strtok(NULL, SEPCHARS)) == NULL)
                break;
        }

        if (strcmp(cmd, "Q") == 0)
            exit(0);
        if (strcmp(cmd, "QUIT") == 0)
            exit(0);
        else if (strcmp(cmd, "IAM") == 0)
            cmdIAM(args);
        else if (strcmp(cmd, "SAM") == 0)
            cmdSAM(args);
        else if (strcmp(cmd, "SAO") == 0)
            cmdSAO(args);
        else if (strcmp(cmd, "ACM") == 0)
            cmdACM(args);
        else if (strcmp(cmd, "ANC") == 0)
            cmdANC(args);
        else if (strcmp(cmd, "CLF") == 0)
            cmdCLF(args);
        else if (strcmp(cmd, "RLG") == 0)
            cmdRLG(args);
        else if (strcmp(cmd, "RSC") == 0)
            cmdRSC(args);
        else if (strcmp(cmd, "BLO") == 0)
            cmdBLO(args);
        else if (strcmp(cmd, "UBL") == 0)
            cmdUBL(args);
        else if (strcmp(cmd, "GRS") == 0)
            cmdGRS(args);
        else if (strcmp(cmd, "MGB") == 0)
            cmdMGB(args);
        else if (strcmp(cmd, "MGU") == 0)
            cmdMGU(args);
        else if (strcmp(cmd, "CBK") == 0)
            cmdCBK(args);
        else if (strcmp(cmd, "CFL") == 0)
            cmdCFL(args);
        else if (strcmp(cmd, "CIRCUIT") == 0)
            cmdCIRCUIT(args);
        else if (strcmp(cmd, "HELP") == 0)
            cmdHELP(args);
        else if (strcmp(cmd, "?") == 0)
            cmdHELP(args);
        else
            printf("Error: Unrecognized command, type HELP for list of commands\n");
    }
}

