/* TUP management test application */

#ifdef UNIX
#include <sys/types.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#endif                                  /* UNIX */

#ifdef WIN32
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* WIN32 */

#ifdef __OS2__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* __OS2__ */

/*#include "txcpi.h"   */
#include "ss7type.h"
#include "tupmgmt.h"
#include "tupver.h"
#include "tupdef.h"
#include "osdep.h"


void resetReq( short *parms );
void deleteReq( short *parms );
void blockReq( short *parms );
void unblockReq( short *parms );
void hblockReq( short *parms );
void hunblockReq( short *parms );
void sblockReq( short *parms );
void sunblockReq( short *parms );
void mblockReq( short *parms );
void munblockReq( short *parms );
void statusReq( short *parms );
void statsReq( short *parms );
void traceReq( short *parms );
void HelpCommand( short *parms );
void setCP( short *parms );
void getConfigReq( short *parms );
void quit( short *parms );

char *circuitState [] =
{
    "Initial",
    "Idle",
    "Idle, Locally Blocked",
    "Idle, Remotely Blocked",
    "Idle, Remotely Blocked, Locally Blocked",
    "Busy Incoming",
    "Busy Incoming, Locally Blocked",
    "Busy Incoming, Remotely Blocked",
    "Busy Incoming, Remotely Blocked, Locally Blocked",
    "Busy OutGoing",
    "Busy OutGoing, Locally Blocked",
    "Busy OutGoing, Remotely Blocked",
    "Busy OutGoing, Remotely Blocked, Locally Blocked",
    "Out of Service locally",
    "Out of Service locally, Locally Blocked",
    "Out of Service locally, Remotely Blocked",
    "Out of Service locally, Remotely Blocked, Locally Blocked",
    "Out of Service remotely",
    "Out of Service remotely, Locally Blocked",
    "Out of Service remotely, Remotely Blocked",
    "Out of Service remotely, Remotely Blocked, Locally Blocked",
    "Out of Service BOTH",
    "Out of Service BOTH, Locally Blocked",
    "Out of Service BOTH, Remotely Blocked",
    "Out of Service BOTH, Remotely Blocked, Locally Blocked"
};

#define MAXCALLSTATE    0x0f

char *callState [2][10] =
{
    {
        "Initial State",
        "Wait CLF State, in call",
        "Wait Info State",
        "Wait Answer State",
        "Wait ACM State",
        "Wait local RLG State",
        "Wait COT State",
        "Wait CCR State",
        "Wait CLF to end cont check State",
        "Dead State",
    },
    {
        "Initial State",
        "Wait RLG State",
        "Wait Release State",
        "Wait Info State",
        "Wait Answer State",
        "Wait ACM State",
        "Wait Local COT State",
        "Wait Local CONT CLF State",
        "Wait Next CCR State",
        "Dead State",
    }
};

#define MAXTRANSSTATE    0x10

/* lookup tables for translating names into values and vice versa */
struct lookupEnt
{
	char 	*name;
	short	value;
};


/* parameter name tables */

struct lookupEnt StatsLookupTbl[] =
{
    { "NSAP",    0 },
    { "CIRCUIT", 1 },
    { "GROUP",   2 },
    { 0,         0 }
};

struct lookupEnt GetConfigLookupTbl[] =
{
    { "GEN",     1 },
    { "USAP",    2 },
    { "NSAP",    3 },
    { "CIRCUIT", 4 },
    { "GROUP",   5 },
    { 0,         0 }
};

struct lookupEnt ResetLookupTbl[] =
{
    { "CIRCUIT", 1 },
    { "GROUP",   2 },
    { 0,         0 }
};

struct lookupEnt DeleteLookupTbl[] =
{
    { "CIRCUIT", 1 },
    { "GROUP",   2 },
    { 0,         0 }
};

struct lookupEnt OnOffLookupTbl[] =
{
    { "ON",  1 },
    { "OFF", 0 },
    { 0,     0 }
};

struct lookupEnt TraceLookupTbl[] =
{
    { "EVENT",   TRACE_EVENT   },
    { "DATA",    TRACE_DATA    },
    { "ERROR",   ENCODE_ERROR  },
    { "WARNING", ENUM_WARNING  },
    { "ELEMENT", INIT_ELMT_DBG },
    { "TIMER",   TMR_TRACE     },
    { 0,         0             }
};


#ifdef JPTEMP
static char validateHelp[]        =   "    Syntax: VALIDATE <number> \n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";
#endif

static char getConfigHelp[]        =   "    Syntax: GET <entity> [<number>] \n\n"
"     where:\n"
"            <entity>      [GEN | USAP | NSAP | CIRCUIT | GROUP]\n"
"            <number>      entity ID\n\n";

static char resetHelp[]        =   "    Syntax: RESET <entity> <number>\n\n"
"     where:\n"
"            <entity>      [CIRCUIT | GROUP]\n"
"            <number>      entity ID\n\n";

static char deleteHelp[]        =   "    Syntax: DELETE <entity> <number> <count>\n\n"
"     where:\n"
"            <entity>      [CIRCUIT | GROUP]\n"
"            <number>      entity ID\n"
"            <count>       if circuit, count of circuits to delete\n\n";

static char blockHelp[]        =   "    Syntax: BLOCK <number>\n\n"
"     where:\n"
"            <number>      circuit ID\n\n";

static char unblockHelp[]        =   "    Syntax: UNBLOCK <number>\n\n"
"     where:\n"
"            <number>      circuit ID\n\n";

static char mblockHelp[]        =   "    Syntax: MBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char hblockHelp[]        =   "    Syntax: HBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char munblockHelp[]        =   "    Syntax: MUNBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char sblockHelp[]        =   "    Syntax: SBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char sunblockHelp[]        =   "    Syntax: SUNBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char hunblockHelp[]        =   "    Syntax: HUNBLOCK <number>\n\n"
"     where:\n"
"            <number>      group ID\n\n";

static char statusHelp[]        =   "    Syntax: STATUS <entity> <number>\n\n"
"     where:\n"
"            <entity>      [CIRCUIT | GROUP]\n"
"            <number>      entity ID\n\n";

static char statsHelp[]        =   "    Syntax: STATS <entity> <number>\n\n"
"     where:\n"
"            <entity>      [NSAP | CIRCUIT | GROUP]\n"
"            <number>      entity ID\n\n";

static char quitHelp[]              =   "    Syntax: Q \n\n"
"    Quit TUPmgr application\n\n\n";

static char traceHelp[]               =   "    Syntax: TRACE <entity> {ON | OFF}\n\n"
"     where:\n"
"            <entity>      [EVENT | DATA | ERROR | WARNING | ELEMENT | TIMER]\n";

static char boardHelp[]        =   "    Syntax: BOARD <new target board number>\n\n";

struct lookupEnt CmdLookupTbl[] =
{
        { "ALL",        0  },
        { "BLOCK",      1  },
        { "UNBLOCK",    2  },
        { "RESET",      3  },
        { "MBLOCK",     4  },
        { "HBLOCK",     5  },
        { "SBLOCK",     6  },
        { "SUNBLOCK",   7  },
        { "MUNBLOCK",   8  },
        { "HUNBLOCK",   9  },
        { "STATUS",    10  },
        { "STATS",     11  },
        { "GET",       12  },
        { "TRACE",     13  },
        { "BOARD",     14  },
        { "Q",         15  },
        { "DELETE",    16  },
};

static char allHelp[]               =   "    BLOCK      Blocks a circuit\n"
"    UNBLOCK    Unblocks a circuit\n"
"    MBLOCK     Maintenance blocks a circuit group\n"
"    HBLOCK     Hardware blocks a circuit group\n"
"    SBLOCK     Software blocks a circuit group\n"
"    MUNBLOCK   Maintenance unblocks a circuit group\n"
"    HUNBLOCK   Hardware unblocks a circuit group\n"
"    SUNBLOCK   Software unblocks a circuit group\n"
"    RESET      Resets a circuit or group\n"
"    DELETE     Deletes a circuit or group\n"
"    STATUS     Retrieves a circuit or group status\n"
"    STATS      NSAP | Circuit | group statistics\n"
"    GET        Gets configuration information\n"
"    TRACE      Turns on/off tracing\n"
"    BOARD      Changes target board\n"
"    Q          Quits tupmgr application\n"
"\n";

static char* helpTbl[] =
{
	    allHelp,
        blockHelp,
        unblockHelp,
        resetHelp,
        mblockHelp,
        hblockHelp,
        sblockHelp,
        sunblockHelp,
        munblockHelp,
        hunblockHelp,
        statusHelp,
	    statsHelp,
        getConfigHelp,
        traceHelp,
		boardHelp,
        quitHelp,
        deleteHelp
};
/* structure for command parse tables */
#define MAX_PARMS	10

struct CmdSpec
{
	char				*command;
	struct lookupEnt	*parmTbl[MAX_PARMS];
	void				(*func)(short *);
};

/* Help Command */
struct CmdSpec  helpCmd =
{
	"?",               /* command name */
	    { CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	    HelpCommand         /* handler function */
};

/* tupmgmt API commands */

struct CmdSpec  resetCmd =
{
    "RESET",                                            /* command name */
    { ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    resetReq                                            /* handler function */
};

struct CmdSpec  deleteCmd =
{
    "DELETE",                                            /* command name */
    { DeleteLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    deleteReq                                            /* handler function */
};

struct CmdSpec  blockCmd =
{
    "BLOCK",                                            /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    blockReq                                            /* handler function */
};

struct CmdSpec  mblockCmd =
{
    "MBLOCK",                                            /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    mblockReq                                            /* handler function */
};

struct CmdSpec  hblockCmd =
{
    "HBLOCK",                                            /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    hblockReq                                            /* handler function */
};

struct CmdSpec  sblockCmd =
{
    "SBLOCK",                                            /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    sblockReq                                            /* handler function */
};

struct CmdSpec  unblockCmd =
{
    "UNBLOCK",                                          /* command name */
    { ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    unblockReq                                          /* handler function */
};

struct CmdSpec  munblockCmd =
{
    "MUNBLOCK",                                          /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    munblockReq                                          /* handler function */
};

struct CmdSpec  hunblockCmd =
{
    "HUNBLOCK",                                          /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    hunblockReq                                          /* handler function */
};

struct CmdSpec  sunblockCmd =
{
    "SUNBLOCK",                                          /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    sunblockReq                                          /* handler function */
};

struct CmdSpec  statusCmd =
{
    "STATUS",                                           /* command name */
    { ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },                   /* parameter lookups */
    statusReq                                           /* handler function */
};

struct CmdSpec	statsCmd =
{
    "STATS",                                            /* command name */
    { StatsLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },      /* parameter lookups */
    statsReq                                            /* handler function */
};

struct CmdSpec  getConfigCmd =
{
    "GET",                                              /* command name */
    { GetConfigLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    getConfigReq                                        /* handler function */
};

struct CmdSpec	boardCmd =
{
    "BOARD",                                            /* command name */
    {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },                  /* parameter lookups */
    setCP                                               /* handler function */
};

struct CmdSpec	quitCmd =
{
    "Q",                                                /* command name */
		{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
		quit											/* handler function */
};

struct CmdSpec  traceCmd =
{
    "TRACE",                                            /* command name */
        {  TraceLookupTbl,OnOffLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 }, /* parameter lookups */
     traceReq                                           /* handler function */
};

struct CmdSpec *CmdTable[] = {
        &blockCmd,
        &unblockCmd,
        &resetCmd,
        &mblockCmd,
        &munblockCmd,
        &hblockCmd,
        &hunblockCmd,
        &sblockCmd,
        &sunblockCmd,
        &statusCmd,
		&statsCmd,
        &getConfigCmd,
        &traceCmd,
		&boardCmd,
		&quitCmd,
        &deleteCmd,
	    &helpCmd,
	    0
};

/* Misc. globals */
char	valStr[20];
unsigned char   cp = 1;
short	quittinTime = 0;

/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char *getname( struct lookupEnt *tbl, short val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( tbl->value == val )
		{
			/* match found */
			return( tbl->name );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed - convert number to ascii */
	sprintf( valStr, "%d", val );
	return( valStr );

}	/* end getname() */


static void hexdump(U8 *in, char *out, U16 length)
{
    U16   i;

    for (i = 0; i < length; i++)
        out += sprintf(out, "%02X ", (unsigned char)in[i]);

}

static int dumpMsg(char *header, U8 *buffer, S16 len)
{
    char line[100];

    printf("\n%16s", header);
    do
    {
        hexdump(buffer, line, (U16)((len >= 16) ? 16 : (len % 16)));
        printf("%s\n", line);
        buffer += 16;
        printf("%s", "                ");
    } while ((len -= 16) > 0);
    printf("\n");


    return(0);
}

/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval( struct lookupEnt *tbl, char *s, short *val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( strcmp( tbl->name, s ) == 0 )
		{
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed */
	return( 0 );
}	/* end getval() */

void printBool(char *label, Bool bool)
{
    printf("%s%s\n", label, ((bool) ? "True" : "False"));
}

void printAddr(char *label, TUPAddr *addr)
{
    if (addr->length)
        printf("%s%*s\n", label, addr->length, addr->string);
    else
        printf("%sNot Present\n", label);
}

void printTimer(char *label, TimerCfg *timer)
{
    printf("%s% 3d, %s\n", label, timer->value,((timer->enable) ? "Enabled" : "Disabled"));
}

void printPC(char *label, U32 pc)
{
    printf("%s%d.%d.%d\n", label, ((pc >> 16) & 0xff), ((pc >> 8) & 0xff), (pc & 0xff));
}

void traceReq( short *parms )
{

    S16     status;

    if ((status = TUPTraceControl(cp, (Bool)parms[1], parms[0])) != TUP_SUCCESS)
        printf( "Board %d Trace Control Request failed: status = %d\n",
                cp, parms[0], status );
}



void resetReq( short *parms )
{
    S16     status;

    switch (parms[0])
    {
        case 1:
            if ((status = TUPResetCircuit(cp, parms[1])) != TUP_SUCCESS)
                printf( "Board %d Circuit %d Reset Request failed: status = %d\n", cp, parms[1], status );
            break;

        case 2:
            if ((status = TUPResetGroup(cp, parms[1])) != TUP_SUCCESS)
                printf( "Board %d Group %d Reset Request failed: status = %d\n", cp, parms[1], status );
            break;
    }
}

void deleteReq( short *parms )
{
    S16     status;

    switch (parms[0])
    {
        case 1:
            if ((status = TUPDeleteCircuit(cp, parms[1], parms[2])) != TUP_SUCCESS)
                printf( "Board %d Circuit %d Delete Request failed: status = %d\n", cp, parms[1], status );
            break;

        case 2:
            if ((status = TUPDeleteGroup(cp, parms[1])) != TUP_SUCCESS)
                printf( "Board %d Group %d Delete Request failed: status = %d\n", cp, parms[1], status );
            break;
    }
}

void blockReq( short *parms )
{
    S16     status;

    if ((status = TUPBlockCircuit(cp, parms[0])) != TUP_SUCCESS)
        printf( "Board %d Circuit %d Block Request failed: status = %d\n", cp, parms[0], status );
}

void mblockReq( short *parms )
{
    S16     status;


    if ((status = TUPBlockGroup(cp, parms[0], TUP_MBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Maintenance Block Request failed: status = %d\n", cp, parms[0], status );
}

void hblockReq( short *parms )
{
    S16     status;


    if ((status = TUPBlockGroup(cp, parms[0], TUP_HBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Hardware Block Request failed: status = %d\n", cp, parms[0], status );
}

void sblockReq( short *parms )
{
    S16     status;


    if ((status = TUPBlockGroup(cp, parms[0], TUP_SBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Software Block Request failed: status = %d\n", cp, parms[0], status );
}

void unblockReq( short *parms )
{
    S16     status;

    if ((status = TUPUnblockCircuit(cp, parms[0])) != TUP_SUCCESS)
        printf( "Board %d Circuit %d Block Request failed: status = %d\n", cp, parms[0], status );
}

void munblockReq( short *parms )
{
    S16     status;

    if ((status = TUPUnblockGroup(cp, parms[0], TUP_MUNBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Maintenance Unblock Request failed: status = %d\n", cp, parms[0], status );
}

void hunblockReq( short *parms )
{
    S16     status;

    if ((status = TUPUnblockGroup(cp, parms[0], TUP_HUNBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Hardware Unblock Request failed: status = %d\n", cp, parms[0], status );
}

void sunblockReq( short *parms )
{
    S16     status;

    if ((status = TUPUnblockGroup(cp, parms[0], TUP_SUNBLOCK)) != TUP_SUCCESS)
        printf( "Board %d  %d Group Software Unblock Request failed: status = %d\n", cp, parms[0], status );
}

/*
 * statusReq command handler
 */
void statusReq( short *parms )
{
    S16             status;

    switch (parms[0])
    {
        case 1:
        {
            TUPCircStatus  sts;

            status = TUPGetCircStatus(cp, parms[1], &sts);
            printf("TUPGetCircStatus returned %d\n", status);
            if (status != TUP_SUCCESS)
                printf( "Board %d Circuit %d Status Request failed: status = %d\n",
                        cp, parms[1], status );
            else
            {
                if (sts.callState)
                  printf("Circuit %d :\n\t circuitState: %s \n\t callState:    %s\n\n",
                        parms[1], circuitState[sts.circuitState - 0x200],
                                  callState[(sts.callState >> 8) - 3][sts.callState & 0xff]);
                else /* if there is no call state, say so */
                  printf("Circuit %d :\n\t circuitState: %s \n\t callState:    %s\n\n",
                        parms[1], circuitState[sts.circuitState - 0x200], "N/A");

            }
            break;
        }

        case 2:
        {
            TUPGrpStatus  sts;
            if ((status = TUPGetGrpStatus(cp, parms[1], &sts)) != TUP_SUCCESS)
                printf( "Board %d Group %d Status Request failed: status = %d\n",
                        cp, parms[1], status );
            else
            {
                printf("Group %d:\n", parms[1]);
                printf("\t dpc: 0x%0x\t opc: 0x%0x\t cic: %d\t range: %d\t count: %d\n",
                       sts.dpc,     sts.altOpc,      sts.cic,  sts.range,  sts.count);
            }
            break;
        }
    }
}

/*
 * statsReq command handler
 */
void statsReq( short *parms )
{
    S16             status;

    switch (parms[0])
    {
        case 0:                         /* NSAP */
        {
            TUPNSapStats sts;

            if ((status = TUPGetNSapStats(cp, parms[1], &sts, 0)) != TUP_SUCCESS)
            {
                printf( "Board %d NSAP %d Statistics Request failed: status = %d\n",
                        cp, parms[1], status );
                return;
            }
            printf("pktTx        : % 8ld   ", sts.pktTx);
            printf("pktRx        : % 8ld   ", sts.pktRx);
            printf("\n");
            break;
        }

        case 1:                         /* Circuit */
        {
            TUPCircStats sts;

            if ((status = TUPGetCircStats(cp, parms[1], &sts, 0)) != TUP_SUCCESS)
            {
                printf( "Board %d Circuit %d Statistics Request failed: status = %d\n",
                        cp, parms[0], status );
                return;
            }
            printf("blockTx      : % 8ld   ", sts.blockTx );
            printf("blockAckTx   : % 8ld   ", sts.blockAckTx );
            printf("\n");

            printf("unblockTx    : % 8ld   ", sts.unblockTx );
            printf("unblockAckTx : % 8ld   ", sts.unblockAckTx );
            printf("\n");

            printf("cirResTx     : % 8ld   ", sts.cirResTx );
            printf("adrCmpltTx   : % 8ld   ", sts.adrCmpltTx);
            printf("answerTx     : % 8ld   ", sts.answerTx);
            printf("\n");

            printf("contiTx      : % 8ld   ", sts.contiTx);
            printf("conChkReqTx  : % 8ld   ", sts.conChkReqTx);
            printf("\n");

            printf("fotTx        : % 8ld   ", sts.fotTx);
            printf("clfTx        : % 8ld   ", sts.clfTx);
            printf("cclTx        : % 8ld   ", sts.cclTx);
            printf("\n");

            printf("rlgTx        : % 8ld   ", sts.rlgTx);
            printf("cbkTx        : % 8ld   ", sts.cbkTx);
            printf("ranTx        : % 8ld   ", sts.ranTx);
            printf("\n");

            printf("oprTx        : % 8ld   ", sts.oprTx);
            printf("mpmTx        : % 8ld   ", sts.mpmTx);
            printf("accTx        : % 8ld   ", sts.accTx);
            printf("\n");

            printf("ubmTx        : % 8ld   ", sts.ubmTx);
            printf("initAdrTx    : % 8ld   ", sts.initAdrTx);
            printf("subsAdrTx    : % 8ld   ", sts.subsAdrTx);
            printf("\n");

            printf("infoTx       : % 8ld   ", sts.infoTx);
            printf("infoReqTx    : % 8ld   ", sts.infoReqTx);
            printf("\n");

            printf("malTx        : % 8ld   ", sts.malTx);
            printf("chargeTx     : % 8ld   ", sts.chargeTx);
            printf("\n");

            printf("blockRx      : % 8ld   ", sts.blockRx );
            printf("blockAckRx   : % 8ld   ", sts.blockAckRx );
            printf("\n");
            printf("unblockRx    : % 8ld   ", sts.unblockRx );
            printf("unblockAckRx : % 8ld   ", sts.unblockAckRx );
            printf("\n");
            printf("cirResRx     : % 8ld   ", sts.cirResRx );
            printf("adrCmpltRx   : % 8ld   ", sts.adrCmpltRx);
            printf("answerRx     : % 8ld   ", sts.answerRx);
            printf("\n");
            printf("contiRx      : % 8ld   ", sts.contiRx);
            printf("conChkReqRx  : % 8ld   ", sts.conChkReqRx);
            printf("\n");
            printf("fotRx        : % 8ld   ", sts.fotRx);
            printf("clfRx        : % 8ld   ", sts.clfRx);
            printf("cclRx        : % 8ld   ", sts.cclRx);
            printf("\n");
            printf("rlgRx        : % 8ld   ", sts.rlgRx);
            printf("cbkRx        : % 8ld   ", sts.cbkRx);
            printf("ranRx        : % 8ld   ", sts.ranRx);
            printf("\n");
            printf("oprRx        : % 8ld   ", sts.oprRx);
            printf("mpmRx        : % 8ld   ", sts.mpmRx);
            printf("accRx        : % 8ld   ", sts.accRx);
            printf("\n");
            printf("ubmRx        : % 8ld   ", sts.ubmRx);
            printf("initAdrRx    : % 8ld   ", sts.initAdrRx);
            printf("\n");
            printf("infoRx       : % 8ld   ", sts.infoRx);
            printf("infoReqRx    : % 8ld   ", sts.infoReqRx);
            printf("subsAdrRx    : % 8ld   ", sts.subsAdrRx);
            printf("\n");
            printf("malRx        : % 8ld   ", sts.malRx);
            printf("chargeRx     : % 8ld   ", sts.chargeRx);
            printf("\n");
            break;
        }

        case 2:                         /* Group */
        {
            TUPGrpStats sts;

            if ((status = TUPGetGrpStats(cp, parms[1], &sts, 0)) != TUP_SUCCESS)
            {
                printf( "Board %d Group %d Statistics Request failed: status = %d\n",
                        cp, parms[0], status );
                return;
            }
            printf("cirHwGrBlockRx      : %8ld     ", sts.cirHwGrBlockRx);
            printf("cirHwGrBlockAckRx   : %8ld     ", sts.cirHwGrBlockAckRx);
            printf("\n");
            printf("cirHwGrUnBlockRx    : %8ld     ", sts.cirHwGrUnBlockRx);
            printf("cirHwGrUnBlockAckRx : %8ld     ", sts.cirHwGrUnBlockAckRx);
            printf("\n");
            printf("cirSwGrBlockRx      : %8ld     ", sts.cirSwGrBlockRx);
            printf("cirSwGrBlockAckRx   : %8ld     ", sts.cirSwGrBlockAckRx);
            printf("\n");
            printf("cirSwGrUnBlockRx    : %8ld     ", sts.cirSwGrUnBlockRx);
            printf("cirSwGrUnBlockAckRx : %8ld     ", sts.cirSwGrUnBlockAckRx);
            printf("\n");
            printf("cirMGrBlockRx       : %8ld     ", sts.cirMGrBlockRx);
            printf("cirMGrBlockAckRx    : %8ld     ", sts.cirMGrBlockAckRx);
            printf("\n");
            printf("cirMGrUnBlockRx     : %8ld     ", sts.cirMGrUnBlockRx);
            printf("cirMGrUnBlockAckRx  : %8ld     ", sts.cirMGrUnBlockAckRx);
            printf("\n");
            printf("cirGrResRx;         : %8ld     ", sts.cirGrResRx);
            printf("cirGrResAckRx;      : %8ld     ", sts.cirGrResAckRx);
            printf("\n");
            printf("\n");
            printf("cirHwGrBlockTx      : %8ld     ", sts.cirHwGrBlockTx);
            printf("cirHwGrBlockAckTx   : %8ld     ", sts.cirHwGrBlockAckTx);
            printf("\n");
            printf("cirHwGrUnBlockTx    : %8ld     ", sts.cirHwGrUnBlockTx);
            printf("cirHwGrUnBlockAckTx : %8ld     ", sts.cirHwGrUnBlockAckTx);
            printf("\n");
            printf("cirSwGrBlockTx      : %8ld     ", sts.cirSwGrBlockTx);
            printf("cirSwGrBlockAckTx   : %8ld     ", sts.cirSwGrBlockAckTx);
            printf("\n");
            printf("cirSwGrUnBlockTx    : %8ld     ", sts.cirSwGrUnBlockTx);
            printf("cirSwGrUnBlockAckTx : %8ld     ", sts.cirSwGrUnBlockAckTx);
            printf("\n");
            printf("cirMGrBlockTx       : %8ld     ", sts.cirMGrBlockTx);
            printf("cirMGrBlockAckTx    : %8ld     ", sts.cirMGrBlockAckTx);
            printf("\n");
            printf("cirMGrUnBlockTx     : %8ld     ", sts.cirMGrUnBlockTx);
            printf("cirMGrUnBlockAckTx  : %8ld     ", sts.cirMGrUnBlockAckTx);
            printf("\n");
            printf("cirGrResTx;         : %8ld     ", sts.cirGrResTx);
            printf("cirGrResAckTx;      : %8ld     ", sts.cirGrResAckTx);
            printf("\n");
            break;
        }
    }

}

/*
 * getConfigReq command handler
 */
void getConfigReq( short *parms )
{
    S16             status;

    switch (parms[0])
    {
        case 1:                         /* General */
        {
            TUPGenParms cfg;

            if ((status = TUPGetGenCfg(cp, &cfg)) != TUP_SUCCESS)
            {
                printf( "Board %d GENERAL: Get Configuration Request failed: status = %d\n",
                        cp, parms[1], status );
                return;
            }

            /* Display the high availability and mate connection states */
            switch (cfg.haState) {
               case TUP_ST_HAST_STARTING:
                  printf("High Availability State: STARTING\n");
                  break;
               case TUP_ST_HAST_STANDALONE:
                  printf("High Availability State: STANDALONE\n");
                  break;
               case TUP_ST_HAST_PRIMARY:
                  printf("High Availability State: PRIMARY\n");
                  break;
               case TUP_ST_HAST_BACKUP:
                  printf("High Availability State: BACKUP\n");
                  break;
               default:
                  printf("High Availability State: UNKNOWN STATE\n");
                  break;
            }

            switch (cfg.mcState) {
               case TUP_ST_MCST_ISOLATED:
                  printf("Mate Connection State: ISOLATED\n");
                  break;
               case TUP_ST_MCST_CONNECTED:
                  printf("Mate Connection State: CONNECTED\n");
                  break;
               default:
                  printf("Mate Connection State: UNKNOWN\n");
                  break;
            }

            printf("maxSaps      : %d\n",  cfg.maxSaps);
            printf("maxNetSaps   : %d\n",  cfg.maxNetSaps);
            printf("maxCircuits  : %d\n",  cfg.maxCircuits);
            printf("txmonTimeout : %d\n",  cfg.txmonTimeout);
            printf("maxCircGrp   : %d\n",  cfg.maxCircGrp);
            printf("maxDpcs      : %d\n",  cfg.maxDpcs);
            printf("maxOpcs      : %d\n",  cfg.maxOpcs);
            printf("maxRoutes    : %d\n",  cfg.maxRoutes);
            printTimer("Puase timer:", &cfg.MtpPauseTmr);
            printTimer("t20      : ", &cfg.dfltGrpCfg.t20);
            printTimer("t21      : ", &cfg.dfltGrpCfg.t21);
            printTimer("t22      : ", &cfg.dfltGrpCfg.t22);
            printTimer("t23      : ", &cfg.dfltGrpCfg.t23);
            printTimer("t24      : ", &cfg.dfltGrpCfg.t24);
            /*printTimer("t25      : ", &cfg.dfltGrpCfg.t25); */
            printTimer("t26      : ", &cfg.dfltGrpCfg.t26);
            printTimer("t27      : ", &cfg.dfltGrpCfg.t27);
            printTimer("t28      : ", &cfg.dfltGrpCfg.t28);
            printTimer("t29      : ", &cfg.dfltGrpCfg.t29);
            printTimer("t30      : ", &cfg.dfltGrpCfg.t30);
            printTimer("t31      : ", &cfg.dfltGrpCfg.t31);
            printTimer("t32      : ", &cfg.dfltGrpCfg.t32);
            printTimer("t33      : ", &cfg.dfltGrpCfg.t33);
            printTimer("t34      : ", &cfg.dfltGrpCfg.t34);
            printTimer("t35      : ", &cfg.dfltGrpCfg.t35);
            printTimer("t36      : ", &cfg.dfltGrpCfg.t36);
            printTimer("t37      : ", &cfg.dfltGrpCfg.t37);
            printTimer("t38      : ", &cfg.dfltGrpCfg.t38);
            printTimer("t39      : ", &cfg.dfltGrpCfg.t39);
            printTimer("t40      : ", &cfg.dfltGrpCfg.t40);
            printTimer("t41      : ", &cfg.dfltGrpCfg.t41);

            break;
        }
        case 2:                         /* USAP */
        {
            TUPUSapCfg cfg;

            if ((status = TUPGetUSapCfg(cp, &cfg, parms[1])) != TUP_SUCCESS)
            {
                printf( "Board %d USAP %d Get Configuration Request failed: status = %d\n",
                        cp, parms[1], status );
                return;
            }
            printf("switchType     : %d\n",  cfg.switchType);
            break;
        }
        case 3:                         /* NSAP */
        {
            TUPNSapCfg cfg;

            if ((status = TUPGetNSapCfg(cp, &cfg, parms[1])) != TUP_SUCCESS)
            {
                printf( "Board %d NSAP %d Get Configuration Request failed: status = %d\n",
                        cp, parms[1], status );
                return;
            }
            printf("switchType  : %d\n", cfg.switchType);
            printf("spId        : %d\n", cfg.spId);
            printf("mtpSpId     : %d\n", cfg.mtpSpId);
            printf("ssf         : %d\n", cfg.ssf);
            printf("priority    : %d\n", cfg.priority);
            break;
        }
        case 4:                         /* Circuit */
        {
            TUPCircCfg cfg;

            if ((status = TUPGetCircCfg(cp, &cfg, (U32)parms[1])) != TUP_SUCCESS)
            {
                printf( "Board %d Circuit %d Get Configuration Request failed: status = %d\n",
                        cp, parms[0], status );
                return;
            }
            printf    ("circuitId    : %ld\n", cfg.circuitId);
            printf    ("cic          : %ld\n", cfg.cic);
            printf    ("dstPointCode : 0x%0x\n",cfg.dstPointCode);
            printf    ("orgPointCode : 0x%0x\n",cfg.altOrgPointCode);

            printf    ("numCircs     : %d\n",  cfg.numCircs);
            printTimer("t1           : ", &cfg.t1);
            printTimer("t2           : ", &cfg.t2);
            printTimer("t3           : ", &cfg.t3);
            printTimer("t4           : ", &cfg.t4);
            printTimer("t5           : ", &cfg.t5);
            printTimer("t6           : ", &cfg.t6);
            printTimer("t7           : ", &cfg.t7);
            printTimer("t8           : ", &cfg.t8);
            printTimer("t9           : ", &cfg.t9);
            printTimer("t10          : ", &cfg.t10);
            printTimer("t11          : ", &cfg.t11);
            printTimer("t12          : ", &cfg.t12);
            printTimer("t13          : ", &cfg.t13);
            printTimer("t14          : ", &cfg.t14);
            printTimer("t15          : ", &cfg.t15);
            printTimer("t16          : ", &cfg.t16);
            printTimer("t17          : ", &cfg.t17);
            printTimer("t18          : ", &cfg.t18);
            printTimer("t19          : ", &cfg.t19);
            printTimer("t25          : ", &cfg.t25);
            break;
        }
        case 5:                         /* General */
        {
            TUPGrpCfg cfg;

            if ((status = TUPGetGrpCfg(cp, &cfg, parms[1])) != TUP_SUCCESS)
            {
                printf( "Board %d GROUP: Get Configuration Request failed: status = %d\n",
                        cp, parms[1], status );
                return;
            }

            printf    ("dpc     : 0x%0x\n",cfg.dpc);
            printf    ("opc     : 0x%0x\n",cfg.altOpc);
            printf    ("cic     : %d\n",  cfg.cic);
            printf    ("count   : %d\n",  cfg.count);
            printf    ("range   : %ld\n", cfg.range);
            printf    ("circId  : %ld\n", cfg.circId);
            printf    ("groupId : %d\n",  cfg.groupId);
            printTimer("t20     : ",     &cfg.t20);
            printTimer("t21     : ",     &cfg.t21);
            printTimer("t22     : ",     &cfg.t22);
            printTimer("t23     : ",     &cfg.t23);
            printTimer("t24     : ",     &cfg.t24);
            /*printTimer("t25     : ",     &cfg.t25);*/
            printTimer("t26     : ",     &cfg.t26);
            printTimer("t27     : ",     &cfg.t27);
            printTimer("t28     : ",     &cfg.t28);
            printTimer("t29     : ",     &cfg.t29);
            printTimer("t30     : ",     &cfg.t30);
            printTimer("t31     : ",     &cfg.t31);
            printTimer("t32     : ",     &cfg.t32);
            printTimer("t33     : ",     &cfg.t33);
            printTimer("t34     : ",     &cfg.t34);
            printTimer("t35     : ",     &cfg.t35);
            printTimer("t36     : ",     &cfg.t36);
            printTimer("t37     : ",     &cfg.t37);
            printTimer("t38     : ",     &cfg.t38);
            printTimer("t39     : ",     &cfg.t39);
            printTimer("t40     : ",     &cfg.t40);
            printTimer("t41     : ",     &cfg.t41);

            break;
        }
    }

}

void HelpCommand( short *parms )
{
	printf( "%s", helpTbl[*parms] );
}

void setCP( short *parms )
{
    unsigned char       newcp;
    S16                 status;

    newcp = (unsigned char)parms[0];

	if( newcp < 1 || newcp > 8 )
	{
		printf( "CP must be in range 1..8\n" );
	}

	/* open/init the MTP3 mgmt API for new board*/
    status = TUPInitMgmtAPI(newcp, TUPMGR_ENT);
    if( status != TUP_SUCCESS )
	{
        printf( "Board %d TUPInitMgmtAPI failed [%d]\n", newcp, status );
	}
	else
	{
		/* successful, close old cp & set new cp as the current cp */
        status = TUPTermMgmtAPI(cp);
		cp = newcp;
	}
}

void quit( short *parms )
{
    UNREFERENCED_PARAMETER(parms);
	quittinTime = 1;
}


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
	short 				i, j;
	short				found;
	short 				params[MAX_PARMS];	/* numeric parameters array */
	struct lookupEnt	*lp;

	/* first search command table for command match */
	for( i = 0; CmdTable[i] != 0; i++ )
	{
		if( strcmp( tokens[0], CmdTable[i]->command ) == 0 )
		{
			break;		/* command matched */
		}
	}	/* end for */

	/* if no match found display error message and return */
	if( CmdTable[i] == 0 )
	{
		printf("Unrecognized command: %s\n", tokens[0] );
		return;
	}

	/* convert remaining tokens (i.e., parameters) to short integers,
	 * using lookup tables  where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
	for( j = 0; j < MAX_PARMS; j++ )
		params[j] = 0;					/* first initialize to defaults */

	for( j = 1; j < cnt; j++, lp++ )
	{
		lp = CmdTable[i]->parmTbl[j - 1];
		if( lp )
		{	/* lookup table for this param ? */
			found = getval( lp, tokens[j], &params[j - 1] );
			if( found )
			{
				continue;				/* parameter succesfully converted */
			}
		}

		/* no lookup table or lookup failed - try to convert as a number */
		if( sscanf( tokens[j], "%hi", &params[j - 1] ) != 1 )
		{
			/* conversion failed */
			printf("Unrecognized parameter: %s\n", tokens[j] );
			return;
		}
	}	/* end for */

	/* call command handler function */

	if( CmdTable[i]->func != 0 )
	{
		CmdTable[i]->func( params );
	}
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}

void usage( void )
{
    printf("\n usage: TUPmgr -b [1] \n");
	printf("              -b [CP Board number]\n");
	exit(1);
}

int main( int argc, char *argv[] )
{

	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	short	cnt;
    short   tmp;
	short	i;
	char	*p;
    short   status;

    printf("\n\n%s: sample TUP management application version %s %s\n\n",
            argv[0], TUPMGR_VERSION, __DATE__);

	/* parse command line for interrupt & cp # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
            switch (*++p)
			{
			case 'b' :    /* Set Board number */
                if (argc == 2)
                {
                    if ((sscanf(p, "%hi", &tmp) != 1))
                        usage();
                    cp = (unsigned char)tmp;
                    if ((cp < 1) || (cp > 8))
                        usage();
                }
                else
                {
                    i++;
                    if ((sscanf(argv[i], "%hi", &tmp) != 1))
                        usage();
                    cp = (unsigned char)tmp;
                    if ((cp < 1) || (cp > 8))
                        usage();
                }
				break;

			default:
				usage();
				break;
			}	/* end switch */
		}
		else
		{
			usage();
		}

	}	/* end for */

	/* open/init the MTP3 mgmt API */
    status = TUPInitMgmtAPI(cp, TUPMGR_ENT);
    if( status != TUP_SUCCESS )
	{
        printf( "TUPInitMgmtAPI failed [0x%x]\n", status );
		exit(1);
	}

	/* command processing loop */
    printf( "TUPmgr[%-1u]>", cp );
    while( (quittinTime == 0) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
		cnt = tokenize( buffer, tokens );
		if( cnt > 0 )
			processCmd( tokens, cnt );
        printf( "TUPmgr[%-1u]>", cp );
	}	/* end while */

    status = TUPTermMgmtAPI(cp);

    return( 0 );
}	/* end main() */
